#!/bin/sh
# =====================================================================
# This file is part of MOSIC (R). MOSIC is a System@tic project.
# 
# MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
# developed by THALES and SELEX SISTEMI INTEGRATI.
# CARDAMOM is derivative work based on PERCO.
# Copyright (C) THALES 2009-2010. All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
# License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

# This script creates the cdmw_env file.
# All required environment variables have to be specified before starting this script. 
# (we supposed it's done by calling scripts or startCdmw.sh)
# If --binaryversion option is specified, CDMW_HOME is not set to a placeholder in cdmw_env.
# If --testversion option is specified, some additional variables for 
# validation and qualification tests are added in cdmw_env.

if [ -z "$CDMW_HOME" ]; then
    echo "ERROR: \$CDMW_HOME must be defined"
    exit 1
fi

script_name=`basename $0`

print_usage() {
    echo "Usage: $script_name [--compilationversion] [--binaryversion] [--testversion] <output_dir>"
}


if [ "$1" = "--compilationversion" ]; then
    COMPIL_VERSION=true
    shift
fi
    
if [ "$1" = "--binaryversion" ]; then
    BIN_VERSION=true
    shift
fi
    
if [ "$1" = "--testversion" ]; then
    TEST_VERSION=true
    shift
fi

if [ -z "$1" ]; then
    print_usage
    exit 1
fi

if [ ! -d "$1" ]; then
    echo "ERROR: $1 is not a valid directory"
    print_usage
    exit 1
fi

OUTPUT_DIR=$1
CDMW_ENV_FILE=$OUTPUT_DIR/cdmw_env


if [ "$BIN_VERSION" = "true" ]; then
    DATE="@date@"
    FUTUR_CDMW_HOME="@CDMW_HOME@"
else
    DATE=`date`
    FUTUR_CDMW_HOME=$OUTPUT_DIR
fi



#
# Generate common content of file
#
cat <<EOF > $CDMW_ENV_FILE
#
# Environment file generated at: $DATE
#

export CDMW_HOME=$FUTUR_CDMW_HOME

export ACE_ROOT=$ACE_ROOT
export TAO_ROOT=$TAO_ROOT
export PERL5LIB=$TAO_ROOT/src/bin
export CIAO_ROOT=$TAO_ROOT
export XERCESCROOT=$XERCESCROOT

export JAVA_HOME=$JAVA_HOME
export JACORB_HOME=$JACORB_HOME
export SAXON_HOME=$SAXON_HOME

export PATH=\$CDMW_HOME/bin:\$ACE_ROOT/bin:\$TAO_ROOT/orbsvcs/Naming_Service:\$TAO_ROOT/orbsvcs/IFR_Service:\$JAVA_HOME/bin:\$PATH
export LD_LIBRARY_PATH=\$CDMW_HOME/lib/c++:\$ACE_ROOT/lib:\$XERCESCROOT/lib:\$LD_LIBRARY_PATH

export PERL5LIB=$TAO_ROOT/src/bin



for i in \`ls \$JACORB_HOME/lib/endorsed/*.jar\`; do
    export CLASSPATH=\$i:\$CLASSPATH
done
# Note: SAXON jars are set in CLASSPATH by appropriate .sh files

# Add Cardamom jar to CLASSPATH if compiled
# (if configure has just been done, they're not compiled yet)
for i in \`ls \$CDMW_HOME/lib/java/*.jar 2>/dev/null\`; do
    export CLASSPATH=\$i:\$CLASSPATH
done

EOF
#
# End of common content
#


if [ "$COMPIL_VERSION" = "true" ]; then
    #
    # Generate specifiec content for compilation
    #
cat <<EOF >> $CDMW_ENV_FILE
#
# Extra variables for compilation
#
export ANT_HOME=$ANT_HOME
export PATH=\$ANT_HOME/bin:\$PATH
for i in \`ls \$ANT_HOME/lib/*.jar\`; do
    export CLASSPATH=\$i:\$CLASSPATH   
done

export JUNIT_HOME=$JUNIT_HOME
for i in \`ls \$JUNIT_HOME/*.jar\`; do
    export CLASSPATH=\$i:\$CLASSPATH   
done

export CPPUNIT_ROOT=$CPPUNIT_ROOT
export LD_LIBRARY_PATH=\$LD_LIBRARY_PATH:\$CPPUNIT_ROOT/lib

export EMMA_HOME=$EMMA_HOME
if [ -n "\$EMMA_HOME" ]; then
    for i in \`ls \$EMMA_HOME/lib/*.jar\`; do
        export CLASSPATH=\$i:\$CLASSPATH
    done
fi


export CODEGENERATOR_HOME=$CODEGENERATOR_HOME
if [ -n "\$CODEGENERATOR_HOME" ]; then
    for i in \`ls \$CODEGENERATOR_HOME/*.jar\`; do
        export CLASSPATH=\$i:\$CLASSPATH
    done
fi


export CDMW_INSTALL_DIR=$CDMW_INSTALL_DIR 

export CDMW_PACKAGES_LIST=$CDMW_PACKAGES_LIST
export CDMW_CPP_DEBUG=$CDMW_CPP_DEBUG
export CDMW_CPP_OPTIM=$CDMW_CPP_OPTIM
export CDMW_CPP_GCOV=$CDMW_CPP_GCOV
export CDMW_JAVA_EMMA=$CDMW_JAVA_EMMA
export CDMW_TRACE_LEVEL=$CDMW_TRACE_LEVEL
export CDMW_CALL_TRACES=$CDMW_CALL_TRACES

## Compilation functions

# for compilation of all makefiles in sub-directories
function compileCSC ()
{
    list=
    # find all GNUmakefiles in current dir
    for i in \`find . -type f -name "GNUmakefile.*"\`; do
        # get target name (extension of GNUmakefile)
        target=\`basename \$i | sed s%GNUmakefile.%%g\`
        #add it to the list
        list="\$target \$list"
    done

    # default makefile to call:
    makefile=GNUmakefile
    # test if we're in a valid or qualif directory
    if [ "\${PWD/\/valid\//}" != \${PWD} ]; then
        makefile=GNUmakefile.validation
    fi
    if [ "\${PWD/\/qualif\//}" != \${PWD} ]; then
        makefile=GNUmakefile.qualification
    fi

    # call make for targets list
    echo make -f \$CDMW_HOME/Build/MPC/\$makefile \$list
    make -f \$CDMW_HOME/Build/MPC/\$makefile \$list
}

function compileCppUT_exclude_testftccmcif ()
{
    list=
    # find all GNUmakefiles in current dir
    for i in \`find ./tests -not \( -name *testftccmcif_ComponentServer* -o -name *testftccmcif_FtCcmCifExecutor* -o -name *testftccmcif_mainComponentServer* -o -name *testftccmcif_main* \) -name "*GNUmakefile*"\`; do
        # get target name (extension of GNUmakefile)
        target=\`basename \$i | sed s%GNUmakefile.%%g\`
        #add it to the list
        list="\$target \$list"
    done

    # default makefile to call:
    makefile=GNUmakefile

    echo "WARNING: not compiling testftccmcif due to problems with the buildsystem"   
    echo make -f \$CDMW_HOME/Build/MPC/\$makefile \$list
    make -f \$CDMW_HOME/Build/MPC/\$makefile \$list
}

function checkBuildFile()
{
    if [ ! -f ./build.xml ] 
        then echo 'Build file build.xml not found in the current directory. Please move to the CSCI directory and retry.';
             return 1
    fi 
    return 0
}

function compileJavaCSC ()
{   
    checkBuildFile
    if [ "\$?" -eq 0 ] 
        then
            echo 'Compiling the java CSCI';
            ant compile-csci
    fi
}

function cleanJavaCSC ()
{   
    checkBuildFile
    if [ "\$?" -eq 0 ] 
        then
            echo 'Cleaning the java CSCI and its unit tests'
            ant clean
            ant clean-tests
    fi
}

function compileJavaUT ()
{
    checkBuildFile
        if [ "\$?" -eq 0 ] 
        then
            echo 'Compiling the java unit tests of the current CSCI'
            ant phase-test-compile
    fi
}

function cleanJavaUT ()
{
    checkBuildFile
        if [ "\$?" -eq 0 ] 
        then
            echo 'Compiling the java unit tests of the current CSCI'
            ant clean-tests
    fi
}
EOF
    #
    # End of specific content for compilation
    #
fi


if [ "$TEST_VERSION" = "true" ]; then
    #
    # Generate specific content for tests
    #
cat <<EOF >> $CDMW_ENV_FILE
#
# Extra variables for validation or qualification tests
#
export JUNIT_HOME=$JUNIT_HOME
for i in \`ls \$JUNIT_HOME/*.jar\`; do
    export CLASSPATH=\$i:\$CLASSPATH   
done

export CPPUNIT_ROOT=$CPPUNIT_ROOT
export LD_LIBRARY_PATH=\$LD_LIBRARY_PATH:\$CPPUNIT_ROOT/lib

export EMMA_HOME=$EMMA_HOME
if [ -n "\$EMMA_HOME" ]; then
    for i in \`ls \$EMMA_HOME/lib/*.jar\`; do
        export CLASSPATH=\$i:\$CLASSPATH
    done
fi

export CDMW_HOSTNAME_1=$CDMW_HOSTNAME_1
export CDMW_HOSTNAME_2=$CDMW_HOSTNAME_2
export CDMW_HOSTNAME_3=$CDMW_HOSTNAME_3
export CDMW_HOSTNAME_4=$CDMW_HOSTNAME_4
export CDMW_HOSTNAME_5=$CDMW_HOSTNAME_5
export CDMW_CPP_ORB=$CDMW_CPP_ORB
export CDMW_CPP_ORB_VERSION=$CDMW_CPP_ORB_VERSION
export CDMW_JAVA_ORB=$CDMW_JAVA_ORB

export PATH=\$PATH:\$TAO_ROOT/orbsvcs/Naming_Service
    
EOF
    #
    # End of specific content for tests
    #

fi

