#!/bin/sh
# =====================================================================
# This file is part of MOSIC (R). MOSIC is a System@tic project.
# 
# MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
# developed by THALES and SELEX SISTEMI INTEGRATI.
# CARDAMOM is derivative work based on PERCO.
# Copyright (C) THALES 2009-2010. All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
# License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

#check if environment variable is set
if [ -z "$CDMW_HOME" ]; then
    echo "ERROR: \$CDMW_HOME is not defined"
    exit 1
fi

if [ ! -d $CDMW_HOME ]; then
    echo "ERROR: \$CDMW_HOME ($CDMW_HOME) is not a valid directory"
    exit 1
fi

#set initial values for variables
docs_dir=$CDMW_INSTALL_DIR/Docs
file_patterns=""
product_version=`grep -w CDMW $CDMW_HOME/Build/data/ReleaseProperties.xml | cut -d'"' -f4`
perl_dir=`which perl`

extract_prv=""
cscs=""
CSCI_LIST="CCMContainer     \
           CCMDeployment    \
           CodeGen          \
           ConfAndPlug      \
           Event            \
           FaultTolerance   \
           Foundation       \
           LifeCycle        \
           LoadBalancing    \
           Repository       \
           SystemMngt       \
           Time             \
           TraceAndPerf     \
           XMLServer"

#get param
if [ -n "$1" ]; then
    if [ "$1" == "genpublicdoc" ]; then
	is_public_doc="true"
    else
	if [ "$1" == "genfulldoc" ]; then
	    is_public_doc="false"
	else
	    echo "ERROR: Incorrect param name. It should be genpublicdoc or genfulldoc"
	    exit 1
	fi    
    fi
else
    echo "ERROR: Param (genpublicdoc or genfulldoc) is not defined"
    exit 1
fi


#
#Prepare data for C++ documentation generation
#
prepare_gendoc_cpp() {
    #set pattern for generation
    file_patterns="*.h *.hpp *.inl *.idl"
    
    #set list of cscs for generation
    for i in $CSCI_LIST; do
        csci=$CDMW_HOME/$i

	#check if csci exists
        if [ ! -d $csci ]; then
            echo "ERROR: $csci does not exist"
        fi
	

	#add include and idl dirs to $cscs
	#add public includes
        #get list of all subdirectories
        csc_list=`ls $csci/c++/include/$i 2>/dev/null`
        for csc in $csc_list; do
	    cscs=$cscs" "$csci/c++/include/$i/$csc
	done
        #get list of all subdirectories
        csc_list=`ls $csci/c++/src 2>/dev/null`
	#add private includes
	if [ "$is_public_doc" == "false" ]; then
	    for csc in $csc_list; do
		cscs=$cscs" "$csci/c++/src/$csc
	    done
	fi
	#add idl dirs if exists
	[ -d $csci/idl ] && cscs=$cscs" "$csci/idl
    done
}


#
#Prepare data for Java documentation generation
#
prepare_gendoc_java() {
    #set pattern for generation
    file_patterns="*.java *.idl"
    
    #set list of cscs for generation
    for i in $CSCI_LIST; do
        csci=$CDMW_HOME/$i

	#check if csci exists
        if [ ! -d $csci ]; then
            echo "ERROR: $csci does not exist"
        fi
	

	#add include and idl dirs to $cscs
	#add includes
        [ -d $csci/java ] && cscs=$cscs" "$csci/java
	#add idl dirs
	[ -d $csci/idl ] && cscs=$cscs" "$csci/idl
    done
}


#
#Generating documentation and index file
#
run_doxygen() {
    lang=$1
    
    cfg_file="doxygen.cfg"
    data_dir=$CDMW_HOME/Build/configuration/data
    
    #set output directory depending on type of documentation
    if [ "$is_public_doc" == "true" ]; then
	output_dir=$docs_dir/html/public/$lang
	extract_prv="NO"
    else
	output_dir=$docs_dir/html/full/$lang
	extract_prv="YES"
    fi
    
    #remove previously generated documentation
    rm -r -f -d $output_dir
    
    #create output directory
    mkdir -p $output_dir
    
    #copy doxygen.cfg to the temprorary location and replace placeholders
    cat $data_dir/$cfg_file | sed "s#\@product\.version\@#$product_version#" | sed "s#\@cscs\@#$cscs#" | sed "s#\@docdir\@#$output_dir#" | sed "s#\@perldir\@#$perl_dir#" | sed "s#\@htmlheaderdir\@#$data_dir#" | sed "s#\@extract\.prv\@#$extract_prv#" | sed "s#\@file\.patterns\@#$file_patterns#" > $docs_dir/$cfg_file

    #generate documentation
    cur_dir=`pwd`
    cd $docs_dir
    doxygen $docs_dir/$cfg_file
    
    #remove temprorary doxygen.cfg
    rm -f $docs_dir/$cfg_file
    
    #generate index
    cd $output_dir
    doxytag -t search.idx
    
    #copy needed files to documentation
    cp $data_dir/logo-doc.gif $output_dir
    
    #return to the base directory
    cd $cur_dir
    
}


#main part
prepare_gendoc_cpp
run_doxygen "c++"
prepare_gendoc_java
run_doxygen "java"
