/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// Cdmw files
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/common/Assert.hpp>
#include <Foundation/logging/CdmwLogging.stub.hpp>


// Cdmw CCM files
#include <CCMContainer/ccmcif/CCMContext_impl.hpp>
#include <CCMContainer/ccmcommon/CCMUtils.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>

namespace 
{
    const std::string UNAVAILABLE_SERVICE_IDENTIFIERS[] = {"RootPOA", 
                                                           "POACurrent", 
                                                           "ORBPolicyManager",
                                                           "PolicyCurrent",
                                                           "PICurrent"};
    // size of the table is sizeof / 4 octets
    const int UNAVAILABLE_SERVICE_IDENTIFIERS_NB 
       = sizeof(UNAVAILABLE_SERVICE_IDENTIFIERS)/sizeof(UNAVAILABLE_SERVICE_IDENTIFIERS[0]);
       
} // end anonymous namespace


namespace Cdmw {

namespace CCM {

namespace CIF {

// initialise static members

std::map<std::string, CORBA::Object_var> CCMContext_impl::m_services_map;

//
// IDL:omg.org/CdmwComponents/CCMContextExt:1.0
//  
CCMContext_impl::CCMContext_impl
(
 Components::CCMContext_ptr ctx,
 const std::string & component_id,
 CdmwLogging::LoggerFactory_ptr logger_factory
 ) : m_component_id(component_id)
{
    // create logger with domain for ccmcif
    m_logger = logger_factory->get_logger(CCMLoggers::CCMCIF_LOG_DOMAIN);
    
    CDMW_LOG_FUNCTION(m_logger.in(), "");
    
    m_context = Components::CCMContext::_narrow(ctx);
    
    if (CORBA::is_nil(m_context.in()))
    {
        // This is to avoid Assertion failure in debug mode as internal ref count
        // equals 1 (CORBA C++ mapping oblige!)
        _set_ref_count(0);
    
        throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAMWrongContextType, 
                               CORBA::COMPLETED_NO);
    }

}

CCMContext_impl::~CCMContext_impl()
{
    CDMW_LOG_FUNCTION(m_logger.in(), "");
}

//
// IDL:omg.org/Components/CCMContext/get_caller_principal:1.0
//
Components::Principal_ptr 
CCMContext_impl::get_caller_principal()
    throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger.in(), "");
    return m_context->get_caller_principal();
}

//
// IDL:omg.org/Components/CCMContext/get_CCM_home:1.0
//
Components::CCMHome_ptr  
CCMContext_impl::get_CCM_home()
    throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger.in(), "");
    return m_context->get_CCM_home();
}

//
// IDL:omg.org/Components/CCMContext/get_rollback_only:1.0
//
CORBA::Boolean  
CCMContext_impl::get_rollback_only()
    throw(Components::IllegalState,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger.in(), "");
    return m_context->get_rollback_only();
}

//
// IDL:omg.org/Components/CCMContext/get_user_transaction:1.0
//
Components::Transaction::UserTransaction_ptr  
CCMContext_impl::get_user_transaction()
    throw(Components::IllegalState,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger.in(), "");
    return m_context->get_user_transaction();
}

//
// IDL:omg.org/Components/CCMContext/is_caller_in_role:1.0
//
CORBA::Boolean  
CCMContext_impl::is_caller_in_role(const char* role)
    throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger.in(), "");
    return m_context->is_caller_in_role(role);
}

//
// IDL:omg.org/Components/CCMContext/set_rollback_only:1.0
//
void  
CCMContext_impl::set_rollback_only()
    throw(Components::IllegalState,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger.in(), "");
    m_context->set_rollback_only();
}


//
// IDL:omg.org/Components/CCMContext/resolve_service_reference:1.0
//
CORBA::Object_ptr 
CCMContext_impl::resolve_service_reference(const char* service_id) 
    throw(Components::CCMException,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger.in(), "service_id="<<service_id);
    
    // Check if service access is authorized
    std::string id_str = service_id;
    for (int i=0 ; i < UNAVAILABLE_SERVICE_IDENTIFIERS_NB ; i++)
    {
        if (id_str == UNAVAILABLE_SERVICE_IDENTIFIERS[i]) {
            CDMW_ERROR(m_logger.in(), 
                "Attempt to access unauthorized service: " << service_id);
            Components::CCMException ex(Components::OBJECT_NOT_FOUND);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
    }
    
    // try to find service in ServicesMap first
    services_map_iterator service_pos = m_services_map.find(service_id);
    if (service_pos != m_services_map.end())
    {
        CDMW_DEBUG(m_logger.in(), 
            "Service " << service_id << " found in ServiceMap");
        // service found, return it
        return CORBA::Object::_duplicate(service_pos->second.in());
    }
    
    // call CCM2Context::resolve_initial_references()
    /*
    try
    {
        CDMW_DEBUG(m_logger.in(), 
            "Service " << service_id << " not found in ServiceMap. "
            << "Try to get it from CCM2Context.");
        return m_context->resolve_initial_references(service_id);
    }
    catch (const CdmwCcmContainer::CCM2Context::InvalidName& )
    {
        CDMW_ERROR(m_logger.in(), 
            "Service " << service_id << " has InvalidName !");
        Components::CCMException ex(Components::OBJECT_NOT_FOUND);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    catch (const CORBA::SystemException& e)
    {
        CDMW_ERROR(m_logger.in(), 
            "Exception received getting service " << service_id << " : "
            << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw;
    }
    
    */

    return CORBA::Object::_nil();
    
}

              
} // end namespace CIF
} // end namespace CCM
} // end namespace Cdmw



