/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

// Cdmw files
#include <Foundation/logging/FunctionLogger.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <CCMContainer/ccmcommon/Cookie_impl.hpp>
// Cdmw CCM files
#include <CCMContainer/ccmcif/ExtensionContext_impl.hpp>
#include <CCMContainer/ccmcommon/CCMUtils.hpp>
#include <string>

namespace Cdmw {

namespace CCM {

namespace CIF {

    ExtensionContext_impl::ExtensionContext_impl
    (
     Components::CCMContext_ptr ctx,
     const ::std::string & component_id,
     CdmwLogging::LoggerFactory_ptr logger_factory
     ) 
        throw (CORBA::SystemException)
        : CCMContext_impl(ctx, component_id, logger_factory)
    {
        this->m_logger = 
            logger_factory->get_logger(Cdmw::CCM::CCMLoggers::CCMCIF_LOG_DOMAIN);
        //CDMW_LOG_FUNCTION(m_logger, "");
        try {
            this->m_extension_context = Components::ExtensionContext::_narrow(ctx);
        } catch (CORBA::BAD_PARAM &e) {
            CDMW_ERROR(m_logger, "Context is not a ExtensionContext");
            CORBA::BAD_PARAM ex = 
                CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMWrongContextType, CORBA::COMPLETED_NO);
            // CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
    }    


    // Implementation skeleton destructor
    ExtensionContext_impl::~ExtensionContext_impl (void)
    {
        // anything to clean up?
    }


    ::Components::Cookie * 
    ExtensionContext_impl::install_service_reference (
                                                      const char * service_id,
                                                      ::CORBA::Object_ptr objref
                                                      )
        throw(::CORBA::SystemException, ::Components::CCMException)
    {
        // CDMW_LOG_FUNCTION(m_logger, "service_id=" + service_id);
       
        std::string service_id_str = std::string(service_id);
       
        if (CORBA::is_nil(objref)) {
            CDMW_ERROR(m_logger, "Attempt to install a Service (" + service_id_str + ") which is nil reference.");
            Components::CCMException ex = 
                Components::CCMException(Components::SERVICE_INSTALLATION_ERROR);
            // CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }    

        CDMW_MUTEX_GUARD(m_mtx);

        services_map_iterator it = m_services_map.find(service_id_str);
        // check if service is already installed
        if (it != m_services_map.end()) {
            // service already installed
            CDMW_ERROR(m_logger, "Service " + service_id_str + " is already installed.");
            Components::CCMException ex = 
                Components::CCMException(Components::SERVICE_INSTALLATION_ERROR);
            // CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        } 

        // add service to ServicesMap
        m_services_map[service_id_str] =  CORBA::Object::_duplicate(objref);

        Cdmw::CCM::Common::Cookie_impl * cookie = 
             Cdmw::CCM::Common::Cookie_impl::create_cookie(service_id_str); 
        
        return cookie;
    }

    ::CORBA::Object_ptr 
    ExtensionContext_impl::uninstall_service_reference (
                                                        ::Components::Cookie * ck
                                                        )
        throw(::CORBA::SystemException, ::Components::CCMException)
    {
        // Add your implementation here
        // this is dodgy, the uninstall operation does not mirrow the install op,
        
        this->m_extension_context->uninstall_service_reference(ck);
        return CORBA::Object::_nil();
    }


    ::Components::ContainerPortableInterceptor::ClientContainerInterceptorRegistration_ptr 
    ExtensionContext_impl::get_client_interceptor_registration ()
        throw(::CORBA::SystemException, ::Components::CCMException)
                     
    {
        // Add your implementation here                
        return this->m_extension_context->get_client_interceptor_registration ();
    }

    ::Components::ContainerPortableInterceptor::ServerContainerInterceptorRegistration_ptr 
    ExtensionContext_impl::get_server_interceptor_registration ()
        throw(::CORBA::SystemException, ::Components::CCMException)
    {
        // Add your implementation here
        return this->m_extension_context->get_server_interceptor_registration ();
    }
                                                                
    ::Components::ContainerPortableInterceptor::StubContainerInterceptorRegistration_ptr 
    ExtensionContext_impl::get_stub_interceptor_registration ()
        throw(::CORBA::SystemException, ::Components::CCMException)
    {
        // Add your implementation here
        return this->m_extension_context->get_stub_interceptor_registration ();
    }

    ::Components::ContainerPortableInterceptor::ServantContainerInterceptorRegistration_ptr 
    ExtensionContext_impl::get_servant_interceptor_registration ()
        throw(::CORBA::SystemException, ::Components::CCMException)
    {
        return this->m_extension_context->get_servant_interceptor_registration ();
    }


} // end namespace CIF
} // end namespace CCM
} // end namespace Cdmw
