/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "CCMContainer/ccmcontainer/ApplicationElementsContextService.hpp"
#include "ApplicationElementsContextServiceFactories.hpp"
#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include "CCMContainer/ccmcommon/CCMLoggers.hpp"
#include "CCMContainer/ccmcontainer/ServiceRegistry.hpp"
#include <CCMContainer/ccmcopi/COPIRegistration_impl.hpp>
#include "ComponentServerElementContext_impl.hpp"

namespace Cdmw
{
namespace CCM
{
namespace Container
{

const std::string ApplicationElementsContextService::ApplicationElementsContextServiceId =
        "CDMW_ApplicationElementsContextService";

ApplicationElementsContextService::ApplicationElementsContextService(const std::string& componentServerId,
        const Components::ConfigValues& values) :
    m_logger(CCMLoggers::Get_ccmcontainer_logger()), m_componentServerContext(0)
{
    CDMW_LOG_FUNCTION(m_logger, "");
    create_element_context(componentServerId, CORBA::Object::_nil(), values);
    try
    {
        m_registration
                = CdmwCcmContainer::LifeCycleInterceptorRegistration::_narrow(
                        ServiceRegistry::get_instance()->lookup(
                                Cdmw::CCM::COPI::COPIRegistration_impl::M_LifeCycleInterceptorRegistrationServiceId));

        m_compFactory = new ComponentElementContextFactory(m_registration);
        m_homeFactory = new HomeElementContextFactory(m_registration);

        m_contextsFactories.push_back(m_compFactory);
        m_contextsFactories.push_back(m_homeFactory);
    }
    catch (Components::CCMException ex)
    {
        CDMW_ERROR(m_logger, "Failed to get the LifeCycleInterceptorRegistrationService reference." <<
                " The  " << ApplicationElementsContextServiceId << " service will not be available");
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
    }
}

ApplicationElementsContextService::~ApplicationElementsContextService()
{
    m_contextsFactories.clear();
    delete m_compFactory;
    m_compFactory = 0;
    delete m_homeFactory;
    m_homeFactory = 0;
    CORBA::release(m_registration);
    m_registration = 0;
    CORBA::release(m_componentServerContext);
    m_componentServerContext = 0;
}

CdmwCcmContainer::ComponentElementContext*
ApplicationElementsContextService::find_component_context_by_id(const std::string& id) const
        throw(Components::CCMException)
{
    CDMW_LOG_FUNCTION(m_logger, "id = "<< id);
    CdmwCcmContainer::ComponentElementContext* ctx =
            dynamic_cast<CdmwCcmContainer::ComponentElementContext*> (m_compFactory->find_element_context_by_id(id));
    if (0 == ctx)
    {
        Components::CCMException ex(Components::OBJECT_NOT_FOUND);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    return ctx;
}

CdmwCcmContainer::HomeElementContext*
ApplicationElementsContextService::find_home_context_by_id(const std::string& id) const throw(Components::CCMException)
{
    CDMW_LOG_FUNCTION(m_logger, "id = "<< id);
    CdmwCcmContainer::HomeElementContext* ctx =
            dynamic_cast<CdmwCcmContainer::HomeElementContext*> (m_homeFactory->find_element_context_by_id(id));
    if (0 == ctx)
    {
        Components::CCMException ex(Components::OBJECT_NOT_FOUND);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    return ctx;
}

CdmwCcmContainer::ComponentServerContext*
ApplicationElementsContextService::get_component_server_context() const throw(Components::CCMException)
{
    CDMW_LOG_FUNCTION(m_logger, "");
    CdmwCcmContainer::ComponentServerContext* ctx =
            dynamic_cast<CdmwCcmContainer::ComponentServerContext*> (this->find_element_context_by_id(""));
    if (0 == ctx)
    {
        Components::CCMException ex(Components::OBJECT_NOT_FOUND);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    return ctx;
}

CdmwCcmContainer::ApplicationElementContext* ApplicationElementsContextService::create_element_context(
        const std::string& id, CORBA::Object_ptr ref, const Components::ConfigValues& configValues)
{
    if (0 == m_componentServerContext)
    {
        m_componentServerContext = new ComponentServerElementContext_impl(configValues, id, ref);
    }
    return m_componentServerContext;
}

CdmwCcmContainer::ApplicationElementContext* ApplicationElementsContextService::find_element_context_by_id(
        const std::string& id) const
{
    return m_componentServerContext;
}

void ApplicationElementsContextService::remove_element_context(const std::string& id)
{
    CORBA::release(m_componentServerContext);
    m_componentServerContext = 0;
}

void ApplicationElementsContextService::clear()
{
}

}
}
}
