/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "ApplicationElementsContextServiceFactories.hpp"
#include "Foundation/logging/FunctionLogger.hpp"
#include "CCMContainer/ccmcommon/CCMLoggers.hpp"

namespace Cdmw
{
namespace CCM
{
namespace Container
{

//----- ComponentElementContextFactory -----//

ApplicationElementsContextService::ComponentElementContextFactory::ComponentElementContextFactory(
        CdmwCcmContainer::LifeCycleInterceptorRegistration_ptr reg) :
    m_logger(CCMLoggers::Get_ccmcontainer_logger()),
    m_registration(reg)
{
    m_ck = reg->register_component_lifecycle_interceptor(this);
}

ApplicationElementsContextService::ComponentElementContextFactory::~ComponentElementContextFactory()
{
    clear();
}

CdmwCcmContainer::ApplicationElementContext*
ApplicationElementsContextService::ComponentElementContextFactory::create_element_context(const std::string& id,
        CORBA::Object_ptr ref, const Components::ConfigValues& configValues)
{
    CDMW_LOG_FUNCTION(m_logger, "id= " << id);
    ComponentElementContext_impl* comp_ctx = 0;
    if (0 == m_componentElementContextsMap.count(id))
    {
        comp_ctx = new ComponentElementContext_impl(configValues, id, ref);
        m_componentElementContextsMap[id] = comp_ctx;
    }
    else CDMW_DEBUG(m_logger, "Component context is already created");

    return comp_ctx;
}

CdmwCcmContainer::ApplicationElementContext*
ApplicationElementsContextService::ComponentElementContextFactory::find_element_context_by_id(const std::string& id) const
{
    CdmwCcmContainer::ApplicationElementContext* ctx = 0;
    ComponentElementContextMap::const_iterator it = m_componentElementContextsMap.find(id);
    if (it != m_componentElementContextsMap.end())
    {
        ctx = it->second;
    }
    return ctx;
}

void ApplicationElementsContextService::ComponentElementContextFactory::remove_element_context(const std::string& id)
{
    ComponentElementContextMap::iterator it = m_componentElementContextsMap.find(id);
    if (it != m_componentElementContextsMap.end())
    {
        delete it->second;
        m_componentElementContextsMap.erase(it);
    }
    else
    {
        CDMW_ERROR(m_logger, "Failed to remove the component context."
                "Component Context not found !!");
    }
}

void ApplicationElementsContextService::ComponentElementContextFactory::clear()
{
    CDMW_LOG_FUNCTION(m_logger, "");
    if (!m_componentElementContextsMap.empty())
    {
        for (ComponentElementContextMap::iterator it = m_componentElementContextsMap.begin(); it
                != m_componentElementContextsMap.end(); it++)
        {
            delete it->second;
        }
        m_componentElementContextsMap.clear();
    }
    try
    {
        m_registration->unregister_component_lifecycle_interceptor(m_ck);
    }
    catch (Components::ContainerPortableInterceptor::InvalidRegistration e)
    {
        CDMW_LOG_FUNCTION_EXCEPTION(e);
    }

}

void ApplicationElementsContextService::ComponentElementContextFactory::after_component_activate(const char*)
        throw (CORBA::SystemException)
{

}

void ApplicationElementsContextService::ComponentElementContextFactory::after_component_connect(const char*)
        throw (CORBA::SystemException)
{

}

void ApplicationElementsContextService::ComponentElementContextFactory::after_component_create(const char* id,
        CORBA::Object_ptr ref) throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger, "id = " << id);
    ComponentElementContext_impl* ctx =
            dynamic_cast<ComponentElementContext_impl*> (this->find_element_context_by_id(id));
    if (0 == ctx)
    {
        CDMW_ERROR(m_logger, "Component Context not found !!");
        return;
    }
    ctx->setRef(ref);
}

void ApplicationElementsContextService::ComponentElementContextFactory::after_component_passivate(const char* arg0)
        throw (CORBA::SystemException)
{

}

void ApplicationElementsContextService::ComponentElementContextFactory::after_component_remove(const char* id)
        throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger, "id = " << id);
    remove_element_context(id);
}

void ApplicationElementsContextService::ComponentElementContextFactory::before_component_activate(const char* id)
        throw (CORBA::SystemException)
{

}

void ApplicationElementsContextService::ComponentElementContextFactory::before_component_connect(const char* id,
        const char* port, CORBA::Object_ptr ref) throw (CORBA::SystemException)
{

}

void ApplicationElementsContextService::ComponentElementContextFactory::before_component_create(const char* id,
       Components::ConfigValues& configuration) throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger, "id = " << id);
    create_element_context(id, CORBA::Object::_nil(), configuration);
}

void ApplicationElementsContextService::ComponentElementContextFactory::before_component_passivate(const char* id)
        throw (CORBA::SystemException)
{

}

void ApplicationElementsContextService::ComponentElementContextFactory::before_component_remove(const char* id)
        throw (CORBA::SystemException)
{

}

void ApplicationElementsContextService::ComponentElementContextFactory::destroy() throw (::CORBA::SystemException)
{

}

char* ApplicationElementsContextService::ComponentElementContextFactory::name() throw (::CORBA::SystemException)
{
    return "";
}

CORBA::UShort ApplicationElementsContextService::ComponentElementContextFactory::priority()
        throw (::CORBA::SystemException)
{
    return 0;
}

Components::ContainerPortableInterceptor::IntegrationPoint*
ApplicationElementsContextService::ComponentElementContextFactory::registration_info() throw (::CORBA::SystemException)
{
    return 0;
}

void ApplicationElementsContextService::ComponentElementContextFactory::set_slot_id(::PortableInterceptor::SlotId)
        throw (::CORBA::SystemException)
{

}

//----- HomeElementContextFactory -----//

ApplicationElementsContextService::HomeElementContextFactory::HomeElementContextFactory(
        CdmwCcmContainer::LifeCycleInterceptorRegistration_ptr reg) :
    m_logger(CCMLoggers::Get_ccmcontainer_logger()),
    m_registration(reg)
{
    m_ck = reg->register_home_lifecycle_interceptor(this);
}

ApplicationElementsContextService::HomeElementContextFactory::~HomeElementContextFactory()
{
    clear();
}

CdmwCcmContainer::ApplicationElementContext*
ApplicationElementsContextService::HomeElementContextFactory::create_element_context(const std::string& id,
        CORBA::Object_ptr ref, const Components::ConfigValues& configValues)
{
    CDMW_LOG_FUNCTION(m_logger, "id= " << id);
    HomeElementContext_impl* comp_ctx = 0;
    if (0 == m_homeElementContextsMap.count(id))
    {
        comp_ctx = new HomeElementContext_impl(configValues, id, ref);
        m_homeElementContextsMap[id] = comp_ctx;
    }
    else CDMW_DEBUG(m_logger, "Component context is already created");

    return comp_ctx;
}

CdmwCcmContainer::ApplicationElementContext*
ApplicationElementsContextService::HomeElementContextFactory::find_element_context_by_id(const std::string& id) const
{
    CdmwCcmContainer::ApplicationElementContext* ctx = 0;
    HomeElementContextMap::const_iterator it = m_homeElementContextsMap.find(id);
    if (it != m_homeElementContextsMap.end())
    {
        ctx = it->second;
    }
    return ctx;
}

void ApplicationElementsContextService::HomeElementContextFactory::remove_element_context(const std::string& id)
{
    HomeElementContextMap::iterator it = m_homeElementContextsMap.find(id);
    if (it != m_homeElementContextsMap.end())
    {
        delete it->second;
        m_homeElementContextsMap.erase(it);
    }
    else
    {
        CDMW_ERROR(m_logger, "Failed to remove the home context."
                "Home Context not found !!");
    }
}

void ApplicationElementsContextService::HomeElementContextFactory::clear()
{
    CDMW_LOG_FUNCTION(m_logger, "");
    if (!m_homeElementContextsMap.empty())
    {
        for (HomeElementContextMap::iterator it = m_homeElementContextsMap.begin(); it
                != m_homeElementContextsMap.end(); it++)
        {
            delete it->second;
        }
        m_homeElementContextsMap.clear();
    }
    try
    {
        m_registration->unregister_home_lifecycle_interceptor(m_ck);
    }
    catch (Components::ContainerPortableInterceptor::InvalidRegistration e)
    {
        CDMW_LOG_FUNCTION_EXCEPTION(e);
    }
}

void ApplicationElementsContextService::HomeElementContextFactory::after_home_create(const char* id,
        CORBA::Object_ptr ref) throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger, "id = " << id);
    HomeElementContext_impl* ctx = dynamic_cast<HomeElementContext_impl*> (this->find_element_context_by_id(id));
    if (0 == ctx)
    {
        CDMW_ERROR(m_logger, "Home Context not found !!");
        return;
    }
    ctx->setRef(ref);
}

void ApplicationElementsContextService::HomeElementContextFactory::after_home_remove(const char* id)
        throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger, "id = " << id);
    remove_element_context(id);
}

void ApplicationElementsContextService::HomeElementContextFactory::before_home_create(const char* id,
        Components::ConfigValues& configuration, const char*) throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger, "id = " << id);
    create_element_context(id, CORBA::Object::_nil(), configuration);
}

void ApplicationElementsContextService::HomeElementContextFactory::before_home_remove(const char* id)
        throw (CORBA::SystemException)
{

}

void ApplicationElementsContextService::HomeElementContextFactory::destroy() throw (::CORBA::SystemException)
{

}

char* ApplicationElementsContextService::HomeElementContextFactory::name() throw (::CORBA::SystemException)
{
    return "";
}

CORBA::UShort ApplicationElementsContextService::HomeElementContextFactory::priority() throw (::CORBA::SystemException)
{
    return 0;
}

Components::ContainerPortableInterceptor::IntegrationPoint*
ApplicationElementsContextService::HomeElementContextFactory::registration_info() throw (::CORBA::SystemException)
{
    return 0;
}

void ApplicationElementsContextService::HomeElementContextFactory::set_slot_id(::PortableInterceptor::SlotId slot_id)
        throw (::CORBA::SystemException)
{

}

}
}
}
