/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <vector>
#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/idllib/Components.stub.hpp"
#include "CCMContainer/idllib/CdmwCcmContainer.stub.hpp"
#include "InterceptorIteratorStackSupport_impl.hpp"


#ifndef HOMELIFECYCLEINTERCEPTORITERATOR_HPP_
#define HOMELIFECYCLEINTERCEPTORITERATOR_HPP_

namespace Cdmw
{
namespace CCM
{
namespace Container
{

    /*
      Note: there is no specified behavior for the LC interceptors.
      Therefore the implementation makes the following guarantees:
      
      Per definition a HomeLifeCycleInterceptor has two independent sets of starting /
      ending interception points:

      Set1: 
      starting ip:
      before_home_create

      ending ip:
      after_home_create


      Set2:
      starting ip:
      before_home_remove

      ending ip:
      after_home_remove


      For each of the two sets the same flow stack rules apply as for the Portable Interceptors,
      e.g. iff a starting ip succeeds and ending ip will be called etc.

      Since there is no receive exception ending point in case of an exception the ending 
      iterception point will be called using 0s as args.
      
      Rational:
      -It is not advisable to keep copis with incremented refs hanging around between
      after_home_create and before_home_remove, because between calling these two operations
      a considerable amount of time could pass what make the unregistration of LC interceptor block

      -Apart from having two sets of starting/ending ips we stick to the PI spec for obvious reasons (well known behaviour)
    */

  
/**
 * Call operations on registered lifecycle interceptors before/after operations on home
 */
//FIXME ContainerInterceptor? HomeLifeCycleInterceptorOperations?
class HomeLifeCycleInterceptorIterator : 
        public Components::ContainerPortableInterceptor::ContainerInterceptor,
        public virtual InterceptorIteratorStackSupport_impl<CdmwCcmContainer::HomeLifeCycleInterceptor_ptr>
{
private:
    

public:
    ///Ctor
    HomeLifeCycleInterceptorIterator();

    void after_home_create(const char* homeId, CORBA::Object_ptr homeRef);
    void after_home_remove(const char* homeId);
    void before_home_create(const char* homeId, Components::ConfigValues& values, const char* entryPoint);
    void before_home_remove(const char* homeId);

    void destroy() throw (::CORBA::SystemException);
    char* name() throw (::CORBA::SystemException);
    CORBA::UShort priority() throw (::CORBA::SystemException);
    Components::ContainerPortableInterceptor::IntegrationPoint* registration_info() throw (::CORBA::SystemException);
    void set_slot_id(::PortableInterceptor::SlotId slot_id) throw (::CORBA::SystemException);

protected:
    void invoke_ip
    (
     CdmwCcmContainer::HomeLifeCycleInterceptor_ptr interceptor,
     InterceptorIteratorStackSupportTrait<CdmwCcmContainer::HomeLifeCycleInterceptor_ptr>::arg_t arg
     );

    InterceptorIteratorStackSupportTrait<CdmwCcmContainer::HomeLifeCycleInterceptor_ptr>::interceptor_seq_t *
    get_interceptors();

    ::CORBA::UShort
    add_interceptor_ref(CdmwCcmContainer::HomeLifeCycleInterceptor_ptr interceptor);
    
    ::CORBA::UShort
    remove_interceptor_ref(CdmwCcmContainer::HomeLifeCycleInterceptor_ptr interceptor);

};

}
}
}

#endif /* HOMELIFECYCLEINTERCEPTORITERATOR_HPP_ */
