/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "CCMContainer/ccmcontainer/ServiceRegistry.hpp"
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>
#include <Foundation/logging/FunctionLogger.hpp>

namespace Cdmw
{
namespace CCM
{
namespace Container
{

// Implementation skeleton constructor
ServiceRegistry::ServiceRegistry(const std::string & name) :
    m_name(name)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
}

// Implementation skeleton destructor
ServiceRegistry::~ServiceRegistry(void)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    //release services
//    for (services_map_t::iterator it = m_services_map.begin(); it != m_services_map.end(); ++it)
//    {
//        CORBA::release(it->second);
//    }
}

ServiceRegistry * ServiceRegistry::m_instance = 0;

OsSupport::Mutex ServiceRegistry::m_mtx;

ServiceRegistry* ServiceRegistry::get_instance()
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    if (m_instance == 0)
    {
        CDMW_MUTEX_GUARD(m_mtx);
        if (m_instance == 0)
        {
            m_instance = new ServiceRegistry("GlobalRegistry");
        }
    }
    return m_instance;
}

void ServiceRegistry::destroy_instance()
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    if (m_instance != 0)
    {
        CDMW_MUTEX_GUARD(m_mtx);
        if (m_instance != 0)
        {
            CORBA::release( m_instance);
            m_instance = 0;
        }
    }
}

void ServiceRegistry::bind(const char * service_id, ::CORBA::Object_ptr service_ref)
                                                                                          throw (::CORBA::SystemException,
                                                                                         ::Components::CCMException )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    CDMW_MUTEX_GUARD(m_mtx);
    services_map_t::iterator it = m_services_map.find(service_id);
    if (it != m_services_map.end())
    {
        Components::CCMException ex(Components::SERVICE_INSTALLATION_ERROR);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    else
    {
        m_services_map.insert(std::make_pair(service_id, service_ref));
    }
}

CdmwCcmContainer::ServicesIds* ServiceRegistry::list() throw (::CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    CdmwCcmContainer::ServicesIds* retSeq = new CdmwCcmContainer::ServicesIds();
    CDMW_MUTEX_GUARD(m_mtx);
    unsigned int len = 0;

    for (services_map_t::iterator it = m_services_map.begin(); it != m_services_map.end(); ++it)
    {
        retSeq->length(++len);
        (*retSeq)[len - 1] = CORBA::string_dup((it->first).c_str());
    }
    return retSeq;
}

::CORBA::Object_ptr ServiceRegistry::lookup(const char * service_id) throw (::CORBA::SystemException,
                                                                         ::Components::CCMException )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    CDMW_MUTEX_GUARD(m_mtx);
    services_map_t::iterator it = m_services_map.find(service_id);
    if (it != m_services_map.end())
    {
        return it->second;
    }
    else
    {
        return CORBA::Object::_nil();
    }
}

void ServiceRegistry::rebind(const char * service_id, ::CORBA::Object_ptr service_ref)
                                                                                            throw (::CORBA::SystemException,
                                                                                           ::Components::CCMException )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    CDMW_MUTEX_GUARD(m_mtx);
    services_map_t::iterator it = m_services_map.find(service_id);
    if (it != m_services_map.end())
        m_services_map.erase(it);

    m_services_map.insert(std::make_pair(service_id, service_ref));
}

::CORBA::Object_ptr ServiceRegistry::unbind(const char * service_id) throw (::CORBA::SystemException,
                                                                         ::Components::CCMException )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    CDMW_MUTEX_GUARD(m_mtx);

    ::CORBA::Object_ptr toRet = CORBA::Object::_nil();

    services_map_t::iterator it = m_services_map.find(service_id);
    if (it != m_services_map.end())
    {
        toRet = it->second;
        m_services_map.erase(it);
    }
    else
    {
        Components::CCMException ex(Components::FIND_ERROR);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }

    return toRet;
}

void ServiceRegistry::clear() throw (::CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    CDMW_MUTEX_GUARD(m_mtx);
    m_services_map.clear();
}

}// End namespace Container
}// End namespace CCM
}// End namespace Cdmw


