/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "Session2ExtensionContext_impl.hpp"
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include "CCMContainer/ccmcontainer/ServiceRegistry.hpp"
#include "CCMContainer/ccmcopi/COPIServiceInit.hpp"

namespace Cdmw
{
namespace CCM
{
namespace Container
{

// ctor
Session2ExtensionContext_impl::Session2ExtensionContext_impl(CORBA::ORB_ptr orb,
        PortableServer::POA_ptr parent, Components::CCMHome_ptr home, const std::string & home_id,
        ComponentServantProvider* servant_provider,
        CdmwDeployment::ServantLifetimeValue servant_lifetime,
        CdmwDeployment::ThreadingPolicyValue threading_policy,
        Components::HomeRegistration_ptr home_registration,
        CdmwEvent::EventChannelFactory_ptr event_channel_factory, const std::string & process_name,
        const std::string & application_name,
        const ValueTypeDependencyList & valuetype_dependencies) :
    Session2Context_impl(orb, parent, home, home_id, servant_provider, servant_lifetime,
            threading_policy, home_registration, event_channel_factory, process_name,
            application_name, valuetype_dependencies),
            m_logger(Cdmw::CCM::CCMLoggers::Get_ccmcontainer_logger()),
            m_copi_current(0),
            m_is_copi_current_initialised(false)

{
    CDMW_LOG_FUNCTION(m_logger, "");

    // resolve COPICurrent from orb
    try {
        if(::Cdmw::CCM::COPI::COPIServiceInit::Is_initialized() == true) 
            this->m_copi_current =
                ::CdmwCcmCopi::COPICurrent::_narrow
                (orb->resolve_initial_references(::Cdmw::CCM::COPI::COPICurrent_impl::COPI_CURRENT_ID.c_str()));
        m_is_copi_current_initialised = true;
        
    } catch (CORBA::ORB::InvalidName & inv_name) {        
        CDMW_LOG_FUNCTION_EXCEPTION(inv_name);
    }
}

// dtor
Session2ExtensionContext_impl::~Session2ExtensionContext_impl(void) throw()
{
    CDMW_LOG_FUNCTION(m_logger, "");
    if (m_is_copi_current_initialised)
    {
        CORBA::release(this->m_copi_current);
    }
}

::Components::Cookie *
Session2ExtensionContext_impl::install_service_reference(const char * service_id,
        ::CORBA::Object_ptr objref) throw (::CORBA::SystemException, ::Components::CCMException )
{
    CDMW_LOG_FUNCTION(m_logger, "");
    try
    {
        this->m_service_registry->bind(service_id, objref);
    }
    catch (Components::CCMException & e)
    {

        CDMW_ERROR(m_logger,
                "Service " << service_id << " is already installed.");
        Components::CCMException ex(Components::SERVICE_INSTALLATION_ERROR);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }

    return new Cookie_impl(service_id);
}

::CORBA::Object_ptr Session2ExtensionContext_impl::uninstall_service_reference(
        ::Components::Cookie * ck) throw (::CORBA::SystemException, ::Components::CCMException )
{
    CDMW_LOG_FUNCTION(m_logger, "");

    Cookie_impl * ck_impl = 0;
    try
    {
        ck_impl = dynamic_cast<Cookie_impl *> (ck);
        if (ck_impl == 0)
        {
            CORBA::BAD_PARAM ex = CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMBADCookie,
                    CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
    }
    catch (std::bad_cast & bd)
    {
        CORBA::BAD_PARAM ex = CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMBADCookie,
                CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }

    std::string service_id = ck_impl->get_cookie_string();

    try
    {
        return this->m_service_registry->unbind(service_id.c_str());
    }
    catch (::Components::CCMException & e)
    {
        CDMW_ERROR(m_logger,
                "Service " << service_id << " is not installed.");
        Components::CCMException ex(Components::SERVICE_INSTALLATION_ERROR);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

Components::ContainerPortableInterceptor::ClientContainerInterceptorRegistration_ptr
Session2ExtensionContext_impl::get_client_interceptor_registration()
        throw (::CORBA::SystemException, ::Components::CCMException )
{
    CDMW_LOG_FUNCTION(m_logger, "");
    if (m_copi_current == 0)
    {
        ::Components::CCMException ex(Components::SERVICE_INSTALLATION_ERROR);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }

    return this->m_copi_current->get_client_interceptor_registration();
}

Components::ContainerPortableInterceptor::ServerContainerInterceptorRegistration_ptr
Session2ExtensionContext_impl::get_server_interceptor_registration()
        throw (::CORBA::SystemException, ::Components::CCMException )
{
    CDMW_LOG_FUNCTION(m_logger, "");
    if (m_copi_current == 0)
    {
        ::Components::CCMException ex(Components::SERVICE_INSTALLATION_ERROR);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }

    return this->m_copi_current->get_server_interceptor_registration();
}

::Components::ContainerPortableInterceptor::StubContainerInterceptorRegistration_ptr
Session2ExtensionContext_impl::get_stub_interceptor_registration()
        throw (::CORBA::SystemException, ::Components::CCMException )
{
    CDMW_LOG_FUNCTION(m_logger, "");
    if (m_copi_current == 0)
    {
        ::Components::CCMException ex(Components::SERVICE_INSTALLATION_ERROR);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }

    return this->m_copi_current->get_stub_interceptor_registration();
}

::Components::ContainerPortableInterceptor::ServantContainerInterceptorRegistration_ptr
Session2ExtensionContext_impl::get_servant_interceptor_registration()
        throw (::CORBA::SystemException, ::Components::CCMException )
{
    CDMW_LOG_FUNCTION(m_logger, "");
    if (m_copi_current == 0)
    {
        ::Components::CCMException ex(Components::SERVICE_INSTALLATION_ERROR);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    return this->m_copi_current->get_servant_interceptor_registration();
}

} // End namespace Container
} // End namespace CCM
} // End namespace Cdmw

