/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

// standard files

// needed for insertion of SystemException into an any
#include "tao/AnyTypeCode/ExceptionA.h"

// Cdmw Files
#include <orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>

// Local Files
#include <CCMContainer/ccmcopi/ServantContainerInterceptorIterator_impl.hpp>


namespace 
{    
} // end of anonymous namespace


namespace Cdmw
{
namespace CCM
{
namespace COPI
{

    ServantContainerInterceptorIterator_impl::ServantContainerInterceptorIterator_impl 
    (
     ::CdmwCcmCopi::ServantContainerInterceptorSeq * servant_interceptors, 
     ::CdmwCcmCopi::RegisteredCOPICollection_ptr registration_util
     ) : m_copis_list(::CdmwCcmCopi::RegisteredCOPICollection::_duplicate(registration_util)),
         m_sending_state_interceptor_index(-1),
         m_receiving_state_interceptor_index(-1),
         m_safe_interceptors(0)
    { 
        CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

        if(servant_interceptors->length() > 32767) {

            // since we will be later iterating / looping using ints, we
            // need to limit the maximum number of copis

            CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(), "too many copis (more than 32767)");
            CORBA::BAD_PARAM ex = CORBA::BAD_PARAM (Cdmw::OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }

        m_servant_interceptors =
            new ::CdmwCcmCopi::ServantContainerInterceptorSeq();

        m_servant_interceptors->length(servant_interceptors->length());
        for(::CORBA::ULong i = 0; i < servant_interceptors->length(); ++i) {
            m_servant_interceptors[i] = (*servant_interceptors)[i];
        }
    };


    ServantContainerInterceptorIterator_impl::~ServantContainerInterceptorIterator_impl () 
    { 
        CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
        // a ServantContainerInterceptorIterator_impl is only good for one
        // request, i.e. it is to be recreated for each request
        delete m_safe_interceptors;
    };


    void ServantContainerInterceptorIterator_impl::servant_receive_request 
    (
     ::Components::ContainerPortableInterceptor::ContainerServantRequestInfo_ptr info,
     ::CORBA::Boolean & proceed_call
     )
        throw (
               ::CORBA::SystemException,
               ::PortableInterceptor::ForwardRequest
               )
    {
        CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
        bool is_call_completed = true;

        int i = 0;

        m_safe_interceptors =
            get_safe_servant_interceptors(m_servant_interceptors.in());

        try {
            for (i = 0; (unsigned int)i < m_safe_interceptors->size(); i++) {
                if(proceed_call) {
                    (*m_safe_interceptors)[i]->servant_receive_request(info, proceed_call);
                    m_receiving_state_interceptor_index = i;

                    if (!proceed_call) {
                        is_call_completed = false;
                        break;
                    }        
                }
            }
        } catch (::PortableInterceptor::ForwardRequest & e) {
            CDMW_DEBUG(CCMLoggers::Get_ccmcommon_logger(),
                       "ForwardRequest exception returned from the ExtCOPI '"
                       << (*m_safe_interceptors)[i]->name() << "' when calling servant_receive_request: "
                       << e);
            ::CORBA::Any ex;
            ex <<= e;
            info->the_exception(ex);
            is_call_completed = false;
            
        } catch (::CORBA::SystemException & se) {
            CDMW_WARN(CCMLoggers::Get_ccmcommon_logger(),
                      "SystemException exception returned from the ExtCOPI '"
                      << (*m_safe_interceptors)[i]->name() << "' when calling servant_receive_request: "
                      << se);

            ::CORBA::Any ex;
            ex <<= se;
            info->the_exception(ex);
            is_call_completed = false;
        }


        // No interceptor is in the SENDING state yet
        if (m_safe_interceptors->size() > 0) {
            if (!proceed_call) {
                m_sending_state_interceptor_index = m_receiving_state_interceptor_index;
            } else {
                m_sending_state_interceptor_index = m_receiving_state_interceptor_index + 1;
            }
            if(!is_call_completed || !proceed_call) {
                m_copis_list->removeServantCOPIRef((*m_safe_interceptors)[i]);
            }
        }                                    
    }


    void ServantContainerInterceptorIterator_impl::servant_send_reply 
    (
     ::Components::ContainerPortableInterceptor::ContainerServantRequestInfo_ptr info,
     ::CORBA::Boolean & proceed_call
     )
        throw (
               ::CORBA::SystemException
               )
    {
        CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
        
        int j = m_receiving_state_interceptor_index;

        try{
            
            for (
                 ;
                 j >= 0 && (unsigned int)j < m_safe_interceptors->size();
                 j--)
                {
                    if (true == proceed_call) {                    
                        m_sending_state_interceptor_index = j;
                        (*m_safe_interceptors)[j]->servant_send_reply(info, proceed_call);
                        m_copis_list->removeServantCOPIRef((*m_safe_interceptors)[j]);
                    }
                    else {
                        servant_send_other(info);
                        break;
                    }                    
                }
                        
        } catch (::CORBA::SystemException & se) {
            CDMW_WARN(CCMLoggers::Get_ccmcommon_logger(),
                      "SystemException exception returned from the ExtCOPI '"
                      << (*m_safe_interceptors)[j]->name() << "' when calling servant_send_reply: "
                      << se);

            m_copis_list->removeServantCOPIRef((*m_safe_interceptors)[j]);

            ::CORBA::Any ex;
            ex <<= se;
            info->the_exception(ex);
            
            servant_send_exception(info, proceed_call);
        } 
    }


    void ServantContainerInterceptorIterator_impl::servant_send_exception 
    (
     ::Components::ContainerPortableInterceptor::ContainerServantRequestInfo_ptr info,
     ::CORBA::Boolean & proceed_call
     )
        throw (
               ::CORBA::SystemException,
               ::PortableInterceptor::ForwardRequest
               )
    {
        CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
        
        int j = m_sending_state_interceptor_index - 1;

        try{
            for (;
                 j >= 0 && (unsigned int)j < m_safe_interceptors->size() && j <= m_sending_state_interceptor_index;
                 j--) 
                {
                    if(true == proceed_call) {
                        m_sending_state_interceptor_index = j;
                        (*m_safe_interceptors)[j]->servant_send_exception(info, proceed_call);
                        m_copis_list->removeServantCOPIRef((*m_safe_interceptors)[j]);
                    }
                    else {
                        servant_send_other(info);
                        break;
                    }
                }
                        
        } catch (::PortableInterceptor::ForwardRequest & e) {

            CDMW_DEBUG(CCMLoggers::Get_ccmcommon_logger(),
                       "ForwardRequest exception returned from the ExtCOPI '"
                       << (*m_safe_interceptors)[j]->name() <<  "' when calling servant_send_exception: " 
                       << e);
            m_copis_list->removeServantCOPIRef((*m_safe_interceptors)[j]);

            ::CORBA::Any ex;
            ex <<= e;
            info->the_exception(ex);
            servant_send_other(info);

        } catch(::CORBA::SystemException & se) {
            CDMW_DEBUG(CCMLoggers::Get_ccmcommon_logger(),
                       "SystemException exception returned from the ExtCOPI '"
                       << (*m_safe_interceptors)[j]->name() <<  "' when calling servant_send_exception: " 
                       << se);

            m_copis_list->removeServantCOPIRef((*m_safe_interceptors)[j]);

            ::CORBA::Any ex;
            ex <<= se;
            info->the_exception(ex);

            // call servant_send_exception recursively
            servant_send_exception(info, proceed_call);
        }
    }

    void ServantContainerInterceptorIterator_impl::servant_send_other 
    (
     ::Components::ContainerPortableInterceptor::ContainerServantRequestInfo_ptr info
     )
        throw (
               ::CORBA::SystemException,
               ::PortableInterceptor::ForwardRequest
               )
    {        
        CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

        int j = m_sending_state_interceptor_index - 1;
        try {
            for (;
                 j >= 0 && (unsigned int)j < m_safe_interceptors->size() && j <= m_sending_state_interceptor_index;
                 j--)
                {               
 
                    m_sending_state_interceptor_index = j;
                    (*m_safe_interceptors)[j]->servant_send_other(info);
                    m_copis_list->removeServantCOPIRef((*m_safe_interceptors)[j]);
                }
        } catch (::PortableInterceptor::ForwardRequest & e) {
            CDMW_WARN(CCMLoggers::Get_ccmcommon_logger(),
                      "ForwardRequest exception returned from the ExtCOPI '"
                      << (*m_safe_interceptors)[j]->name() << "' when calling servant_send_other: "
                      << e);

            m_copis_list->removeServantCOPIRef((*m_safe_interceptors)[j]);
            ::CORBA::Any ex;
            ex <<= e;
            info->the_exception(ex);

            servant_send_other(info);
        } catch(::CORBA::SystemException & se) {
            CDMW_WARN(CCMLoggers::Get_ccmcommon_logger(),
                      "SystemException exception returned from the ExtCOPI '"
                      << (*m_safe_interceptors)[j]->name() << "' when calling servant_send_other: "
                      << se);


            m_copis_list->removeServantCOPIRef((*m_safe_interceptors)[j]);

            ::CORBA::Any ex;
            ex <<= se;
            info->the_exception(ex);

            CORBA::Boolean boolval = true;
            servant_send_exception(info, boolval);

        }
    }

    servant_copis_list_t *
    ServantContainerInterceptorIterator_impl::get_safe_servant_interceptors
    (
     const CdmwCcmCopi::ServantContainerInterceptorSeq & interceptors
     )
    {
        servant_copis_list_t * safe_interceptors =
            new servant_copis_list_t();

        // try to increment ref for each interceptor, if this fails then the
        // interceptor was unregistered, i.e MUST not be used
        for (unsigned int i = 0; i < interceptors.length(); ++i) {
            if(0 == m_copis_list->addServantCOPIRef(interceptors[i])) {
                safe_interceptors->push_back(interceptors[i]);
            }
        }

        return safe_interceptors;
    }

} // End of namespace COPI
} // End of namespace CCM
} // End of namespace Cdmw
