/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <sstream>

// Cdmw Files
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>
#include <CCMContainer/ccmcommon/CCMUtils.hpp>
#include <CCMContainer/ccmcif/CCMUtils.hpp>

// Local Files
#include <CCMContainer/ccmcopi/COPIUtils_impl.hpp>
#include <CCMContainer/ccmcopi/COPIServiceInit.hpp>
#include <CCMContainer/ccmcopi/ContainerServerRequestInfo_impl.hpp>
#include <CCMContainer/ccmcopi/ServerRequestInterceptorIterator_impl.hpp>


namespace 
{
    const PortableInterceptor::ORBInitInfo::ObjectId COPI_SERVER_REQUEST_INTERCEPTOR_ITERATOR_NAME
            = "96a0cb5d-3276-425d-ab41-8d84060b9a9b";
    
} // end of anonymous namespace


namespace Cdmw
{
namespace CCM
{
namespace COPI
{

// static members / methods
const std::string ServerRequestInterceptorIterator_impl::M_name = "eca0dabc-097a-44cb-acad-0b83dc7ae136";
OsSupport::Mutex ServerRequestInterceptorIterator_impl::M_mtx;

ServerRequestInterceptorIterator_impl::ServerRequestInterceptorIterator_impl
(
 int origin_id, 
 int target_id,
 int target_feature_name_id, 
 int custom_slot_item_seq_id,
 ::CdmwCcmCopi::RegisteredCOPICollection_ptr registration
 )
    : m_origin_id_slot_id(origin_id),
      m_target_id_slot_id(target_id),
      m_target_feature_name_slot_id(target_feature_name_id),
      m_custom_slot_item_seq_slot_id(custom_slot_item_seq_id),
      m_registration(::CdmwCcmCopi::RegisteredCOPICollection::_duplicate(registration))
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
}


char*
ServerRequestInterceptorIterator_impl::name() 
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    return ::CORBA::string_dup(M_name.c_str());      
}

void
ServerRequestInterceptorIterator_impl::destroy() 
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    throw CORBA::NO_IMPLEMENT();
}


void
ServerRequestInterceptorIterator_impl::cdmw_ft_interception_point(
      PortableInterceptor::ServerRequestInfo_ptr ri,
      CORBA::OctetSeq_out ocs)
    throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
}

void
ServerRequestInterceptorIterator_impl::receive_request_service_contexts(
        PortableInterceptor::ServerRequestInfo_ptr info)
    throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
   
    // Populating () the RSC from the received service context
    CORBA::ULong req_id;
    if(0 != extract_req_id_from_request_service_context(info, &req_id)) {
        CDMW_DEBUG(CCMLoggers::Get_ccmcommon_logger(), "No request id in request service context, no basic copis called "); 
        return;
    }

    extract_from_request_service_context(info);
    ContainerServerRequestInfo_impl * csri =
          build_container_server_request_info(info);

    ::CORBA::OctetSeq_var origin_id = csri->origin_id();
    ::CORBA::OctetSeq_var target_id = csri->target_id();
    ::CORBA::String_var name = csri->name();

    // TODO if debug...
    ::CORBA::String_var origin_id_str =
        origin_id->length() != 0 ? ::Cdmw::CCM::Common::OctetSeq_to_string(origin_id.in()) : ::CORBA::string_dup("NA");

    ::CORBA::String_var target_id_str =
        target_id->length() != 0 ? ::Cdmw::CCM::Common::OctetSeq_to_string(target_id.in()) : ::CORBA::string_dup("NA");

    ::CORBA::String_var empty_name = CORBA::string_dup("");
    
    if (0 == strcmp(empty_name.in(), name.in())) {
        name = CORBA::string_dup("NA");
    }

    CDMW_DEBUG(CCMLoggers::Get_ccmcommon_logger(),  
              "Component " 
               << origin_id_str << " has sent a request to component "
               << target_id_str << " on its port " << name << std::endl);
    

    ::Components::ContainerPortableInterceptor::IntegrationPoint ip;

    ip.port = csri->name();
    ip.operation = info->operation();
    
    ::CdmwCcmCopi::ServerContainerInterceptorSeq_var interceptors =
          m_registration->getServerInterceptorByIPOrderedByPriority(ip);

    if(interceptors->length() > 32767) {

        // since we will be later iterating / looping using ints, we
        // need to limit the maximum number of copis

        CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(), "copi registry returned too many copis (more than 32767)");
        csri->_remove_ref();
        return;
    }

    CopisStackHolderTrait< ::Components::ContainerPortableInterceptor::ServerContainerInterceptor_ptr >::copis_list_t * safe_interceptors =
        get_safe_server_interceptors(interceptors);

    CopisStackHolder_impl< ::Components::ContainerPortableInterceptor::ServerContainerInterceptor_ptr > * req_dat =
        new CopisStackHolder_impl< ::Components::ContainerPortableInterceptor::ServerContainerInterceptor_ptr >();    
        
    try {
        for (unsigned int i = 0; i < safe_interceptors->size(); ++i) {
            (*safe_interceptors)[i]->receive_request_service_contexts(csri);
            req_dat->set_recv_idx (i);
        }        
    } catch (...) {
        if (safe_interceptors->size() > 0) {
            // Greater indexes are those of SENDING interceptors. At this point,
            // no interceptor is SENDING so the SENDINGStateInterceptorIndex
            // should be
            // initialized with the greatest index value of the list
            req_dat->set_send_idx(req_dat->get_recv_idx() + 1);
            req_dat->set_copis(safe_interceptors);
            set_copis_stack(req_id, req_dat);
        }

        csri->_remove_ref();
        throw;
    }

    req_dat->set_send_idx(req_dat->get_recv_idx() + 1);
    req_dat->set_copis(safe_interceptors);    
    set_copis_stack(req_id, req_dat);
    csri->_remove_ref();
}


void
ServerRequestInterceptorIterator_impl::receive_request
(
PortableInterceptor::ServerRequestInfo_ptr info
)
    throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    CORBA::ULong req_id;
    if(0 != extract_req_id_from_request_service_context(info, &req_id)) {
        CDMW_DEBUG(CCMLoggers::Get_ccmcommon_logger(), "No request id in request service context, no basic copis called ");        
        return;
    }

    CDMW_DEBUG(CCMLoggers::Get_ccmcommon_logger(), "request id = " << req_id);

    extract_feature_name(info);
   
    ContainerServerRequestInfo_impl * csri =
          build_container_server_request_info(info);

    CDMW_DEBUG(CCMLoggers::Get_ccmcommon_logger(),
              "Component "
              << (csri->origin_id() != 0 ? ::Cdmw::CCM::Common::OctetSeq_to_string(*(csri->origin_id())) : "null")
              << " has sent a request to component "
              << (csri->target_id() != 0 ? ::Cdmw::CCM::Common::OctetSeq_to_string(*(csri->target_id())) : "null")
              << " on its port "
              << (csri->name() != 0 ? csri->name() : ""));

   
    CopisStackHolder_impl< ::Components::ContainerPortableInterceptor::ServerContainerInterceptor_ptr > * req_dat =
        get_copis_stack(req_id);

    CopisStackHolderTrait< ::Components::ContainerPortableInterceptor::ServerContainerInterceptor_ptr >::copis_list_t *
        interceptors = req_dat->get_copis();

    try {
        for (int i = 0; i <= req_dat->get_recv_idx(); i++) {
            (*interceptors)[i]->receive_request(csri);
        }
    } catch (...) {
        CDMW_WARN(CCMLoggers::Get_ccmcommon_logger(), "A copi raised an execption on receive_request");  
        // ignore exception, receive_request is neither a beginning not an ending ip
    }

    csri->_remove_ref();
}


void
ServerRequestInterceptorIterator_impl::send_reply
(
 PortableInterceptor::ServerRequestInfo_ptr info
 )
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    do_sending_ip(info, SEND_REPLY);
}


void
ServerRequestInterceptorIterator_impl::send_other
(
 PortableInterceptor::ServerRequestInfo_ptr info
)
    throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    do_sending_ip(info, SEND_OTHER);
}


void
ServerRequestInterceptorIterator_impl::send_exception
(
 PortableInterceptor::ServerRequestInfo_ptr info
 )
    throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    do_sending_ip(info, SEND_EXCEPTION);
}


CopisStackHolder_impl< 
    ::Components::ContainerPortableInterceptor::ServerContainerInterceptor_ptr > *
ServerRequestInterceptorIterator_impl::get_copis_stack
(
 CORBA::ULong request_id
 )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    CDMW_MUTEX_GUARD(M_mtx);

    request_context_t::iterator it =
        m_request_context.find(request_id);

    if(it == m_request_context.end()) {
        return 0;
    }

    if(it->second == 0) {
        CORBA::INTERNAL ex = CORBA::INTERNAL (Cdmw::OrbSupport::INTERNALBug, CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }

    return it->second;
}


void
ServerRequestInterceptorIterator_impl::remove_copis_stack
(
 CORBA::ULong request_id
 )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    // clean up data which is assosiated with
    // the request
    CDMW_MUTEX_GUARD(M_mtx);

    request_context_t::iterator it =
        m_request_context.find(request_id);

    if(it == m_request_context.end()) {
        return;
    }

    delete it->second;
    m_request_context.erase(it);
}


void
ServerRequestInterceptorIterator_impl::set_copis_stack
(
 CORBA::ULong request_id,
 CopisStackHolder_impl< ::Components::ContainerPortableInterceptor::ServerContainerInterceptor_ptr > * copis_stack
 )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    CDMW_MUTEX_GUARD(M_mtx);
    m_request_context.insert(std::make_pair(request_id, copis_stack));
}


ContainerServerRequestInfo_impl *
ServerRequestInterceptorIterator_impl::build_container_server_request_info
(
 ::PortableInterceptor::ServerRequestInfo_ptr info
 ) 
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    ContainerServerRequestInfo_impl * csri = 0;
    CORBA::OctetSeq * origin_id = 0;
    CORBA::OctetSeq * target_id = 0;
    const char * feature_name = 0;

    try {

        CORBA::Any_var any = info->get_slot(m_origin_id_slot_id);
        any.in() >>= origin_id;

        any = info->get_slot(m_target_id_slot_id);
        any.in() >>= target_id;

        any = info->get_slot(m_target_feature_name_slot_id);
        any.in() >>= feature_name;

    } catch (::PortableInterceptor::InvalidSlot & is) {
        CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(), "During extraction of origin_id, target_id, feature_name: " << is);
        throw;
    }

    csri = new ContainerServerRequestInfo_impl
        (
         feature_name == 0 ? "" : feature_name,
         origin_id,
         target_id,
         info
         );

    return csri;
}
    

void
ServerRequestInterceptorIterator_impl::extract_from_request_service_context
(
 ::PortableInterceptor::ServerRequestInfo_ptr info
 )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    try {
        ::IOP::ServiceContext_var svc_ctx_received =
              info->get_request_service_context(COPIServiceInit::CDMW_COPI_SERVICE_CONTEXT_ID);

        
        ::Components::ContainerPortableInterceptor::COPIServiceContext_var copi_ctx_received =
              COPIUtils_impl::Service_to_copi_context(svc_ctx_received.in());
        

        /*
        CORBA::Any_var decoded_data;
        decoded_data = COPIUtils_impl::Get_codec()->decode (svc_ctx_received->context_data);
        ::Components::ContainerPortableInterceptor::COPIServiceContext * copi_ctx = 0;
        (decoded_data.in()) >>= copi_ctx;
        ::Components::ContainerPortableInterceptor::COPIServiceContext_var copi_ctx_received
              = copi_ctx;
        */

        


        {
            CORBA::Any any;
            if(0 != copi_ctx_received->origin_id) {
                any <<= copi_ctx_received->origin_id;
            }
            info->set_slot(m_origin_id_slot_id, any);
        }

        {
            CORBA::Any any;
            if(0 != copi_ctx_received->target_id) {
                any <<= copi_ctx_received->target_id;
            }
            info->set_slot(m_target_id_slot_id, any);
        }
        
        {
            CORBA::Any any;
            ::Components::ContainerPortableInterceptor::CustomSlotItemSeq csi =
                  copi_ctx_received->slot_info;

            any <<= csi;
            info->set_slot(m_custom_slot_item_seq_slot_id, any);
        }

    } catch (CORBA::BAD_PARAM & bp) {
        CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(), "Caught exception during extraction from request context: " << bp);
        throw;
    } catch (::IOP::Codec::FormatMismatch & fm) {
        CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(), "Caught exception during extraction from request context: " << fm);
        throw;
    } catch (::PortableInterceptor::InvalidSlot & is) {
        CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(), "Caught exception during extraction from request context: " << is);
        throw;
    } catch (::Cdmw::BadParameterException & bp) {
        CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(), "Caught exception during extraction from request context: " << bp);
        throw;
    }
       
    //extract the request Id and put it into the reply service context

    try {
        ::IOP::ServiceContext_var svc_ctx_received =
              info->get_request_service_context(COPIServiceInit::CDMW_CLIENT_COPI_SERVICE_CONTEXT_ID);

        if(0 != svc_ctx_received) {
            CDMW_DEBUG(CCMLoggers::Get_ccmcommon_logger(), "Adding the request Id into the reply service context");
            info->add_reply_service_context(*svc_ctx_received, true);
        } 
    } catch(CORBA::BAD_PARAM &) {
        CDMW_DEBUG(CCMLoggers::Get_ccmcommon_logger(), "No request Id service context found !!");
    }
}


int 
ServerRequestInterceptorIterator_impl::extract_req_id_from_request_service_context
(
 ::PortableInterceptor::ServerRequestInfo_ptr info,
 CORBA::ULong * request_id 
 )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    try {
        ::IOP::ServiceContext_var svc_ctx =
              info->get_request_service_context(COPIServiceInit::CDMW_CLIENT_COPI_SERVICE_CONTEXT_ID);

        CORBA::Any_var decoded_data = 0;
        decoded_data = COPIUtils_impl::Get_codec()->decode (svc_ctx->context_data);
        CDMW_ASSERT(decoded_data);

        if (!(decoded_data >>= (*request_id))) {
            CDMW_WARN(CCMLoggers::Get_ccmcommon_logger(), "no request id in request service context");
            return 1;
        }
    } catch (CORBA::BAD_PARAM & bp) {
        CDMW_DEBUG(CCMLoggers::Get_ccmcommon_logger(), "failed to exctract the request id because of " << bp << ". request id was not set");
        return 1;
    }


    return 0;
}


void 
ServerRequestInterceptorIterator_impl::build_reply_service_context
(
 ::PortableInterceptor::ServerRequestInfo_ptr info
 )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    ::Components::ContainerPortableInterceptor::CustomSlotItemSeq * csi = 0;

    try {
        CORBA::Any_var any = info->get_slot(m_custom_slot_item_seq_slot_id);
        if(!(any.in() >>= csi)) {
            CDMW_DEBUG(CCMLoggers::Get_ccmcommon_logger(), "no data");
        }

        ::CORBA::OctetSeq_var o_var;

        // building the service context
        CORBA::OctetSeq * origin_id = 0;
        CORBA::OctetSeq * target_id = 0;

        any = info->get_slot(m_origin_id_slot_id);
        if(!(any.in() >>= origin_id)) {
            CDMW_DEBUG(CCMLoggers::Get_ccmcommon_logger(), "no data in origin id  slot");
        }

        any = info->get_slot(m_target_id_slot_id);
        if(!(any.in() >>= target_id)) {
            CDMW_DEBUG(CCMLoggers::Get_ccmcommon_logger(), "no data in target id  slot");
        }

        ::Components::ContainerPortableInterceptor::COPIServiceContext * copi_ctx_to_send =
              new ::Components::ContainerPortableInterceptor::COPIServiceContext();
        
        
        if(origin_id) {
            copi_ctx_to_send->origin_id = *origin_id;
        } else {
            ::CORBA::OctetSeq_var os = new ::CORBA::OctetSeq();
            os->length(0);
            copi_ctx_to_send->origin_id = os;
        } 

        if(target_id) { 
            copi_ctx_to_send->target_id = *target_id;            
        } else {  
           ::CORBA::OctetSeq_var os = new ::CORBA::OctetSeq();
           os->length(0);
           copi_ctx_to_send->origin_id = os;
        } 

        if(csi) {
            copi_ctx_to_send->slot_info = *csi;
        } else {
            ::Components::ContainerPortableInterceptor::CustomSlotItemSeq_var csi_var
                  = new ::Components::ContainerPortableInterceptor::CustomSlotItemSeq();
            csi_var->length(0);
            copi_ctx_to_send->slot_info = csi_var;
        }
                 
        ::IOP::ServiceContext_var svc_ctx_to_send =
              COPIUtils_impl::Copi_to_service_context(copi_ctx_to_send);

        // setting the copi reply service context
        info->add_reply_service_context(svc_ctx_to_send.in(), true);

    } catch (::PortableInterceptor::InvalidSlot &) {
        // no error condition
        CDMW_DEBUG(CCMLoggers::Get_ccmcommon_logger(), "caught ::PortableInterceptor::InvalidSlot");
    } catch (::IOP::Codec::InvalidTypeForEncoding &) {
        CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(),
                   "::IOP::Codec::InvalidTypeForEncoding");

        throw;
    }

}


void 
ServerRequestInterceptorIterator_impl::extract_feature_name
(
 ::PortableInterceptor::ServerRequestInfo_ptr info
 )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
    // getting the server-side port name
    char * feature_name = 0;
    char * str_oid = 0;

    try {
        // might throw BAD PARAM if object id contains 0 what could be the case
        // if the servant USER_ID policy was not used.
        CORBA::OctetSeq_var object_id = info->object_id();
        
        str_oid =
            ::Cdmw::CCM::Common::OctetSeq_to_string(object_id);

    } catch (CORBA::BAD_PARAM & bp) {
        CDMW_WARN(CCMLoggers::Get_ccmcommon_logger(), "converting info->object_id raised BAD_PARAM");
    }

    ::Cdmw::CCM::CIF::CCMUtils::NameVector names;

    if(str_oid) {
        names =
            ::Cdmw::CCM::CIF::CCMUtils::Read_oid(str_oid);    

        CORBA::string_free(str_oid);
    }

    switch (names.size()) {
    case 1: { // oid is a component oid         
        feature_name = CORBA::string_dup("");
        break;
    }
    case 2: { // oid is a facet/consumer oid
        feature_name = CORBA::string_dup(names[1].c_str());
        break;
    }
    case 3: { // oid is a local push consumer"
        feature_name = CORBA::string_dup(names[1].c_str());
        break;
    }
    default: {        
        feature_name = CORBA::string_dup("NA"); //Not Available
    }
    }

    CORBA::Any any;
    any <<= feature_name;
    CORBA::string_free(feature_name);

    try {
        info->set_slot(m_target_feature_name_slot_id, any);
    } catch (::PortableInterceptor::InvalidSlot &) {
        CDMW_DEBUG(CCMLoggers::Get_ccmcommon_logger(), "caught ::PortableInterceptor::InvalidSlot");
    }
}

void
ServerRequestInterceptorIterator_impl::do_sending_ip
(
 PortableInterceptor::ServerRequestInfo_ptr info,
 sending_ip sip
 )
    throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

    CORBA::ULong req_id;
    if(extract_req_id_from_request_service_context(info, &req_id) != 0) {
        CDMW_DEBUG(CCMLoggers::Get_ccmcommon_logger(), "No request id in request service context, no basic copis called ");        
        return;
    }

    ContainerServerRequestInfo_impl * csri =
          build_container_server_request_info(info);

    CopisStackHolder_impl< 
        ::Components::ContainerPortableInterceptor::ServerContainerInterceptor_ptr> * req_dat =
        get_copis_stack(req_id);

    if(0 == req_dat) {        
        CDMW_WARN(CCMLoggers::Get_ccmcommon_logger(), "no copi context could be restored, no basic server copi will be called");
        return;
    }

    CopisStackHolderTrait<
        ::Components::ContainerPortableInterceptor::ServerContainerInterceptor_ptr >::copis_list_t *
        interceptors = req_dat->get_copis();

    int i = 0;
    try {
        if(sip == SEND_REPLY) {
            for (i = req_dat->get_recv_idx();
                 i < (long)interceptors->size() && i >= 0;
                 i--) 
                {                
                    req_dat->set_send_idx(i);
                    (*interceptors)[i]->send_reply(csri);
                    m_registration->removeServerCOPIRef((*interceptors)[i]);
                }
        } else {
            for (i = req_dat->get_send_idx() - 1;
                 i < (long)interceptors->size() && i >= 0 && i <= req_dat->get_recv_idx();
                 i--) 
                {
                    req_dat->set_send_idx (i);
                    switch(sip) {
                    case SEND_EXCEPTION: {
                        (*interceptors)[i]->send_exception(csri);
                        break;
                    }
                    case SEND_OTHER: {
                        (*interceptors)[i]->send_other(csri);
                        break;
                    }
                    default: {
                        CORBA::INTERNAL ex = CORBA::INTERNAL (Cdmw::OrbSupport::INTERNALBug, CORBA::COMPLETED_NO);
                        CDMW_LOG_FUNCTION_EXCEPTION(ex);
                        remove_copis_stack(req_id);
                        throw ex;
                    }
                    }

                    m_registration->removeServerCOPIRef((*interceptors)[i]);
                }
        }

    } catch(...) {
        if(i < (long)interceptors->size() && i >= 0) {
            m_registration->removeServerCOPIRef((*interceptors)[i]);
        }
        // copy the RSC to the reply service context
        build_reply_service_context(info);
        csri->_remove_ref();
        remove_copis_stack(req_id);
        throw;
    }

    if(i < (long)interceptors->size() && i >= 0) {
        m_registration->removeServerCOPIRef((*interceptors)[i]);
    }
    // copy the RSC to the reply service context
    build_reply_service_context(info); 
    csri->_remove_ref();
    remove_copis_stack(req_id);
}


// caller must delete return valu
CopisStackHolderTrait< ::Components::ContainerPortableInterceptor::ServerContainerInterceptor_ptr >::copis_list_t *
ServerRequestInterceptorIterator_impl::get_safe_server_interceptors
(
 const ::CdmwCcmCopi::ServerContainerInterceptorSeq * interceptors
 )
{
    CopisStackHolderTrait< ::Components::ContainerPortableInterceptor::ServerContainerInterceptor_ptr >::copis_list_t * safe_interceptors =
        new CopisStackHolderTrait< ::Components::ContainerPortableInterceptor::ServerContainerInterceptor_ptr >::copis_list_t();

    // try to increment ref for each interceptor, if this fails then the
    // interceptor was unregistered, i.e MUST not be used
    for (unsigned int i = 0; i < interceptors->length(); ++i) {
        if(m_registration->addServerCOPIRef((*interceptors)[i]) == 0) {
            safe_interceptors->push_back((*interceptors)[i]);
        }
    }

    return safe_interceptors;
}



} // End of namespace COPI
} // End of namespace CCM
} // End of namespace Cdmw
