/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

// standard files


// needed for insertion of SystemException into an any
#include "tao/AnyTypeCode/ExceptionA.h"

// Cdmw Files
#include <orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>

// Local Files
#include <CCMContainer/ccmcopi/StubContainerInterceptorIterator_impl.hpp>


namespace 
{    
} // end of anonymous namespace


namespace Cdmw
{
namespace CCM
{
namespace COPI
{
    StubContainerInterceptorIterator_impl::StubContainerInterceptorIterator_impl 
    (
     ::CdmwCcmCopi::StubContainerInterceptorSeq * stub_interceptors, 
     ::CdmwCcmCopi::RegisteredCOPICollection_ptr registration_util
     ) : m_copis_list(::CdmwCcmCopi::RegisteredCOPICollection::_duplicate(registration_util)),
         m_sending_state_interceptor_index(-1),
         m_receiving_state_interceptor_index(-1),
         m_safe_interceptors(0)
    { 
        CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");        

        if(stub_interceptors->length() > 32767) {

            // since we will be later iterating / looping using ints, we
            // need to limit the maximum number of copis

            CDMW_ERROR(CCMLoggers::Get_ccmcommon_logger(), "too many copis (more than 32767)");
            CORBA::BAD_PARAM ex = CORBA::BAD_PARAM (Cdmw::OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }

        m_stub_interceptors =
            new CdmwCcmCopi::StubContainerInterceptorSeq();
        
        m_stub_interceptors->length(stub_interceptors->length());
        for(::CORBA::ULong i = 0; i < stub_interceptors->length(); ++i) {
            m_stub_interceptors[i] = (*stub_interceptors)[i];
        }

    };


    StubContainerInterceptorIterator_impl::~StubContainerInterceptorIterator_impl () 
    { 
        CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
        // a StubContainerInterceptorIterator_impl is only good for one
        // request, i.e. it is to be recreated for each request
        delete m_safe_interceptors;
        m_safe_interceptors = 0;
    };

    
    void StubContainerInterceptorIterator_impl::stub_send_request 
    (
     ::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr info,
     ::CORBA::Boolean & proceed_call
     )
        throw (
               ::CORBA::SystemException,
               ::PortableInterceptor::ForwardRequest
               ) 
    {
        CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
        bool is_call_completed = true;
        unsigned int i = 0;


        m_safe_interceptors = get_safe_stub_interceptors(m_stub_interceptors);
        

        try {
            for (i = 0; i < m_safe_interceptors->size(); i++) {

                if (proceed_call){
                    (*m_safe_interceptors)[i]->stub_send_request(info, proceed_call);
                    m_sending_state_interceptor_index = i;
                    
                    if (!proceed_call) {
                        is_call_completed = false;
                        break;
                    }
                }
            }
        } catch (::PortableInterceptor::ForwardRequest & e) {
            CDMW_WARN(CCMLoggers::Get_ccmcommon_logger(),
                                        "ForwardRequest exception returned from the ExtCOPI '"
                                        << (*m_safe_interceptors)[i]->name() << "' when calling stub_send_request: " 
                                        << e);

            ::CORBA::Any ex;
            ex <<= e; 
            info->the_exception(ex);
            is_call_completed = false;
            
        } catch(::CORBA::SystemException & se) {
            CDMW_WARN(CCMLoggers::Get_ccmcommon_logger(),
                      "SystemException exception returned from the ExtCOPI '"
                      << (*m_safe_interceptors)[i]->name() << "' when calling stub_send_request: " 
                      << se);

            ::CORBA::Any ex;
            ex <<= se; 
            info->the_exception(ex);
            is_call_completed = false; 
        } 

        // No interceptor is in the RECEIVING state yet
        if (m_safe_interceptors->size() > 0) {
            if (!proceed_call) {
                m_receiving_state_interceptor_index =
                    m_sending_state_interceptor_index;
            } else {
                m_receiving_state_interceptor_index =
                    m_sending_state_interceptor_index + 1;
            }
            if(!is_call_completed || !proceed_call) {
                m_copis_list->removeStubCOPIRef((*m_safe_interceptors)[i]);
            }               
        }

    };


    void StubContainerInterceptorIterator_impl::stub_receive_reply 
    (
     ::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr info,
     ::CORBA::Boolean & proceed_call
     )
        throw (
               ::CORBA::SystemException
               )
    {
        CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");

        int j = m_sending_state_interceptor_index;

        CDMW_ASSERT(0 != m_safe_interceptors);
        
        try {            
            for (;
                 (unsigned int)j >= 0 && (unsigned int)j < m_safe_interceptors->size();
                 j--) 
                {
                    if(true == proceed_call) {                    
                        m_receiving_state_interceptor_index = j;
                        (*m_safe_interceptors)[j]->stub_receive_reply(info, proceed_call);
                        m_copis_list->removeStubCOPIRef((*m_safe_interceptors)[j]);
                    }
                    else {
                        stub_receive_other(info);
                        break;
                    }
                }                    
        } catch(::CORBA::SystemException & se) {

            CDMW_WARN(CCMLoggers::Get_ccmcommon_logger(),
                                        "SystemException exception returned from the ExtCOPI '"
                                        << (*m_safe_interceptors)[j]->name() << "' when calling stub_receive_exception: " 
                                        << se);

            ::CORBA::Any ex;
            ex <<= se; 
            info->the_exception(ex);

            m_copis_list->removeStubCOPIRef((*m_safe_interceptors)[j]);
            stub_receive_exception(info, proceed_call);
        } catch(...) {
            m_copis_list->removeStubCOPIRef((*m_safe_interceptors)[j]); 
            throw;
        }
    }


    void StubContainerInterceptorIterator_impl::stub_receive_exception 
    (
     ::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr info,
     ::CORBA::Boolean & proceed_call
     )
        throw (
               ::CORBA::SystemException,
               ::PortableInterceptor::ForwardRequest
               ) 
    {
        CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcommon_logger(), "");
        CDMW_ASSERT(0 != m_safe_interceptors);

        int j = m_receiving_state_interceptor_index - 1;

        try {
            for (; 
                 j >= 0 && (unsigned int)j < m_safe_interceptors->size() && j <= m_sending_state_interceptor_index;
                 j--)
                {
                    if(true == proceed_call) {
                        m_receiving_state_interceptor_index = j;
                        (*m_safe_interceptors)[j]->stub_receive_exception(info, proceed_call);
                        m_copis_list->removeStubCOPIRef((*m_safe_interceptors)[j]);
                    }
                    else {
                        stub_receive_other(info);
                        break;
                    }
                }
                        
        } catch (::PortableInterceptor::ForwardRequest & e) {
            CDMW_WARN(CCMLoggers::Get_ccmcommon_logger(),
                                        "ForwardRequest exception returned from the ExtCOPI '"
                                        << (*m_safe_interceptors)[j]->name() << "' when calling stub_receive_exception: " 
                                        << e);

            m_copis_list->removeStubCOPIRef((*m_safe_interceptors)[j]);

            ::CORBA::Any ex;
            ex <<= e; 
            info->the_exception(ex);

            stub_receive_other(info);
            
        } catch(::CORBA::SystemException & se) {
            CDMW_WARN(CCMLoggers::Get_ccmcommon_logger(),
                                        "SystemException exception returned from the ExtCOPI '"
                                        << (*m_safe_interceptors)[j]->name() << "' when calling stub_receive_exception: " 
                                        << se);
            m_copis_list->removeStubCOPIRef((*m_safe_interceptors)[j]);

            ::CORBA::Any ex;
            ex <<= se; 
            info->the_exception(ex);

            // recursive call for further exception processing 
            stub_receive_exception(info, proceed_call);
        } catch(...) {
            CDMW_WARN(CCMLoggers::Get_ccmcommon_logger(),
                                        "Unknown exception returned from the ExtCOPI '"
                                        << (*m_safe_interceptors)[j]->name() << "' when calling stub_receive_exception");
            m_copis_list->removeStubCOPIRef((*m_safe_interceptors)[j]); 


            throw;
        }
    }
    


    void StubContainerInterceptorIterator_impl::stub_receive_other 
    (
     ::Components::ContainerPortableInterceptor::ContainerStubRequestInfo_ptr info
     )
        throw (
               ::CORBA::SystemException,
               ::PortableInterceptor::ForwardRequest
               )
    {
        int j = m_receiving_state_interceptor_index - 1;

        try {
            
            for (;
                 j >= 0 && (unsigned int)j < m_safe_interceptors->size() && j <= m_sending_state_interceptor_index;
                 j--)
                {
                    m_receiving_state_interceptor_index = j;
                    (*m_safe_interceptors)[j]->stub_receive_other(info);
                    m_copis_list->removeStubCOPIRef((*m_safe_interceptors)[j]);
                }          
                        
        } catch (::PortableInterceptor::ForwardRequest & e) {
            CDMW_DEBUG(CCMLoggers::Get_ccmcommon_logger(),
                    "ForwardRequest exception returned from the ExtCOPI '"
                       << (*m_safe_interceptors)[j]->name() << "' when calling stub_receive_other: " << e);

            m_copis_list->removeStubCOPIRef((*m_safe_interceptors)[j]);

            ::CORBA::Any ex;
            ex <<= e;
            info->the_exception(ex);
            stub_receive_other(info);
            
        } catch(::CORBA::SystemException & se) {
            CDMW_DEBUG(CCMLoggers::Get_ccmcommon_logger(),
                    "System exception returned from the ExtCOPI '"
                       << (*m_safe_interceptors)[j]->name() << "' when calling stub_receive_other: " << se);

            m_copis_list->removeStubCOPIRef((*m_safe_interceptors)[j]);

            ::CORBA::Any ex;
            ex <<= se; 
            info->the_exception(ex);
            CORBA::Boolean boolvar = true;
            stub_receive_exception(info, boolvar);

        } catch(...) {
            CDMW_DEBUG(CCMLoggers::Get_ccmcommon_logger(),
                    "Unknown exception returned from the ExtCOPI '"
                       << (*m_safe_interceptors)[j]->name() << "' when calling stub_receive_other");

            m_copis_list->removeStubCOPIRef((*m_safe_interceptors)[j]); 

            throw;
        }

    }


    stub_copis_list_t *
    StubContainerInterceptorIterator_impl::get_safe_stub_interceptors
    (
     const CdmwCcmCopi::StubContainerInterceptorSeq & interceptors
     )
    {
        stub_copis_list_t * safe_interceptors =
            new stub_copis_list_t();

        // try to increment ref for each interceptor, if this fails then the
        // interceptor was unregistered, i.e MUST not be used
        for (unsigned int i = 0; i < interceptors.length(); ++i) {
            if(0 == m_copis_list->addStubCOPIRef(interceptors[i])) {
                safe_interceptors->push_back(interceptors[i]);
            }
        }

        return safe_interceptors;
    }



} // End of namespace COPI
} // End of namespace CCM
} // End of namespace Cdmw
