/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.ccmcopi;

import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;

import org.jacorb.orb.SystemExceptionHelper;
import org.omg.CORBA.Any;
import org.omg.CORBA.BooleanHolder;
import org.omg.CORBA.SystemException;
import org.omg.Components.ContainerPortableInterceptor.ContainerServantRequestInfo;
import org.omg.Components.ContainerPortableInterceptor.ServantContainerInterceptor;
import org.omg.PortableInterceptor.ForwardRequest;
import org.omg.PortableInterceptor.ForwardRequestHelper;

import com.thalesgroup.CdmwCcmCopi.RegisteredCOPICollection;

/**
 *
 */
public class ServantContainerInterceptorIterator {
    
    private cdmw.logging.Logger logger = cdmw.ccm.common.CCMLoggers.GetCcmCopiLogger();
    
    
    /**
     * The list of registered servant interceptors
     */
    private ServantContainerInterceptor[] servantInterceptors;
    
    
    /**
     * A servant COPI enters the SENDING state whenever it processes
     * servant_send_request operations successfully. it gives the index (in the servant COPI
     * list) of the last Server COPI that has already processed a sending end
     * point. All greater indexes are in the SENDING state
     */
    private int SENDINGStateInterceptorIndex = -1;
    
    /**
     * A servant COPI enters the RECEIVING state whenever it processes the
     * servant_receive_request operation successfully. it gives the
     * index (in the servant COPI list) of the last Server COPI that processes
     * stub_receive_request successfully.
     */
    private int RECEIVINGStateInterceptorIndex = -1; 
    
    
    private RegisteredCOPICollection COPIsList;

    
    public ServantContainerInterceptorIterator (ServantContainerInterceptor[] servant_interceptors, RegisteredCOPICollection registrationUtil){
        servantInterceptors = new ServantContainerInterceptor[servant_interceptors.length];
        System.arraycopy(servant_interceptors, 0, servantInterceptors, 0, servant_interceptors.length);
        this.COPIsList = registrationUtil;        
    }
    
    
    public void servant_receive_request(ContainerServantRequestInfo info,
                                        BooleanHolder proceed_call) {
        CDMW_LOG_FUNCTION(logger, "operation["+info.operation()+"]");
        boolean isCallCompleted = true;
        int i = 0;
        try {
            for (i = 0; i < servantInterceptors.length; i++) {

                if (proceed_call.value){
                    if (COPIsList.addServantCOPIRef(servantInterceptors[i]) != 0){
                        servantInterceptors = COPIUtils.removeArrayElement(servantInterceptors, i);
                        i--;
                        continue;
                    }
                    servantInterceptors[i].servant_receive_request(info, proceed_call);
                    RECEIVINGStateInterceptorIndex = i;
                    
                    if (!proceed_call.value){
                        isCallCompleted = false;
                        break;
                    }
                }
            }
        } catch (ForwardRequest e) {
            CDMW_WARN(logger,
                    "ForwardRequest exception returned from the ExtCOPI '"
                            + servantInterceptors[i].name() + "' when calling servant_receive_request");
            Any ex = COPIUtils.createAny();
            ForwardRequestHelper.insert(ex, e);
            info.the_exception(ex);
            isCallCompleted = false;
            
        } catch(SystemException se) {
            CDMW_WARN(logger, "SystemException generated !");
            se.printStackTrace();
            Any ex = COPIUtils.createAny();
            SystemExceptionHelper.insert(ex, se);
            info.the_exception(ex);
            isCallCompleted = false;
        } finally {

            // No interceptor is in the SENDING state yet
            if (servantInterceptors.length != 0){
                if (!proceed_call.value) {
                    SENDINGStateInterceptorIndex = RECEIVINGStateInterceptorIndex;
                } else {
                    SENDINGStateInterceptorIndex = RECEIVINGStateInterceptorIndex + 1;
                }
                if(!isCallCompleted || !proceed_call.value) {
                    COPIsList.removeServantCOPIRef(servantInterceptors[i]);

                
            }
            CDMW_LOG_FUNCTION_RETURN(logger, "isCallCompleted=["+isCallCompleted+"]");
                
        }
            CDMW_LOG_FUNCTION_RETURN(logger, "isCallCompleted=["+isCallCompleted+"]");
        }

    }


    public void servant_send_exception(ContainerServantRequestInfo info,
            BooleanHolder proceed_call) {
        CDMW_LOG_FUNCTION(logger, "operation["+info.operation()+"]");
        
        int j = SENDINGStateInterceptorIndex - 1;
        try{
            for (; j >= 0 && j < servantInterceptors.length && j <= SENDINGStateInterceptorIndex ; j--) {
                if(proceed_call.value) {

                    SENDINGStateInterceptorIndex = j;
                    servantInterceptors[j].servant_send_exception(info, proceed_call);
                    COPIsList.removeServantCOPIRef(servantInterceptors[j]);
                }
                else {
                    servant_send_other(info);
                    break;
                }
            }
                        
        }catch (ForwardRequest e) {
            COPIsList.removeServantCOPIRef(servantInterceptors[j]);
            CDMW_WARN(logger,
                    "ForwardRequest exception returned from the ExtCOPI '"
                            + servantInterceptors[j].name() + "' when calling servant_send_exception");
            Any ex = COPIUtils.createAny();
            ForwardRequestHelper.insert(ex, e);
            info.the_exception(ex);
            servant_send_other(info);
        } catch(SystemException se) {
            COPIsList.removeServantCOPIRef(servantInterceptors[j]);
            Any ex = COPIUtils.createAny();
            SystemExceptionHelper.insert(ex, se);
            info.the_exception(ex);
            servant_send_exception(info, proceed_call);
        }catch(RuntimeException re){
            COPIsList.removeServantCOPIRef(servantInterceptors[j]); 
            CDMW_LOG_FUNCTION_EXCEPTION(logger,re);
            throw re;
        }
        CDMW_LOG_FUNCTION_RETURN(logger);
    }

    public void servant_send_other(ContainerServantRequestInfo info) {
        CDMW_LOG_FUNCTION(logger, "operation["+info.operation()+"]");
        
        int j = SENDINGStateInterceptorIndex - 1;
        try{
            for (; j >= 0 && j < servantInterceptors.length && j <= SENDINGStateInterceptorIndex ; j--){               
 
                    SENDINGStateInterceptorIndex = j;
                    servantInterceptors[j].servant_send_other(info);
                    COPIsList.removeServantCOPIRef(servantInterceptors[j]);
            }
        }catch (ForwardRequest e) {
            COPIsList.removeServantCOPIRef(servantInterceptors[j]);
            CDMW_WARN(logger,
                    "ForwardRequest exception returned from the ExtCOPI '"
                            + servantInterceptors[j].name() + "' when calling servant_send_other");
            Any ex = COPIUtils.createAny();
            ForwardRequestHelper.insert(ex, e);
            info.the_exception(ex);
            servant_send_other(info);
        } catch(SystemException se) {
            COPIsList.removeServantCOPIRef(servantInterceptors[j]);
            Any ex = COPIUtils.createAny();
            SystemExceptionHelper.insert(ex, se);
            info.the_exception(ex);
            servant_send_exception(info, new BooleanHolder(true));
        } catch(RuntimeException re){
            COPIsList.removeServantCOPIRef(servantInterceptors[j]); 
            CDMW_LOG_FUNCTION_EXCEPTION(logger,re);
            throw re;
        }
        CDMW_LOG_FUNCTION_RETURN(logger);
    }

    public void servant_send_reply(ContainerServantRequestInfo info,
            BooleanHolder proceed_call) {
        CDMW_LOG_FUNCTION(logger, "operation["+info.operation()+"]");
        
        int j = RECEIVINGStateInterceptorIndex;
        try{
            
            for (; j >= 0 && j < servantInterceptors.length; j--){
                if (proceed_call.value){
                    
                    SENDINGStateInterceptorIndex = j;
                    servantInterceptors[j].servant_send_reply(info, proceed_call);
                    COPIsList.removeServantCOPIRef(servantInterceptors[j]);
                }
                else {
                    servant_send_other(info);
                    break;
                }
                
            }
                        
        }catch(SystemException se) {
            COPIsList.removeServantCOPIRef(servantInterceptors[j]);
            Any ex = COPIUtils.createAny();
            SystemExceptionHelper.insert(ex, se);
            info.the_exception(ex);
            servant_send_exception(info, proceed_call);
        }catch(RuntimeException re){
            COPIsList.removeServantCOPIRef(servantInterceptors[j]); 
            CDMW_LOG_FUNCTION_EXCEPTION(logger,re);
            throw re;
        }
        CDMW_LOG_FUNCTION_RETURN(logger);
    }
}
