/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.ccmcopi;

import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;

import org.jacorb.orb.SystemExceptionHelper;
import org.omg.CORBA.Any;
import org.omg.CORBA.BooleanHolder;
import org.omg.CORBA.SystemException;
import org.omg.Components.ContainerPortableInterceptor.ContainerStubRequestInfo;
import org.omg.Components.ContainerPortableInterceptor.StubContainerInterceptor;
import org.omg.PortableInterceptor.ForwardRequest;
import org.omg.PortableInterceptor.ForwardRequestHelper;

import com.thalesgroup.CdmwCcmCopi.RegisteredCOPICollection;

/**
 *
 */
public class StubContainerInterceptorIterator {

    private cdmw.logging.Logger logger = cdmw.ccm.common.CCMLoggers.GetCcmCopiLogger();


    /**
     * The list of registered stub interceptors
     */
    private StubContainerInterceptor[] stubInterceptors;


    /**
     * A Stub COPI enters the SENDING state whenever it processes
     * stub_send_reuqest operations successfully. it gives the index (in the stub COPI
     * list) of the last Server COPI that has already processed a sending end
     * point. All lesser indexes are in the SENDING state
     */
    private int SENDINGStateInterceptorIndex = -1;

    /**
     * A stub COPI enters the RECEIVING state whenever it processes one of the
     * stub_receive_* operations successfully. it gives the
     * index (in the server COPI list) of the last Server COPI that processes
     * stub_receive_* successfully.
     */
    private int RECEIVINGStateInterceptorIndex = -1;


    private RegisteredCOPICollection COPIsList;

    public StubContainerInterceptorIterator (StubContainerInterceptor[] stub_interceptors, RegisteredCOPICollection registrationUtil){
        CDMW_LOG_FUNCTION(logger, "");
        stubInterceptors = new StubContainerInterceptor[stub_interceptors.length];
        System.arraycopy(stub_interceptors, 0, stubInterceptors, 0, stub_interceptors.length);
        this.COPIsList = registrationUtil;
        CDMW_LOG_FUNCTION_RETURN(logger);
    }

    public void stub_receive_exception(ContainerStubRequestInfo info,
            BooleanHolder proceed_call) {
        CDMW_LOG_FUNCTION(logger, "operation["+info.operation()+"]");

        int j = RECEIVINGStateInterceptorIndex - 1;
        try{
            for (; j >= 0 && j < stubInterceptors.length && j <= SENDINGStateInterceptorIndex ; j--)
                if(proceed_call.value) {
                    RECEIVINGStateInterceptorIndex = j;
                    stubInterceptors[j].stub_receive_exception(info, proceed_call);
                    COPIsList.removeStubCOPIRef(stubInterceptors[j]);
                }
                else {
                    stub_receive_other(info);
                    break;
                }

        }catch (ForwardRequest e) {
            COPIsList.removeStubCOPIRef(stubInterceptors[j]);
            CDMW_WARN(logger,
                    "ForwardRequest exception returned from the ExtCOPI '"
                            + stubInterceptors[j].name() + "' when calling stub_receive_other");
            Any ex = COPIUtils.createAny();
            ForwardRequestHelper.insert(ex, e);
            info.the_exception(ex);
            stub_receive_other(info);

        } catch(SystemException se) {
            COPIsList.removeStubCOPIRef(stubInterceptors[j]);
            Any ex = COPIUtils.createAny();
            SystemExceptionHelper.insert(ex, se);
            info.the_exception(ex);
            stub_receive_exception(info, proceed_call);
        } catch(RuntimeException re){
            COPIsList.removeStubCOPIRef(stubInterceptors[j]); 
            CDMW_LOG_FUNCTION_EXCEPTION(logger, re);
            throw re;
        }
        CDMW_LOG_FUNCTION_RETURN(logger);
    }


    public void stub_receive_other(ContainerStubRequestInfo info){
        CDMW_LOG_FUNCTION(logger, "operation["+info.operation()+"]");

        int j = RECEIVINGStateInterceptorIndex - 1;
        try{

            for (; j >= 0 && j < stubInterceptors.length && j <= SENDINGStateInterceptorIndex; j--){

                RECEIVINGStateInterceptorIndex = j;
                stubInterceptors[j].stub_receive_other(info);
                COPIsList.removeStubCOPIRef(stubInterceptors[j]);
            }

            Any ex =info.the_exception();
            if(ex!=null && ForwardRequestHelper.type().equal(ex.type())) {
             // TODO : invoking a strategy object that handles the Exception
                //new COPIForwardRequestHandler().proceed(proceed_call, info,...);
            }

        }catch (ForwardRequest e) {
            COPIsList.removeStubCOPIRef(stubInterceptors[j]);
            CDMW_DEBUG(logger,
                    "ForwardRequest exception returned from the ExtCOPI '"
                            + stubInterceptors[j].name() + "' when calling stub_receive_other");
            Any ex = COPIUtils.createAny();
            ForwardRequestHelper.insert(ex, e);
            info.the_exception(ex);
            stub_receive_other(info);

        } catch(SystemException se) {
            COPIsList.removeStubCOPIRef(stubInterceptors[j]);
            Any ex = COPIUtils.createAny();
            SystemExceptionHelper.insert(ex, se);
            info.the_exception(ex);
            BooleanHolder proceed_call =new BooleanHolder(true);
            stub_receive_exception(info, proceed_call);
        }catch(RuntimeException re){
            COPIsList.removeStubCOPIRef(stubInterceptors[j]); 
            CDMW_LOG_FUNCTION_EXCEPTION(logger, re);
            throw re;
        }
        CDMW_LOG_FUNCTION_RETURN(logger);
    }


    public void stub_receive_reply(ContainerStubRequestInfo info,
            BooleanHolder proceed_call) {
        CDMW_LOG_FUNCTION(logger, "operation["+info.operation()+"]");

        int j = SENDINGStateInterceptorIndex;

        try{

            for (; j >= 0 && j < stubInterceptors.length; j--){
                if(proceed_call.value) {
                    
                    RECEIVINGStateInterceptorIndex = j;
                    stubInterceptors[j].stub_receive_reply(info, proceed_call);
                    COPIsList.removeStubCOPIRef(stubInterceptors[j]);
                }
                else {
                    stub_receive_other(info);
                    break;
                }
            }

        }catch(SystemException se) {
            Any ex = COPIUtils.createAny();
            SystemExceptionHelper.insert(ex, se);
            info.the_exception(ex);
            COPIsList.removeStubCOPIRef(stubInterceptors[j]);
            stub_receive_exception(info, proceed_call);
        }catch(RuntimeException re){
            COPIsList.removeStubCOPIRef(stubInterceptors[j]); 
            CDMW_LOG_FUNCTION_EXCEPTION(logger, re);
            throw re;
        }
        CDMW_LOG_FUNCTION_RETURN(logger);
    }


    public void stub_send_request(ContainerStubRequestInfo info,
            BooleanHolder proceed_call){
        CDMW_LOG_FUNCTION(logger, "operation["+info.operation()+"]");
        boolean isCallCompleted = true;
        int i = 0;
        try {
            for (i = 0; i < stubInterceptors.length; i++) {

                if (proceed_call.value){
                    if(COPIsList.addStubCOPIRef(stubInterceptors[i]) != 0){
                        stubInterceptors = COPIUtils.removeArrayElement(stubInterceptors, i);
                        i--;
                        continue;
                    }
                    stubInterceptors[i].stub_send_request(info, proceed_call);
                    SENDINGStateInterceptorIndex = i;
                    
                    if (!proceed_call.value){
                        isCallCompleted = false;
                        break;
                    }
                }
            }
        } catch (ForwardRequest e) {
            CDMW_WARN(logger,
                    "ForwardRequest exception returned from the ExtCOPI '"
                            + stubInterceptors[i].name() + "' when calling stub_send_request");
            Any ex = COPIUtils.createAny();
            ForwardRequestHelper.insert(ex, e);
            info.the_exception(ex);
            isCallCompleted = false;

        } catch(SystemException se) {
            Any ex = COPIUtils.createAny();
            SystemExceptionHelper.insert(ex, se);
            info.the_exception(ex);
            isCallCompleted = false;
        } finally {

            // No interceptor is in the RECEIVING state yet
            if (stubInterceptors.length != 0){
                if (!proceed_call.value) {
                    RECEIVINGStateInterceptorIndex = SENDINGStateInterceptorIndex;
                } else {
                    RECEIVINGStateInterceptorIndex = SENDINGStateInterceptorIndex + 1;
                }
                if(!isCallCompleted || !proceed_call.value) {
                    COPIsList.removeStubCOPIRef(stubInterceptors[i]);
                }
 
            }
            CDMW_LOG_FUNCTION_RETURN(logger);
        }
    }
}
