/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.container;

import org.omg.CORBA.Object;
import org.omg.Components.ConfigValuesHolder;
import org.omg.Components.ContainerPortableInterceptor.IntegrationPoint;

import cdmw.ccm.cif.CCMHomeImpl;
import cdmw.ccm.common.CCMLoggers;

import com.thalesgroup.CdmwCcmContainer.ComponentLifeCycleInterceptor;
import com.thalesgroup.CdmwCcmContainer.ComponentLifeCycleInterceptorOperations;
import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;

/**
 * call operations on registered lifecycle interceptors before/after operations on components
 */
public class ComponentLifecycleInterceptorIterator implements
        ComponentLifeCycleInterceptorOperations {

    private ComponentLifeCycleInterceptor[] interceptors;

    private CCMHomeImpl compIdentityCreator;

    public ComponentLifecycleInterceptorIterator(CCMHomeImpl ref){
        interceptors = LifeCycleInterceptorRegistration.getInstance().listComponentLifeCycleInterceptor();
        this.compIdentityCreator = ref;
    }


    public void after_component_activate(String compId) {

        for(ComponentLifeCycleInterceptor copi: interceptors){
            try{
                copi.after_component_activate(compId);
            }
            catch(Exception e){
                CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(), "One of the COPIs has failed to process after_component_activate");
                CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcontainerLogger(), e);
             }
        }
    }


    public void after_component_connect(String compId) {

        for(ComponentLifeCycleInterceptor copi: interceptors){
            try{
                copi.after_component_connect(compId);
            }
            catch(Exception e){
                CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(), "One of the COPIs has failed to process after_component_connect");
                CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcontainerLogger(), e);
             }
        }
    }


    public void after_component_create(String compId, Object compRef) {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcontainerLogger(), "compId="+compId);
        CDMW_DEBUG(CCMLoggers.GetCcmcontainerLogger(), "There are "+interceptors.length+" interceptors");
        for(ComponentLifeCycleInterceptor copi: interceptors){
            try{
                copi.after_component_create(compId, compRef);
            }
            catch(Exception e){
                CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(), "One of the COPIs has failed to process after_component_create");
               CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcontainerLogger(), e);
            }
        }
       CDMW_LOG_FUNCTION_RETURN(CCMLoggers.GetCcmcontainerLogger());
    }


    public void after_component_passivate(String compId) {

        for(ComponentLifeCycleInterceptor copi: interceptors){
            try{
                copi.after_component_passivate(compId);
            }
            catch(Exception e){
               CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(), "One of the COPIs has failed to process after_component_passivate");
               CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcontainerLogger(), e);
            }
        }

    }


    public void after_component_remove(String compId) {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcontainerLogger(), "compId="+compId);
        CDMW_DEBUG(CCMLoggers.GetCcmcontainerLogger(), "There are "+interceptors.length+" interceptors");

        for(ComponentLifeCycleInterceptor copi: interceptors){
            try{
                copi.after_component_remove(compId);
            }
            catch(Exception e){
                CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(), "One of the COPIs has failed to process after_component_remove");
                CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcontainerLogger(), e);
             }
        }
       CDMW_LOG_FUNCTION_RETURN(CCMLoggers.GetCcmcontainerLogger());
    }

    public void before_component_activate(String compId) {

        for(ComponentLifeCycleInterceptor copi: interceptors){
            copi.before_component_activate(compId);
        }

    }


    public void before_component_connect(String compId, String featureName, Object compRef) {

        for(ComponentLifeCycleInterceptor copi: interceptors){
            try{
                copi.before_component_connect(compId, featureName, compRef);
            }
            catch(Exception e){
                CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(), "One of the COPIs has failed to process before_component_connect");
                CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcontainerLogger(), e);
             }
        }

    }


    public void before_component_create(String compId, ConfigValuesHolder compConfigVals) {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcontainerLogger(), "compId="+compId);
        CDMW_DEBUG(CCMLoggers.GetCcmcontainerLogger(), "There are "+interceptors.length+" interceptors");

        String id = compId;
        for(ComponentLifeCycleInterceptor copi: interceptors){
            try{
                copi.before_component_create(id, compConfigVals);
                // we need to re-compute the comp id because it is based on the component Name
                // config value that can be modified
                id = compIdentityCreator.create_component_identity(compConfigVals.value);
            }
            catch(Exception e){
                CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(), "One of the COPIs has failed to process before_component_create");
                CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcontainerLogger(), e);
             }
        }
        CDMW_LOG_FUNCTION_RETURN(CCMLoggers.GetCcmcontainerLogger());
    }


    public void before_component_passivate(String compId) {

        for(ComponentLifeCycleInterceptor copi: interceptors){
            try{
                copi.before_component_passivate(compId);
            }
            catch(Exception e){
                CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(), "One of the COPIs has failed to process before_component_passivate");
                CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcontainerLogger(), e);
             }
        }

    }


    public void before_component_remove(String compId) {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcontainerLogger(), "compId="+compId);
        CDMW_DEBUG(CCMLoggers.GetCcmcontainerLogger(), "There are "+interceptors.length+" interceptors");

        for(ComponentLifeCycleInterceptor copi: interceptors){
            try{
                copi.before_component_remove(compId);
            }
            catch(Exception e){
                CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(), "One of the COPIs has failed to process before_component_remove");
                CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcontainerLogger(), e);
             }
        }
       CDMW_LOG_FUNCTION_RETURN(CCMLoggers.GetCcmcontainerLogger());
    }

    public void destroy() {
    }


    public String name() {
        return "ComponentLifecycleIterator";
    }


    public short priority() {
        return 0;
    }


    public IntegrationPoint registration_info() {
        return null;
    }


    public void set_slot_id(int arg0) {
    }

}
