/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.qosenablers.dds;

import org.omg.CCM_DDS.DDS_TopicBase;
import org.omg.CCM_DDS.InternalError;
import org.omg.CORBA.BAD_OPERATION;
import org.omg.CORBA.BAD_PARAM;
import org.omg.Components.CCMException;

import DDS.ANY_STATUS;
import DDS.DataReader;
import DDS.DataReaderQosHolder;
import DDS.DataWriterQosHolder;
import DDS.DataWriter;
import DDS.DomainParticipant;
import DDS.DomainParticipantFactory;
import DDS.DomainParticipantQosHolder;
import DDS.Publisher;
import DDS.PublisherQosHolder;
import DDS.Subscriber;
import DDS.SubscriberQosHolder;
import DDS.Topic;
import DDS.TopicQosHolder;
import DDS.TypeSupport;

/**
 *
 */
public abstract class DDS_TopicBaseFragmentImpl extends DDS_BaseFragmentImpl
        implements DDS_TopicBase {

    private String topic_name; // = "DefaultTOPIC";
    private String[] key_fields;

    /**
     * Generic DDS entities
     */
    protected DomainParticipant participant;
    protected Topic topic;
    protected Subscriber subscriber;
    protected DataReader reader;
    protected Publisher publisher;
    protected DataWriter writer;

    /**
     * Qos policies to be applied on the entities
     */
    protected DDS.DomainParticipantQos participantQos = DDS.PARTICIPANT_QOS_DEFAULT.value;
    protected DDS.TopicQos topicQos = DDS.TOPIC_QOS_DEFAULT.value;
    protected DDS.PublisherQos publisherQos = DDS.PUBLISHER_QOS_DEFAULT.value;
    protected DDS.SubscriberQos subscriberQoS = DDS.SUBSCRIBER_QOS_DEFAULT.value;
    protected DDS.DataReaderQos datareaderQoS = DDS.DATAREADER_QOS_DEFAULT.value;
    protected DDS.DataWriterQos datawriterQoS = DDS.DATAWRITER_QOS_DEFAULT.value;

    public String[] key_fields() {
        return this.key_fields;
    }

    public void key_fields(String[] keys) {
        if (this.key_fields == null) {
            this.key_fields = new String[keys.length];
            System.arraycopy(keys, 0, this.key_fields, 0, keys.length);
        } else
            throw new BAD_OPERATION("The key_fields is already set !");

    }

    public String topic_name() {
        return this.topic_name;
    }

    public void topic_name(String name) {
        if (name == null) {
            logger.warning("invalid topic name: " + name);
            throw new BAD_PARAM("invalid topic name: " + name);
        }

        if (this.topic_name == null) {
            this.topic_name = new String(name);
            logger.info("topic_name set to " + name);
        } else
            throw new BAD_OPERATION("The topic_name is already set !");

    }

    public DDS_TopicBaseFragmentImpl() {

    }

    public DDS_TopicBaseFragmentImpl(String domain_id, String qos_profile,
            String topic_name, String[] key_fields) {
        super(domain_id, qos_profile);
        this.topic_name(topic_name);
        this.key_fields(key_fields);
    }

    public synchronized void setupDDSInfrastructure(boolean forPublication,
            boolean forSubscription) throws InternalError {
        // getting the DomainParticipant
        logger.info("Getting the DomainParticipant ...");
        prepareParticiantQos();
        participant = ((DomainParticipantImpl) getDomainParticipant(this.participantQos)).participant;

        // Register the required datatype for Donnee
        logger.info("Register the required datatype ...");
        TypeSupport dataTS = getDataTypeSupport();
        ErrorHandler.checkHandle(dataTS, "new DonneeTypeSupport");

        String dataTypeName = dataTS.get_type_name();

        int status = dataTS.register_type(participant, dataTypeName);
        ErrorHandler.checkStatus(status,
                "Donnee.DonneeTypeSupport.register_type");

        // Use the changed policy when defining the topic data
        logger.info("Creating the Topic ...");
        prepareTopicQoS();
        topic = participant.create_topic(this.topic_name, dataTypeName,
                this.topicQos, null, ANY_STATUS.value);

        ErrorHandler.checkHandle(topic, "DDS.DomainParticipant.create_topic");

        if (forPublication) {
            // getting the Publisher
            logger.info("Creating the Publisher ...");
            preparePublisherQos();
            publisher = participant.create_publisher(this.publisherQos, null,
                    ANY_STATUS.value);
            ErrorHandler.checkHandle(publisher,
                    "DDS.DomainParticipant.create_publisher");

            /*
             * Create a DataWriter for the Topic (using the appropriate QoS).
             */
            prepareDataWriterQoS();
            logger.info("creating the generic DataWriter ...");
            writer = publisher.create_datawriter(topic, this.datawriterQoS,
                    null, ANY_STATUS.value);
            ErrorHandler.checkHandle(writer, "DDS.Publisher.create_datawriter");
        }
        if (forSubscription) {
            // getting the Subscriber
            logger.info("Creating the subscriber ...");
            prepareSubscriberQos();
            subscriber = participant.create_subscriber(this.subscriberQoS,
                    null, ANY_STATUS.value);
            ErrorHandler.checkHandle(subscriber,
                    "DDS.DomainParticipant.create_subscriber");

            prepareDataReaderQoS();
            logger.info("creating the generic DataReader ...");
            reader = subscriber.create_datareader(topic, this.datareaderQoS,
                    null, ANY_STATUS.value);
            ErrorHandler
                    .checkHandle(reader, "DDS.Subscriber.create_datareader");
        }

        refCount++;
        logger.info("Setting up a DDS environment for a DDS ports. " + refCount
                + " times");
    }

    protected abstract TypeSupport getDataTypeSupport();

    public synchronized void teardownDDSInfrastructure(boolean forPublication,
            boolean forSubscription) throws InternalError {

        if (forPublication) {

            int status = publisher.delete_datawriter(writer);
            ErrorHandler.checkStatus(status,
                    "DDS.DomainParticipant.delete_publisher");
            logger.info("DataWriter deleted ...");

            status = participant.delete_publisher(publisher);
            ErrorHandler.checkStatus(status,
                    "DDS.DomainParticipant.delete_publisher");
            logger.info("Publisher deleted ...");
        }
        if (forSubscription) {
            /* Remove the DataReader */
            int status = reader.delete_contained_entities();
            ErrorHandler.checkStatus(status,
                    "DDS.Subscriber.delete_contained_entities");
            status = subscriber.delete_datareader(reader);
            ErrorHandler
                    .checkStatus(status, "DDS.Subscriber.delete_datareader");
            logger.info("DataReader deleted ...");

            /* Remove the Subscriber. */
            status = participant.delete_subscriber(subscriber);
            ErrorHandler.checkStatus(status,
                    "DDS.DomainParticipant.delete_subscriber");
            logger.info("Subscriber deleted ...");
        }

        /* Remove the Topics. */
        int status = participant.delete_topic(topic);
        ErrorHandler.checkStatus(status,
                "DDS.DomainParticipant.delete_topic (donneeTopic)");
        logger.info("Topic deleted ...");

        refCount--;

        if (refCount == 0) {
            // logger.info("Removing the DomainParticipant");
            // removeDomainParticipant();
        }

    }

    private void prepareDataReaderQoS() throws InternalError {
        if (super.getQosProfile() == null) {
            logger.finest("Default DataReader QoS policies are used for topic "
                    + this.topic_name);
            return;
        }

        if (super.getQosProfile().datareader_qos != null && subscriber != null) {
            DataReaderQosHolder dataReaderQos = new DataReaderQosHolder();
            int status = subscriber.get_default_datareader_qos(dataReaderQos);
            ErrorHandler.checkStatus(status,
                    "subscriber.get_default_datareader_qos");

            if (super.getQosProfile().datareader_qos.durability != null)
                dataReaderQos.value.durability = super.getQosProfile().datareader_qos.durability
                        .toDDSPolicy();
            if (super.getQosProfile().datareader_qos.deadline != null)
                dataReaderQos.value.deadline = super.getQosProfile().datareader_qos.deadline;
            if (super.getQosProfile().datareader_qos.latency_budget != null)
                dataReaderQos.value.latency_budget = super.getQosProfile().datareader_qos.latency_budget;
            if (super.getQosProfile().datareader_qos.liveliness != null)
                dataReaderQos.value.liveliness = super.getQosProfile().datareader_qos.liveliness
                        .toDDSPolicy();
            if (super.getQosProfile().datareader_qos.reliability != null)
                dataReaderQos.value.reliability = super.getQosProfile().datareader_qos.reliability
                        .toDDSPolicy();
            if (super.getQosProfile().datareader_qos.destination_order != null)
                dataReaderQos.value.destination_order = super.getQosProfile().datareader_qos.destination_order
                        .toDDSPolicy();
            if (super.getQosProfile().datareader_qos.history != null)
                dataReaderQos.value.history = super.getQosProfile().datareader_qos.history
                        .toDDSPolicy();
            if (super.getQosProfile().datareader_qos.resource_limits != null)
                dataReaderQos.value.resource_limits = super.getQosProfile().datareader_qos.resource_limits;
            if (super.getQosProfile().datareader_qos.user_data != null)
                dataReaderQos.value.user_data = super.getQosProfile().datareader_qos.user_data;
            if (super.getQosProfile().datareader_qos.ownership != null)
                dataReaderQos.value.ownership = super.getQosProfile().datareader_qos.ownership
                        .toDDSPolicy();
            if (super.getQosProfile().datareader_qos.time_based_filter != null)
                dataReaderQos.value.time_based_filter = super.getQosProfile().datareader_qos.time_based_filter;
            if (super.getQosProfile().datareader_qos.reader_data_lifecycle != null)
                dataReaderQos.value.reader_data_lifecycle = super
                        .getQosProfile().datareader_qos.reader_data_lifecycle;

            this.datareaderQoS = dataReaderQos.value;
        }
    }

    private void prepareDataWriterQoS() throws InternalError {
        if (super.getQosProfile() == null) {
            logger.finest("Default DataWriter QoS policies are used for topic "
                    + this.topic_name);
            return;
        }

        if (super.getQosProfile().datawriter_qos != null && publisher != null) {
            DataWriterQosHolder holder = new DataWriterQosHolder();
            int status = publisher.get_default_datawriter_qos(holder);
            ErrorHandler.checkStatus(status,
                    "publisher.get_default_datawriter_qos");

            if (super.getQosProfile().datawriter_qos.durability != null)
                holder.value.durability = super.getQosProfile().datawriter_qos.durability
                        .toDDSPolicy();
            if (super.getQosProfile().datawriter_qos.deadline != null)
                holder.value.deadline = super.getQosProfile().datawriter_qos.deadline;
            if (super.getQosProfile().datawriter_qos.latency_budget != null)
                holder.value.latency_budget = super.getQosProfile().datawriter_qos.latency_budget;
            if (super.getQosProfile().datawriter_qos.liveliness != null)
                holder.value.liveliness = super.getQosProfile().datawriter_qos.liveliness
                        .toDDSPolicy();
            if (super.getQosProfile().datawriter_qos.reliability != null)
                holder.value.reliability = super.getQosProfile().datawriter_qos.reliability
                        .toDDSPolicy();
            if (super.getQosProfile().datawriter_qos.destination_order != null)
                holder.value.destination_order = super.getQosProfile().datawriter_qos.destination_order
                        .toDDSPolicy();
            if (super.getQosProfile().datawriter_qos.history != null)
                holder.value.history = super.getQosProfile().datawriter_qos.history
                        .toDDSPolicy();
            if (super.getQosProfile().datawriter_qos.resource_limits != null)
                holder.value.resource_limits = super.getQosProfile().datawriter_qos.resource_limits;
            if (super.getQosProfile().datawriter_qos.user_data != null)
                holder.value.user_data = super.getQosProfile().datawriter_qos.user_data;
            if (super.getQosProfile().datawriter_qos.ownership != null)
                holder.value.ownership = super.getQosProfile().datawriter_qos.ownership
                        .toDDSPolicy();
            if (super.getQosProfile().datawriter_qos.ownership_strength != null)
                holder.value.ownership_strength = super.getQosProfile().datawriter_qos.ownership_strength;
            if (super.getQosProfile().datawriter_qos.writer_data_lifecycle != null)
                holder.value.writer_data_lifecycle = super.getQosProfile().datawriter_qos.writer_data_lifecycle;
            if (super.getQosProfile().datawriter_qos.transport_priority != null)
                holder.value.transport_priority = super.getQosProfile().datawriter_qos.transport_priority;
            if (super.getQosProfile().datawriter_qos.lifespan != null)
                holder.value.lifespan = super.getQosProfile().datawriter_qos.lifespan;

            this.datawriterQoS = holder.value;
        }
    }

    private void prepareTopicQoS() throws InternalError {
        if (super.getQosProfile() == null) {
            logger.finest("Default Topic QoS policies are used for topic "
                    + this.topic_name);
            return;
        }

        if (super.getQosProfile().topic_qos != null && participant != null) {
            TopicQosHolder holder = new TopicQosHolder();
            int status = participant.get_default_topic_qos(holder);
            ErrorHandler.checkStatus(status,
                    "participant.get_default_topic_qos");

            if (super.getQosProfile().topic_qos.deadline != null)
                holder.value.deadline = super.getQosProfile().topic_qos.deadline;
            if (super.getQosProfile().topic_qos.destination_order != null)
                holder.value.destination_order = super.getQosProfile().topic_qos.destination_order
                        .toDDSPolicy();
            if (super.getQosProfile().topic_qos.durability != null)
                holder.value.durability = super.getQosProfile().topic_qos.durability
                        .toDDSPolicy();
            if (super.getQosProfile().topic_qos.durability_service != null)
                holder.value.durability_service = super.getQosProfile().topic_qos.durability_service
                        .toDDSPolicy();
            if (super.getQosProfile().topic_qos.history != null)
                holder.value.history = super.getQosProfile().topic_qos.history
                        .toDDSPolicy();
            if (super.getQosProfile().topic_qos.latency_budget != null)
                holder.value.latency_budget = super.getQosProfile().topic_qos.latency_budget;
            if (super.getQosProfile().topic_qos.lifespan != null)
                holder.value.lifespan = super.getQosProfile().topic_qos.lifespan;
            if (super.getQosProfile().topic_qos.liveliness != null)
                holder.value.liveliness = super.getQosProfile().topic_qos.liveliness
                        .toDDSPolicy();
            if (super.getQosProfile().topic_qos.ownership != null)
                holder.value.ownership = super.getQosProfile().topic_qos.ownership
                        .toDDSPolicy();
            if (super.getQosProfile().topic_qos.reliability != null)
                holder.value.reliability = super.getQosProfile().topic_qos.reliability
                        .toDDSPolicy();
            if (super.getQosProfile().topic_qos.resource_limits != null)
                holder.value.resource_limits = super.getQosProfile().topic_qos.resource_limits;
            if (super.getQosProfile().topic_qos.topic_data != null)
                holder.value.topic_data = super.getQosProfile().topic_qos.topic_data;
            if (super.getQosProfile().topic_qos.transport_priority != null)
                holder.value.transport_priority = super.getQosProfile().topic_qos.transport_priority;

        }
    }
    
    private void preparePublisherQos() throws InternalError {
        if (super.getQosProfile() == null) {
            logger.finest("Default Publisher QoS policies are used for topic "
                    + this.topic_name);
            return;
        }

        if (super.getQosProfile().publisher_qos != null && participant != null) {
            PublisherQosHolder holder = new PublisherQosHolder();
            int status = participant.get_default_publisher_qos(holder);
            ErrorHandler.checkStatus(status,
                    "participant.get_default_publisher_qos");

            if (super.getQosProfile().publisher_qos.entity_factory != null)
                holder.value.entity_factory = super.getQosProfile().publisher_qos.entity_factory;
            if (super.getQosProfile().publisher_qos.group_data != null)
                holder.value.group_data = super.getQosProfile().publisher_qos.group_data;
            if (super.getQosProfile().publisher_qos.partition != null)
                holder.value.partition = super.getQosProfile().publisher_qos.partition;
            if (super.getQosProfile().publisher_qos.presentation != null)
                holder.value.presentation = super.getQosProfile().publisher_qos.presentation
                        .toDDSPolicy();
            if (super.getQosProfile().publisher_qos.entity_factory != null)
                holder.value.entity_factory = super.getQosProfile().publisher_qos.entity_factory;
        }
    }
    
    private void prepareSubscriberQos() throws InternalError {
        if (super.getQosProfile() == null) {
            logger.finest("Default Subscriber QoS policies are used for topic "
                    + this.topic_name);
            return;
        }

        if (super.getQosProfile().subscriber_qos != null && participant != null) {
            SubscriberQosHolder holder = new SubscriberQosHolder();
            int status = participant.get_default_subscriber_qos(holder);
            ErrorHandler.checkStatus(status,
                    "participant.get_default_subscriber_qos");

            if (super.getQosProfile().subscriber_qos.entity_factory != null)
                holder.value.entity_factory = super.getQosProfile().subscriber_qos.entity_factory;
            if (super.getQosProfile().subscriber_qos.group_data != null)
                holder.value.group_data = super.getQosProfile().subscriber_qos.group_data;
            if (super.getQosProfile().subscriber_qos.partition != null)
                holder.value.partition = super.getQosProfile().subscriber_qos.partition;
            if (super.getQosProfile().subscriber_qos.presentation != null)
                holder.value.presentation = super.getQosProfile().subscriber_qos.presentation
                        .toDDSPolicy();
            if (super.getQosProfile().subscriber_qos.entity_factory != null)
                holder.value.entity_factory = super.getQosProfile().subscriber_qos.entity_factory;
        }
    }
    
    private void prepareParticiantQos() throws InternalError {
        if (super.getQosProfile() == null) {
            logger.finest("Default Domain Participant QoS policies are used for topic "
                    + this.topic_name);
            return;
        }

        if (super.getQosProfile().domainparticipant_qos != null ) {
            DomainParticipantQosHolder holder = new DomainParticipantQosHolder();
            int status = DomainParticipantFactory.get_instance().get_default_participant_qos(holder);
            ErrorHandler.checkStatus(status,
                    "participant.get_default_subscriber_qos");
            if(super.getQosProfile().domainparticipant_qos.entity_factory != null) holder.value.entity_factory = super.getQosProfile().domainparticipant_qos.entity_factory;
            if(super.getQosProfile().domainparticipant_qos.user_data != null) holder.value.user_data = super.getQosProfile().domainparticipant_qos.user_data;
        }
    }
              
    

}
