/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.qosenablers.recording;

import static cdmw.logging.FunctionLogger.CDMW_LOG_FUNCTION;
import static cdmw.logging.FunctionLogger.CDMW_LOG_FUNCTION_EXCEPTION;
import static cdmw.logging.FunctionLogger.CDMW_LOG_FUNCTION_RETURN;
import static cdmw.logging.Logger.CDMW_DEBUG;
import static cdmw.logging.Logger.CDMW_ERROR;

import java.util.ArrayList;
import java.util.Hashtable;

import org.omg.CORBA.BAD_PARAM;
import org.omg.CORBA.Object;
import org.omg.Components.CCMException;
import org.omg.Components.CCMExceptionReason;
import org.omg.Components.CCMObject;
import org.omg.Components.CCMObjectHelper;
import org.omg.Components.ConfigValue;
import org.omg.Components.ConfigValuesHolder;
import org.omg.Components.ConsumerDescription;
import org.omg.Components.Cookie;
import org.omg.Components.ExtensionContext;
import org.omg.Components.FacetDescription;
import org.omg.Components.SessionContext;
import org.omg.Components.ContainerPortableInterceptor.IntegrationPoint;
import org.omg.Components.ContainerPortableInterceptor.InvalidRegistration;

import cdmw.ccm.ccmcopi.COPIUtils;
import cdmw.ccm.common.CCMLoggers;
import cdmw.ccm.container.ApplicationElementsContextService;
import cdmw.ccm.qosenablers.common.QOSEnablerBase;
import cdmw.common.Assert;
import cdmw.commonsvcs.naming.NamingInterface;
import cdmw.logging.Logger;

import com.thalesgroup.CdmwCcmContainer.ApplicationElementContextService;
import com.thalesgroup.CdmwCcmContainer.ApplicationElementContextServiceHelper;
import com.thalesgroup.CdmwCcmContainer.LifeCycleInterceptorRegistration;
import com.thalesgroup.CdmwCcmContainer.LifeCycleInterceptorRegistrationHelper;
import com.thalesgroup.CdmwCcmPlugins.CCM_CdmwQosEnabler;
import com.thalesgroup.CdmwCcmPlugins.NamedValue;
import com.thalesgroup.CdmwRecording.REC_CONTAINER_MNGR_ID;

/**
 *
 */
public class RecordingServiceComponentImpl extends QOSEnablerBase implements
        CCM_CdmwQosEnabler,
        com.thalesgroup.CdmwCcmContainer.ComponentLifeCycleInterceptor {

    public static final String CCM_RECORDING_LOG_DOMAIN = "CDMW.CCM.QOSENABLERS.recording";
    static Logger recordingLogger = Logger.Get_Logger(CCM_RECORDING_LOG_DOMAIN);

    public static final String serviceName = "CDMW.CCM.QOSENABLERS.Recording";
    public static final String PROPERTY_RECORD_USING_COMPONENT_NAME = "Cdmw.CCMContainer.Recording.UseComponentName";

    /**
     * Stores the configvalues of each created component. The mappings are
     * removed once the component is recorded
     */
    private Hashtable<String, ConfigValue[]> compConfigValues = new Hashtable<String, ConfigValue[]>();

    /**
     * Stores the list of objects to record for each component
     */
    private Hashtable<String, org.omg.CORBA.Object[]> compRegsiteredObjects = new Hashtable<String, org.omg.CORBA.Object[]>();

    /**
     * The Recording ComponentLifeCycleInterceptor Cookie obtained upon its
     * registration
     */
    private Cookie ck;

    /**
     * the LifeCycleInterceptor registration interface
     */
    private LifeCycleInterceptorRegistration registrationInterface;

    private ExtensionContext context;

    /**
     * The service properties
     */
    private NamedValue[] settings;

    /**
     *
     */
    private com.thalesgroup.CdmwRecording.ContainerManager recordingMgr;

    @Override
    public void ccm_activate() throws CCMException {
        CDMW_DEBUG(RecordingServiceComponentImpl.recordingLogger,
                "Recording QoSEnabler ccm_activated ...");
    }

    @Override
    public void ccm_passivate() throws CCMException {
        CDMW_DEBUG(RecordingServiceComponentImpl.recordingLogger,
        "Recording QoSEnabler ccm_passivated ...");

    }

    @Override
    public void ccm_remove() throws CCMException {
        CDMW_DEBUG(RecordingServiceComponentImpl.recordingLogger,
        "Recording QoSEnabler ccm_removed ...");
        try {
            registrationInterface
                    .unregister_component_lifecycle_interceptor(ck);
        } catch (InvalidRegistration ir) {
            CDMW_ERROR(recordingLogger,
                    "Failed to unregister the component lifecycle interceptor !");
        }

    }

    @Override
    public void set_extension_context(ExtensionContext ctx) throws CCMException {
        CDMW_DEBUG(RecordingServiceComponentImpl.recordingLogger,
                "set_extension_context called on the Recording QoSEnabler ...");
        this.context = ctx;

        // get CdmwRecording::ContainerManager object
        try {
            org.omg.CORBA.Object o = this.context
                    .resolve_service_reference(REC_CONTAINER_MNGR_ID.value);

            recordingMgr = com.thalesgroup.CdmwRecording.ContainerManagerHelper
                    .narrow(o);

        } catch (CCMException e) {
            CDMW_ERROR(RecordingServiceComponentImpl.recordingLogger,
                    REC_CONTAINER_MNGR_ID.value
                            + " object is not registered as initial reference",
                    new Throwable());

        } catch (org.omg.CORBA.BAD_PARAM e) {
            CDMW_ERROR(
                    RecordingServiceComponentImpl.recordingLogger,
                    REC_CONTAINER_MNGR_ID.value
                            + " object is not a CdmwRecording::ContainerManager",
                    new Throwable());
        }

        try {
            Object obj = ctx
                    .resolve_service_reference(cdmw.ccm.container.LifeCycleInterceptorRegistration.LifeCycleInterceptorRegistrationServiceId);
            registrationInterface = LifeCycleInterceptorRegistrationHelper
                    .narrow(obj);
            ck = registrationInterface
                    .register_component_lifecycle_interceptor(this);
        } catch (BAD_PARAM bp) {
            CDMW_ERROR(
                    recordingLogger,
                    "The Recording Service won't be activated! "
                            + "Failed to get the LifeCycleInterceptorRegistration service");
            CCMException ccme = new CCMException(CCMExceptionReason.FIND_ERROR);
            throw ccme;
        }

    }

    @Override
    public void set_session_context(SessionContext ctx) throws CCMException {
        // TODO Auto-generated method stub

    }

    public NamedValue[] service_settings() {
        return this.settings;
    }

    public void service_settings(NamedValue[] serviceSettings) {
        this.settings = new NamedValue[serviceSettings.length];
        System.arraycopy(serviceSettings, 0, settings, 0,
                serviceSettings.length);
    }

    public NamedValue[] service_statistics() {
        return null;
    }

    public void service_statistics(NamedValue[] arg0) {

    }

    public String get_name() {
        return serviceName;
    }

    public NamedValue[] get_settings() {
        // TODO Auto-generated method stub
        return null;
    }

    public NamedValue[] get_statistics() {
        // TODO Auto-generated method stub
        return null;
    }

    public int get_status() {
        // TODO Auto-generated method stub
        return 0;
    }

    /**
     * Implementation of the ComponentLifeCycleInterceptor interface
     */

    public void after_component_activate(String arg0) {

    }

    public void after_component_connect(String arg0) {

    }

    public void after_component_create(String compId, Object compRef) {
        CDMW_LOG_FUNCTION(RecordingServiceComponentImpl.recordingLogger,
                "compId= " + compId);

        if (!compConfigValues.containsKey(compId)) {
            CDMW_ERROR(RecordingServiceComponentImpl.recordingLogger,
                    "Enable to find the component config values !!!");
            return;
        }

        try {
            Object[] objs = this.registerForRecording(compId, compRef,
                    compConfigValues.get(compId));
            compConfigValues.remove(compId);
            compRegsiteredObjects.put(compId, objs);
        } catch (Exception e) {
            CDMW_ERROR(RecordingServiceComponentImpl.recordingLogger,
                    "Failed to register component " + compId
                            + "for recording !");
            CDMW_LOG_FUNCTION_EXCEPTION(
                    RecordingServiceComponentImpl.recordingLogger, e);
        }

        CDMW_LOG_FUNCTION_RETURN(RecordingServiceComponentImpl.recordingLogger);
    }

    public void after_component_passivate(String arg0) {

    }

    public void after_component_remove(String compId) {
        if (!compRegsiteredObjects.containsKey(compId)) {
            CDMW_DEBUG(recordingLogger, "Component " + compId
                    + " not to be recorded !");
            return;
        }

        if (recordingMgr != null) {
            recordingMgr.discard_objects(compRegsiteredObjects.remove(compId));
        }

    }

    public void before_component_activate(String arg0) {

    }

    public void before_component_connect(String arg0, String arg1, Object arg2) {

    }

    public void before_component_create(String compId,
            ConfigValuesHolder configValues) {
        CDMW_LOG_FUNCTION(RecordingServiceComponentImpl.recordingLogger,
                "compId= " + compId);

        if (compConfigValues.containsKey(compId))
            return;

        // to save the component config values
        compConfigValues.put(compId, configValues.value);

        CDMW_LOG_FUNCTION_RETURN(RecordingServiceComponentImpl.recordingLogger);
    }

    public void before_component_passivate(String arg0) {

    }

    public void before_component_remove(String arg0) {

    }

    public void destroy() {

    }

    public String name() {
        return serviceName;
    }

    public short priority() {
        // To catch the latest component configValues
        return Short.MIN_VALUE;
    }

    public IntegrationPoint registration_info() {
        return null;
    }

    public void set_slot_id(int arg0) {

    }

    private org.omg.CORBA.Object[] registerForRecording(String compOid,
            org.omg.CORBA.Object obj, ConfigValue[] configValues) {

        CDMW_LOG_FUNCTION(RecordingServiceComponentImpl.recordingLogger,
                "compOid=" + compOid);

        Object[] objToRecord = null;

        // check if Recording ConfigValue is set
        org.omg.CORBA.AnyHolder recordingFlagAny = new org.omg.CORBA.AnyHolder();
        if (cdmw.ccm.common.CCMUtils.ReadConfigValue(
                com.thalesgroup.CdmwDeployment.RECORDING.value, configValues,
                recordingFlagAny)) {
            boolean recordingFlag;
            try {
                recordingFlag = recordingFlagAny.value.extract_boolean();
            } catch (org.omg.CORBA.BAD_OPERATION e) {
                CDMW_ERROR(RecordingServiceComponentImpl.recordingLogger,
                        "Recording ConfigValue is not a CORBA::Boolean",
                        new Throwable());
                throw e;
            }

            if (recordingFlag && recordingMgr != null) {

                // if SystemProperty PROPERTY_RECORD_USING_COMPONENT_NAME is
                // "true" use the component name and the server Id for
                // recording
                String serverId = "";
                if (getPROPERTY_RECORD_USING_COMPONENT_NAME().equals("true")) {

                    // get the server id

                    try {
                        Object o = context
                                .resolve_service_reference(ApplicationElementsContextService.ApplicationElementsContextServiceId);
                        ApplicationElementContextService svc = ApplicationElementContextServiceHelper
                                .narrow(o);
                        serverId = svc.get_component_server_context().id();
                    } catch (Exception e) {
                        CDMW_ERROR(
                                RecordingServiceComponentImpl.recordingLogger,
                                "Failed to get the ApplicationElementContextService");
                        CDMW_LOG_FUNCTION_EXCEPTION(
                                RecordingServiceComponentImpl.recordingLogger,
                                e);
                    }

                    // get COMPONENT_NAME
                    org.omg.CORBA.Any componentNameAny = cdmw.ccm.common.CCMUtils
                            .ReadConfigValue(
                                    com.thalesgroup.CdmwDeployment.COMPONENT_NAME.value,
                                    configValues);

                    if (componentNameAny != null) {
                        java.lang.String componentName;
                        componentName = componentNameAny.extract_string();

                        // ConfigValue found, add component name to server_id
                        try {
                            org.omg.CosNaming.NameComponent[] name = NamingInterface
                                    .toName(serverId);
                            java.util.ArrayList<org.omg.CosNaming.NameComponent> nameAsList = new java.util.ArrayList<org.omg.CosNaming.NameComponent>(
                                    java.util.Arrays.asList(name));
                            org.omg.CosNaming.NameComponent comp = new org.omg.CosNaming.NameComponent();
                            comp.id = componentName;
                            comp.kind = "";
                            nameAsList.add(comp);
                            serverId = NamingInterface.toString(nameAsList
                                    .toArray(name));
                        } catch (cdmw.commonsvcs.naming.InvalidNameException e) {
                            // shouldn't happen
                            e.printStackTrace();
                            Assert.check(false);
                        }
                    }
                }

                // register object for recording
                CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(),
                        "Register object in Recording service with service_id="
                                + serverId, new Throwable());
                try {
                    ArrayList<Object> list = new ArrayList<Object>();
                    list.add(obj);
                    // get facets and consumers
                    try {
                        CCMObject ccmObj = CCMObjectHelper.narrow(obj);
                        FacetDescription[] facets = ccmObj.get_all_facets();
                        for (FacetDescription facet : facets) {
                            CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(), "registering facet "+facet.name);
                            list.add(facet.facet_ref);
                        }
                        ConsumerDescription[] consumers = ccmObj
                                .get_all_consumers();
                        for (ConsumerDescription consumer : consumers) {
                            list.add(consumer.consumer);
                            CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(), "registering consumer "+consumer.name);
                        }
                    } catch (BAD_PARAM bp) {
                        CDMW_ERROR(
                                RecordingServiceComponentImpl.recordingLogger,
                                "The object to record is not a CCMObject !");
                        throw bp;
                    }
                    objToRecord = list.toArray(new org.omg.CORBA.Object[0]);
                    recordingMgr.record_objects(serverId, objToRecord);

                } catch (org.omg.CORBA.SystemException e) {
                    CDMW_ERROR(CCMLoggers.GetCcmcifLogger(),
                            "Error registering object in Recording service : "
                                    + e, new Throwable());
                    throw e;
                }

            } else {
                // recording_flag = false or the recordingMgr == null
                CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(),
                        "No recording for this component "
                                + "(Recording ConfigValue set to "
                                + recordingFlag
                                + " and the recordingManager is "
                                + (recordingMgr == null ? "null" : "not null")
                                + ")");
            }
        } else {
            // Recording ConfigValue not specified
            CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(),
                    "No recording required for this component"
                            + "(no Recording ConfigValue specified)",
                    new Throwable());
        }

        return objToRecord;
    }

    private String getPROPERTY_RECORD_USING_COMPONENT_NAME() {

        for (NamedValue property : service_settings()) {
            if (PROPERTY_RECORD_USING_COMPONENT_NAME.equals(property.name)) {
                return property.value;
            }
        }

        return "false";
    }

}
