/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.qosenablers.requestfollowup;

import org.omg.PortableInterceptor.Current;
import org.omg.PortableInterceptor.CurrentHelper;
import org.omg.Components.Cookie;
import org.omg.Components.CCMException;
import org.omg.Components.SessionContext;
import org.omg.Components.ExtensionContext;
import org.omg.Components.ContainerPortableInterceptor.InvalidRegistration;

import cdmw.ccm.qosenablers.common.QOSEnablerBase;
import com.thalesgroup.CdmwCcmPlugins.CCM_CdmwQosEnabler;
import com.thalesgroup.Cdmw_CdmwCcmPlugins.CCM_CdmwQosEnabler_ExtensionContext;
import com.thalesgroup.Cdmw_CdmwCcmPlugins.CdmwPluginHelper;
import com.thalesgroup.CdmwCcmPlugins.NamedValue;

import cdmw.logging.Logger;
import cdmw.commonsvcs.requestfollowup.NDCLogger;

import static cdmw.logging.FunctionLogger.*;
import static cdmw.logging.Logger.*;


/**
 *
 */
public class BasicNDCExecutorImpl
    extends QOSEnablerBase
    implements CCM_CdmwQosEnabler {

    /**
     * Internal logging
     */
    private Logger m_logger = NDCLogger.getInstance().getLogger();

    /**
     * Extension context
     */
    private CCM_CdmwQosEnabler_ExtensionContext m_ext_ctx = null;

    /**
     * Server Basic COPI
     */
    private NDCServerContainerInterceptorImpl m_serverCOPI = null;

    /**
     * Server Cookie
     */
    private Cookie m_serverCookie = null;

    /**
     * Client Basic  COPI
     */
    private NDCClientContainerInterceptorImpl m_clientCOPI = null;

    /**
     * Stub cookie
     */
    private Cookie m_clientCookie = null;

    /**
     * Status of the Executor
     */
    private int m_status = com.thalesgroup.CdmwCcmPlugins.UNKNOWN_STATUS.value;

    /**
     * Cookie of the service
     */
    private Cookie m_RFUCookie;

    /**
     * PICurrent
     */
    private Current m_PICurrent = null;

    /**
     * Name of the service Request Follow Up
     */
    public static final String RFU_SERVICE_NAME = "RequestFollowUp_SERVICE";

    /**
     * Component attribute : settings
     */
    private NamedValue[] m_service_settings = null;

    /**
     * Component attribute : statistics
     */
    private NamedValue[] m_service_statistics = null;




    /**
     * Remove the component
     *
     */
    //
    // IDL:org.omg/Components/ExtensionComponent/ccm_remove:1.0
    //
    public void ccm_remove()
        throws CCMException {
        CDMW_LOG_FUNCTION(m_logger,"");

        // Unregister interceptors
        try {
            m_ext_ctx.get_server_interceptor_registration().unregister_server_interceptor(m_serverCookie);
            m_serverCookie = null;
            m_ext_ctx.get_client_interceptor_registration().unregister_client_interceptor(m_clientCookie);
            m_clientCookie = null;
        }
        catch(InvalidRegistration ire) {
            CDMW_ERROR(m_logger, "Exception raised when unregeistering interceptors: " + ire);
            CDMW_LOG_FUNCTION_EXCEPTION(m_logger, ire);
        }

        m_status = com.thalesgroup.CdmwCcmPlugins.STOPPED_STATUS.value;

        CDMW_LOG_FUNCTION_RETURN(m_logger,"");
    }


    /**
     * Passivate the component
     *
     */
    //
    // IDL:org.omg/Components/SessionComponent/ccm_passivate:1.0
    //
    public void ccm_passivate()
        throws CCMException {
        CDMW_LOG_FUNCTION(m_logger,"");

        m_status = com.thalesgroup.CdmwCcmPlugins.STOPPING_STATUS.value;

        // Uninstall the service
        m_ext_ctx.uninstall_service_reference(m_RFUCookie);

        CDMW_LOG_FUNCTION_RETURN(m_logger,"");
    }


    /**
     * Activate the component
     *
     */
    //
    // IDL:org.omg/Components/SessionComponent/ccm_activate:1.0
    //
    public void ccm_activate()
        throws CCMException {
        CDMW_LOG_FUNCTION(m_logger,"");

        // Install the service
        m_RFUCookie =
            m_ext_ctx.install_service_reference(RFU_SERVICE_NAME,
                                                CdmwPluginHelper.narrow(this));

        m_status = com.thalesgroup.CdmwCcmPlugins.RUNNING_STATUS.value;

        CDMW_LOG_FUNCTION_RETURN(m_logger,"");
    }


    /**
     * Set the session context
     *
     * @param ctx the session context
     *
     */
    //
    // IDL:org.omg/Components/SessionComponent/set_session_context:1.0
    //
    public void set_session_context(SessionContext ctx)
        throws CCMException {
        CDMW_LOG_FUNCTION(m_logger,"");

        m_status = com.thalesgroup.CdmwCcmPlugins.STARTED_STATUS.value;

        CDMW_LOG_FUNCTION_RETURN(m_logger,"");
    }


    /**
     * Set the extension context
     *
     * @param ctx the extension context
     *
     */
    //
    // IDL:org.omg/Components/ExtensionComponent/set_extension_context:1.0
    //
    public void set_extension_context(ExtensionContext ctx)
        throws CCMException {
        CDMW_LOG_FUNCTION(m_logger,"");

        m_status = com.thalesgroup.CdmwCcmPlugins.STARTING_STATUS.value;
        m_ext_ctx = 
            com.thalesgroup.Cdmw_CdmwCcmPlugins.
            CCM_CdmwQosEnabler_ExtensionContextHelper.narrow(ctx);

        // Get the PICurrent
        try {
            m_PICurrent = 
                CurrentHelper.
                narrow(m_ext_ctx.resolve_service_reference("PICurrent"));
        }
        catch(CCMException ccme) {
            CDMW_ERROR(m_logger, "Exception raised when getting the PICurrent: " + ccme);
            CDMW_LOG_FUNCTION_EXCEPTION(m_logger, ccme);
            throw ccme;
        }

        // Create and register interceptors
        m_serverCOPI = new NDCServerContainerInterceptorImpl();
        m_serverCookie = m_ext_ctx.get_server_interceptor_registration().register_server_interceptor(m_serverCOPI);
        m_clientCOPI = new NDCClientContainerInterceptorImpl();
        m_clientCookie = m_ext_ctx.get_client_interceptor_registration().register_client_interceptor(m_clientCOPI);

        CDMW_LOG_FUNCTION_RETURN(m_logger,"");
    }


    /**
     * Get the name
     *
     * @return the name of the component
     *
     */
    //
    // IDL:com.thalesgroup/CdmwCcmPlugins/CdmwPlugin/get_name:1.0
    //
    public java.lang.String get_name() {
        //         CDMW_LOG_FUNCTION(m_logger,"");
        //         CDMW_LOG_FUNCTION_RETURN(m_logger,"name=["+RFU_SERVICE_NAME+"]");
        return RFU_SERVICE_NAME;
    }

    public int get_status() {
        //         CDMW_LOG_FUNCTION(m_logger,"");
        //         CDMW_LOG_FUNCTION_RETURN(m_logger,"status=["+m_status+"]");
        return m_status;
    }

    public NamedValue[] get_settings() {
        //         CDMW_LOG_FUNCTION(m_logger,"");
        //         CDMW_LOG_FUNCTION_RETURN(m_logger,"settings=["+null+"]");
        return null;
    }

    public NamedValue[] get_statistics() {
        //         CDMW_LOG_FUNCTION(m_logger,"");
        //         CDMW_LOG_FUNCTION_RETURN(m_logger,"statistics=["+null+"]");
        return null;
    }


    //
    // IDL:com.thalesgroup/CdmwCcmPlugins/CdmwPlugin/service_settings:1.0
    //
    public NamedValue[] service_settings() {
        return m_service_settings;
    }


    //
    // IDL:com.thalesgroup/CdmwCcmPlugins/CdmwPlugin/service_settings:1.0
    //
    public void service_settings(NamedValue[] arg) {
        m_service_settings = new NamedValue[arg.length];
        System.arraycopy(arg, 0, m_service_settings, 0, arg.length);
    }


    //
    // IDL:com.thalesgroup/CdmwCcmPlugins/CdmwPlugin/service_statistics:1.0
    //
    public NamedValue[] service_statistics() {
        return m_service_statistics;
    }


    //
    // IDL:com.thalesgroup/CdmwCcmPlugins/CdmwPlugin/service_statistics:1.0
    //
    public void service_statistics(NamedValue[] arg) {
        m_service_statistics = new NamedValue[arg.length];
        System.arraycopy(arg, 0, m_service_statistics, 0, arg.length);
    }
}
