/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_CDMW_CCM_COPI_COPITESTER_IMPL_HPP
#define INCL_CDMW_CCM_COPI_COPITESTER_IMPL_HPP

#include <map>
#include <Foundation/common/Assert.hpp>
#include <Foundation/idllib/Components.stub.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include <CCMContainer/idllib/CdmwCcmCopi.stub.hpp>
#include <CCMContainer/ccmcopi/COPIUtils_impl.hpp>
#include "TestService_IDL2.skel.hpp"
#include "LoggerClientCOPI_impl.hpp"
#include "LoggerServerCOPI_impl.hpp"




namespace Cdmw {

namespace CCM {

namespace COPI {

class  COPITester_impl
  : public virtual TestService::COPITester,
    public Cdmw::OrbSupport::RefCountLocalObject
{
public:
    // Constructor 
    COPITester_impl 
    (
     ::Components::ContainerPortableInterceptor::ClientContainerInterceptorRegistration_ptr client_ci_reg,
     ::Components::ContainerPortableInterceptor::ServerContainerInterceptorRegistration_ptr server_ci_reg,
     ::Components::ContainerPortableInterceptor::StubContainerInterceptorRegistration_ptr stub_ci_reg,
     ::Components::ContainerPortableInterceptor::ServantContainerInterceptorRegistration_ptr servant_ci_reg
     );

    COPITester_impl 
    (
     ::Components::ExtensionContext_ptr context
     );

    // Destructor 
    virtual ~COPITester_impl (void);

    
    // operations defined via idl
    virtual
    void register_interceptor 
    (
     const char * name,
     ::TestService::COPIType type
     )
        throw (
               ::CORBA::SystemException
               );
  
    virtual
    void unregister_interceptor 
    (
     const char * name
     )
        throw (
               ::CORBA::SystemException,
               ::Components::ContainerPortableInterceptor::InvalidRegistration
               );
  
    virtual
    void test_interception_stacks ()
        throw (
               ::CORBA::SystemException
               );
  
    virtual
    void start_listening ()
        throw (
               ::CORBA::SystemException
               );
  
    virtual
    void end_listening ()
        throw (
               ::CORBA::SystemException
               );

    // additional non idl methods, NOTE, obviously all the methods which are invoked
    // from the ContainerInterceptors should be defined in idl, this has not been done in
    // order to keep the C++ implementation in sync with the Java one

    void 
    notify_operation
    (
     const std::string & op_name,
     const std::string & copi_name,
     const Cdmw::OsSupport::OS::Timeval timestamp,
     const std::string & intercepted_op
     );

private:
    // needs to be called under mtx protection
    bool is_started();

    class COPI_info {
        friend class COPITester_impl;
    public:
        COPI_info
        (
         const ::Components::Cookie * ck,
         const ::TestService::COPIType & type,
         const std::string & name
         ) : m_ck(ck), m_type(type), m_used(0), m_name(name)
        {};

        virtual ~COPI_info() {
        }
        
        void push() {
            CDMW_MUTEX_GUARD(m_mtx);
            m_used++;
        }
        
        void pop() {
            CDMW_MUTEX_GUARD(m_mtx);
            m_used--;
            if(m_used < 0) {
                CDMW_NEVER_HERE();
            }
        }

    private:
        COPI_info& operator=(const COPI_info & other);
        COPI_info(const COPI_info & other);

        const ::Components::Cookie * m_ck;
        const ::TestService::COPIType m_type;
        int m_used;
        const std::string m_name;        

        OsSupport::Mutex m_mtx;
    };


    class InterceptionPoint {
        friend class COPITester_impl;
    public:
        
        InterceptionPoint
        (
         const std::string & name,
         const std::string & intercepted_op,
         const std::string & interception_point,
         const Cdmw::OsSupport::OS::Timeval timestamp
         ) : m_copi_name(name), m_intercepted_op(intercepted_op), m_interception_point(interception_point), m_timestamp(timestamp)
        {
        };
        
        friend std::ostream& operator<<(std::ostream& output, const InterceptionPoint & ip) {
            output << COPIUtils_impl::Timestamp_to_string(ip.m_timestamp, "%a, %d %b %Y %H:%M:%S %z") << "\t" 
                   << ip.m_copi_name <<  "." 
                   << ip.m_interception_point << "\t" 
                   << ip.m_intercepted_op;
            return output;            
        }

    private:
        InterceptionPoint& operator=(const InterceptionPoint & other);
        InterceptionPoint(const InterceptionPoint & other);

        const std::string m_copi_name;
        const std::string m_intercepted_op;
        const std::string m_interception_point;
        const Cdmw::OsSupport::OS::Timeval m_timestamp;   
    };

    typedef std::map< std::string, COPI_info * > registered_copis_t;
    typedef std::map< unsigned int, InterceptionPoint * > histo_t;

    OsSupport::Mutex m_mtx;
    registered_copis_t m_registered_copis;
    bool m_started;
    histo_t m_histo;
    int m_i;

    
    std::map< ::Components::Cookie *, LoggerClientCOPI_impl *> client_copis;
    std::map< ::Components::Cookie *, LoggerServerCOPI_impl *> server_copis;
    // std::map< ::Components::Cookie *, LoggerStubCOPI_impl *> stub_copis;
    // std::map< ::Components::Cookie *, LoggerServantCOPI_impl *> servant_copis;


    ::Components::ContainerPortableInterceptor::ClientContainerInterceptorRegistration_ptr
    m_client_ci_reg;

    ::Components::ContainerPortableInterceptor::ServerContainerInterceptorRegistration_ptr
    m_server_ci_reg;

    ::Components::ContainerPortableInterceptor::StubContainerInterceptorRegistration_ptr
    m_stub_ci_reg;

    ::Components::ContainerPortableInterceptor::ServantContainerInterceptorRegistration_ptr
    m_servant_ci_reg;
};

} // End namespace COPI
} // End namespace CCM
} // End namespace Cdmw


#endif // INCL_CDMW_CCM_COPI_COPITESTER_IMPL_HPP
