/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <Foundation/common/Assert.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Foundation/idllib/CdmwDeployment.stub.hpp>
#include <Foundation/common/Locations.hpp>

#include <CCMContainer/ccmcontainer/Container_impl.hpp>
#include <CCMContainer/ccmcontainer/HomesServantActivator.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>
#include <CCMContainer/ccmcontainer/ServiceRegistry_impl.hpp>
#include <CCMContainer/ccmcomponentserver/ComponentServer_impl.hpp>

#include <Repository/repositoryinterface/RepositoryInterface.hpp>

#include "MyComponentServer_impl.hpp"


#include <algorithm>
#include <functional>

#include <osthreads/MutexGuard.hpp>

namespace
{

     
}; // End of anonymous namespace



namespace Cdmw {

namespace CCM {

namespace COPI {

    const char * SINGLE_THREAD_HOMES_POA_NAME = "HomeSerializePOA";
    const char * MULTI_THREAD_HOMES_POA_NAME = "HomeMultithreadPOA";
    

    
MyComponentServer_impl::MyComponentServer_impl
(
 CORBA::ORB_ptr                      orb,
 PortableServer::POA_ptr             poa,
 ::CdmwDeployment::ComponentInstallation_ptr component_installation
 )
    throw (CORBA::SystemException)
    : m_orb(CORBA::ORB::_duplicate(orb)),
      m_poa(PortableServer::POA::_duplicate(poa)),
      m_current(PortableServer::Current::_nil()),
      m_component_installation(component_installation),
      m_process_name("CCMCOPITestProcess"),
      m_application_name("CCMCOPITestApplication"),
      m_is_removed(false)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcomponentserver_logger(), "");
    
    CDMW_ASSERT(!CORBA::is_nil(orb));

    try {
        CORBA::Object_var obj =
            orb->resolve_initial_references("POACurrent");
        m_current = PortableServer::Current::_narrow(obj.in());
    } catch (::CORBA::ORB::InvalidName & in) {
        CDMW_ASSERT(false);
    }
 
    try {
        // installing the default services
        ::Cdmw::CCM::Container::ServiceRegistry_impl::get_instance()->bind
              (
               LifeCycleInterceptorRegistration::M_LifeCycleInterceptorRegistrationServiceId,
               LifeCycleInterceptorRegistration::get_instance()
               );

        ApplicationElementsContextService_impl * applicationElementsContextService_impl =
            new ApplicationElementsContextService_impl
            (
             ::Cdmw::CCM::ComponentServer::ComponentServer_impl::compute_server_id
             (
              m_application_name,
              m_process_name
              ),
             this->configuration()
             );

        ::Cdmw::CCM::Container::ServiceRegistry_impl::get_instance()->bind
              (
               ApplicationElementsContextService_impl::M_ApplicationElementsContextServiceId,
               applicationElementsContextService_impl
               );
              
                        
    } catch (::Components::CCMException & ex) {
        CDMW_ERROR(CCMLoggers::Get_ccmcomponentserver_logger(),
                   "Failed to install the " << LifeCycleInterceptorRegistration_impl::M_LifeCycleInterceptorRegistrationServiceId
                   );
                   
        // TODO minor code Cdmw::OrbSupport::INTERNALLifeCycleFrameworkError
        ::CORBA::INTERNAL ex;
        throw ex;
    }   
}

    
MyComponentServer_impl::~MyComponentServer_impl()
    throw ()
{
    // empty
}

    
//
// IDL:omg.org/Components/Deployment/ComponentServer/configuration:1.0
//
Components::ConfigValues*
MyComponentServer_impl::configuration()
    throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcomponentserver_logger(), "");
    return 0;
}
        
//
// IDL:omg.org/Components/Deployment/ComponentServer/get_server_activator:1.0
//
Components::Deployment::ServerActivator_ptr
MyComponentServer_impl::get_server_activator()
    throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcomponentserver_logger(), "");
    return 0;
}

//
// IDL:omg.org/Components/Deployment/ComponentServer/create_container:1.0
//
Components::Deployment::Container_ptr
MyComponentServer_impl::create_container(const Components::ConfigValues& config)
    throw(Components::CreateFailure,
          Components::Deployment::InvalidConfiguration,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcomponentserver_logger(), "");
    CDMW_MUTEX_GUARD(m_component_server_mutex);
    
    ::Components::HomeRegistration_ptr home_registration = 0;
    ::Components::Deployment::ComponentServer_ptr component_server = 0;

    try {
        ::PortableServer::POA_ptr poa = this->m_current->get_POA();
        ::CORBA::OctetSeq_var oid = this->m_current->get_object_id();

        ::CORBA::Object_var obj = poa->id_to_reference(oid);

        component_server =
            ::Components::Deployment::ComponentServer::_narrow(obj);

        if (true == ::CORBA::is_nil(component_server)) {
            Components::CreateFailure ex(CdmwDeployment::CONTAINER_ERROR);
            throw ex;
        }
            
        ::Cdmw::CCM::Container::ServiceRegistry_impl::get_instance()->bind
              (
               ::Cdmw::CCM::ComponentServer::ComponentServer_impl::M_componentServerRefServiceProvider,
               component_server
               );
            
    } catch (...) {
        Components::CreateFailure ex(CdmwDeployment::CONTAINER_ERROR);
        throw ex;
    }

    ::PortableServer::POA_var homes_serialize_poa;
    ::PortableServer::POA_var homes_multithread_poa;


    try {
        using namespace Cdmw::OrbSupport;        
        PortableServer::POAManager_var poaManager = m_poa -> the_POAManager();
        CORBA::PolicyList policies;
        policies.length(7);
        policies[0] =
            m_poa -> create_id_assignment_policy(PortableServer::USER_ID);
        // Force homes to have TRANSIENT references though LifeCycle factories
        // have persistent references
        policies[1] =
            m_poa -> create_lifespan_policy(PortableServer::PERSISTENT);
        policies[2] =
            m_poa -> create_servant_retention_policy(PortableServer::RETAIN);
        policies[3] =
            m_poa -> create_id_uniqueness_policy(PortableServer::UNIQUE_ID);
        policies[4] = m_poa ->
            create_request_processing_policy(PortableServer::USE_SERVANT_MANAGER);
        policies[5] = m_poa ->
            create_implicit_activation_policy(PortableServer::NO_IMPLICIT_ACTIVATION);
        // Create a single thread poa
        policies[6] = m_poa ->
            create_thread_policy(PortableServer::SINGLE_THREAD_MODEL);
                             

        StrategyList poaStrategies;

        
        homes_serialize_poa = Cdmw::OrbSupport::OrbSupport::create_POA(m_poa.in(),
                                                                       SINGLE_THREAD_HOMES_POA_NAME,
                                                                       poaManager.in(), 
                                                                       policies,
                                                                       poaStrategies);
        //
        // Create and install servant activator
        //
        {
            PortableServer::ServantActivator_var the_activator =
                new Container::HomesServantActivator();
            homes_serialize_poa -> set_servant_manager(the_activator.in());
        }
        
        
        // Create a multi thread poa
        policies[6] = m_poa ->
            create_thread_policy(PortableServer::ORB_CTRL_MODEL);
        

        homes_multithread_poa = Cdmw::OrbSupport::OrbSupport::create_POA(m_poa.in(),
                                                                         MULTI_THREAD_HOMES_POA_NAME,
                                                                         poaManager.in(), 
                                                                         policies,
                                                                         poaStrategies);
        //
        // Create and install servant activator
        //
        {
            PortableServer::ServantActivator_var the_activator =
                new Container::HomesServantActivator();
            homes_multithread_poa -> set_servant_manager(the_activator.in());
        }

    } catch (const PortableServer::POA::AdapterAlreadyExists &) {
        throw CORBA::INTERNAL(Cdmw::OrbSupport::INTERNALLifeCycleFrameworkError,
                              CORBA::COMPLETED_YES);
    } catch (const PortableServer::POA::InvalidPolicy &) {
        throw CORBA::INTERNAL(Cdmw::OrbSupport::INTERNALLifeCycleFrameworkError,
                              CORBA::COMPLETED_YES);
    } catch (const CORBA::SystemException &) {
        throw; // rethrow
    }

           
    // GET EVENT CHANNEL FACTORY
        
    // Get NamingInterface object from RepositoryInterface
    //Cdmw::CommonSvcs::Naming::NamingInterface ni;

    CdmwEvent::EventChannelFactory_var event_channel_factory;
    try {
        Cdmw::CommonSvcs::Naming::NamingInterface ni =
            Cdmw::NamingAndRepository::RepositoryInterface::get_domain_naming_interface
            (
             Cdmw::Common::Locations::CDMW_SERVICES_NAME_DOMAIN
             );

        ::CORBA::Object_var obj =
              ni.resolve("EVENT/FACTORIES/Factories/DefaultEventChannelFactory");
        
        event_channel_factory
            = CdmwEvent::EventChannelFactory::_narrow(obj.in());
        
    } catch (...) {
        CDMW_ASSERT(false);
    }
       
    if  (true == CORBA::is_nil(event_channel_factory.in())) {
        std::cerr << "defaultEventChannelFactory is a nil object reference" << std::endl;
        CDMW_ASSERT(false);
    }

    // Create a container instance
    Cdmw::CCM::Container::Container_impl* cont =
        new Cdmw::CCM::Container::Container_impl(m_orb.in(),
                                                 m_poa.in(),
                                                 homes_serialize_poa.in(),
                                                 homes_multithread_poa.in(),
                                                 m_component_installation.in(),
                                                 component_server,
                                                 home_registration,
                                                 0,
                                                 event_channel_factory.in(),
                                                 m_process_name,
                                                 m_application_name,
                                                 config);

    PortableServer::ServantBase_var servant = cont;

    // Components::Deployment::Container_var 
    m_container = cont->_this();
   
    return m_container._retn();
}
        
//
// IDL:omg.org/Components/Deployment/ComponentServer/remove_container:1.0
//
void
MyComponentServer_impl::remove_container(Components::Deployment::Container_ptr cref)
    throw(Components::RemoveFailure,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcomponentserver_logger(), "");
    CDMW_MUTEX_GUARD(m_component_server_mutex);
    if(false != ::CORBA::is_nil(m_container)) {
        m_container->remove();
    } else {
        ::CORBA::BAD_PARAM bp;
        // TODO set minor code BAD_PARAMUnknownContainerReference
        throw bp;
    }

}
        
//
// IDL:omg.org/Components/Deployment/ComponentServer/get_containers:1.0
//
Components::Deployment::Containers*
MyComponentServer_impl::get_containers()
    throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcomponentserver_logger(), "");
    CDMW_MUTEX_GUARD(m_component_server_mutex);

    using namespace Components;
    Deployment::Containers_var containers;
    if(true == ::CORBA::is_nil(m_container)) {
        containers = new Deployment::Containers(0);
        containers->length(0); 
    } else {        
        containers = new Deployment::Containers(1);
        containers->length(1); 
        CORBA::ULong idx = 0L;
        containers[idx] = m_container;
    }

    return containers._retn();
}


//
// IDL:omg.org/Components/Deployment/ComponentServer/remove:1.0
//
void
MyComponentServer_impl::remove()
    throw(Components::RemoveFailure,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcomponentserver_logger(), "");
    CDMW_MUTEX_GUARD(m_component_server_mutex);
    if (m_is_removed) {
        CDMW_ERROR(CCMLoggers::Get_ccmcomponentserver_logger(), 
            "ComponentServer already removed");
        CORBA::OBJECT_NOT_EXIST ex(Cdmw::OrbSupport::OBJECT_NOT_EXIST,
                                   CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    try {
        PortableServer::ObjectId_var oid = m_poa->servant_to_id(this);
        m_poa->deactivate_object(oid.in());
        
        // remove_all_containers();
        
        m_is_removed = true;
    } catch (const CORBA::Exception & e ) {
        CDMW_ERROR(CCMLoggers::Get_ccmcomponentserver_logger(), 
            "CORBA Exception received removing ComponentServer: " <<
            Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        Components::RemoveFailure ex(CdmwDeployment::CONTAINER_ERROR);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


}; // End namespace ComponentServer
}; // End namespace CCM
}; // End namespace Cdmw

