/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <ostream>

#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/Exception.hpp>
#include <Foundation/common/String.hpp>
#include <Foundation/common/Assert.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/CdmwObjectIds.hpp>
#include <Foundation/ossupport/SystemProperties.hpp>
#include <Foundation/logging/LogManager.hpp>

#include <ccmcopi/COPIServiceInit.hpp>
#include "TestCopi_impl.hpp"

#include "tao/CodecFactory/CodecFactory.h"
#include "tao/Codeset/Codeset.h"
#include <CCMContainer/ccmcopi/COPIUtils_impl.hpp>

class TestThread : public Cdmw::OsSupport::Thread
{
   public:
      TestThread(CORBA::ORB_ptr orb)
         : m_orb(CORBA::ORB::_duplicate(orb))
      {
      }
      virtual ~TestThread() throw() {}
      
      virtual void run() throw()
      {
         std::cout << "TestThread: Enter in run()." << std::endl;
         m_orb->run();
      }
   private:
      CORBA::ORB_var m_orb;
};

class TestDriver : public Cdmw::OsSupport::Thread
{
   public:
      TestDriver(CORBA::ORB_ptr orb)
         : m_orb(CORBA::ORB::_duplicate(orb))
      {
      }
      virtual ~TestDriver() throw() {}
      
      virtual void run() throw()
      {
         std::cout << "TestDriver: Enter in run()." << std::endl;
         int timescale = Cdmw::TestUtils::get_timescale();
         Cdmw::OsSupport::OS::sleep(timescale*2000);
         
         Cdmw::TestUtils::CORBATestManager::instance()->run_tests();
         Cdmw::TestUtils::CORBATestManager::instance()->summary();
         // ORB destroyed by test
      }
   private:
      CORBA::ORB_var m_orb;
};

int main (int argc, char* argv[])
{

    std::cout << "Main: Start the Test..." << std::endl;
    int result = 0;

    try {
        Cdmw::OrbSupport::StrategyList strategyList;
        strategyList.add_OrbThreaded();
        strategyList.add_PoaThreadPool( 10 );
        strategyList.add_multicast();

        Cdmw::Logging::LogManager::Init(argc, argv);

        std::cout << "Main: Call to COPIServiceInit::Init()." << std::endl;
        Cdmw::CCM::COPI::COPIServiceInit::Init(argc, argv);

        std::cout << "Main: Call to OrbSupport::ORB_init()." << std::endl;
        CORBA::ORB_var orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, strategyList ); 
       
        std::cout << "Main: Call to COPIServiceInit::PostInit()." << std::endl;
        Cdmw::CCM::COPI::COPIServiceInit::Post_init(orb.in());
        
        // Activate the POA manager
        std::cout << "Main: Retrieve the RootPOA." << std::endl;
        CORBA::Object_var obj = orb->resolve_initial_references( "RootPOA" );
        PortableServer::POA_var rootPOA = PortableServer::POA::_narrow( obj.in() );
      
        std::cout << "Main: Activate the POA manager." << std::endl;
        PortableServer::POAManager_var poaMgr = rootPOA->the_POAManager();
        poaMgr->activate();

        std::cout << "Main: Init the Test manager." << std::endl;
        Cdmw::TestUtils::CORBATestManager::instance()->init(orb.in(),rootPOA.in(),argc,argv);
        std::auto_ptr<TestDriver> thr(new TestDriver(orb.in()));
        std::cout << "Main: Run the Test Suite." << std::endl;
        thr->start();        
        orb->run();
        thr->join();     

        std::cout << "Main: Stop the Test." << std::endl;
        // ORB destroyed by test
        /*
        Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
        orb->destroy();      
        */
        result = Cdmw::TestUtils::CORBATestManager::instance()->was_successful();

    } catch (const CORBA::SystemException& e) {
        std::cerr << "Unexpected CORBA::SystemException:\n"
                  << e << " - minor code: " << e.minor()
                  << std::endl;
    }

    std::cout << "Main: Test ended." << std::endl;
    return result;
}

