/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <vector>
#include <sstream>
#include <cppunit/extensions/HelperMacros.h>
#include "COPITester_impl.hpp"

namespace Cdmw {

namespace CCM {

namespace COPI {


COPITester_impl::COPITester_impl 
(
 Components::ExtensionContext_ptr context
 ) : m_started(false), m_context(::Components::ExtensionContext::_duplicate(context))
{}


COPITester_impl::~COPITester_impl () {

    registered_copis_t::iterator it;
    for(
        registered_copis_t::iterator it = m_registered_copis.begin();
        it !=  m_registered_copis.end();
        ++it
        ) {
        delete it->second;
    };
}
    

void COPITester_impl::register_interceptor 
(
 const char * name,
 ::TestService::COPIType type
 )
    throw (
           ::CORBA::SystemException
           )
{
    ::Components::Cookie * ck = 0;

    try {
        switch(type) {
        case ::TestService::CLIENT : {
            LoggerClientCOPI_impl * copi =
                new LoggerClientCOPI_impl(name, this);
            ::Components::ContainerPortableInterceptor::ClientContainerInterceptorRegistration_var client_ci_reg
                  = m_context->get_client_interceptor_registration();
            ck = client_ci_reg->register_client_interceptor(copi);
            client_copis.insert(std::make_pair(ck, copi));
            break;
        }
        case ::TestService::SERVER : {
            LoggerServerCOPI_impl * copi =
                new LoggerServerCOPI_impl(name, this);
            ::Components::ContainerPortableInterceptor::ServerContainerInterceptorRegistration_var server_ci_reg
                  = m_context->get_server_interceptor_registration();

            ck = server_ci_reg->register_server_interceptor(copi);
            server_copis.insert(std::make_pair(ck, copi));
            break;
        }
        case ::TestService::SERVANT : {            
            LoggerServantCOPI_impl * copi =
                new LoggerServantCOPI_impl(name, this);
            ::Components::ContainerPortableInterceptor::ServantContainerInterceptorRegistration_var servant_ci_reg
                  = m_context->get_servant_interceptor_registration();            
            ck = servant_ci_reg->register_servant_interceptor(copi);
            servant_copis.insert(std::make_pair(ck, copi));
            break;
        }
        case ::TestService::STUB : {
            LoggerStubCOPI_impl * copi =
                new LoggerStubCOPI_impl(name, this);
            ::Components::ContainerPortableInterceptor::StubContainerInterceptorRegistration_var stub_ci_reg
                  = m_context->get_stub_interceptor_registration();            
            ck = stub_ci_reg->register_stub_interceptor(copi);
            stub_copis.insert(std::make_pair(ck, copi));
            break;            
        }
        default:       
            CDMW_NEVER_HERE();
        }

        this->m_registered_copis.insert(std::make_pair(name, new COPI_info(ck, type, name)));

        std::cout << "==================================" << std::endl;
        std::cout << "Interceptor " << name << " REGISTERED" << std::endl;
        std::cout << "==================================" << std::endl;
    }
    catch (const Components::CCMException& e) {
        std::cout << "Caught: " << e << " -> exiting..." << std::endl;
        CDMW_ASSERT(false);
    }
}

void COPITester_impl::unregister_interceptor 
(
 const char * name
 )
    throw (
           ::CORBA::SystemException,
           ::Components::ContainerPortableInterceptor::InvalidRegistration
           )
{

    COPI_info * info = 0;

    registered_copis_t::iterator it =
        m_registered_copis.find(name);

    if(it != m_registered_copis.end()) {
        info = it->second;
    }

    if(0 == info) {
        std::cout << "COPI info not found" << std::endl;
        throw ::Components::ContainerPortableInterceptor::InvalidRegistration();        
    }
        
    try {
        ::Components::Cookie * ck = (::Components::Cookie *)info->m_ck;

        switch(info->m_type) {
            // cast constness away because of CORBA mapping
        case ::TestService::CLIENT : {
            ::Components::ContainerPortableInterceptor::ClientContainerInterceptorRegistration_var client_ci_reg
                  = m_context->get_client_interceptor_registration();            
            client_ci_reg->unregister_client_interceptor(ck);
            std::map< ::Components::Cookie *, LoggerClientCOPI_impl *>::iterator it =
                client_copis.find(ck);
            
            if(it != client_copis.end()) {
                delete it->second;
                ::Components::Cookie_var v = it->first;
                client_copis.erase(it);
            }

            break;
        }
        case  ::TestService::SERVER : {
            ::Components::ContainerPortableInterceptor::ServerContainerInterceptorRegistration_var server_ci_reg
                  = m_context->get_server_interceptor_registration();            
            server_ci_reg->unregister_server_interceptor(ck);
            std::map< ::Components::Cookie *, LoggerServerCOPI_impl *>::iterator it =
                server_copis.find(ck);

            if(it != server_copis.end()) {
                delete it->second;
                ::Components::Cookie_var v = it->first;
                server_copis.erase(it);
            }

            break;
        }
        case ::TestService::SERVANT : {
            ::Components::ContainerPortableInterceptor::ServantContainerInterceptorRegistration_var servant_ci_reg
                  = m_context->get_servant_interceptor_registration();            
            servant_ci_reg->unregister_servant_interceptor(ck);
            std::map< ::Components::Cookie *, LoggerServantCOPI_impl *>::iterator it =
                servant_copis.find(ck);

            if(it != servant_copis.end()) {
                delete it->second;
                ::Components::Cookie_var v = it->first;
                servant_copis.erase(it);
            }

            break;
        }


        case ::TestService::STUB : {
            ::Components::ContainerPortableInterceptor::StubContainerInterceptorRegistration_var stub_ci_reg
                  = m_context->get_stub_interceptor_registration();            
            stub_ci_reg->unregister_stub_interceptor(ck);
            std::map< ::Components::Cookie *, LoggerStubCOPI_impl *>::iterator it =
                stub_copis.find(ck);

            if(it != stub_copis.end()) {
                delete it->second;
                ::Components::Cookie_var v = it->first;
                stub_copis.erase(it);
            }

            break;
        }



        default:       
            CDMW_NEVER_HERE();
        }

        delete it->second;
        this->m_registered_copis.erase(it);

        std::cout << "==================================" << std::endl;
        std::cout << "Interceptor " << name << " UNREGISTERED" << std::endl;
        std::cout << "==================================" << std::endl;
    }
    catch (const Components::CCMException& e) {
        std::cout << "Caught: " << e << " -> exiting..." << std::endl;
        CDMW_ASSERT(false);
    }

}

void COPITester_impl::test_interception_stacks ()
    throw (
           ::CORBA::SystemException
           )
{
    std::cout << "****************************************" << std::endl;
    std::cout << "****** COPIs Testing report *************" << std::endl;
    std::cout << "****************************************" << std::endl;
        
    if(true == m_histo.empty()) {
        std::cout << "No interception points registered. May be the COPITester_SERVICE is not started !" << std::endl;
        return;
    }
    std::cout << "Checking that every COPI processed an ending interception point ...." << std::endl;

   

    for(registered_copis_t::iterator it = m_registered_copis.begin(); 
        it != m_registered_copis.end(); 
        ++it)
        {
            std::cout << it->second->m_name << "\t"
                      << (it->second->m_used == 0 ? "OK" : "Not OK")
                      << std::endl;     

            CPPUNIT_ASSERT_MESSAGE("A COPI has not processed an ending ip.", it->second->m_used == 0);
        }
        
    std::cout << "Checking state changes of every COPI ..." << std::endl;
    
    std::vector< InterceptionPoint * > histo_tab;

    // don't see the reason why the histo should be a map when we are converting it into a
    // vector, it is not used otherwise!
    for(histo_t::iterator it = m_histo.begin(); 
        it != m_histo.end(); 
        ++it)
        {
            histo_tab.push_back(it->second);
        }

          
    int client_basic_step = 0;
    int server_basic_step = 0;
    int stub_extended_step = 0;
    int servant_extended_step = 0;
       
    for(unsigned int i=0; i < histo_tab.size(); /* empty */ ) {
               
        while(i < histo_tab.size() && "stub_send_request" == histo_tab[i]->m_interception_point) {
            stub_extended_step++;
            std::cout << *(histo_tab[i]) << std::endl;
            i++;
        }
           
           
        while(i < histo_tab.size() && "send_request" == histo_tab[i]->m_interception_point) {
            client_basic_step++;
            std::cout << *(histo_tab[i]) << std::endl;
            i++;
        }
           
        while(i < histo_tab.size() && "receive_request_service_contexts" == histo_tab[i]->m_interception_point) {
            server_basic_step++;
            std::cout << *(histo_tab[i]) << std::endl;
            i++;
        }
           
        while(i < histo_tab.size() && "receive_request" == histo_tab[i]->m_interception_point) {
            // TODO: step missing?!   
            std::cout << *(histo_tab[i]) << std::endl;
            i++;
        }
           
        while(i < histo_tab.size() && "servant_receive_request" == histo_tab[i]->m_interception_point) {
            servant_extended_step++;
            std::cout << *(histo_tab[i]) << std::endl;
            i++;
        }
           
           
        while(i < histo_tab.size() && 0 == histo_tab[i]->m_interception_point.find("servant_send_")) {
            std::ostringstream msg;
            msg << "Unexpected interception point: " << *(histo_tab[i]);

            CPPUNIT_ASSERT_MESSAGE(msg.str(), servant_extended_step != 0);
            servant_extended_step--;
            std::cout << *(histo_tab[i]) << std::endl;
            i++;
        }
           
        while(i < histo_tab.size() && (0 == histo_tab[i]->m_interception_point.find("send_reply") || 0 == histo_tab[i]->m_interception_point.find("send_exception") ||
                                      0 == histo_tab[i]->m_interception_point.find("send_other"))) {
            std::ostringstream msg;
            msg << "Unexpected interception point: " << *(histo_tab[i]);

            CPPUNIT_ASSERT_MESSAGE(msg.str(), server_basic_step != 0);
            server_basic_step--;
            std::cout << *(histo_tab[i]) << std::endl;
            i++;
        }
           
        while(i < histo_tab.size() && (0 == histo_tab[i]->m_interception_point.find("receive_reply") || 0 == histo_tab[i]->m_interception_point.find("receive_exception") ||
                                      0 == histo_tab[i]->m_interception_point.find("receive_other"))) {
            std::ostringstream msg;
            msg << "Unexpected interception point: " << *(histo_tab[i]);

            CPPUNIT_ASSERT_MESSAGE(msg.str(), client_basic_step != 0);
            client_basic_step--;
            std::cout << *(histo_tab[i]) << std::endl;
            i++;
        }
           
        while(i < histo_tab.size() && (0 == histo_tab[i]->m_interception_point.find("stub_receive_"))) {
            std::ostringstream msg;
            msg << "Unexpected interception point: " << *(histo_tab[i]);

            CPPUNIT_ASSERT_MESSAGE(msg.str(), stub_extended_step != 0);
            stub_extended_step--;
            std::cout << *(histo_tab[i]) << std::endl;
            i++;
        }
           
        //(*ExtendedStep=1) expresses the case where one of the servant/stub COPIs raise an exception or blocks the call
        CPPUNIT_ASSERT_MESSAGE("Illegal Stub COPI state transitions", stub_extended_step == 0 || stub_extended_step == 1);
        CPPUNIT_ASSERT_MESSAGE("Illegal Servant COPI state transitions", servant_extended_step == 0 || servant_extended_step == 1); 
        servant_extended_step = 0;
        stub_extended_step = 0;
        CPPUNIT_ASSERT_MESSAGE("Illegal Client COPI state transitions", client_basic_step == 0);
        CPPUNIT_ASSERT_MESSAGE("Illegal Server COPI state transitions", server_basic_step == 0);
           
           
    } // for(...)
       
       
       
    std::cout << "Checking state changes of every COPI ... OK :-)" << std::endl;
}

void COPITester_impl::start_listening ()
    throw (
           ::CORBA::SystemException
           )
{
    CDMW_MUTEX_GUARD(m_mtx);
    m_started = true;
      
    for(histo_t::iterator it = m_histo.begin(); it !=  m_histo.end(); ++it) {
        delete it->second;
    }
    m_histo.clear();    

    m_i = 0;
}

void COPITester_impl::end_listening ()
    throw (
           ::CORBA::SystemException
           )
{
    CDMW_MUTEX_GUARD(m_mtx);
    m_started = false;
    
}

// non idl spec'ed methods

void 
COPITester_impl::notify_operation
(
 const std::string & op_name,
 const std::string & copi_name,
 const Cdmw::OsSupport::OS::Timeval timestamp,
 const std::string & intercepted_op
 ) 
{
    CDMW_MUTEX_GUARD(m_mtx);

    if(!is_started()) {
        return;
    }
                        
    m_histo.insert(std::make_pair(m_i, new InterceptionPoint(copi_name, intercepted_op, op_name, timestamp))); 

    std::cout << "Operation " << op_name << " notified ! rank : " << m_i << std::endl; 
    m_i++;
        
    COPI_info * info = 0;

    registered_copis_t::iterator it =
        m_registered_copis.find(copi_name);

    if(it != m_registered_copis.end()) {
        info = it->second;
    }

    if(0 == info) {
        std::cout << "COPI info not found" << std::endl;
        throw ::Components::ContainerPortableInterceptor::InvalidRegistration();        
    }
        
    if(std::string::npos != intercepted_op.find("send_request", 0)) {
        info->push();
    }        
        
    if("receive_service_context" == intercepted_op) {
        info->push();
    }
        
    if(std::string::npos != intercepted_op.find("receive_request", 0)) {
        info->push();
    }
                
    if(0 == intercepted_op.find("servant_send_")) {
        info->pop();
    }
        
    if(0 == intercepted_op.find("send_reply") || 0 == intercepted_op.find("send_exception") ||
       0 == intercepted_op.find("send_other")) {
        info->pop();
    }
        
    if(0 == intercepted_op.find("receive_reply") || 0 == intercepted_op.find("receive_exception") ||
       0 == intercepted_op.find("receive_other")) {           
        info->pop();
    }
        
    if(0 == intercepted_op.find("stub_receive_")) {
        info->pop();
    }
}

// needs to be called under mtx protection
bool COPITester_impl::is_started() {
    return m_started;
}
        
} // End namespace COPI
} // End namespace CCM
} // End namespace Cdmw

