/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "ftmanager/InitUtils.hpp"
#include <Foundation/common/Options.hpp>
#include <Foundation/common/Exception.hpp>
#include <Foundation/common/String.hpp>
#include <Foundation/common/Assert.hpp> 
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/commonsvcs/initutils/InitUtils.hpp>
#include <FaultTolerance/ftinit/FTServiceInit.hpp>
#include "FaultTolerance/ftreplicationmanager/PrimaryBackupGroupRepositoryFTManager_impl.hpp"
#include <FaultTolerance/ftcommon/UltimateFallbackFactory.hpp>
#include <FaultTolerance/ftcommon/MacroDef.hpp>
#include <FaultTolerance/ftcommon/FTConfiguration.hpp>
#include <ftgroupcreator/GroupConfigurationMngr.hpp>
#include "ftmanager/ServiceLocator_impl.hpp"
#include <FaultTolerance/ftcommon/FTManagerDSID.hpp>
#include <FaultTolerance/ftcommon/FTLogger.hpp>
#include <Foundation/logging/LogManager.hpp>
#include <FaultTolerance/ftcommon/FTObjectIds.hpp>
// include XML library utils
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>

#include <FaultTolerance/ftcommon/FTSystemProperties.hpp>
#include <Foundation/ossupport/SystemProperties.hpp>
#include <Foundation/commonsvcs/naming/InvalidNameException.hpp>
#include <ios>
#include <string>
#include <iomanip>
#include <cstdlib>

using namespace xercesc;

namespace 
{
   const char* FT_MANAGER_NAME = "fault_manager";
   std::string HELP_OPTION = "--help";
   std::string GROUP_CONFIG_OPTION = "--groupConf";
   void notifyPreInitialisationError(bool platformManaged, const char* message)
   {
      CDMW_ASSERT(0); // Let's see if this used!
      //        CDMW_FT_ERROR << "[FTManager] " << message << std::endl;
   }   
   // Default value for ORB connection timeout
   std::string CONNECTION_TIMEOUT_PROPERTY_VALUE="50"; // 50 ms
} // end anonymous namespace


void print_option(std::ostream& os, 
                  const std::string & option,
                  const char * value,
                  const char*  description,
                  const char*  default_value = "")
{
   std::string opt(option);
   if (strlen(value) != 0) {
      opt += "=";
      opt += value;
   }
   os << "\t" << std::setw(45) << std::left << opt << ": " << description;
   if (strlen(default_value) != 0) 
      os << "Default is " << default_value << ".";
   os << "\n";
}

void print_syntax(std::ostream & os, char* program_name)
{
   os << "CARDAMOM Fault Tolerance Manager\n"
      << "Syntax :\n"
      << "\t" << program_name <<" " << Cdmw::Common::Options::PROCESS_XML_FILE_OPTION << "=XMLConfigFile [OPTIONS]\n"
      << "\n"
      << "Options:\n";
   print_option(os,Cdmw::Common::Options::PROCESS_XML_FILE_OPTION,"XMLConfigFile","FT Manager configuration file");
   print_option(os,GROUP_CONFIG_OPTION,"XMLGroupConfigurationFile","Optional. Initial object groups definitions.");
   print_option(os,Cdmw::Common::Options::VALIDATE_XML_OPTION,"","Optional. Validate XML files");
   print_option(os,HELP_OPTION,"","Optional. Print this Usage.");  
   os << std::endl; 
}


namespace Cdmw {
namespace FT {
namespace FTManager {
    FTManagerDataStore * InitUtils::M_ftManagerDataStore = NULL;
    IOGRInfoDataStore * InitUtils::M_iogrInfoDataStore = NULL;
    ConsumerIdDataStore * InitUtils::M_consumerIdDataStore = NULL;
    SequencePushConsumerDataStore * InitUtils::M_sequencePushConsumerDataStore = NULL;
    StructuredPushConsumerDataStore * InitUtils::M_structuredPushConsumerDataStore = NULL;
    HostDataStore * InitUtils::M_hostDataStore = NULL;
    MonitoringIdDataStore * InitUtils::M_monitoringIdDataStore = NULL;
    HostStatusDataStore * InitUtils::M_hostStatusDataStore = NULL;
    LocInfoDataStore * InitUtils::M_locInfoDataStore = NULL;
    OGIDDataStore * InitUtils::M_ogidDataStore = NULL;
    OGDDataStore * InitUtils::M_ogdDataStore = NULL;
    PropertiesDataStore * InitUtils::M_propertiesDataStore = NULL;
    Cdmw::FT::DataStoreContext * InitUtils::M_context = NULL;
    std::string InitUtils::M_xml_conf = "";
    std::string InitUtils::M_xml_group_conf = "";
    bool InitUtils::M_validate = false;


/**
 * Purpose:
 * <p> pre_init
 * 
 *@exception CORBA::SystemException
 */        
    
std::string
InitUtils::Pre_init(int& argc, char** argv)
    throw(ConfigurationException, CORBA::SystemException)
{

    // Initialize the XML4C2 system 
    try
    {   
        XMLPlatformUtils::Initialize();
    }
    catch(const XMLException& toCatch)
    {
       std::string xmlExceptionMsg (XMLString::transcode(toCatch.getMessage()));
       std::string msg = "Error during Xerces-c Initialization: " + xmlExceptionMsg;

       CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                    CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL,CORBA::COMPLETED_NO),
                    msg);
    }


    try {
       // First initialise the CARDAMOM SystemProperties and the 
       // logging library is initialised.
       (void*) Cdmw::OsSupport::SystemProperties::Init(argc, argv);
       Cdmw::Logging::LogManager::Init(argc, argv);
       // From now on, CARDAMOM logging library is initialised.
    } catch (const std::exception & e ) {
       std::cerr << "Cannot initialise CARDAMOM SystemProperties or Logging library! \n"
                 << "Caught exception : " << e.what() << std::endl;
        ::exit(EXIT_FAILURE);
    }
    // Print usage when no arguments are specified
    if (argc == 1) 
    {  
       std::ostringstream ostr;
       print_syntax (ostr,argv[0]);
       CDMW_ERROR(Cdmw::FT::FTLogger::GetLogger(),
                  "The xml configuration file must be specified\n" << ostr.str());
        ::exit(EXIT_FAILURE);
    }
    std::string help_option =
       Cdmw::OsSupport::OS::get_option_value( argc, argv, HELP_OPTION);
    if (help_option == "yes")
    {  
       print_syntax (std::cout,argv[0]);
       ::exit(EXIT_SUCCESS);
    }
  
    // checks the xml configuration file option
    std::string xml_conf_option =
    Cdmw::OsSupport::OS::get_option_value( argc, argv, Cdmw::Common::Options::PROCESS_XML_FILE_OPTION);
    
    if ( xml_conf_option != "no" )
    {
        if (xml_conf_option != "yes")
        {
            if (Cdmw::OsSupport::OS::file_exists(xml_conf_option))
            {
                M_xml_conf = xml_conf_option;          
                
                // checks the optional validate option
                std::string validateXML_option =
                Cdmw::OsSupport::OS::get_option_value( argc, argv, Cdmw::Common::Options::VALIDATE_XML_OPTION);

                if (validateXML_option != "no")
                {
                    M_validate = true;
                }

            }
            else
            {
                CDMW_ERROR(Cdmw::FT::FTLogger::GetLogger(),"The xml configuration file doesn't exist");
                ::exit(EXIT_FAILURE);
            }
                
        }
        else
        {
            CDMW_ERROR(Cdmw::FT::FTLogger::GetLogger(),"The xml configuration file must be specified");
            ::exit(EXIT_FAILURE);
        }
    }
    else
    {
        CDMW_ERROR(Cdmw::FT::FTLogger::GetLogger(),"The xml configuration file must be specified");
        ::exit(EXIT_FAILURE);
    }


    // checks the xml group configuration file option
    CDMW_INFO(Cdmw::FT::FTLogger::GetLogger(),"Checking the xml configuration file option.");
    std::string xml_group_conf_option =
    Cdmw::OsSupport::OS::get_option_value(argc, argv, GROUP_CONFIG_OPTION);

    if (xml_group_conf_option != "no")
    {
        if (xml_group_conf_option != "yes")
        {
            if (Cdmw::OsSupport::OS::file_exists(xml_group_conf_option))
            {
                M_xml_group_conf = xml_group_conf_option;          

                // checks the optional validate option
                std::string validateXML_option =
                Cdmw::OsSupport::OS::get_option_value( argc, argv, GROUP_CONFIG_OPTION);

                if (validateXML_option != "no")
                {
                    M_validate = true;
                }
            }
            else
            {
                CDMW_ERROR(Cdmw::FT::FTLogger::GetLogger(),"No group configuration file present in the current directory");
                ::exit(EXIT_FAILURE);
            }
        }
        // else this option is not mandatory!
    }

    // The default configuration
    Cdmw::FT::ConfigurationMngr conf;
    // If an XML configuration file has been transmit, we use it

    if (!M_xml_conf.empty())
    {
        conf.parse(M_xml_conf, M_validate);
    }

    // Read XML values and then set Cdmw.FaultTolerance.SimpleIsAliveRetry & 
    // Cdmw.FaultTolerance.MutualIsAliveRetry properties from xml conf file.
    int nbSimpleIsAliveRetry = conf.getSimpleIsAliveRetry();
    int nbMutualIsAliveRetry = conf.getMutualIsAliveRetry();
 
    if (-1 != nbSimpleIsAliveRetry)
        Cdmw::OsSupport::SystemProperties::Set_Property(
            Cdmw::FT::FT_SIMPLE_ISALIVE_RETRY_PROPERTY_NAME, nbSimpleIsAliveRetry);
    if (-1 != nbMutualIsAliveRetry)
        Cdmw::OsSupport::SystemProperties::Set_Property(
            Cdmw::FT::FT_MUTUAL_ISALIVE_RETRY_PROPERTY_NAME, nbMutualIsAliveRetry);

    // Allow specification of a value for the ORB connection timeout from
    // command-line arguments or a property file
    std::string dummy_prop = 
       OsSupport::SystemProperties::Get_Property(OrbSupport::CDMW_ORB_CONNECTION_TIMEOUT_PROPERTY_NAME);
    if (dummy_prop.empty())
       OsSupport::SystemProperties::Set_Property(OrbSupport::CDMW_ORB_CONNECTION_TIMEOUT_PROPERTY_NAME,
                                                 CONNECTION_TIMEOUT_PROPERTY_VALUE);
    // pre_init orb for ReplicationManager purpose
    Cdmw::FT::ReplicationManager::InitUtils::Pre_init(argc, argv);

    return conf.getSystemPortString();
}
    
    
    
/** Purpose:
 * <p> post_init 
 * 
 *@exception CORBA::SystemException
 */    
    
void
InitUtils::Post_init (int& argc, char** argv, CORBA::ORB_ptr orb)
    throw(ConfigurationException, CORBA::SystemException)
{
    
    // init the traces delegation to the Daemon LogCollector
    //
    std::string daemonPort =
        Cdmw::OsSupport::OS::get_option_value(
            argc, argv, Common::Options::DAEMON_PORT_OPTION);
    Cdmw::CommonSvcs::InitUtils* initUtils = 
        Cdmw::CommonSvcs::InitUtils::Instance();
    try
    {
        initUtils->activate_LogCollector(
            orb, (short) atoi(daemonPort.c_str()));
    }
    catch (const Cdmw::Exception& ex)
    {
        CDMW_WARN(Cdmw::FT::FTLogger::GetLogger(), 
                  "Trace logs will not be sent to Trace Collector: "
                  << ex.what());
    }

    // The default configuration
    Cdmw::FT::ConfigurationMngr conf;
    // If an XML configuration file has been transmit, we use it
    if (!M_xml_conf.empty())
    {
        conf.parse(M_xml_conf, M_validate);
    }


    // If an XML Group configuration file has been transmited, we use it to
    // create default group needed by the system
    if (!M_xml_group_conf.empty() && M_validate)
    {
        CDMW_INFO(Cdmw::FT::FTLogger::GetLogger(),"Validate default Group configuration..." );
        // The default Group configuration
        CDMW_DEBUG(Cdmw::FT::FTLogger::GetLogger(),"create GroupConfigurationMngr " );
        Cdmw::FT::GroupConfigurationMngr group_conf(true);
        CDMW_DEBUG(Cdmw::FT::FTLogger::GetLogger(),"GroupConfigurationMngr parse"<< M_xml_group_conf );
        group_conf.validate(M_xml_group_conf);
    }
    else
        CDMW_INFO(Cdmw::FT::FTLogger::GetLogger(),"GroupConfigurationMngr parse"<< M_xml_group_conf );


    // TO be removed walk around!!!!!
    Cdmw::OrbSupport::OrbSupport::set_endpoint_selector_factory(Cdmw::FT::FTConfiguration::Get_FT_endpoint_selector_factory());


    CORBA::Object_var obj;
    obj = orb->resolve_initial_references("ORBPolicyManager");

    CORBA::PolicyManager_var policy_manager 
    = CORBA::PolicyManager::_narrow(obj.in());
    TimeBase::TimeT duration_time = conf.getRequestDurationTime();        
    
    CORBA::Any any;
    any <<= duration_time;
    CORBA::PolicyList policies(1);
    policies.length(1);
    policies[0] =
       orb->create_policy(::FT::REQUEST_DURATION_POLICY, any);
    
    policy_manager->set_policy_overrides(policies, CORBA::ADD_OVERRIDE);
    policies[0]->destroy();

    // Activate the POA manager
    obj = orb->resolve_initial_references( "RootPOA" );
    PortableGroup::GOA_var goa = PortableGroup::GOA::_narrow( obj.in() );
    
    PortableServer::POAManager_var mgr = goa->the_POAManager();
    mgr->activate();

    
    // Create datastore Context


    //create the properties data store and storage home
    M_context = new Cdmw::FT::DataStoreContext(orb,
                                               goa.in(),
                                               //                                                    conf.getSystemPortString(),
                                               //                                                    conf.getDomainID(),
                                               //                                                    conf.getStateTransfertIPAddress(),
                                               conf.getMaxTransactionInProgress(),
                                               conf.getMaxTransactionDone(),
                                               conf.getCohortTimeOut(),
                                               conf.getCoordinatorTimeOut());


    Cdmw::FT::TXManager* manager = new Cdmw::FT::TXManager(*M_context);

    // create datastore
    M_propertiesDataStore = new PropertiesDataStore(*manager,
                                                    PROPERTIES_DATA_STORE,
                                                    orb,
                                                    goa.in());
        
    OsSupport::OS::Timeval time;
    time.seconds = 10;
    time.microseconds = 0;
        
    M_ogdDataStore = new OGDDataStore (*manager,
                                       OGD_DATA_STORE,
                                       orb,
                                       goa.in(), 20, 100, time);
        
        
    M_ogidDataStore = new OGIDDataStore (*manager,
                                         OGID_DATA_STORE,
                                         orb,
                                         goa.in());
        
        
    M_locInfoDataStore = new LocInfoDataStore(*manager,
                                              LOCATIONS_DATA_STORE,
                                              orb,
                                              goa.in());
        
        
    //simple monitoring observer
    M_hostStatusDataStore = new HostStatusDataStore (*manager,
                                                     HOST_DATA_STORE2,
                                                     orb,
                                                     goa.in());
        
    // fault detector
    M_monitoringIdDataStore = new MonitoringIdDataStore(*manager,
                                                        MONITORING_ID_DATA_STORE,
                                                        orb,
                                                        goa.in());
        
    M_hostDataStore = new HostDataStore(*manager,
                                        HOST_DATA_STORE,
                                        orb,
                                        goa.in());
        
    // fault notifier
    M_structuredPushConsumerDataStore = new StructuredPushConsumerDataStore(*manager,
                                                                            STRUCTURED_PUSH_CONSUMER_DATA_STORE,
                                                                            orb,
                                                                            goa.in());
        
        
    M_sequencePushConsumerDataStore = new SequencePushConsumerDataStore(*manager,
                                                                        SEQUENCE_PUSH_CONSUMER_DATA_STORE,
                                                                        orb,
                                                                        goa.in());
        
    M_consumerIdDataStore = new ConsumerIdDataStore(*manager,
                                                    CONSUMER_ID_DATA_STORE,
                                                    orb,
                                                    goa.in()); 
        
        
    //ftmanager
    M_iogrInfoDataStore = new IOGRInfoDataStore(*manager,
                                                IOGR_INFOS_DATA_STORE,
                                                orb,
                                                goa.in());
        
        
        
    M_ftManagerDataStore = new FTManagerDataStore(*manager,
                                                  FT_MANAGER_DATA_STORE,
                                                  orb,
                                                  goa.in());
        

    // Create the TypeRepository object and register it
    CdmwFT::TypeRepository_var typeRepository =
    new Cdmw::FT::TypeRepository_impl(conf.getSupportedInterfaceMap());
    orb->register_initial_reference(::Cdmw::FT::FT_TYPE_REPOSITORY_ObjectId, typeRepository.in());

    // Check that Fault Tolerance Domain ID is well formed!
    try {
       std::string FTDomainID(conf.getDomainID());
       CosNaming::Name_var name = 
          Cdmw::CommonSvcs::Naming::NamingInterface::to_name(FTDomainID);
       if (name->length() != 1) {
          CDMW_FATAL(Cdmw::FT::FTLogger::GetLogger(),
                     "Invalid format for Fault Tolerance Domain Identifier. "
                     << "Only one name element is allowed!");
          throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAMInvalidConfigValues, CORBA::COMPLETED_NO);
       }
       
    } catch (const Cdmw::CommonSvcs::Naming::InvalidNameException & ex) {
       CDMW_FATAL(Cdmw::FT::FTLogger::GetLogger(),
                  "Invalid format for Fault Tolerance Domain Identifier ("
                  << ex.what() << "). Use OMG compliant stringified name!");
       throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAMInvalidConfigValues, CORBA::COMPLETED_NO);
    }

    // Create a replication manager
    CdmwFT::ReplicationManager_var replication_manager
    = Cdmw::FT::ReplicationManager::InitUtils::Create_replication_manager(argc,
                                                                          argv,
                                                                          orb,
                                                                          goa.in(),
                                                                          conf.getDomainID(),
                                                                          conf.getTypeSafetyLevel(),
                                                                          conf.getHostCollocationLevel(),
                                                                          std::cout);

    // post_init orb for ReplicationManager purpose
    Cdmw::FT::ReplicationManager::InitUtils::Post_init(replication_manager.in());

    std::string str_opt_mt("");
    str_opt_mt += "--SIMPLE_PULL_CORBALOC ";
    str_opt_mt += conf.getSimplePullCorbaloc();
    str_opt_mt += " --SIMPLE_PUSH_CORBALOC ";
    str_opt_mt += conf.getSimplePushCorbaloc();
    str_opt_mt += " --MUTUAL_PULL_CORBALOC ";
    str_opt_mt += conf.getMutualPullCorbaloc();
    str_opt_mt += " --MUTUAL_PUSH_CORBALOC ";
    str_opt_mt += conf.getMutualPushCorbaloc();
    int argc_mt(8);

    if (conf.getMasterInterrogationTime() != "0")
    {            
        str_opt_mt += " --MASTER_INTERROGATION ";
        str_opt_mt += conf.getMasterInterrogationTime();
        argc_mt = argc_mt + 2;
    }
    if (conf.getSlaveAcknowledgementTime() != "0")
    {
        str_opt_mt += " --SLAVE_ACKNOWLEDGEMENT ";
        str_opt_mt += conf.getSlaveAcknowledgementTime();            
        argc_mt = argc_mt + 2;
    }
    if (conf.getMasterConfirmationTime() != "0")
    {
        str_opt_mt += " --MASTER_CONFIRMATION ";
        str_opt_mt += conf.getMasterConfirmationTime();            
        argc_mt = argc_mt + 2;
    }
    if (conf.getMasterElectionTime() != "0")
    {
        str_opt_mt += " --MASTER_ELECTION ";
        str_opt_mt += conf.getMasterElectionTime();            
        argc_mt = argc_mt + 2;
    }
    if (conf.getDaemonAcknowledgementTime() != "0")
    {
        str_opt_mt += " --DAEMON_ACKNOWLEDGEMENT ";
        str_opt_mt += conf.getDaemonAcknowledgementTime();
        argc_mt = argc_mt + 2;
    }
/*
    // Read XML values and then set Cdmw.FaultTolerance.SimpleIsAliveRetry & 
    // Cdmw.FaultTolerance.MutualIsAliveRetry properties from xml conf file.
    int nbSimpleIsAliveRetry = conf.getSimpleIsAliveRetry();
    int nbMutualIsAliveRetry = conf.getMutualIsAliveRetry();
 
    if (-1 != nbSimpleIsAliveRetry)
        Cdmw::OsSupport::SystemProperties::Set_Property(
            Cdmw::FT::FT_SIMPLE_ISALIVE_RETRY_PROPERTY_NAME, nbSimpleIsAliveRetry);
    if (-1 != nbMutualIsAliveRetry)
        Cdmw::OsSupport::SystemProperties::Set_Property(
            Cdmw::FT::FT_MUTUAL_ISALIVE_RETRY_PROPERTY_NAME, nbMutualIsAliveRetry);
*/
    char** argv_mt = Cdmw::Common::String::to_char_array
    (Cdmw::Common::String::to_strings(str_opt_mt));

    CDMW_INFO(Cdmw::FT::FTLogger::GetLogger(),"Creating the monitoringLibraryInterface");
    CdmwFT::MonitoringLibrary::MonitoringLibraryInterface_var monitoringLibraryInterface = 
    Cdmw::FT::InitUtils::Create_monitoring_library( argc_mt, argv_mt, orb);

    // Create a supervision fault detector 
    CDMW_INFO(Cdmw::FT::FTLogger::GetLogger(),"Creating the fault detector");    
    Cdmw::FT::FaultManagement::FaultDetector_impl* observer_impl
    = new Cdmw::FT::FaultManagement::FaultDetector_impl(orb,
                                                        goa.in(),
                                                        monitoringLibraryInterface.in());

    CdmwFT::FaultManagement::FaultDetector_var observer = observer_impl->_this();
    // observer_impl will be managed by the POA
    observer_impl->_remove_ref();

    // Create a fault notifier 
    CDMW_INFO(Cdmw::FT::FTLogger::GetLogger(),"Creating the fault notifier");
    Cdmw::FT::FaultNotifier_impl* notifier_impl
    = new Cdmw::FT::FaultNotifier_impl(orb,
                                       goa.in());
        
    ::FT::FaultNotifier_var notifier = notifier_impl->_this();
        
    // register a local onsumer into the faultNotifier
    CosNotifyComm::StructuredPushConsumer_var rm_consumer
    = CosNotifyComm::StructuredPushConsumer::_narrow(replication_manager.in());
    notifier_impl->register_local_consumer(rm_consumer.in());
        
    //register the fault notifier to the fault detector
    CDMW_DEBUG(Cdmw::FT::FTLogger::GetLogger(),"Set the fault notifier");
    observer->set_fault_notifier( notifier.in() );

    // create the primary backup admin 
    CDMW_DEBUG(Cdmw::FT::FTLogger::GetLogger(),"Creating the PrimaryBackupGroupRepository");
    ::Cdmw::FT::ReplicationManager::PrimaryBackupGroupRepositoryFTManager_impl * primary_backup_repository
      = ::Cdmw::FT::ReplicationManager::PrimaryBackupGroupRepositoryFTManager_impl::Get_instance();
        
    ::FT::Location loc;
    loc.length(1);
    loc[0].id = CORBA::string_dup(Cdmw::OsSupport::OS::get_hostname().c_str());
        
    // Get those FT System Properties, if any.
    Cdmw::Common::Properties* sp = Cdmw::OsSupport::SystemProperties::Instance();

    // Read XML values and then set Cdmw.FaulTolerance.UltimateFallback.host & 
    // Cdmw.FaulTolerance.UltimateFallback.port properties from xml conf file.
    std::pair<std::string,std::string> ufi = 
        conf.getUltimateFallbackInfo();
    if (!ufi.first.empty())
        sp->set_property(Cdmw::FT::ULTIMATE_FALLBACK_HOST_PROPERTY_NAME,ufi.first);
    if (!ufi.second.empty())
        sp->set_property(Cdmw::FT::ULTIMATE_FALLBACK_PORT_PROPERTY_NAME,ufi.second);
         
    std::string ufallback_host = sp->get_property(Cdmw::FT::ULTIMATE_FALLBACK_HOST_PROPERTY_NAME);
    unsigned short int ufallback_port(0);
    if (!ufallback_host.empty()) {
        std::string s = sp->get_property(Cdmw::FT::ULTIMATE_FALLBACK_PORT_PROPERTY_NAME);
        if (!s.empty())
            ufallback_port = atoi(s.c_str());
        if (ufallback_port != 0)
            (void*)UltimateFallbackFactory::Init(orb,
                                                 ufallback_host.c_str(),
                                                 ufallback_port);
    }
    
    
    // create the ft manger object
    CDMW_INFO(Cdmw::FT::FTLogger::GetLogger(),"Creating the FTManager");
    Cdmw::FT::FTManager_impl* ftManagerObj 
       = new Cdmw::FT::FTManager_impl(orb,
                                      goa.in(),
                                      notifier_impl,
                                      observer_impl,
                                      replication_manager.in(),
                                      conf.getDomainID(),
                                      conf.getSystemPortString(),
                                      M_xml_group_conf);
        
    CdmwFT::FTManager_var ftManager = ftManagerObj->_this();
    // ftManagerObj is kept by POA, thus remove local ref
    ftManagerObj->_remove_ref();
        

    std::string multicast_addr = conf.getLocalisationServiceAddress();
    CORBA::Object_var group = orb->string_to_object(multicast_addr.c_str());
    PortableServer::ObjectId_var  service_locator_id = goa->create_id_for_reference(group.in());
    Cdmw::FT::ServiceLocator_impl* service_locator_obj 
       = new  Cdmw::FT::ServiceLocator_impl(orb, 
                                            conf.getDomainID(),
                                            ftManagerObj->get_IOGR_dsid());
    goa->activate_object_with_id (service_locator_id.in (),service_locator_obj);
        

    // initialise the primary_backup_repository
    primary_backup_repository->init(orb,
                                    loc,
                                    ftManager.in());

    //bind the ft manager to the corbaloc
    OrbSupport::OrbSupport::bind_object_to_corbaloc(orb, FT_MANAGER_NAME, ftManager.in());
    // bind the NamingContextExt object to corbaloc
    CosNaming::NamingContextExt_var root_nc 
       = CosNaming::NamingContextExt::_narrow(replication_manager.in());
    if (!CORBA::is_nil(root_nc.in()))
       OrbSupport::OrbSupport::bind_object_to_corbaloc(orb, "NameService", root_nc.in());

        
    //create the mutual monitoring object

    CDMW_DEBUG(Cdmw::FT::FTLogger::GetLogger(),"Create the locations");
    CdmwFT::MonitoringLibrary::Locations_var locations = new CdmwFT::MonitoringLibrary::Locations;
    locations->length(1);
    locations[0L]=OsSupport::OS::get_hostname().c_str();
        
        
    CDMW_DEBUG(Cdmw::FT::FTLogger::GetLogger(),"Create the mutualMonitoring");        
    CdmwFT::MonitoringLibrary::PrimaryBackup::MutualMonitoring_var mutualMonitoring 
    = monitoringLibraryInterface->create_mutual_monitoring(locations.in());

    CDMW_DEBUG(Cdmw::FT::FTLogger::GetLogger(),"Create the mutualMonitoringObserver");        
    Cdmw::FT::MutualMonitoringObserver_impl* mutualMonitoringObserver 
    = new Cdmw::FT::MutualMonitoringObserver_impl(notifier.in(), ftManagerObj);
        
    mutualMonitoring->attach_observer(mutualMonitoringObserver);
    ftManagerObj->register_mutual_monitoring(mutualMonitoring.in());        
    mutualMonitoring->start_monitoring();
    // Print out the corbalocs
    CDMW_INFO(Cdmw::FT::FTLogger::GetLogger(),"corbaloc::" << OsSupport::OS::get_hostname()
              << ":" << conf.getSystemPortString()
              << "/" << FT_MANAGER_NAME );
}

       
    
/** Purpose:
 * <p> cleanup
 * 
 *@exception CORBA::SystemException
 */    
    
void
InitUtils::Cleanup()
throw(CORBA::SystemException)
{

    delete M_ftManagerDataStore;
    delete M_iogrInfoDataStore;
    delete M_consumerIdDataStore;
    delete M_sequencePushConsumerDataStore;
    delete M_structuredPushConsumerDataStore;
    delete M_hostDataStore;
    delete M_monitoringIdDataStore;
    delete M_hostStatusDataStore;
    delete M_locInfoDataStore;
    delete M_ogidDataStore;
    delete M_ogdDataStore;
    delete M_propertiesDataStore;

    delete M_context;

    try
    {
        XMLPlatformUtils::Terminate();
    }
    catch(const XMLException& toCatch)
    {
       std::string xmlExceptionMsg (XMLString::transcode(toCatch.getMessage()));
       std::string msg = "Error during Xerces-c Termination: " + xmlExceptionMsg;

       CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                    CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL,CORBA::COMPLETED_NO),
                    msg);
    }
}


// Dump of FTManagerDataStore

class FTManagerDataStorePrinter: public FTManagerDataStore::Functor
{
    public:
        FTManagerDataStorePrinter(Cdmw::Logging::LevelType level)
            : m_level(level) {};
    
        void operator()(const std::pair< _Oid_var_type, _Data_var_type > & elt)
        {
            CDMW_LOG(Cdmw::FT::FTLogger::GetLogger(), m_level,
                     "   FTManagerObjects for " << elt.first.c_str() << " : "
                     << "(not displayed)"
                     );
        };
        
    private:
        Cdmw::Logging::LevelType m_level;
};

void
InitUtils::Dump_FTManagerDataStore (Cdmw::Logging::LevelType level)
{
    CDMW_LOG(Cdmw::FT::FTLogger::GetLogger(), level,
             "Dump of FTManagerDataStore...");
    FTManagerDataStorePrinter printer(level);
    M_ftManagerDataStore->do_for_each(printer);
}


// Dump of IOGRInfoDataStore

class IOGRInfoDataStorePrinter: public IOGRInfoDataStore::Functor
{
    public:
        IOGRInfoDataStorePrinter(Cdmw::Logging::LevelType level)
            : m_level(level) {};
    
        void operator()(const std::pair< _Oid_var_type, _Data_var_type > & elt)
        {
            CDMW_LOG(Cdmw::FT::FTLogger::GetLogger(), m_level,
                     "   IOGR num. " << elt.first << " : "
                     << "version: " << elt.second->version << " - "
                     << "location: "  
                     << Cdmw::CommonSvcs::Naming::NamingInterface::to_string(
                           elt.second->the_location)
                     );
        };
        
    private:
        Cdmw::Logging::LevelType m_level;
};

void
InitUtils::Dump_IOGRInfoDataStore (Cdmw::Logging::LevelType level)
{
    CDMW_LOG(Cdmw::FT::FTLogger::GetLogger(), level,
             "Dump of IOGRInfoDataStore...");
    IOGRInfoDataStorePrinter printer(level);
    M_iogrInfoDataStore->do_for_each(printer);
}



// Dump of ConsumerIdDataStore

class ConsumerIdDataStorePrinter: public ConsumerIdDataStore::Functor
{
    public:
        ConsumerIdDataStorePrinter(Cdmw::Logging::LevelType level)
            : m_level(level) {};
    
        void operator()(const std::pair< _Oid_var_type, _Data_var_type > & elt)
        {
            CDMW_LOG(Cdmw::FT::FTLogger::GetLogger(), m_level,
                     "   " << elt.first.c_str() << " : "
                     << "last used id: " << elt.second
                     );
        };
        
    private:
        Cdmw::Logging::LevelType m_level;
};

void
InitUtils::Dump_ConsumerIdDataStore (Cdmw::Logging::LevelType level)
{
    CDMW_LOG(Cdmw::FT::FTLogger::GetLogger(), level,
             "Dump of ConsumerIdDataStore...");
    ConsumerIdDataStorePrinter printer(level);
    M_consumerIdDataStore->do_for_each(printer);
}



// Dump of SequencePushConsumerDataStore

class SequencePushConsumerDataStorePrinter: public SequencePushConsumerDataStore::Functor
{
    public:
        SequencePushConsumerDataStorePrinter(Cdmw::Logging::LevelType level)
            : m_level(level) {};
    
        void operator()(const std::pair< _Oid_var_type, _Data_var_type > & elt)
        {
            CDMW_LOG(Cdmw::FT::FTLogger::GetLogger(), m_level,
                     "   consumerId " << elt.first << " : "
                     << "(IOR not displayed)"
                     );
        };
        
    private:
        Cdmw::Logging::LevelType m_level;
};

void
InitUtils::Dump_SequencePushConsumerDataStore (Cdmw::Logging::LevelType level)
{
    CDMW_LOG(Cdmw::FT::FTLogger::GetLogger(), level,
             "Dump of SequencePushConsumerDataStore...");
    SequencePushConsumerDataStorePrinter printer(level);
    M_sequencePushConsumerDataStore->do_for_each(printer);
}



// Dump of StructuredPushConsumerDataStore

class StructuredPushConsumerDataStorePrinter: public StructuredPushConsumerDataStore::Functor
{
    public:
        StructuredPushConsumerDataStorePrinter(Cdmw::Logging::LevelType level)
            : m_level(level) {};
    
        void operator()(const std::pair< _Oid_var_type, _Data_var_type > & elt)
        {
            CDMW_LOG(Cdmw::FT::FTLogger::GetLogger(), m_level,
                     "   consumerId " << elt.first << " : "
                     << "(IOR not displayed)"
                     );
        };
        
    private:
        Cdmw::Logging::LevelType m_level;
};

void
InitUtils::Dump_StructuredPushConsumerDataStore (Cdmw::Logging::LevelType level)
{
    CDMW_LOG(Cdmw::FT::FTLogger::GetLogger(), level,
             "Dump of StructuredPushConsumerDataStore...");
    StructuredPushConsumerDataStorePrinter printer(level);
    M_structuredPushConsumerDataStore->do_for_each(printer);
}



// Dump of HostDataStore

class HostDataStorePrinter: public HostDataStore::Functor
{
    public:
        HostDataStorePrinter(Cdmw::Logging::LevelType level)
            : m_level(level) {};
    
        void operator()(const std::pair< _Oid_var_type, _Data_var_type > & elt)
        {
            CDMW_LOG(Cdmw::FT::FTLogger::GetLogger(), m_level,
                     "   monitoringId " << elt.first << " : "
                     << "host: " << elt.second.c_str()
                     );
        };
        
    private:
        Cdmw::Logging::LevelType m_level;
};

void
InitUtils::Dump_HostDataStore (Cdmw::Logging::LevelType level)
{
    CDMW_LOG(Cdmw::FT::FTLogger::GetLogger(), level,
             "Dump of HostDataStore...");
    HostDataStorePrinter printer(level);
    M_hostDataStore->do_for_each(printer);
}



// Dump of MonitoringIdDataStore

class MonitoringIdDataStorePrinter: public MonitoringIdDataStore::Functor
{
    public:
        MonitoringIdDataStorePrinter(Cdmw::Logging::LevelType level)
            : m_level(level) {};
    
        void operator()(const std::pair< _Oid_var_type, _Data_var_type > & elt)
        {
            CDMW_LOG(Cdmw::FT::FTLogger::GetLogger(), m_level,
                     "   " << elt.first.c_str() << " : "
                     << "last used id: " << elt.second
                     );
        };
        
    private:
        Cdmw::Logging::LevelType m_level;
};

void
InitUtils::Dump_MonitoringIdDataStore (Cdmw::Logging::LevelType level)
{
    CDMW_LOG(Cdmw::FT::FTLogger::GetLogger(), level,
             "Dump of MonitoringIdDataStore...");
    MonitoringIdDataStorePrinter printer(level);
    M_monitoringIdDataStore->do_for_each(printer);
}



// Dump of HostStatusDataStore

class HostStatusDataStorePrinter: public HostStatusDataStore::Functor
{
    public:
        HostStatusDataStorePrinter(Cdmw::Logging::LevelType level)
            : m_level(level) {};
    
        void operator()(const std::pair< _Oid_var_type, _Data_var_type > & elt)
        {
            CDMW_LOG(Cdmw::FT::FTLogger::GetLogger(), m_level,
                     "   host " << elt.first.c_str() << " : "
                     << "status: " << (elt.second ? "TRUE" : "FALSE")
                     );
        };
        
    private:
        Cdmw::Logging::LevelType m_level;
};

void
InitUtils::Dump_HostStatusDataStore (Cdmw::Logging::LevelType level)
{
    CDMW_LOG(Cdmw::FT::FTLogger::GetLogger(), level,
             "Dump of HostStatusDataStore...");
    HostStatusDataStorePrinter printer(level);
    M_hostStatusDataStore->do_for_each(printer);
}



// Dump of LocInfoDataStore

class LocInfoDataStorePrinter: public LocInfoDataStore::Functor
{
    public:
        LocInfoDataStorePrinter(Cdmw::Logging::LevelType level)
            : m_level(level) {};
    
        void operator()(const std::pair< _Oid_var_type, _Data_var_type > & elt)
        {
            std::string state;
            switch (elt.second->state) {
                case CdmwFT::LocationState_INITIAL:
                    state = "INITIAL";
                    break;
                case CdmwFT::LocationState_INSERTING_PRIMARY:
                    state = "INSERTING_PRIMARY";
                    break;
                case CdmwFT::LocationState_PRIMARY:
                    state = "PRIMARY";
                    break;
                case CdmwFT::LocationState_INSERTING_BACKUP:
                    state = "INSERTING_BACKUP";
                    break;
                case CdmwFT::LocationState_BACKUP:
                    state = "BACKUP";
                    break;
                case CdmwFT::LocationState_FAILED:
                    state = "FAILED";
                    break;
                default:
                    state = "UNSPECIFIED STATE !!";
            }
            
            std::string stateness;
            switch (elt.second->stateness) {
                case CdmwFT::LocationStateness_STATELESS:
                    stateness = "STATELESS";
                    break;
                case CdmwFT::LocationStateness_STATEFULL:
                    stateness = "STATEFULL";
                    break;
                default:
                    state = "UNSPECIFIED STATENESS !!";
            }
            CDMW_LOG(Cdmw::FT::FTLogger::GetLogger(), m_level,
                     "   " << elt.first.c_str() << " : "
                     << "state: " << state << " - "
                     << "expected_add_member: " << elt.second->expected_add_member << " - "
                     << "stateness: " << stateness
                     );
        };
        
    private:
        Cdmw::Logging::LevelType m_level;
};

void
InitUtils::Dump_LocInfoDataStore (Cdmw::Logging::LevelType level)
{
    CDMW_LOG(Cdmw::FT::FTLogger::GetLogger(), level,
             "Dump of LocInfoDataStore...");
    LocInfoDataStorePrinter printer(level);
    M_locInfoDataStore->do_for_each(printer);
}



// Dump of OGIDDataStore

class OGIDDataStorePrinter: public OGIDDataStore::Functor
{
    public:
        OGIDDataStorePrinter(Cdmw::Logging::LevelType level)
            : m_level(level) {};
    
        void operator()(const std::pair< _Oid_var_type, _Data_var_type > & elt)
        {
            CDMW_LOG(Cdmw::FT::FTLogger::GetLogger(), m_level,
                     "   " << elt.first.c_str() << " : "
                     << "last used id: " << elt.second
                     );
        };
        
    private:
        Cdmw::Logging::LevelType m_level;
};

void
InitUtils::Dump_OGIDDataStore (Cdmw::Logging::LevelType level)
{
    CDMW_LOG(Cdmw::FT::FTLogger::GetLogger(), level,
             "Dump of OGIDDataStore...");
    OGIDDataStorePrinter printer(level);
    M_ogidDataStore->do_for_each(printer);
}



// Dump of OGDDataStore

class OGDDataStorePrinter: public OGDDataStore::Functor
{
    public:
        OGDDataStorePrinter(Cdmw::Logging::LevelType level)
            : m_level(level) {};
    
        void operator()(const std::pair< _Oid_var_type, _Data_var_type > & elt)
        {
            std::string state;
            switch (elt.second->state) {
                case CdmwFT::ObjectGroupState_INITIAL:
                    state = "INITIAL";
                    break;
                case CdmwFT::ObjectGroupState_FAILED:
                    state = "FAILED";
                    break;
                case CdmwFT::ObjectGroupState_LOST_REDUNDANCY:
                    state = "LOST_REDUNDANCY";
                    break;
                case CdmwFT::ObjectGroupState_DEGRADED_REDUNDANCY:
                    state = "DEGRADED_REDUNDANCY";
                    break;
                case CdmwFT::ObjectGroupState_FULLY_REDUNDANT:
                    state = "FULLY_REDUNDANT";
                    break;
                default:
                    state = "UNSPECIFIED STATE !!";
            }
            CDMW_LOG(Cdmw::FT::FTLogger::GetLogger(), m_level,
                     "   OGID " << elt.first << " : "
                     << "alias: " << elt.second->alias.in() << " - "
                     << "RepId: " << elt.second->repository_id.in() << " - "
                     << "state: " << state << " - "
                     << "version: " << elt.second->object_group_ref_version
                     );
        };
        
    private:
        Cdmw::Logging::LevelType m_level;
};

void
InitUtils::Dump_OGDDataStore (Cdmw::Logging::LevelType level)
{
    CDMW_LOG(Cdmw::FT::FTLogger::GetLogger(), level,
             "Dump of OGDDataStore...");
    OGDDataStorePrinter printer(level);
    M_ogdDataStore->do_for_each(printer);
}



// Dump of PropertiesDataStore

class PropertiesDataStorePrinter: public PropertiesDataStore::Functor
{
    public:
        PropertiesDataStorePrinter(Cdmw::Logging::LevelType level)
            : m_level(level) {};
    
        void operator()(const std::pair< _Oid_var_type, _Data_var_type > & elt)
        {
            
            CDMW_LOG(Cdmw::FT::FTLogger::GetLogger(), m_level,
                     "   " << elt.first.c_str() << " : "
                     << elt.second->length() << " properties"
                     );
        };
        
    private:
        Cdmw::Logging::LevelType m_level;
};

void
InitUtils::Dump_PropertiesDataStore (Cdmw::Logging::LevelType level)
{
    CDMW_LOG(Cdmw::FT::FTLogger::GetLogger(), level,
             "Dump of PropertiesDataStore...");
    PropertiesDataStorePrinter printer(level);
    M_propertiesDataStore->do_for_each(printer);
}



}; // namespace FTManager
}; // namespace FT
}; // namespace Cdmw

