/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2007. 
 * All rights reserved.
 * Copyright (C) THALES 2008-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "SystemMngt/platformlibrary/PropertyEventNotifier.hpp"

namespace Cdmw
{
    namespace PlatformMngt
    {


        PropertyEventNotifier::PropertyEventNotifier (unsigned long callTimeout,
                                                      CORBA::ORB_ptr the_orb)
        {
            m_observers_regs_for_all.clear();
            m_observers_regs_for_prop.clear();
            m_bMustStop = false;
            m_bRunning = false;
            m_beventInQueue = false;
            m_callTimeout = callTimeout;
            m_evt_primary_key = 0;
            m_evt_seconds = 0;
            m_evt_microsec = 0;
            m_evt_index = 0;
            m_the_orb = CORBA::ORB::_duplicate (the_orb);
            m_event_notif_ds = 0;
            m_logger_pointer =
            Cdmw::Logging::Logger::Get_Logger (CdmwPlatformMngtBase::PFMMNG);
            CDMW_ASSERT (m_logger_pointer != 0);

            CDMW_DEBUG (m_logger_pointer, "Property Event Notifier Instance ");

            m_boundSyncCallFactory = BoundSyncCallFactory::getFactory ();


            // instance to datastore notifiers
            try
            {
            Cdmw::CommonSvcs::DataStore::DataStoreBase * ds =
                Cdmw::CommonSvcs::DataStore::DataStoreRegistry::
                Get_data_store (Configuration::
                        Get_event_notifier_datastore_id ());
            m_event_notif_ds =
                dynamic_cast < Cdmw::PlatformMngt::EventNotifierDataStore * >(ds);
            }
            catch (const NotExistException & e)
            {
            CDMW_ASSERT (false);
            throw;
            }
            catch (std::bad_cast & e)
            {
            CDMW_FATAL (m_logger_pointer, e.what ());
            throw;
            }
        }

        PropertyEventNotifier::~PropertyEventNotifier ()throw ()
        {
            CORBA::release (m_the_orb);
            stop ();
            m_PropEvQueue.destroy ();
        }



        std::string PropertyEventNotifier::eventToString (CdmwPlatformMngt::PropertyChange *
                           event)
        {
            CORBA::String_var str = event->to_string ();

            std::string eventStr (str.in ());

            return eventStr;
        }


        void PropertyEventNotifier::insertEventInDataStore (CdmwPlatformMngt::
                         PropertyChange * event)
        {
            if (Configuration::Is_primary ())
            {
            // update the datastore

            // get the event key
        CdmwPlatformMngtBase::EventHeader & event_header = event->header ();
            try
            {
                m_event_notif_ds->insert (event_header.event_key, event);
            }
            catch (const Common::AlreadyExistException &)
            {
            CDMW_WARN (Cdmw::Logging::Logger::Get_Logger (CdmwPlatformMngtBase::PFMMNG),
                       "event notif record already exists in datastore : "
                   << event_header.event_key.primary_key << ":" 
                   << event_header.event_key.seconds << ":" 
                   << event_header.event_key.microseconds << ":" 
                   << event_header.event_key.counter_inx);
            }
            catch (...)
            {
            CDMW_WARN (Cdmw::Logging::Logger::Get_Logger (CdmwPlatformMngtBase::PFMMNG),
                       "event notif record unexpected exception in insertion : "
                   << event_header.event_key.primary_key << ":" 
                   << event_header.event_key.seconds << ":" 
                   << event_header.event_key.microseconds << ":" 
                   << event_header.event_key.counter_inx);
            }
            }
        }
        


        void PropertyEventNotifier::removeEventInDataStore (CdmwPlatformMngtBase::EventKey event_key)
        {
            if (Configuration::Is_primary ())
            {
            try
            {
                m_event_notif_ds->remove (event_key);
            }
            catch (const Common::NotFoundException &)
            {
            }
            }
        }

        bool PropertyEventNotifier::register_observer (PropertyChangeAspect & prop_aspect,
                                                       CdmwPlatformMngt::SupervisionObserver_ptr observer_ptr)
    throw (BadParameterException, OutOfMemoryException, CdmwPlatformMngt::ElementAlreadyExists)
{
    if (m_bMustStop) {
            return false;
    }

    std::string target_element_path = prop_aspect.get_targetelement_path();
            try
            {
            CDMW_MUTEX_GUARD (m_mutex);

        if (CORBA::is_nil (observer_ptr)) {
                CDMW_THROW2 (BadParameterException, "observer_ptr", "NIL");
        }

            //insert into map elements observer
        if (prop_aspect.is_register_for_all_properties())
            {
                // return false if no key has the observer_name in its value
            bool bool_ret = PropUtils::check_obs_name_in_map_for_all(m_observers_regs_for_all, 
                prop_aspect.get_obsname(), target_element_path);

                // if the element is present throw an Element Already Present Exception 
            if (bool_ret) {
                throw CdmwPlatformMngt::ElementAlreadyExists();
            }
            
                // if the element is not present go on 
            ObserverInfo obs_info;
            obs_info.observer_name = prop_aspect.get_obsname ();
            obs_info.observer_ptr = CdmwPlatformMngt::SupervisionObserver::_duplicate(observer_ptr);
          
            m_observers_regs_for_all.insert (MapObsForAll::value_type (target_element_path, obs_info));
            }

            // insert in PropertiesObserverData (Property, SupervisionObserver_ptr)
            else
            {
                ObserverInfo obs_info;                
                obs_info.observer_name = prop_aspect.get_obsname ();
                obs_info.observer_ptr =   CdmwPlatformMngt::SupervisionObserver::_duplicate(observer_ptr);

                // if the element is present throw an Element Already Present Exception
                 //PropUtils::print_map_for_prop(m_observers_regs_for_prop);
             
                PropUtils::update_keys (m_observers_regs_for_prop,
                            target_element_path,
                            prop_aspect.get_propertyset (),
                            obs_info);
        
                 PropUtils::print_map_for_prop(m_observers_regs_for_prop);
            }
            }
            catch (const std::bad_alloc &)
            {
            CDMW_THROW (OutOfMemoryException);

            }
    return true;
        }


        bool PropertyEventNotifier::unregister_observer (PropertyChangeAspect & prop_aspect)
        throw (BadParameterException, OutOfMemoryException,
               CdmwPlatformMngt::PropertyObserverNotFound)
        {
            bool bret = false;
            bool bsync = false;
            MapObsForAll::iterator it_for_all;
            std::string target_element_path =
            prop_aspect.get_targetelement_path ();
            std::string observer_name =
            prop_aspect.get_obsname();
            CDMW_DEBUG (m_logger_pointer,
                "UnRegister the observer to Property Event Notifier");

            // make a copy of the observers in order to release the mutex
            m_mutex.lock ();
            MapObsForAll observers_regs_for_all (m_observers_regs_for_all);
            MapObsForProp observers_regs_for_prop (m_observers_regs_for_prop);
            m_mutex.unlock ();
    
            // int to count the occurrencies in Multimap for Prop or for All
            int props_entries = 0;

            // case of the Map for All (map for observer registered for all properties)
            if (prop_aspect.is_register_for_all_properties () == true)
            {
            // PropUtils::print_map_for_all(observers_regs_for_all);
 
            // find all observer occurrencies for the element_path
            props_entries =    observers_regs_for_all.count (target_element_path);
            std::string observer_name_in_map = "";
            if (props_entries!=0)
            { 
            it_for_all = observers_regs_for_all.find (target_element_path);
            observer_name_in_map = it_for_all->second.observer_name;
            }
            // if the observer is present in the map for all for that element path, erase the entry in the Map
            int found = 0;  
            
            while(it_for_all != observers_regs_for_all.end ()){
                if (it_for_all->second.observer_name == prop_aspect.get_obsname () && it_for_all->first == target_element_path)
                {
                    found++;
                    //release the SupervisionObserver_ptr
                    CORBA::release(it_for_all->second.observer_ptr);
                    observers_regs_for_all.erase (it_for_all);
                }
                bsync = true;
                it_for_all++;
            }
            
            if (found==0) throw CdmwPlatformMngt::PropertyObserverNotFound(); 
                //PropUtils::print_map_for_all(observers_regs_for_all);
            }
            
            // case of the Map for Prop (map for observer registered for properties)
            if (prop_aspect.is_register_for_all_properties () == false)
            {
            //PropUtils::print_map_for_prop(observers_regs_for_prop);
            
            PropObsKey prop_obs_key;
            prop_obs_key.set_element_path (target_element_path);
        
            std::vector < std::string >* property_names_to_unreg = prop_aspect.get_propertyset();

            if (property_names_to_unreg == 0) 
                CDMW_THROW2(BadParameterException, "property_names_to_unreg", "NIL");
            int found = 0;          
            for (CORBA::ULong i = 0; i < property_names_to_unreg->size (); i++)
            {
                prop_obs_key.set_property ((*property_names_to_unreg)[i]);
                props_entries = observers_regs_for_prop.count (prop_obs_key);
                MapObsForProp::iterator it_for_prop = observers_regs_for_prop.find (prop_obs_key);
                std::string observer_name_in_map = "";
                while(it_for_prop != observers_regs_for_prop.end ()){
                    observer_name_in_map = it_for_prop->second.observer_name;        
                    if(observer_name==observer_name_in_map && it_for_prop->first == prop_obs_key)
                    {
                        found++;  
                        // Release of SupervisionObserver_ptr 
                        CdmwPlatformMngt::SupervisionObserver_ptr observer_ptr =
                        it_for_prop->second.observer_ptr;
                        //if (CORBA::is_nil(observer_ptr)) 
                        //    std::cerr << " NIL >>>>> " << std::endl;
                        if (!CORBA::is_nil(observer_ptr))
                            CORBA::release (observer_ptr);
                        else 
                            CDMW_DEBUG (m_logger_pointer, "observer_ptr has a nil reference in property map"); 
                        // Erasing the observer from MapObsForProp 
                        observers_regs_for_prop.erase (it_for_prop);
                    }
                it_for_prop++;
                }
            }  
            if (found==0) throw CdmwPlatformMngt::PropertyObserverNotFound();    
            bsync = true;    
            //PropUtils::print_map_for_prop(observers_regs_for_prop);

            }
            if (bsync)
            {
        // m_mutex.lock ();
            CDMW_MUTEX_GUARD (m_mutex);
            m_observers_regs_for_prop = observers_regs_for_prop;
            m_observers_regs_for_all = observers_regs_for_all;
            // m_mutex.unlock ();
            bret = true;
            }
            return bret;
        }


        bool PropertyEventNotifier::is_registered_observer_empty ()
        {
            bool bret = false;
            m_mutex.lock ();
            bret = m_observers_regs_for_all.empty ();
            m_mutex.unlock ();
            return bret;
        }

        bool PropertyEventNotifier::is_running ()
        {
            return m_bRunning;
        }

        bool PropertyEventNotifier::add_prop_event (CdmwPlatformMngt::PropertyChange *
                 event) throw (InternalErrorException)
        {
            try
            {
            CDMW_MUTEX_GUARD (m_mutex);
            CDMW_DEBUG (m_logger_pointer, "add property event");

            if (m_bMustStop)
                return false;

            set_event_key (event);
            insertEventInDataStore (event);
            m_PropEvQueue.push (event);
            m_beventInQueue = true;
            return true;
            }
            catch (...)
            {
            CDMW_THROW (InternalErrorException);
            }
            return false;
        }

        void PropertyEventNotifier::stop () throw ()
        {
            if (m_bMustStop)
            return;

            CDMW_MUTEX_GUARD (m_mutex);
            CDMW_DEBUG (m_logger_pointer, "Stop Property Event Notifier ");
            m_bMustStop = true;
            m_beventInQueue = false;
            
            m_PropEvQueue.destroy();
            this->join();
        }

        void PropertyEventNotifier::run () throw ()
        {
            std::string strace;
            std::string lpszfrm;

            //TypeNotificationCallResult result=SUCCESS;
            CdmwPlatformMngt::PropertyChange * PropEvent = NULL;

            // SupervisionObserver_ptr to be notified
            CdmwPlatformMngt::SupervisionObserver_ptr observer_ptr = CdmwPlatformMngt::SupervisionObserver::_nil();
            if (!m_bRunning)
            m_bRunning = true;

            while (m_bRunning)
            {
            bool bSync = false;
            std::string selement_path = "";
            std::string selement_path_chk = "";

            try
            {

                PropEvent = NULL;
                if (m_bMustStop)
                break;

                // Get the Event
                PropEvent = m_PropEvQueue.pop ();
                CDMW_DEBUG (m_logger_pointer,
                    "Property Event Notifier Event on Queue !");


                // make a copy of the observers in order to release the mutex
                m_mutex.lock ();
                /*ObserverElementMap */
                MapObsForAll observers_regs_for_all (m_observers_regs_for_all);
                MapObsForProp
                observers_regs_for_prop (m_observers_regs_for_prop);
                m_mutex.unlock ();


                // get element_path affected
                selement_path = PropEvent->element_path ();

                // find map of registered observer for this element
                int props_entries =
                observers_regs_for_all.count (selement_path);

                MapObsForAll::iterator it_for_all =
                observers_regs_for_all.find (selement_path);

                if (it_for_all != observers_regs_for_all.end ())
                {
                for (int j = 0; j < props_entries; ++j)
                {

                    observer_ptr = it_for_all->second.observer_ptr;    
                    // Notify 
                    if (do_notification_call (observer_ptr, PropEvent) != SUCCEED)
                    {
                    // The Observer has crashed, remove the entry
                    
                    observers_regs_for_all.erase(it_for_all);
                    bSync=true;
                    }
                    it_for_all++;
    
                    if (CORBA::is_nil(observer_ptr))
                    {
                    observers_regs_for_all.erase(it_for_all);
                                        bSync=true;
                    }
                }
                }
        
                PropObsKey prop_obs_key;
                prop_obs_key.set_element_path (selement_path);
                for (CORBA::ULong i = 0;
                 i < (PropEvent->property_infos ()).length (); i++)
                {
                // prepare the ObsKey with the target element path and the current property    
                prop_obs_key.set_property (std::string ((PropEvent->property_infos ()[i]).property_name));
                // find how many keys with value prop_obs_key are in the Map    

                int props_entries =
                    observers_regs_for_prop.count (prop_obs_key);
            
                // Find the first occourrence with prop_obs_key
                MapObsForProp::iterator it_for_prop =
                    observers_regs_for_prop.find (prop_obs_key);

                if (it_for_prop != observers_regs_for_prop.end())
                { 
                    /* iterate for all keys in observers_regs_for_prop with value
                       prop_obs_key to iterate.
                    */
                    for (int i=0; i < props_entries; i++)
                    { 
                    observer_ptr = it_for_prop->second.observer_ptr;
                    if (do_notification_call (observer_ptr, PropEvent) != SUCCEED)
                    {
                        // The Observer has crashed, remove the entry
                        observers_regs_for_prop.erase(prop_obs_key);
                        bSync=true;
                    }
                    if (CORBA::is_nil(observer_ptr))
                    {
                        observers_regs_for_prop.erase(it_for_prop);
                        bSync=true;
                    }
                    it_for_prop++;
                    }
                }
                }
    
        
                ///////////////////////////////////////////////////////////////////
                // Sync registered observer map.
                ///////////////////////////////////////////////////////////////////
                if (bSync)
                {
                m_mutex.lock ();
                m_observers_regs_for_all = observers_regs_for_all;
                m_observers_regs_for_prop = observers_regs_for_prop;
                m_mutex.unlock ();
                bSync = false;
                }



                // remove event from datastore
                // get header
                CdmwPlatformMngtBase::EventHeader event_header =
                PropEvent->header ();
                // request removing
                removeEventInDataStore (event_header.event_key);

                // deallocates the event
                QueueElementDeallocator <
                CdmwPlatformMngt::PropertyChange >::deallocate (PropEvent);
            }
                            
            catch (const MessageQueueDestroyedException&)
            {
                m_bRunning = false;
            }
            catch (const CORBA::SystemException &)
            {
                //LogMngr::logMessage(ERR, MSG_ID_EVENT_NOTIF_UNEXP_EXCEPT);
                QueueElementDeallocator <
                CdmwPlatformMngt::PropertyChange >::deallocate (PropEvent);
        throw;
            }
            catch ( ... )
            {
                // due to an unexpected exception
                // => the event is lost for all observers
                //LogMngr::logMessage(ERR, MSG_ID_EVENT_NOTIF_UNEXP_EXCEPT);
                QueueElementDeallocator <
                CdmwPlatformMngt::PropertyChange >::deallocate (PropEvent);
            }        
            }
    
            m_bRunning = false;

            CDMW_DEBUG (m_logger_pointer, "Property Event Notifier stopping...");
        }


        void PropertyEventNotifier::set_event_key (CdmwPlatformMngt::PropertyChange * event)
        {

            // set the event key
            CdmwPlatformMngtBase::EventKey event_key;

            // set primary key
            event_key.primary_key = m_evt_primary_key;

            // get time val
            OsSupport::OS::Timeval time_val = OsSupport::OS::get_time ();

            event_key.seconds = static_cast < unsigned long >(time_val.seconds);
            event_key.microseconds =
            static_cast < unsigned long >(time_val.microseconds);

            bool evt_inx_incr = false;

            if (event_key.seconds < m_evt_seconds)
            {
            event_key.seconds = m_evt_seconds;
            event_key.microseconds = m_evt_microsec;
            evt_inx_incr = true;
            }

            else if (event_key.seconds == m_evt_seconds)
            {
            if (event_key.microseconds < m_evt_microsec)
            {
                event_key.microseconds = m_evt_microsec;
                evt_inx_incr = true;
            }
            else if (event_key.microseconds == m_evt_microsec)
            {
                evt_inx_incr = true;
            }
            }

            // if there is no difference between previous event time stamp
            // inrement the evt index to build an unique event key
            if (evt_inx_incr)
            {
            m_evt_index++;
            }
            else
            {
            // reset the index (time is different)
            // counter index in header is already set to 0
            m_evt_index = 0;
            }

            event_key.counter_inx = m_evt_index;

            // store event key

            // get header
            CdmwPlatformMngtBase::EventHeader & event_header = event->header ();

            // set event key in header
            event_header.event_key = event_key;

            // save the current time used for index
            m_evt_seconds = event_key.seconds;
            m_evt_microsec = event_key.microseconds;
        }


        CORBA::Object_ptr PropertyEventNotifier::set_timeout_policy_on (CORBA::Object_ptr obj)
        {
            // Create the timeout policy
            CORBA::Any timeout;
            CORBA::Object_var capsule;

#if CDMW_ORB_VDR == orbacus
            CORBA::PolicyList policies (1);
            timeout <<= (CORBA::ULong) m_callTimeout;
            policies.length (1);
            policies[0] =
            m_the_orb->create_policy (OB::TIMEOUT_POLICY_ID, timeout);
            // Set the policy
            capsule =
            m_the_orb->_set_policy_overrides (policies, CORBA::ADD_OVERRIDE);

#elif CDMW_ORB_VDR == tao
            // Timeout success
            TimeBase::TimeT _timeout = ((unsigned long) (m_callTimeout * 1.0e4));
            CORBA::PolicyList policy_list;

            timeout <<= _timeout;
            // Set the policy
            policy_list.length (1);
            policy_list[0] =
            m_the_orb->
            create_policy (Messaging::RELATIVE_RT_TIMEOUT_POLICY_TYPE, timeout);

            CDMW_DEBUG (m_logger_pointer, "call obj->_set_policy_overrides");
            capsule =
            obj->_set_policy_overrides (policy_list, CORBA::SET_OVERRIDE);
#endif

            return capsule._retn ();
        }

        
         BoundSyncCallReturn  PropertyEventNotifier::do_notification_call (
                                                                CdmwPlatformMngt::SupervisionObserver_ptr observer_ptr,
                                                                CdmwPlatformMngt::PropertyChange*  PropEvent) 
                                                                    throw ()
        {
            CDMW_DEBUG (m_logger_pointer, "DoNotificationCall");
            BoundSyncCall *notifCall =
            m_boundSyncCallFactory->createEventNotificationCall (observer_ptr,
                                         PropEvent,
                                         m_callTimeout);

            // performs the notification via the
            // bound synchronous call
            BoundSyncCallReturn ret = notifCall->execute ();

            delete notifCall;
            return ret;
        }


        BoundSyncCall* PropertyEventNotifier::createEventNotificationCall (CdmwPlatformMngt::
                            SupervisionObserver_ptr
                            observer,
                            CdmwPlatformMngt::
                            PropertyChange *
                            event)
        throw (OutOfMemoryException)
        {

            BoundSyncCall *notifCall =
            m_boundSyncCallFactory->createEventNotificationCall (observer,
                                         event,
                                         m_callTimeout);

            return notifCall;

        }
    }                // End namespace PlatformMngt
}                // End namespace Cdmw



