/* ========================================================================== *
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2007. All rights reserved.
 * Copyright (C) THALES 2008-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * ========================================================================= */

#ifndef CARDAMOM_CLOCK_FT_CONTROLLER_IMPL_HPP
#define CARDAMOM_CLOCK_FT_CONTROLLER_IMPL_HPP

/// @cond JUMP_FILE

#include <Time/clockservice/ClockStateDataStore.hpp>
#include "clockservice/ControllerEx_impl.hpp"


namespace Cdmw
{
    namespace clock
    {
        /**
         * @brief This class implements the fault-tolerant Controller functionalities.
         *
         * To this end it adds fault-tolerance capabilities to the <code>Controller</code>
         * interface described in the EVoT specification. Allows control of periodic execution
         * after the appropriate object has been registered with the clock.
         *
         * @author Angelo Corsaro <acorsaro@selex-si.com>
         * @author Marco Macellari <mmacellari@progesi.it>
         */
        class FTControllerEx_impl : public virtual ControllerEx_impl
        {
            public:

                /**
                 * Creates a <code>FTControllerEx_impl</code> with an associated 
                 * <code>Periodic</code> and <code>Reactor</code>
                 *
                 * @param controllerId Identifyer of internal controller.
                 *
                 * @param clock The clock used by this controller to measure time.
                 *
                 * @param periodic The <code>Periodic</code> instance that will
                 *         be notified when the programmed time interval expires.
                 *
                 * @param controllerDataStorageHome information to manage state transfert.
                 *
                 * @param controllerDataStore information to manage state transfert.
                 *
                 * @param reactor The reactor that will be used to register timeout 
                 *         handlers.
                 */
                FTControllerEx_impl(Cdmw::clock::Ident                               controllerId,
                                  Cdmw::clock::ControllerDataStorageHome&          controllerDataStorageHome,
                                  Cdmw::clock::Executor_impl&                      clock,
                                  CosClockService::PeriodicExecution::Periodic_ptr periodic,
                                  ACE_Reactor*                                     reactor);

                /**
                 * Releases resources associated with the instance.
                 */
                virtual ~FTControllerEx_impl();

            public:

                /**
                 * Retrieve storage object data from data storage home related to a specified object id.
                 */
                static void retrieve_status_lock_free(Cdmw::clock::Ident                      oid,
                                                      Cdmw::clock::ControllerDataStorageHome& cdsh,
                                                      Cdmw::clock::ControllerData_var&        data_var)
                    throw (Cdmw::FT::NotFoundException);

                /**
                 * Activate replica in order to restore primary instance behaviour.
                 */
                void activate();

                // Methods added to allow for FT state transfer.
                virtual void notify_status_lock_free();
                virtual void refresh_data_lock_free(Cdmw::clock::ControllerData& data);

            private:

                Cdmw::clock::Ident                      m_controller_id;
                Cdmw::clock::ControllerDataStorageHome& m_controller_data_storage_home;
        };
    } /* clock */
} /* Cdmw */
/// @endcond
#endif /* CARDAMOM_CLOCK_FT_CONTROLLER_IMPL_HPP */
