/* ========================================================================== *
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati.
 * Copyright (C) SELEX Sistemi Integrati 2004-2006. All rights reserved.
 * Copyright (C) THALES 2008-2010. All rights reserved.
 *
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 *
 *
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * ========================================================================= */

/**
 * @author Raffaele Mele <rmele@progesi.it>
 * @brief TimeObserver - Federated Controlled Clock Observer
 */

package cdmw.clock.federated;

import org.omg.CORBA.ORB;
import org.omg.CORBA.Any;
import org.omg.CORBA.SystemException;
import org.omg.CORBA.ORBPackage.InvalidName;
import org.omg.CosClockService.*;

import org.omg.CosClockService.ClockCatalogPackage.ClockEntry;
import org.omg.CosClockService.ClockCatalogPackage.UnknownEntry;
import org.omg.CosClockService.ControlledClockHelper;
import Cdmw.clock.ClockObserver;
import Cdmw.clock.ControlledClockEx;
import Cdmw.clock.ControllerEx;
import Cdmw.clock.ControlledClockExHelper;

import cdmw.common.BadParameterException;
import cdmw.orbsupport.ORBSupport;
import cdmw.orbsupport.StrategyList;
import cdmw.ossupport.OS;

class ORBWorker extends Thread {
    private ORB m_ORB;

    public ORBWorker(ORB orb) {
        m_ORB = orb;
    }

    public void run() {
        try {
            m_ORB.run();
        } catch (SystemException ex) {
            ex.printStackTrace();
        }
    }
}

class ClockObserverImpl extends Cdmw.clock.ClockObserverPOA 
                     implements Cdmw.clock.ClockObserverOperations 
{

    int m_event;
    static long m_total_events;


    public ClockObserverImpl() {
        m_event = Cdmw.clock.NONE.value;
        m_total_events = 0;
    }

    public void handle_pause(Any arg0) throws org.omg.CORBA.SystemException {
        m_event |= Cdmw.clock.PAUSE.value;
        m_total_events++;
    }

    public void handle_resume(Any arg0) throws org.omg.CORBA.SystemException {
        m_event |= Cdmw.clock.RESUME.value;
        m_total_events++;
    }

    public void handle_set(Any arg0) throws org.omg.CORBA.SystemException {
        m_event |= Cdmw.clock.SET_TIME.value;
        m_total_events++;
    }

    public void handle_set_rate(Any arg0) throws org.omg.CORBA.SystemException {
        m_event |= Cdmw.clock.SET_RATE.value;
        m_total_events++;
    }

    public void handle_terminate(Any arg0) throws org.omg.CORBA.SystemException {
        m_event |= Cdmw.clock.TERMINATE.value;
        m_total_events++;
    }

    public int getProcessedEvent() {
        return m_event;
    }

    public long getTotalEvents() {
        return m_total_events;
    }

    public void resetEventsCounter() {
        m_total_events = 0;
    }
}

public class TimeObserver {

    static final int SUCCESS = 0;

    static final int ERROR = -1;

    private static ORB m_orb;

    private static ClockCatalog m_catalog;

    private static ControlledClockEx m_fedClock;

    private static ClockObserverImpl my_observer;

    private static Cdmw.clock.ClockObserver observer;

    public static void main(String[] args) {
        StrategyList strategyList = new StrategyList();
        strategyList.addORBThreaded();
        try {
            strategyList.addPOAThreadPerConnection();
        } catch (BadParameterException e) {
            e.printStackTrace();
            System.exit(ERROR);
        }

        m_orb =  org.omg.CORBA.ORB.init(args, null);

        // ORB in a thread
        ORBWorker orb_worker = new ORBWorker(m_orb);
        orb_worker.start();

        try {
            OS.sleep(2000);
        } catch (InterruptedException e) {
            e.printStackTrace();
            System.exit(ERROR);
        }

        org.omg.CORBA.Object obj;
        try {
            obj = m_orb.resolve_initial_references("ClockService");
            m_catalog = ClockCatalogHelper.narrow(obj);
        } catch (InvalidName ex) {
            ex.printStackTrace();
            System.exit(ERROR);
        }

        if (m_catalog == null) {
            System.out.println("Error : Unable to resolve clock catalog");
        }

        ClockEntry entry = null;

        try {

            entry = m_catalog.get_entry("FederatedControlledClock");
            m_fedClock = ControlledClockExHelper.narrow(entry.subject);

            my_observer = new ClockObserverImpl();
            observer = my_observer._this(m_orb);

            org.omg.CORBA.Any aparam;
            aparam = m_orb.create_any();

            m_fedClock.attach(Cdmw.clock.ALL.value, observer, aparam);

        } catch (UnknownEntry ex) {
            System.out.println("ClockCatalog::UnknownEntry");
        } catch (org.omg.CosClockService.ControlledClockPackage.NotSupported ex) {
            System.err.println("Unable to attach Observer!");
            ORBSupport.ORBCleanup(m_orb);
            System.exit(ERROR);
        } catch (Cdmw.clock.ControlledClockExPackage.AlreadyPresentEntry ex) {
            System.err.println("Already attacched!");
        }
        if (m_fedClock == null) {
            System.err.println("Unable to resolve ControlledClock, Aborting!");
            ORBSupport.ORBCleanup(m_orb);
            System.exit(ERROR);
        }

        long old_events = my_observer.getTotalEvents();
        while (true) {
            long new_events = my_observer.getTotalEvents();
            if ( new_events > old_events) 
            {
                int event = my_observer.getProcessedEvent();
                if ( event ==  Cdmw.clock.SET_TIME.value ) {
                    System.out.println("Observer: SET_TIME processed.");
                } else if (event ==  Cdmw.clock.SET_RATE.value) {
                    System.out.println("Observer: SET_RATE processed.");
                } else if (event == Cdmw.clock.PAUSE.value) {
                    System.out.println("Observer: PAUSE processed.");
                } else if (event == Cdmw.clock.RESUME.value) {
                    System.out.println("Observer: RESUME processed.");
                } else if (event == Cdmw.clock.TERMINATE.value) {
                    System.out.println("Observer: TERMINATE processed.");
                }
                old_events = new_events;
                
            }
            try {
                OS.sleep(2000);
            } catch (InterruptedException e) {
                e.printStackTrace();
                System.exit(ERROR);
            }
            System.out.println("Total events: " + my_observer.getTotalEvents());
        }
    }
}
