/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2007. 
 * Copyright (C) THALES 2008-2010. All rights reserved.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */


#include <fstream>
#include <iostream>

#include <Foundation/logging/LogManager.hpp>
#include <Foundation/logging/Logger.hpp>
#include "TraceAndPerf/recordinglibrary/OperationMap.hpp"
#include "TraceAndPerf/recordinglibrary/Recording_Default_Parameters.hpp"

namespace
{
    const char C_SEPARATOR = ' ';
    const unsigned short LENGTH_MAX = 1000;
}
namespace Cdmw
{
   namespace Recording
   {

      Cdmw::Logging::Logger* operationmap_logger;
      OperationMap::OperationMap (const char * filename) throw()
      {
         operationmap_logger = Cdmw::Logging::Logger::Get_Logger("CDMW.TRC.RecordingDaemon");
         std::ifstream a_file ( filename );
         char str[LENGTH_MAX];

         if ( !a_file.is_open() ) {
            // The file could not be opened
            m_operation_list.clear();
            CDMW_WARN(operationmap_logger, 
                      "!!! OperationMap : The file "<< filename << " could not be opened ");
         }
         else {
            std::string my_str;
            while (!a_file.eof())
            {
               //Read one string from the file and add it to the
               //operation list
               a_file.getline(str, LENGTH_MAX);
               my_str = str;
               addOperation (my_str);
            }
            a_file.close();
         }   
      }

      OperationMap::~OperationMap () throw()
      {
         m_operation_list.clear();
      }

      /**
       *Purpose:
       *<p> suppress blanks at the head and at
       * the tail of a string
       *
       */
      inline std::string suppr_head_tail_blanks (std::string & str)
      {
        std::string l_str = str;
        int each = 0;
        int pos1 = 0;
        int str_len = str.length();
        while ((each < str_len) && ((str[each] == ' ') || (str[each] == TAB_CHARACTER)))
        {
          each ++;
        }
        pos1 = each;
        each = str_len;
        while ((each > pos1) && ((str[each - 1] == ' ') || (str[each - 1] == TAB_CHARACTER)) )
        {
          each --;
        }
        l_str = str.substr(pos1, each - pos1);
        return l_str;        
      }

      /**
       *Purpose: 
       *<p> checks "if an object in the Object Map
       * implements one of the stored operations then flag
       * it to be recorded".
       *
       */
      bool OperationMap::checkOperation (std::string & operation)
            throw()
      {
         // if operation list is empty, all operations will be recorded
         if (m_operation_list.empty())
         {
            return true;
         }
         std::string l_str = suppr_head_tail_blanks(operation);
         if (l_str.length() == 0)
         {
           return false;
         }           
         for(unsigned int each=0; each<m_operation_list.size(); each++)
         {
           if (l_str == m_operation_list[each])
             return true;
         }
        return false;
      }

      /**
       *Purpose:
       *<p> adds the requested operation to the operations store.
       *
       */
      OperationMap::Status OperationMap::addOperation (std::string & operation)
            throw()
      {
         std::string l_str = suppr_head_tail_blanks(operation);
         if (l_str.length() == 0)
         {
           return OperationMap::OPERATION_NOP;
         }           
         std::vector<std::string>::iterator each;
         for(each = m_operation_list.begin(); 
             each != m_operation_list.end();
             each++)
         {
           if (l_str == *each)
           {
             return OperationMap::OPERATION_ALREADY_EXISTS;
           }
         }
         m_operation_list.push_back(l_str);
         return OperationMap::OPERATION_ADDED;
      }

      /**
       *Purpose:
       *<p> adds the requested operation to the operation store.
       *
       */
      OperationMap::Status OperationMap::addOperations
            (CdmwRecording::String_List & list)
            throw() 
      {
         std::string operation;
         int len = list.length();
         for (int each = 0 ; each < len ; each ++)
         {
            operation = list[each];
            addOperation(operation);
         }
         return OPERATION_OK;
      }

      /**
       *Purpose:
       *<p> removes the requested interface from the operations store.
       *
       */
      OperationMap::Status OperationMap::removeOperation (std::string & operation)
          throw()
      {
        std::string l_str = suppr_head_tail_blanks(operation);
        std::vector<std::string>::iterator each;
        if (l_str.length() == 0)
        {
          return OperationMap::OPERATION_NOP;
        }           
        for(each = m_operation_list.begin(); 
            each != m_operation_list.end();
            each++)
        {
          if (l_str == *each)
          {
            m_operation_list.erase(each);
            return OperationMap::OPERATION_REMOVED;
          }
        }
        return OperationMap::OPERATION_NOT_FOUND;
      }

      /**
       *Purpose:
       *<p> removes the requested operations from the operation store.
       *
       */
      OperationMap::Status OperationMap::removeOperations
            (CdmwRecording::String_List & list)
            throw() 
      {
         std::string operation;
         int len = list.length();

         for (int each = 0 ; each < len ; each ++)
         {
            operation = list[each];
            removeOperation(operation);
         }
         return OPERATION_OK;
      }

      /**
       *Purpose:
       *<p> clear the operation store.
       *
       */
      OperationMap::Status OperationMap::removeAllOperations ()
            throw() 
      {
         m_operation_list.clear();
         return OPERATION_OK;
      }

      /**
       *Purpose:
       *<p> return the operation map size
       *
       */
       int OperationMap::list_size()
       {
          return m_operation_list.size();
       }

      /**
       *Purpose:
       *<p> map iterator.
       *
       */
      void OperationMap::mapLoop (display print)
          throw()
      {

        for (size_t i = 0; i < m_operation_list.size(); i++)
        {
          print (m_operation_list[i], i);
        }
      }

   }
} 
