/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ccm.ccmcopi;

import static cdmw.logging.FunctionLogger.CDMW_LOG_FUNCTION;
import static cdmw.logging.FunctionLogger.CDMW_LOG_FUNCTION_RETURN;
import static cdmw.logging.Logger.CDMW_DEBUG;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.TreeMap;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import org.omg.CORBA.CompletionStatus;
import org.omg.Components.Cookie;
import org.omg.Components.ContainerPortableInterceptor.ClientContainerInterceptor;
import org.omg.Components.ContainerPortableInterceptor.ContainerInterceptor;
import org.omg.Components.ContainerPortableInterceptor.InvalidRegistration;
import org.omg.Components.ContainerPortableInterceptor.ServantContainerInterceptor;
import org.omg.Components.ContainerPortableInterceptor.ServerContainerInterceptor;
import org.omg.Components.ContainerPortableInterceptor.StubContainerInterceptor;

import cdmw.ccm.common.CCMLoggers;
import cdmw.orbsupport.ExceptionMinorCodes;
import cdmw.orbsupport.ExceptionStrings;
import cdmw.orbsupport.ORBSupport;

/**
 * A singleton class that gives access to the four Container Interceptor
 * Registration interfaces
 * 
 */
public class COPIRegistration extends org.omg.CORBA.LocalObject
        implements com.thalesgroup.CdmwCcmCopi.COPIRegistrar {

    protected cdmw.logging.Logger logger = CCMLoggers.GetCcmCopiLogger();

    /**
     * Those maps include all the registered Client/Server/Stub/Servant Container Interceptors, 
     * indexed by their integration points
     */
    private Hashtable clientCOPIMap;

    private Hashtable serverCOPIMap;
    
    private Hashtable stubCOPIMap;
    
    private Hashtable servantCOPIMap;

    /**
     * This is an ArrayList that keeps track for each client interceptor, the number of requests
     * in which it is invloved 
     */
    private Hashtable<ClientContainerInterceptor, RefCountedContainerInterceptor> clientCOPIRefCount;
    
    private Hashtable<ServerContainerInterceptor, RefCountedContainerInterceptor> serverCOPIRefCount;
    
    private Hashtable<StubContainerInterceptor, RefCountedContainerInterceptor> stubCOPIRefCount;
    
    private Hashtable<ServantContainerInterceptor, RefCountedContainerInterceptor> servantCOPIRefCount;

    /**
     * 
     */
    private class RefCountedContainerInterceptor {
        public Cookie cookie;
        private int ref_count;

        public RefCountedContainerInterceptor(Cookie ck) {
            this.cookie = ck;
            this.ref_count = 0;
        }

        public synchronized void add_ref() {
            ref_count++;
        }

        public synchronized void remove_ref() {
            ref_count--;
        }

        public synchronized boolean isStillUsed() {
            return (ref_count == 0 ? false : true);
        }
    }
    


    /**
     * The lock used to synchronize operations done on the client interceptors, namely:
     * registration, unregistration, incrementing and decrementing usage counting 
     */
    final private Lock clientLock = new ReentrantLock();

    /**
     * A Condition variable used to block the unregistration operation of client interceptors
     * until they are no more involved in a request processing  
     */
    final private Condition clientStillUsed = clientLock.newCondition();

    /**
     * The lock used to synchronize operations done on the server interceptors, namely:
     * registration, unregistration, incrementing and decrementing usage counting 
     */
    final private Lock serverLock = new ReentrantLock();

    /**
     * A Condition variable used to block the unregistration operation of server interceptors
     * until they are no more involved in a request processing  
     */
    final private Condition serverStillUsed = serverLock.newCondition();
    
    
    
    final private Lock stubLock = new ReentrantLock();
    
    final private Condition stubStillUsed = stubLock.newCondition();
    
    final private Lock servantLock = new ReentrantLock();
    
    final private Condition servantStillUsed = servantLock.newCondition();
    
    /**
     * The Custom SlotsItemSeq Slot id that has to be provided to the registered interceptors
     */
    private int clientSlotId,
                serverSlotId;
    

    public COPIRegistration(int client_slot_id, int server_slot_id) {
        CDMW_LOG_FUNCTION(logger, "");
        serverCOPIMap = new Hashtable<String, TreeMap<Cookie, ServerContainerInterceptor>>();//LinkedHashMap<Cookie, ServerContainerInterceptor>();
        clientCOPIMap = new Hashtable<String, TreeMap<Cookie, ClientContainerInterceptor>>();//new LinkedHashMap<Cookie, ClientContainerInterceptor>();
        stubCOPIMap = new Hashtable<String, TreeMap<Cookie, StubContainerInterceptor>>();//new LinkedHashMap<Cookie, StubContainerInterceptor>();
        servantCOPIMap = new Hashtable<String, TreeMap<Cookie, ServantContainerInterceptor>>();//new LinkedHashMap<Cookie, ServantContainerInterceptor>();

        clientCOPIRefCount = new Hashtable<ClientContainerInterceptor, RefCountedContainerInterceptor>();
        serverCOPIRefCount = new Hashtable<ServerContainerInterceptor, RefCountedContainerInterceptor>();
        stubCOPIRefCount = new Hashtable<StubContainerInterceptor, RefCountedContainerInterceptor>();
        servantCOPIRefCount = new Hashtable<ServantContainerInterceptor, RefCountedContainerInterceptor>();
        
        this.clientSlotId = client_slot_id;
        this.serverSlotId = server_slot_id;
        CDMW_LOG_FUNCTION_RETURN(logger);
    }


    public Cookie register_client_interceptor(
            ClientContainerInterceptor clientCOPI) {
        CDMW_LOG_FUNCTION(logger, "");
        Cookie ck = null;

        clientLock.lock();
        try {
            clientCOPI.set_slot_id(this.clientSlotId);
            ck = this.register_container_interceptor(clientCOPIMap, clientCOPI);
            // initialize the ref counting of the interceptor
            clientCOPIRefCount.put(clientCOPI,new RefCountedContainerInterceptor(ck));
        } finally {
            clientLock.unlock();
        }
        CDMW_LOG_FUNCTION_RETURN(logger);
        return ck;
    }

    public ClientContainerInterceptor unregister_client_interceptor(Cookie ck)
            throws InvalidRegistration {
        CDMW_LOG_FUNCTION(logger, "");
        ClientContainerInterceptor removed = null;

        COPICookieImpl ckImpl = (COPICookieImpl)ck;
        String key = ckImpl.port + "/" + ckImpl.operation;
        
        if (!clientCOPIMap.containsKey(key) || !((TreeMap<Cookie, ClientContainerInterceptor>)clientCOPIMap.get(key)).containsKey(ck))
            throw new InvalidRegistration(
                    "Unable to find Client COPI associated to this cookie");

        clientLock.lock();
        try {
            RefCountedContainerInterceptor ref_interceptor = null;
            for (RefCountedContainerInterceptor ref : clientCOPIRefCount.values())
                if (ref.cookie.equals(ck)) {
                    ref_interceptor = ref;
                    break;
                }

            while (ref_interceptor.isStillUsed()) {
                clientStillUsed.await();
            }

            removed = (ClientContainerInterceptor) this
                    .unregister_container_interceptor(clientCOPIMap, ck);
            clientCOPIRefCount.remove(removed);

        } catch (InvalidRegistration ir) {
            throw new InvalidRegistration(
                    "Unable to find Client COPI associated to this cookie");
        } catch (InterruptedException e) {
            e.printStackTrace();
        } finally {
            clientLock.unlock();
        }
        CDMW_LOG_FUNCTION_RETURN(logger);
        return removed;
    }

    public Cookie register_server_interceptor(
            ServerContainerInterceptor serverCOPI) {
        CDMW_LOG_FUNCTION(logger, "");
        Cookie ck = null;

        serverLock.lock();
        try {
            serverCOPI.set_slot_id(this.serverSlotId);
            ck = this.register_container_interceptor(serverCOPIMap, serverCOPI);
            // initialize the ref counting of the interceptor
            serverCOPIRefCount.put(serverCOPI, new RefCountedContainerInterceptor(ck));
        } finally {
            serverLock.unlock();
        }
        CDMW_LOG_FUNCTION_RETURN(logger);
        return ck;
    }

    public ServerContainerInterceptor unregister_server_interceptor(Cookie ck)
            throws InvalidRegistration {
        CDMW_LOG_FUNCTION(logger, "");
        ServerContainerInterceptor removed = null;
        
        COPICookieImpl ckImpl = (COPICookieImpl)ck;
        String key = ckImpl.port + "/" + ckImpl.operation;
        
        if (!serverCOPIMap.containsKey(key) || !((TreeMap<Cookie, ServerContainerInterceptor>)serverCOPIMap.get(key)).containsKey(ck))
            throw new InvalidRegistration(
                    "Unable to find Server COPI associated to this cookie");

        serverLock.lock();
        try {
            RefCountedContainerInterceptor ref_interceptor = null;
            for (RefCountedContainerInterceptor ref : serverCOPIRefCount.values())
                if (ref.cookie.equals(ck)) {
                    ref_interceptor = ref;
                    continue;
                }

            while (ref_interceptor.isStillUsed()) {
                serverStillUsed.await();
            }

            removed = (ServerContainerInterceptor) this
                    .unregister_container_interceptor(serverCOPIMap, ck);
            serverCOPIRefCount.remove(removed);

        } catch (InvalidRegistration ir) {
            throw new InvalidRegistration(
                    "Unable to find Server COPI associated to this cookie");
        } catch (InterruptedException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } finally {
            serverLock.unlock();
        }
        CDMW_LOG_FUNCTION_RETURN(logger);
        return removed;
    }

    public Cookie register_stub_interceptor(StubContainerInterceptor stubCOPI) {
        CDMW_LOG_FUNCTION(logger, "");
        Cookie ck = null;

        stubLock.lock();
        try {
            stubCOPI.set_slot_id(this.clientSlotId);
            ck = this.register_container_interceptor(stubCOPIMap, stubCOPI);
            // initialize the ref counting of the interceptor
            stubCOPIRefCount.put(stubCOPI, new RefCountedContainerInterceptor(ck));
        } finally {
            stubLock.unlock();
        }

        CDMW_LOG_FUNCTION_RETURN(logger);
        return ck;
    }

    public StubContainerInterceptor unregister_stub_interceptor(Cookie ck)
            throws InvalidRegistration {
        CDMW_LOG_FUNCTION(logger, "");
        StubContainerInterceptor removed = null;
        
        COPICookieImpl ckImpl = (COPICookieImpl)ck;
        String key = ckImpl.port + "/" + ckImpl.operation;
        
        if (!stubCOPIMap.containsKey(key) || !((TreeMap<Cookie, StubContainerInterceptor>)stubCOPIMap.get(key)).containsKey(ck))
            throw new InvalidRegistration(
                    "Unable to find Stub COPI associated to this cookie");
        
        stubLock.lock();
        try {
            RefCountedContainerInterceptor ref_interceptor = null;
            for (RefCountedContainerInterceptor ref : stubCOPIRefCount.values())
                if (ref.cookie.equals(ck)) {
                    ref_interceptor = ref;
                    continue;
                }

            while (ref_interceptor.isStillUsed()) {
                stubStillUsed.await();
            }

            removed = (StubContainerInterceptor) this
                    .unregister_container_interceptor(stubCOPIMap, ck);
            stubCOPIRefCount.remove(removed);

        } catch (InvalidRegistration ir) {
            throw new InvalidRegistration(
                    "Unable to find Stub COPI associated to this cookie");
        } catch (InterruptedException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } finally {
            stubLock.unlock();
        }
        CDMW_LOG_FUNCTION_RETURN(logger);
        return removed;
    }

    public Cookie register_servant_interceptor(ServantContainerInterceptor servantCOPI) {
        CDMW_LOG_FUNCTION(logger, "");
        Cookie ck = null;

        servantLock.lock();
        try {
            servantCOPI.set_slot_id(this.serverSlotId);
            ck = this.register_container_interceptor(servantCOPIMap, servantCOPI);
            // initialize the ref counting of the interceptor
            servantCOPIRefCount.put(servantCOPI, new RefCountedContainerInterceptor(ck));
        } finally {
            servantLock.unlock();
        }

        CDMW_LOG_FUNCTION_RETURN(logger);
        return ck;
    }

    public ServantContainerInterceptor unregister_servant_interceptor(
            Cookie ck) throws InvalidRegistration {
        CDMW_LOG_FUNCTION(logger, "");
        ServantContainerInterceptor removed = null;

        COPICookieImpl ckImpl = (COPICookieImpl)ck;
        String key = ckImpl.port + "/" + ckImpl.operation;
        
        if (!servantCOPIMap.containsKey(key) || !((TreeMap<Cookie, ServantContainerInterceptor>)servantCOPIMap.get(key)).containsKey(ck))
            throw new InvalidRegistration(
                    "Unable to find Servant COPI associated to this cookie");

        servantLock.lock();
        try {
            RefCountedContainerInterceptor ref_interceptor = null;
            for (RefCountedContainerInterceptor ref : servantCOPIRefCount.values())
                if (ref.cookie.equals(ck)) {
                    ref_interceptor = ref;
                    continue;
                }

            while (ref_interceptor.isStillUsed()) {
                servantStillUsed.await();
            }

            removed = (ServantContainerInterceptor) this
                    .unregister_container_interceptor(servantCOPIMap, ck);
            servantCOPIRefCount.remove(removed);

        } catch (InvalidRegistration ir) {
            throw new InvalidRegistration(
                    "Unable to find Servant COPI associated to this cookie");
        } catch (InterruptedException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } finally {
            servantLock.unlock();
        }
        CDMW_LOG_FUNCTION_RETURN(logger);
        return removed;
    }
    
    protected Cookie register_container_interceptor(Hashtable<String, TreeMap<Cookie, ContainerInterceptor>> map,
            ContainerInterceptor interceptor) throws org.omg.CORBA.BAD_PARAM{
        
        Cookie ck = new COPICookieImpl(interceptor);
        String key = "/";
        if(interceptor.registration_info() != null)
            key = interceptor.registration_info().port + "/" + interceptor.registration_info().operation;
        
        if(map.containsKey(key)){
            
            if (map.get(key).containsValue(interceptor)) {
                org.omg.CORBA.BAD_PARAM ex = new org.omg.CORBA.BAD_PARAM("COPI Already Registered !!");
                try{
                  int minor_code = ExceptionMinorCodes.BAD_PARAMCOPIAlreadyExist;
                  String message = ExceptionStrings.BAD_PARAM_reasons[minor_code & ExceptionMinorCodes.MINOR_CODE_MASK];
                  ex = (org.omg.CORBA.BAD_PARAM) ORBSupport.createSystemException("org.omg.CORBA.BAD_PARAM", minor_code, 
                          ExceptionStrings.BAD_PARAM_reasons, CompletionStatus.COMPLETED_YES);
                }
                catch(Exception e){
                    e.printStackTrace();
                }
                
                throw ex;
            } else {
                map.get(key).put(ck, interceptor);
            }
            
        } else {
            TreeMap<Cookie, ContainerInterceptor> copisMap = new TreeMap<Cookie, ContainerInterceptor>();
            copisMap.put(ck, interceptor);
            map.put(key, copisMap);
        }
        return ck;
    }
    

    protected ContainerInterceptor unregister_container_interceptor(
            Hashtable<String, TreeMap<Cookie, ContainerInterceptor>> map, Cookie ck)
            throws InvalidRegistration {

        COPICookieImpl ckImpl = (COPICookieImpl)ck;
        String key = ckImpl.port + "/" + ckImpl.operation;
        
        if (!map.containsKey(key) || !map.get(key).containsKey(ck))
            throw new InvalidRegistration();
        
        ContainerInterceptor copi = map.get(key).remove(ck);
        
        if(map.get(key).isEmpty()) map.remove(key);
        
        return copi; 

    }

    public ClientContainerInterceptor[] getAllClientContainerInterceptors() {
        ClientContainerInterceptor[] copis = new  ClientContainerInterceptor[0];
        try{
            clientLock.lock();
            return (ClientContainerInterceptor[])getAllContainerInterceptors(clientCOPIMap, copis);
        }
        finally{
            clientLock.unlock(); 
        }
    }

    public ServerContainerInterceptor[] getAllServerContainerInterceptors() {
        ServerContainerInterceptor[] copis = new ServerContainerInterceptor[0]; 
        try{
            serverLock.lock();
            return (ServerContainerInterceptor[])getAllContainerInterceptors(serverCOPIMap, copis);
        }
        finally{
            serverLock.unlock(); 
        }
    }
    
    public ServantContainerInterceptor[] getAllServantContainerInterceptors() {
        ServantContainerInterceptor[] copis = new ServantContainerInterceptor[0];
        try{
            servantLock.lock();
            return (ServantContainerInterceptor[])getAllContainerInterceptors(servantCOPIMap, copis);
        }
        finally{
            servantLock.unlock(); 
        }    
    }

    public StubContainerInterceptor[] getAllStubContainerInterceptors() {
        StubContainerInterceptor[] copis = new StubContainerInterceptor[0];
        try{
            stubLock.lock();
            return (StubContainerInterceptor[])getAllContainerInterceptors(stubCOPIMap, copis);
        }
        finally{
            stubLock.unlock(); 
        }        
    }
    
    
    public ClientContainerInterceptor[] getClientInterceptorByIPOrderedByPriority(org.omg.Components.ContainerPortableInterceptor.IntegrationPoint ip){
        CDMW_LOG_FUNCTION(logger, "");
        ClientContainerInterceptor[] copis = new  ClientContainerInterceptor[0];
                                   
        try{
            clientLock.lock();
            copis = (ClientContainerInterceptor[])getContainerPortableInterceptorsByIPOrderedByPriority(clientCOPIMap, ip, copis);
            CDMW_LOG_FUNCTION_RETURN(logger);
            return copis;
            
        }finally{
            clientLock.unlock(); 
        }
    }

    public ServerContainerInterceptor[] getServerInterceptorByIPOrderedByPriority(org.omg.Components.ContainerPortableInterceptor.IntegrationPoint ip){
        CDMW_LOG_FUNCTION(logger, "");
        ServerContainerInterceptor[] copis = new ServerContainerInterceptor[0]; 
        try{
            serverLock.lock();
            copis = (ServerContainerInterceptor[])getContainerPortableInterceptorsByIPOrderedByPriority(serverCOPIMap, ip, copis);
            CDMW_LOG_FUNCTION_RETURN(logger);
            return copis; 
            
        }finally{
            serverLock.unlock(); 
        }
        
    }    
 
    public ServantContainerInterceptor[] getServantInterceptorByIPOrderedByPriority(org.omg.Components.ContainerPortableInterceptor.IntegrationPoint ip){
        CDMW_LOG_FUNCTION(logger, "");
        ServantContainerInterceptor[] copis = new ServantContainerInterceptor[0];
        try{
            servantLock.lock();
            copis = (ServantContainerInterceptor[])getContainerPortableInterceptorsByIPOrderedByPriority(servantCOPIMap, ip, copis);
            CDMW_LOG_FUNCTION_RETURN(logger);
            return copis;
            
        }finally{
            servantLock.unlock(); 
        }
        
    }
    
    public StubContainerInterceptor[] getStubInterceptorByIPOrderedByPriority(org.omg.Components.ContainerPortableInterceptor.IntegrationPoint ip){
        CDMW_LOG_FUNCTION(logger, "");
        StubContainerInterceptor[] copis = new StubContainerInterceptor[0];
        try{
            stubLock.lock();
            copis = (StubContainerInterceptor[])getContainerPortableInterceptorsByIPOrderedByPriority(stubCOPIMap, ip, copis); 
            CDMW_LOG_FUNCTION_RETURN(logger);
            return copis;
            
        }finally{
            stubLock.unlock(); 
        }
        
    }   
    
   public short addClientCOPIRef(ClientContainerInterceptor copi){
        
        return this.addCOPIRef(copi, clientLock, clientCOPIRefCount);
    }

    public short removeClientCOPIRef(ClientContainerInterceptor copi){

        return this.removeCOPIRef(copi, clientLock, clientCOPIRefCount, clientStillUsed);

    }
    
    public short addServerCOPIRef(ServerContainerInterceptor copi) {
        
        return this.addCOPIRef(copi, serverLock, serverCOPIRefCount);
    }

    public short removeServerCOPIRef(ServerContainerInterceptor copi) {

        return this.removeCOPIRef(copi, serverLock, serverCOPIRefCount, serverStillUsed);

    }
    
    public short addServantCOPIRef(ServantContainerInterceptor copi) {
        
        return this.addCOPIRef(copi, servantLock, servantCOPIRefCount);
    }


    public short addStubCOPIRef(StubContainerInterceptor copi) {
        
        return this.addCOPIRef(copi, stubLock, stubCOPIRefCount); 
    }


    public short removeServantCOPIRef(ServantContainerInterceptor copi) {

        return this.removeCOPIRef(copi, servantLock, servantCOPIRefCount, servantStillUsed); 
    }

    public short removeStubCOPIRef(StubContainerInterceptor copi) {
        
        return this.removeCOPIRef(copi, stubLock, stubCOPIRefCount, stubStillUsed);
    }
    
    protected short addCOPIRef(ContainerInterceptor copi, Lock lock, Hashtable copi_ref_list){
        lock.lock();
        try {
            if (!copi_ref_list.containsKey(copi)){
                CDMW_DEBUG(logger,"COPI Not Found. It was probably unregistered !");
                return 1;
            }

            ((RefCountedContainerInterceptor)copi_ref_list.get(copi)).add_ref();
        } finally {
            lock.unlock();
        }
        return 0;
    }

    
    protected short removeCOPIRef(ContainerInterceptor copi, Lock lock, Hashtable copi_ref_list, Condition condition){
        lock.lock();
        try {
            if (!copi_ref_list.containsKey(copi)){
                CDMW_DEBUG(logger,"COPI Not Found. It was probably unregistered !");
                return 1; 
            }
            // decrement the ref count of the COPI
            ((RefCountedContainerInterceptor)copi_ref_list.get(copi)).remove_ref();
            // if not used wake up a waiting thread for unregistration
            if (!((RefCountedContainerInterceptor)copi_ref_list.get(copi)).isStillUsed())
                condition.signal();
        } finally {
            lock.unlock();
        }
        
        return 0;
    }
    
   protected ContainerInterceptor[] getContainerPortableInterceptorsByIPOrderedByPriority(Hashtable copisMap, 
                                           org.omg.Components.ContainerPortableInterceptor.IntegrationPoint ip,
                                           ContainerInterceptor[] outCopis){
       TreeMap map = new TreeMap();
        
        synchronized (copisMap) {
            String key1 = createTheCOPIsMapKey(ip.port, ip.operation);
            if (copisMap.containsKey(key1)){
                map.putAll((TreeMap)copisMap.get(key1));
            }

            // To add all the COPIs intercepting the given operation whatever
            // the port
            String key2 = createTheCOPIsMapKey("", ip.operation);
            if (!key1.equals(key2) && copisMap.containsKey(key2)) {
                map.putAll((TreeMap) copisMap.get(key2));
            }

            // To add all the COPIs intercepting the given port whatever the
            // operation
            key2 = createTheCOPIsMapKey(ip.port, "");
            if (!key1.equals(key2) && copisMap.containsKey(key2)) {
                map.putAll((TreeMap) copisMap.get(key2));
            }

            // To add all the COPIs intercepting any operation of any port (all
            // the operations)
            key2 = createTheCOPIsMapKey("", "");
            if (!key1.equals(key2) && copisMap.containsKey(key2)) {
                map.putAll((TreeMap) copisMap.get(key2));
            }
        }
        //putting all the copis in one map will reorder them
        return ((ContainerInterceptor[]) map.values().toArray(outCopis));
    }
   
   protected ContainerInterceptor[] getAllContainerInterceptors(Hashtable map, ContainerInterceptor[] outCopis) {
       ArrayList<TreeMap> copisMaps = new ArrayList<TreeMap>(map.values());
       ArrayList<ContainerInterceptor> copis = new ArrayList<ContainerInterceptor>();
       
       for(TreeMap copisMap: copisMaps){
           copis.addAll(copisMap.values());
       }
      return (ContainerInterceptor[])copis.toArray(outCopis); 
       
   }
   
    private String createTheCOPIsMapKey(String port, String operation){
        return (port + "/" + operation);
    }


    public short removeComponentLifeCycleCOPIRef(com.thalesgroup.CdmwCcmContainer.ComponentLifeCycleInterceptor lc) {
        throw new RuntimeException("Not implemented");
    }

    public short removeHomeLifeCycleCOPIRef(com.thalesgroup.CdmwCcmContainer.HomeLifeCycleInterceptor lc) {
        throw new RuntimeException("Not implemented");

    }

    public short addComponentLifeCycleCOPIRef(com.thalesgroup.CdmwCcmContainer.ComponentLifeCycleInterceptor lc) {
        throw new RuntimeException("Not implemented");
    }

    public short addHomeLifeCycleCOPIRef(com.thalesgroup.CdmwCcmContainer.HomeLifeCycleInterceptor lc) {
        throw new RuntimeException("Not implemented");

    }

  public com.thalesgroup.CdmwCcmContainer.ComponentLifeCycleInterceptor[]
   getComponentLifeCycleInterceptorByIPOrderedByPriority(org.omg.Components.ContainerPortableInterceptor.IntegrationPoint ip) {
        throw new RuntimeException("Not implemented");
  }

  public com.thalesgroup.CdmwCcmContainer.HomeLifeCycleInterceptor[]
   getHomeLifeCycleInterceptorByIPOrderedByPriority(org.omg.Components.ContainerPortableInterceptor.IntegrationPoint ip) {
        throw new RuntimeException("Not implemented");
  }

    public com.thalesgroup.CdmwCcmContainer.ComponentLifeCycleInterceptor  unregister_component_lifecycle_interceptor(Cookie ck) {
        throw new RuntimeException("Not implemented");
    }

    public com.thalesgroup.CdmwCcmContainer.HomeLifeCycleInterceptor  unregister_home_lifecycle_interceptor(Cookie ck) {
        throw new RuntimeException("Not implemented");
    }


  public Cookie register_component_lifecycle_interceptor(com.thalesgroup.CdmwCcmContainer.ComponentLifeCycleInterceptor lci) {
        throw new RuntimeException("Not implemented");
  }

  public Cookie register_home_lifecycle_interceptor(com.thalesgroup.CdmwCcmContainer.HomeLifeCycleInterceptor lci) {
        throw new RuntimeException("Not implemented");
  }



}
