/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.ccmcopi;

import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.omg.CORBA.Any;
import org.omg.CORBA.BAD_PARAM;
import org.omg.CORBA.LocalObject;
import org.omg.CORBA.OctetSeqHelper;
import org.omg.CORBA.TCKind;
import org.omg.Components.ContainerPortableInterceptor.COPIServiceContext;
import org.omg.Components.ContainerPortableInterceptor.ClientContainerInterceptor;
import org.omg.Components.ContainerPortableInterceptor.ContainerClientRequestInfo;
import org.omg.Components.ContainerPortableInterceptor.CustomSlotItem;
import org.omg.Components.ContainerPortableInterceptor.CustomSlotItemSeqHelper;
import org.omg.Components.ContainerPortableInterceptor.IntegrationPoint;
import org.omg.IOP.ServiceContext;
import org.omg.IOP.CodecPackage.FormatMismatch;
import org.omg.IOP.CodecPackage.InvalidTypeForEncoding;
import org.omg.PortableInterceptor.ClientRequestInfo;
import org.omg.PortableInterceptor.ClientRequestInterceptor;
import org.omg.PortableInterceptor.ForwardRequest;
import org.omg.PortableInterceptor.InvalidSlot;

import cdmw.ccm.common.CCMLoggers;
import cdmw.common.Assert;
import cdmw.common.BadParameterException;

import com.thalesgroup.CdmwCcmCopi.RegisteredCOPICollection;


/**
 * This class is the main client request interceptor that delegates all the
 * request interceptions points calls to the registered basic container client
 * interceptors.
 */

public class ClientRequestInterceptorIterator extends LocalObject implements
        ClientRequestInterceptor {

    private static final String name = "ClientRequestInterceptorIterator";
    
    private static int requestId = 0;

    /**
     * needed slots ids for the COPI service :
     *  1- The client component identity
     *  2- The server component identity
     *  3- The client component port name (receptacle or source)
     *  4- The Custom slots     
     */
    public int Origin_ID_SlotID, Target_ID_SlotID, OriginFeatureName_SlotID,
            CustomSlotItemSeq_SlotID;

    cdmw.logging.Logger logger = CCMLoggers.GetCcmCopiLogger();

    
    private class CopisStackHolder{
        /**
         * A Client COPI enters the RECEIVING state whenever it process one of the
         * receive_* operations successfully.
         * it gives the index (in the client COPI list) of the last Client COPI that
         * processes a receiving end point
         * successfully. All greater indexes are in the RECEIVING state; due to the
         * fact that receiving end points are called in reverse order
         */
        private int RECEIVINGStateInterceptorIndex;

        /**
         * A Client COPI enters the SENDING state whenever it processes one the
         * send_* operations successfully.
         * it gives the index (in the client COPI
         * list) of the last Client COPI that processes send_* successfully. All
         * lesser indexes are in the SENDING state.
         */
        private int SENDINGStateInterceptorIndex;
        
        /**
         * The copis list
         */
        ClientContainerInterceptor[] copisList;
    }
    
    private ThreadLocal<CopisStackHolder> copisStack = new ThreadLocal<CopisStackHolder>(){
        protected CopisStackHolder initialValue() {
            CopisStackHolder stack= new CopisStackHolder();
            stack.SENDINGStateInterceptorIndex = -1;
            stack.RECEIVINGStateInterceptorIndex = -1;
            stack.copisList = new ClientContainerInterceptor[0];
            return stack;
        }
    };

    
    private int RECEIVINGStateInterceptorIndex(){
        return this.copisStack.get().RECEIVINGStateInterceptorIndex;
    }

    private void RECEIVINGStateInterceptorIndex(int idx){
        this.copisStack.get().RECEIVINGStateInterceptorIndex = idx;
    }

    
    private int SENDINGStateInterceptorIndex(){
        return this.copisStack.get().SENDINGStateInterceptorIndex;
    }
    
    private void SENDINGStateInterceptorIndex(int idx){
        this.copisStack.get().SENDINGStateInterceptorIndex = idx;
    }
    
    /**
     * The Client COPIs list
     */
    private ClientContainerInterceptor[] interceptors(){
       return this.copisStack.get().copisList; 
    }
    
    private void interceptors(ClientContainerInterceptor[] copis){
        this.copisStack.get().copisList = new ClientContainerInterceptor[copis.length];
       System.arraycopy(copis, 0, this.copisStack.get().copisList, 0, copis.length); 
    }
    
    /**
     * used to store the thread local context of each request so that it can be retrieved at 
     * the reception points. It contains (request_id, ThreadLocal) mappings. 
     */
    private Map<Integer, ThreadLocal<CopisStackHolder>> ThreadsLocalContext = Collections.
        synchronizedMap(new HashMap<Integer, ThreadLocal<CopisStackHolder>>());

    
    private RegisteredCOPICollection registration;
    
    
    public ClientRequestInterceptorIterator(int originID, int targetID,
            int originFeatureNameID, int customSlotItemSeqID,  RegisteredCOPICollection registration) {
        Origin_ID_SlotID = originID;
        Target_ID_SlotID = targetID;
        OriginFeatureName_SlotID = originFeatureNameID;
        CustomSlotItemSeq_SlotID = customSlotItemSeqID;
        this.registration = registration;
    }
    
    synchronized private static int getRequestId(){
       return requestId++; 
    }
    

    public void receive_exception(ClientRequestInfo info) throws ForwardRequest {
        CDMW_LOG_FUNCTION(logger, "operation["+info.operation()+"]");

        // setting the thread context
        try{
            int reqId = extractReqIdFromReplyServiceContext(info);
            if (ThreadsLocalContext.get(reqId) != null) {
                copisStack = ThreadsLocalContext.get(reqId);
                ThreadsLocalContext.remove(reqId);
            }
        }
        catch(BAD_PARAM bp){  
            CDMW_DEBUG(logger, "No reply service context containing the request Id." +
                    " The request target is not CDMW-compliant server !");
        }
        
        // extracting the target_id from the reply
        byte[] target_id = extractFromReplyServiceContext(info);

        // building the ContainerRequestInfo
        ContainerClientRequestInfo ccri = buildContainerClientRequestInfo(info, target_id);
        ClientContainerInterceptor[] interceptors = interceptors(); 
        int i = 0;
        try {
            for (i = RECEIVINGStateInterceptorIndex() - 1; i < interceptors.length
                    && i >= 0 && i <= SENDINGStateInterceptorIndex(); i--) {

                RECEIVINGStateInterceptorIndex(i);
                interceptors[i].receive_exception(ccri);
                registration.removeClientCOPIRef(interceptors[i]);
            }
        } catch (RuntimeException e) {
            if(i < interceptors.length
                    && i >= 0 && i <= SENDINGStateInterceptorIndex()) registration.removeClientCOPIRef(interceptors[i]);
            CDMW_LOG_FUNCTION_EXCEPTION(logger,e);
            throw e;
        }
        CDMW_LOG_FUNCTION_RETURN(logger);       
    }

    public void receive_other(ClientRequestInfo info) throws ForwardRequest {
        CDMW_LOG_FUNCTION(logger, "operation["+info.operation()+"]");

        // setting the thread context
        try{
            int reqId = extractReqIdFromReplyServiceContext(info);
            if (ThreadsLocalContext.get(reqId) != null) {
                copisStack = ThreadsLocalContext.get(reqId);
                ThreadsLocalContext.remove(reqId);
            }
        }
        catch(BAD_PARAM bp){  
            CDMW_DEBUG(logger, "No reply service context containing the request Id." +
            " The request target is not CDMW-compliant server !");
        }

        // extracting the target_id from the reply
        byte[] target_id = extractFromReplyServiceContext(info);

        // building the ContainerRequestInfo
        ContainerClientRequestInfo ccri = buildContainerClientRequestInfo(info,
                target_id);
        ClientContainerInterceptor[] interceptors = interceptors(); 
        int i = 0;
        try {
            for (i = RECEIVINGStateInterceptorIndex() - 1; i < interceptors.length
                    && i >= 0 && i <= SENDINGStateInterceptorIndex(); i--) {

                RECEIVINGStateInterceptorIndex(i);
                interceptors[i].receive_other(ccri);
                registration.removeClientCOPIRef(interceptors[i]);
            }
        } catch (RuntimeException e) {
            if(i < interceptors.length
                    && i >= 0 && i <= SENDINGStateInterceptorIndex()) registration.removeClientCOPIRef(interceptors[i]);
            CDMW_LOG_FUNCTION_EXCEPTION(logger,e);
            throw e;
        }
        CDMW_LOG_FUNCTION_RETURN(logger);       
    }

    public void receive_reply(ClientRequestInfo info) {
        CDMW_LOG_FUNCTION(logger, "operation["+info.operation()+"]");

        // setting the thread context
        try{
            int reqId = extractReqIdFromReplyServiceContext(info);
            copisStack = ThreadsLocalContext.get(reqId);
            Assert.check(ThreadsLocalContext.remove(reqId) != null); 
        }
        catch(BAD_PARAM bp){  
            CDMW_DEBUG(logger, "No reply service context containing the request Id." +
            " The request target is not CDMW-compliant server !");
        }
        

        // extracting the target_id from the reply
        byte[] target_id = extractFromReplyServiceContext(info);

        // building the ContainerRequestInfo
        ContainerClientRequestInfo ccri = buildContainerClientRequestInfo(info,
                target_id);
        ClientContainerInterceptor[] interceptors = interceptors(); 
        int i = 0;
        
        try {
            for (i = SENDINGStateInterceptorIndex(); i < interceptors.length

                    && i >= 0; i--) {

                RECEIVINGStateInterceptorIndex(i);
                interceptors[i].receive_reply(ccri);
                registration.removeClientCOPIRef(interceptors[i]);
            }
        } 
        catch(RuntimeException e) {
            if(i < interceptors.length && i >= 0) registration.removeClientCOPIRef(interceptors[i]);
            CDMW_LOG_FUNCTION_EXCEPTION(logger,e);
            throw e;
        }
        CDMW_LOG_FUNCTION_RETURN(logger);
    }


    public void send_poll(ClientRequestInfo info) {
        CDMW_LOG_FUNCTION(logger, "operation["+info.operation()+"]");

        // building the ContainerRequestInfo
        ContainerClientRequestInfo ccri = buildContainerClientRequestInfo(info);
        CDMW_INFO(logger,
                "Component "
                        + (ccri.origin_id() != null ? new String(ccri
                                .origin_id()) : "")
                        + " has sent a request to component "
                        + (ccri.target_id() != null ? new String(ccri
                                .target_id()) : "null") + " on its port "
                        + (ccri.name() != null ? ccri.name() : ""));
        
        this.copisStack.remove();
        
        ClientContainerInterceptor[] interceptors = registration
                .getClientInterceptorByIPOrderedByPriority(new IntegrationPoint(ccri.name(), info.operation())); 
    
        // calling the registered interceptors
        try {
            for (int i = 0; i < interceptors.length; i++) {
                if(registration.addClientCOPIRef(interceptors[i]) != 0) {
                    interceptors = COPIUtils.removeArrayElement(interceptors, i);
                    i--;
                    continue;
                }
                interceptors[i].send_poll(ccri);
                SENDINGStateInterceptorIndex(i);
            }
        } finally {

            if (interceptors().length != 0){
                // No interceptor is in the RECEIVING state yet
                RECEIVINGStateInterceptorIndex(SENDINGStateInterceptorIndex() + 1);
                interceptors(interceptors);
            }

            // building the copi service context
            int reqId = getRequestId();
            buildRequestServiceContext(info, reqId);
            
            //store the thread context
            ThreadsLocalContext.put(reqId, copisStack);
            CDMW_LOG_FUNCTION_RETURN(logger);
        }
    }

    public void send_request(ClientRequestInfo info) throws ForwardRequest {
        CDMW_LOG_FUNCTION(logger, "operation["+info.operation()+"]");

        // building the ContainerRequestInfo
        ContainerClientRequestInfo ccri = buildContainerClientRequestInfo(info);
        CDMW_INFO(logger,
                "Component "
                        + (ccri.origin_id() != null ? new String(ccri
                                .origin_id()) : "")
                        + " has sent a request to component "
                        + (ccri.target_id() != null ? new String(ccri
                                .target_id()) : "null") + " on its port "
                        + (ccri.name() != null ? ccri.name() : ""));
        
        this.copisStack.remove();
        ClientContainerInterceptor[] interceptors = registration
                .getClientInterceptorByIPOrderedByPriority(new IntegrationPoint(ccri.name(), info.operation())); 
       
        
        // calling the registered interceptors
        try {
            for (int i = 0; i < interceptors.length; i++) {
                if(registration.addClientCOPIRef(interceptors[i]) != 0) {
                    interceptors = COPIUtils.removeArrayElement(interceptors, i);
                    //System.arraycopy(interceptors, i+1, interceptors, i, interceptors.length-i);
                    i--;
                    continue;
                }
                interceptors[i].send_request(ccri);
                SENDINGStateInterceptorIndex(i);
            }
        } finally {

            if (interceptors.length != 0){
                // No interceptor is in the RECEIVING state yet
                RECEIVINGStateInterceptorIndex(SENDINGStateInterceptorIndex() + 1);
                interceptors(interceptors);
            }

            // building the copi service context
            int reqId = getRequestId();
            buildRequestServiceContext(info, reqId);
            
            //store the thread context
            ThreadsLocalContext.put(reqId, copisStack);
            CDMW_LOG_FUNCTION_RETURN(logger);
        }
    }

    

    public void destroy() {
        // TODO Auto-generated method stub

    }

    public String name() {
        return name;
    }

    private ContainerClientRequestInfo buildContainerClientRequestInfo(
            ClientRequestInfo info) {
        ContainerClientRequestInfo ccri = null;
        byte[] origin_id = null, target_id = null;
        String feature_name = null;

        try {
            Any any = info.get_slot(Origin_ID_SlotID);
            if (!any.type().kind().equals(TCKind.tk_null))
                origin_id = OctetSeqHelper.extract(any);

            any = info.get_slot(Target_ID_SlotID);
            if (any != null && !any.type().kind().equals(TCKind.tk_null))
                target_id = OctetSeqHelper.extract(any);

            any = info.get_slot(OriginFeatureName_SlotID);
            if (any != null && !any.type().kind().equals(TCKind.tk_null))
                feature_name = any.extract_string();

            ccri = new ContainerClientRequestInfoImpl(feature_name, origin_id,
                    target_id, info);
        } catch (InvalidSlot is) {
            CDMW_ERROR(logger, "Failed to get the slots");
            is.printStackTrace();
        }

        return ccri;
    }
    
    private ContainerClientRequestInfo buildContainerClientRequestInfo(
            ClientRequestInfo info, byte[] targetId) {
        ContainerClientRequestInfo ccri = null;
        byte[] origin_id = null, target_id = null;
        String feature_name = null;

        try {
            Any any = info.get_slot(Origin_ID_SlotID);
            if (!any.type().kind().equals(TCKind.tk_null))
                origin_id = OctetSeqHelper.extract(any);

            any = info.get_slot(OriginFeatureName_SlotID);
            if (any != null && !any.type().kind().equals(TCKind.tk_null))
                feature_name = any.extract_string();

            any = info.get_slot(Target_ID_SlotID);
            if (any != null && !any.type().kind().equals(TCKind.tk_null))
                {
                    target_id = OctetSeqHelper.extract(any);
                    if(!target_id.equals(targetId)){
                        String local_t_id = (target_id != null? new String(target_id):"null"),
                        received_t_id = (targetId != null? new String(targetId):"null");
                    
                        CDMW_WARN(logger, "the local target component id ("+ local_t_id + ") is different than " +
                                        "that sent by the target component itself ("+ received_t_id +") !");
                    }
                }
            
            ccri = new ContainerClientRequestInfoImpl(feature_name, origin_id,
                    targetId, info);
        } catch (InvalidSlot is) {
            CDMW_ERROR(logger, "Failed to get the slots");
            is.printStackTrace();
        }

        return ccri;
    }

    private void buildRequestServiceContext(ClientRequestInfo info) {
        // CustomSlotItem[] csi = null;
        CustomSlotItem[] csi = new CustomSlotItem[0];

        try {

            Any any = info.get_slot(CustomSlotItemSeq_SlotID);
            if (any != null && !any.type().kind().equals(TCKind.tk_null))
                csi = CustomSlotItemSeqHelper.extract(any);

            // building the request service context
            byte[] origin_id = new byte[0], target_id = new byte[0];

            any = info.get_slot(Origin_ID_SlotID);
            if (any != null && !any.type().kind().equals(TCKind.tk_null))
                origin_id = OctetSeqHelper.extract(any);

            any = info.get_slot(Target_ID_SlotID);
            if (any != null && !any.type().kind().equals(TCKind.tk_null))
                target_id = OctetSeqHelper.extract(any);

            COPIServiceContext copi_ctx_to_send = new COPIServiceContext(
                    origin_id, target_id, csi);

            ServiceContext svc_ctx_to_send = COPIUtils
                    .copiToServiceContext(copi_ctx_to_send);
            
            // setting the copi service context
            info.add_request_service_context(svc_ctx_to_send, true);

        } catch (InvalidSlot is) {
            CDMW_ERROR(logger, "Failed to get the slots");
            is.printStackTrace();
        } catch (InvalidTypeForEncoding ite) {
            CDMW_ERROR(CCMLoggers.GetCcmCopiLogger(),
                    "Failed to encode the COPI service Context");
        }

    }
    
    private void buildRequestServiceContext(ClientRequestInfo info, int requestId) {
        this.buildRequestServiceContext(info);
        ServiceContext svcCtx = new ServiceContext(COPIServiceInit.CDMW_CLIENT_COPI_SERVICE_CONTEXT_ID,
                        String.valueOf(requestId).getBytes());
        info.add_request_service_context(svcCtx, true);
    }
    
    
    /**
     * extract the target_id from the reply service context
     * @param info
     * @return target_id
     */
    private byte[] extractFromReplyServiceContext(ClientRequestInfo info) {

        byte[] target_id = null;
        
        try {
            ServiceContext svc_ctx_received = info
                    .get_reply_service_context(COPIServiceInit.CDMW_COPI_SERVICE_CONTEXT_ID);
            COPIServiceContext copi_ctx_received = COPIUtils
                    .serviceToCopiContext(svc_ctx_received);
            return copi_ctx_received.target_id;

        } catch (BAD_PARAM bp) {
            // No COPI service context found !! TODO: to manage
            CDMW_WARN(logger, "No COPI Reply Service Context found !!");
        } catch (FormatMismatch fm) {
            // TODO : 
            CDMW_WARN(logger, "Failed to decode the reply service context : "+ fm.getMessage());
            fm.printStackTrace();
        } catch (BadParameterException e) {
            // TODO Auto-generated catch block
            CDMW_WARN(logger, "Failed to decode the reply service context : "+ e.getMessage());
            e.printStackTrace();
        }
        
        return target_id;

    }
    
    private int extractReqIdFromReplyServiceContext(ClientRequestInfo info) {
        int id = 0;
        ServiceContext svcCtx = info
                .get_reply_service_context(COPIServiceInit.CDMW_CLIENT_COPI_SERVICE_CONTEXT_ID);
        id = Integer.valueOf(new String(svcCtx.context_data)).intValue();
        return id;
    }
    

}
