/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.ccmcopi;

import static cdmw.logging.Logger.CDMW_DEBUG;
import static cdmw.logging.Logger.CDMW_ERROR;
import static cdmw.logging.Logger.CDMW_INFO;
import static cdmw.logging.Logger.CDMW_WARN;

import org.omg.CORBA.Any;
import org.omg.CORBA.BAD_PARAM;
import org.omg.CORBA.LocalObject;
import org.omg.CORBA.OctetSeqHelper;
import org.omg.CORBA.TCKind;
import org.omg.Components.ContainerPortableInterceptor.COPIServiceContext;
import org.omg.Components.ContainerPortableInterceptor.ContainerServerRequestInfo;
import org.omg.Components.ContainerPortableInterceptor.CustomSlotItem;
import org.omg.Components.ContainerPortableInterceptor.CustomSlotItemSeqHelper;
import org.omg.Components.ContainerPortableInterceptor.IntegrationPoint;
import org.omg.Components.ContainerPortableInterceptor.ServerContainerInterceptor;
import org.omg.IOP.ServiceContext;
import org.omg.IOP.CodecPackage.FormatMismatch;
import org.omg.IOP.CodecPackage.InvalidTypeForEncoding;
import org.omg.PortableInterceptor.ForwardRequest;
import org.omg.PortableInterceptor.InvalidSlot;
import org.omg.PortableInterceptor.ServerRequestInfo;
import org.omg.PortableInterceptor.ServerRequestInterceptor;

import cdmw.ccm.cif.CCMUtils;
import cdmw.ccm.common.CCMLoggers;
import cdmw.common.BadParameterException;

import com.thalesgroup.CdmwCcmCopi.RegisteredCOPICollection;

/**
 * This class is the main server request interceptor that delegates all the
 * request interceptions points calls to the registered basic container server
 * interceptors.
 */

// TODO: managing dynamic registration and unregistration of COPIs
public class ServerRequestInterceptorIerator extends LocalObject implements
        ServerRequestInterceptor {

    private static final String name = "ServerRequestInterceptorIerator";
    
    
    /**
     * needed slots ids for the COPI service :
     *  1- The client component identity
     *  2- The server component identity
     *  3- The server component port name (facet or sink)
     *  4- the server custom slots    
     */   
    public int Origin_ID_SlotID, 
                Target_ID_SlotID, 
                TargetFeatureName_SlotID,
                CustomSlotItemSeq_SlotID;

    cdmw.logging.Logger logger = CCMLoggers.GetCcmCopiLogger();
    
    private class CopisStackHolder {
        /**
         * A Server COPI enters the RECEIVING state whenever it processes the
         * receive_request_service_contexts operation successfully. it gives the
         * index (in the server COPI list) of the last Server COPI that
         * processes receive_request_service_contexts successfully. They are in
         * the RECEIVING state
         */
        private int RECEIVINGStateInterceptorIndex;

        /**
         * A Server COPI enters the SENDING state whenever it process one of the
         * send_* operations successfully. it gives the index (in the server
         * COPI list) of the last Server COPI that has already processed a
         * sending end point. All greater indexes are in the SENDING state; due
         * to the fact that sending end points are called in reverse order
         */
        private int SENDINGStateInterceptorIndex;

        /**
         * The Server COPIs list
         */
        ServerContainerInterceptor[] interceptors;
    }

    /**
     * Thread local data
     */
    private ThreadLocal<CopisStackHolder> copisStack = new ThreadLocal<CopisStackHolder>(){
        protected CopisStackHolder initialValue() {
            CopisStackHolder stack= new CopisStackHolder();
            stack.RECEIVINGStateInterceptorIndex = -1;
            stack.SENDINGStateInterceptorIndex = -1;
            stack.interceptors = new ServerContainerInterceptor[0];
            return stack;
        }
    };

    private RegisteredCOPICollection registration;
    
    public ServerRequestInterceptorIerator(int originID, int targetID,
            int originFeatureNameID, int customSlotItemSeqID, RegisteredCOPICollection registration) {
        Origin_ID_SlotID = originID;
        Target_ID_SlotID = targetID;
        TargetFeatureName_SlotID = originFeatureNameID;
        CustomSlotItemSeq_SlotID = customSlotItemSeqID;
        this.registration = registration;
    }
    
    private int RECEIVINGStateInterceptorIndex(){
        return this.copisStack.get().RECEIVINGStateInterceptorIndex;
    }

    private void RECEIVINGStateInterceptorIndex(int idx){
        this.copisStack.get().RECEIVINGStateInterceptorIndex = idx;
    }
    
    private int SENDINGStateInterceptorIndex(){
        return this.copisStack.get().SENDINGStateInterceptorIndex;
    }
    
    private void SENDINGStateInterceptorIndex(int idx) {
        this.copisStack.get().SENDINGStateInterceptorIndex = idx;
    }

    private ServerContainerInterceptor[] interceptors() {
        return this.copisStack.get().interceptors;
    }

    private void interceptors(ServerContainerInterceptor[] copis) {
        this.copisStack.get().interceptors = new ServerContainerInterceptor[copis.length];
        System.arraycopy(copis, 0, this.copisStack.get().interceptors, 0,
                copis.length);
    }
    
    public void receive_request(ServerRequestInfo info) throws ForwardRequest {
        CDMW_INFO(logger, "receive_request called ...");

        // retrieving the feature name (object_id available here)
        extractFeatureName(info);
        
        ContainerServerRequestInfo csri = buildContainerServerRequestInfo(info);

        CDMW_INFO(logger, "Component "
                + (csri.target_id() != null ? new String(csri.target_id())
                        : "")
                + " has received a request from component "
                + (csri.origin_id() != null ? new String(csri.origin_id())
                        : "") + " on its port " + csri.name());
        
        ServerContainerInterceptor[] interceptors = interceptors();
        for (int i = 0; i <= RECEIVINGStateInterceptorIndex(); i++) {
                interceptors[i].receive_request(csri);
        }

    }


    /**
     * 
     */
    public void receive_request_service_contexts(ServerRequestInfo info)
            throws ForwardRequest {
        CDMW_INFO(logger, " receive_request_service_contexts called ...");

        copisStack.remove();

        // populating the RSC from the received service context
        extractFromRequestServiceContext(info);
        ContainerServerRequestInfo csri = buildContainerServerRequestInfo(info);
        ServerContainerInterceptor[] interceptors = registration
                .getServerInterceptorByIPOrderedByPriority(new IntegrationPoint(
                        csri.name(), info.operation()));
        
        try {
            for (int i = 0; i < interceptors.length; i++) {
                if (registration.addServerCOPIRef(interceptors[i]) != 0) {
                    interceptors = COPIUtils.removeArrayElement(interceptors, i);
                    i--;
                    continue;
                }
                interceptors[i].receive_request_service_contexts(csri);
                RECEIVINGStateInterceptorIndex(i);
            }
        } finally {
            
            if (interceptors.length != 0) {
             // Greater indexes are those of SENDING interceptors. At this point,
                // no interceptor is SENDING so the SENDINGStateInterceptorIndex
                // should be
                // initialized with the greatest index value of the list
                SENDINGStateInterceptorIndex(RECEIVINGStateInterceptorIndex() + 1);
                interceptors(interceptors);
            }

        }
    }

    public void send_exception(ServerRequestInfo info) throws ForwardRequest {
        CDMW_INFO(logger, " send_exception called ...");

        ContainerServerRequestInfo csri = buildContainerServerRequestInfo(info);

        ServerContainerInterceptor[] interceptors = interceptors();
        int i = 0;
        try {
            for (i = SENDINGStateInterceptorIndex() - 1; i < interceptors.length
                    && i >= 0 && i <= RECEIVINGStateInterceptorIndex(); i--) {

                SENDINGStateInterceptorIndex(i);
                interceptors[i].send_exception(csri);
                registration.removeServerCOPIRef(interceptors[i]);
            }
        } finally {
            if(i < interceptors.length && i >= 0) registration.removeServerCOPIRef(interceptors[i]);
            // copy the RSC to the reply service context
            buildReplyServiceContext(info);
        }
    }

    public void send_other(ServerRequestInfo info) throws ForwardRequest {
        CDMW_INFO(logger, " send_other called ...");

        ContainerServerRequestInfo csri = buildContainerServerRequestInfo(info);
        ServerContainerInterceptor[] interceptors = interceptors();
        int i = 0;
        try {
            
            for (i = SENDINGStateInterceptorIndex() - 1; i < interceptors.length && i >= 0
                    && i <= RECEIVINGStateInterceptorIndex() ; i--) {
                
                SENDINGStateInterceptorIndex(i);
                interceptors[i].send_other(csri);
                registration.removeServerCOPIRef(interceptors[i]);

            }
        } finally {
            if(i < interceptors.length && i >= 0) registration.removeServerCOPIRef(interceptors[i]);
            // copy the RSC to the reply service context
            buildReplyServiceContext(info);
        }

    }

    public void send_reply(ServerRequestInfo info) {
        CDMW_INFO(logger, " send_reply called ...");
        
        ContainerServerRequestInfo csri = buildContainerServerRequestInfo(info);
        ServerContainerInterceptor[] interceptors = interceptors();
        int i = 0;
        try {

            for (i = RECEIVINGStateInterceptorIndex(); i < interceptors.length && i >= 0; i--) {
                
                SENDINGStateInterceptorIndex(i);
                interceptors[i].send_reply(csri);
                registration.removeServerCOPIRef(interceptors[i]);
            }
        } finally {
            if(i < interceptors.length && i >= 0) 
                registration.removeServerCOPIRef(interceptors[i]);
            // copy the RSC to the reply service context
            buildReplyServiceContext(info);
        }

    }

    public void destroy() {
        // TODO Auto-generated method stub

    }

    public String name() {
        return name;
    }

    private ContainerServerRequestInfo buildContainerServerRequestInfo(
            ServerRequestInfo info) {
        ContainerServerRequestInfo csri = null;
        byte[] origin_id = null, target_id = null;
        String feature_name = null;

        try {

            Any any = info.get_slot(Origin_ID_SlotID);
            if (any != null && !any.type().kind().equals(TCKind.tk_null))
                origin_id = OctetSeqHelper.extract(any);

            any = info.get_slot(Target_ID_SlotID);
            if (any != null && !any.type().kind().equals(TCKind.tk_null))
                target_id = OctetSeqHelper.extract(any);

            any = info.get_slot(TargetFeatureName_SlotID);
            if (any != null && !any.type().kind().equals(TCKind.tk_null))
                feature_name = any.extract_string();

        } catch (InvalidSlot e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        csri = new ContainerServerRequestInfoImpl(feature_name, origin_id,
                target_id, info);
        return csri;
    }

    /**
     * Copy the Service Context to the Request Scope Context
     */
    private void extractFromRequestServiceContext(ServerRequestInfo info) {

        try {
            ServiceContext svc_ctx_received = info
                    .get_request_service_context(COPIServiceInit.CDMW_COPI_SERVICE_CONTEXT_ID);
            COPIServiceContext copi_ctx_received = COPIUtils
                    .serviceToCopiContext(svc_ctx_received);

            Any any = COPIUtils.createAny();
            if(copi_ctx_received.origin_id != null) 
                OctetSeqHelper.insert(any, copi_ctx_received.origin_id);
            info.set_slot(Origin_ID_SlotID, any);
            any = COPIUtils.createAny();
            if(copi_ctx_received.target_id != null) 
                OctetSeqHelper.insert(any, copi_ctx_received.target_id);
            info.set_slot(Target_ID_SlotID, any);
            any = COPIUtils.createAny();
            if(copi_ctx_received.slot_info != null) 
                CustomSlotItemSeqHelper.insert(any, copi_ctx_received.slot_info);
            info.set_slot(CustomSlotItemSeq_SlotID, any);
            
        } catch (BAD_PARAM bp) {
            // No COPI service context found !! TODO: to manage
            CDMW_WARN(logger, "No COPI service context found !!");
        } catch (FormatMismatch fm) {
            // TODO
            fm.printStackTrace();
        } catch (InvalidSlot e) {
            // TODO
            e.printStackTrace();
        } catch (BadParameterException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        
        //extract the request Id and put it into the reply service context
        try{
            ServiceContext svc_ctx_received = info.get_request_service_context(COPIServiceInit.CDMW_CLIENT_COPI_SERVICE_CONTEXT_ID);
            if(svc_ctx_received != null){
                CDMW_DEBUG(logger, "Adding the request Id into the reply service context");
                info.add_reply_service_context(svc_ctx_received, true);
            } 
        }catch(BAD_PARAM bp){
          CDMW_DEBUG (logger, "No request Id service context found !!");  
        }

    }

    /**
     * builds the reply service context
     * 
     * @param info
     */
    private void buildReplyServiceContext(ServerRequestInfo info) {

        CustomSlotItem[] csi = new CustomSlotItem[0];

        try {
            Any any = info.get_slot(CustomSlotItemSeq_SlotID);
            if (any != null && !any.type().kind().equals(TCKind.tk_null))
                csi = CustomSlotItemSeqHelper.extract(any);

            // building the service context
            byte[] origin_id = new byte[0], target_id = new byte[0];

            any = info.get_slot(Origin_ID_SlotID);
            if (any != null && !any.type().kind().equals(TCKind.tk_null))
                origin_id = OctetSeqHelper.extract(any);

            any = info.get_slot(Target_ID_SlotID);
            if (any != null && !any.type().kind().equals(TCKind.tk_null))
                target_id = OctetSeqHelper.extract(any);

            COPIServiceContext copi_ctx_to_send = new COPIServiceContext(
                    origin_id, target_id, csi);

            ServiceContext svc_ctx_to_send = COPIUtils
                    .copiToServiceContext(copi_ctx_to_send);

            // setting the copi reply service context
            info.add_reply_service_context(svc_ctx_to_send, true);

        } catch (InvalidSlot is) {
            CDMW_ERROR(logger, "Failed to get the slots");
            is.printStackTrace();
        } catch (InvalidTypeForEncoding ite) {
            CDMW_ERROR(CCMLoggers.GetCcmCopiLogger(),
                    "Failed to encode the COPI service Context");
        }

    }
    
    
    private void extractFeatureName(ServerRequestInfo info) {
        
        // getting the server-side port name
        String feature_name = null;
        java.util.Vector names = CCMUtils.readOid(new String(info
                .object_id()));
        switch (names.size()) {
        case 1: // oid is a component oid
            feature_name = "";
            break;

        case 2: // oid is a facet/consumer oid
            feature_name = (String) names.elementAt(1);
            break;

        case 3: // oid is a local push consumer"
            feature_name = (String) names.elementAt(1);
            break;
        default:
            feature_name = "NA"; //Not Available
        }

        Any any = COPIUtils.createAny();
        any.insert_string(feature_name);
        
        try {
            info.set_slot(TargetFeatureName_SlotID, any);
        } catch (InvalidSlot e) {
            CDMW_ERROR(logger, "Failed to get the slots");
            e.printStackTrace();
        }
        
    }

}
