/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ccm.cif;

import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;

import org.omg.Components.CCMException;
import org.omg.Components.CCMExceptionReason;


/**
 * This class implement the org.omg.Components.CCMContext interface.
 * It will be inherited by all components context implementation.
 *
 */
public class CCMContextImpl
    extends org.omg.CORBA.LocalObject
    implements org.omg.Components.CCMContext {    
    
    /*
     * logger to be used in CCMContext
     */
    protected com.thalesgroup.CdmwLogging.Logger logger;
                                                               
    /*
     * container context
     */
    private org.omg.Components.CCMContext context;
    
    /*
     * component identifier of the component this context belongs to
     */
    private String componentId;
    
    /*
     * List of forbidden service identifiers
     */
    protected String[] getUnavailableServices(){
        return new String[] {"RootPOA", 
                                "POACurrent", 
                                "ORBPolicyManager",
                                "PolicyCurrent",
                                "PICurrent"};
    }
     

    public CCMContextImpl(org.omg.Components.CCMContext ctx,
                             String componentId, com.thalesgroup.CdmwLogging.LoggerFactory loggerFactory) 
    {
        if(componentId == null) throw new IllegalArgumentException();
            
        // create logger with domain for ccmcif
        this.logger = loggerFactory.get_logger(
            cdmw.ccm.common.CCMLoggers.CCMCIF_LOG_DOMAIN);
            
        CDMW_LOG_FUNCTION(logger, "");
            
        this.context = ctx;
        this.componentId = componentId;
    }

    //
    // IDL:omg.org/Components/CCMContext/get_caller_principal:1.0
    //
    public org.omg.SecurityLevel2.Credentials get_caller_principal() 
    {
        CDMW_LOG_FUNCTION(logger, "");
        return this.context.get_caller_principal();
    }

    //
    // IDL:omg.org/Components/CCMContext/get_CCM_home:1.0
    //
    public org.omg.Components.CCMHome get_CCM_home() 
    {
        CDMW_LOG_FUNCTION(logger, "");
        return this.context.get_CCM_home();
    }

    //
    // IDL:omg.org/Components/CCMContext/get_rollback_only:1.0
    //
    public boolean get_rollback_only() 
        throws org.omg.Components.IllegalState 
    {
        CDMW_LOG_FUNCTION(logger, "");
        return this.context.get_rollback_only();
    }

    //
    // IDL:omg.org/Components/CCMContext/get_user_transaction:1.0
    //
    public org.omg.Components.Transaction.UserTransaction 
    get_user_transaction() 
        throws org.omg.Components.IllegalState 
    {
        CDMW_LOG_FUNCTION(logger, "");
        return this.context.get_user_transaction();
    }

    //
    // IDL:omg.org/Components/CCMContext/is_caller_in_role:1.0
    //
    public boolean is_caller_in_role(String role) 
    {
        CDMW_LOG_FUNCTION(logger, "role="+role);
        return this.context.is_caller_in_role(role);
    }

    //
    // IDL:omg.org/Components/CCMContext/set_rollback_only:1.0
    //
    public void set_rollback_only() 
        throws org.omg.Components.IllegalState 
    {
        CDMW_LOG_FUNCTION(logger, "");
        this.context.set_rollback_only();
    }
    
    /**
     * Resolves service_id
     * 
     * @param service_id The service_id to resolve
     * 
     * @return The resolved object reference or null, if the service was not found
     */
    protected org.omg.CORBA.Object do_resolve_service_reference(String service_id) 
                        throws org.omg.Components.CCMException
    {
        CDMW_LOG_FUNCTION(logger, "service_id="+service_id);
        org.omg.CORBA.Object obj = null;
        
        String scopedServiceId = this.componentId + "@" + service_id;
        CDMW_DEBUG(logger, "Trying to resolve the per-component service " + scopedServiceId);
        try{
            obj = this.context.resolve_service_reference(scopedServiceId);
        }catch (CCMException e) {
            CDMW_DEBUG(logger, "Failed to resolve the per-component service. " +
                    "Trying to resolve the global service ... " + service_id);
            obj = null;
        }
        
        if(obj == null){
            try{
                obj = this.context.resolve_service_reference(service_id);
            }catch (CCMException e) {
                CDMW_DEBUG(logger, "Failed to resolve the global service ref. " +
                "Trying to get it from the ORB's initial references");
                obj = null;
            }
            
            if(obj == null) {
                // delegate resolution to the CCM2Context, this might do lookups within the ORB's 
                // initial references
                com.thalesgroup.CdmwCcmContainer.CCM2Context ctx 
                        = com.thalesgroup.CdmwCcmContainer.CCM2ContextHelper.narrow(this.context);
                try{
                    
                    obj = ctx.resolve_initial_references(service_id); 
                    
                }catch (com.thalesgroup.CdmwCcmContainer.CCM2ContextPackage.InvalidName in) {
                    CDMW_DEBUG(logger,
                            "Failed to resolve service \"" + service_id + "\" within CCM2Context ",
                            new Throwable());
                
                } catch (org.omg.CORBA.SystemException e) {
                    CDMW_ERROR(logger,
                            "Exception received getting service " + service_id + " : " + e,
                            new Throwable());
                    CDMW_LOG_FUNCTION_EXCEPTION(logger, e);
                    throw e;
                }

            }
        }
        return obj;
    }



    //
    // IDL:omg.org/Components/CCMContext/resolve_service_reference:1.0
    //
    public org.omg.CORBA.Object resolve_service_reference(String service_id) 
                throws org.omg.Components.CCMException
    {
        CDMW_LOG_FUNCTION(logger, "");
        
        // Block a specific set of services which are not supposed
        // to be used by application components
        // Check if service access is authorized
        String[] UNAVAILABLE_SERVICE_IDENTIFIERS = getUnavailableServices();
        for (int i=0; i<UNAVAILABLE_SERVICE_IDENTIFIERS.length ; i++) {
            if (UNAVAILABLE_SERVICE_IDENTIFIERS[i].equals(service_id)) {
                CDMW_WARN(logger,
                        "Surpressed attempt by component instance id \"" + this.componentId + "\"to access unauthorized service \"" + service_id + "\"",
                        new Throwable());
                
                return null;                                
            }
        }
        
        org.omg.CORBA.Object obj = do_resolve_service_reference(service_id);
        
        if(obj == null) {
            CDMW_ERROR(logger, "Failed to resolve the given service id "+
                    service_id);
            CCMException ex = new CCMException(CCMExceptionReason.OBJECT_NOT_FOUND);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
       
        return obj;
        
    }
    

}

