/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.container;

import java.util.HashMap;

import org.omg.CORBA.LocalObject;
import org.omg.CORBA.Object;
import org.omg.Components.CCMException;
import org.omg.Components.CCMExceptionReason;
import org.omg.Components.ConfigValue;
import org.omg.Components.ConfigValuesHolder;
import org.omg.Components.Cookie;
import org.omg.Components.ContainerPortableInterceptor.IntegrationPoint;
import org.omg.Components.ContainerPortableInterceptor.InvalidRegistration;

import cdmw.ccm.common.CCMLoggers;

import com.thalesgroup.CdmwCcmContainer.ApplicationElementContext;
import com.thalesgroup.CdmwCcmContainer.ApplicationElementContextService;
import com.thalesgroup.CdmwCcmContainer.ComponentElementContext;
import com.thalesgroup.CdmwCcmContainer.ComponentLifeCycleInterceptor;
import com.thalesgroup.CdmwCcmContainer.ComponentServerContext;
import com.thalesgroup.CdmwCcmContainer.ContainerElementContext;
import com.thalesgroup.CdmwCcmContainer.HomeElementContext;
import com.thalesgroup.CdmwCcmContainer.HomeLifeCycleInterceptor;
import com.thalesgroup.CdmwCcmContainer.LifeCycleInterceptorRegistrationHelper;

import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;


/**
 *
 */
public class ApplicationElementsContextService extends LocalObject implements
        ApplicationElementContextService, AbstractApplicationElementContextFactory {

    public static final String ApplicationElementsContextServiceId = "CDMW_ApplicationElementsContextService";
    
    private ComponentElementContextFactory compFactory;
    private HomeElementContextFactory homeFactory;
    
    private ComponentServerContext componentServerContext;
    private com.thalesgroup.CdmwCcmContainer.LifeCycleInterceptorRegistration registration;
    
    
    private AbstractApplicationElementContextFactory[] contextsFactories = 
        new AbstractApplicationElementContextFactory[]{compFactory, homeFactory};
    
    
    protected cdmw.logging.Logger logger = CCMLoggers.GetCcmcontainerLogger();
    
    
    public ApplicationElementsContextService(String componentServerId, ConfigValue[] values ){
        this.create_element_context(componentServerId, null, values);
        try{
            registration = LifeCycleInterceptorRegistrationHelper.narrow(
                    ServiceRegistry.getInstance().lookup(LifeCycleInterceptorRegistration.LifeCycleInterceptorRegistrationServiceId));
            
            compFactory = new ComponentElementContextFactory(registration);
            homeFactory = new HomeElementContextFactory(registration);
        }
        catch(CCMException ex){
            CDMW_ERROR(logger, "Failed to get the LifeCycleInterceptorRegistrationService reference." +
                          " The  " + ApplicationElementsContextServiceId + " service will not be available");
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
        }
        
    }
    

    public ComponentElementContext find_component_context_by_id(String id)
            throws CCMException {
        CDMW_LOG_FUNCTION(logger, "id = "+id);
        ComponentElementContext ctx = (ComponentElementContext)compFactory.find_element_context_by_id(id);
        if(ctx == null) {
            CCMException ex = new CCMException(CCMExceptionReason.OBJECT_NOT_FOUND);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
        return ctx;
    }


    public HomeElementContext find_home_context_by_id(String id)
            throws CCMException {
        CDMW_LOG_FUNCTION(logger, "id = "+id);
        HomeElementContext ctx = (HomeElementContext)homeFactory.find_element_context_by_id(id);
        if(ctx == null) {
            CCMException ex = new CCMException(CCMExceptionReason.OBJECT_NOT_FOUND);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
        return ctx;
    }

 
    public ComponentServerContext get_component_server_context() throws CCMException {
        CDMW_LOG_FUNCTION(logger, "");
        ComponentServerContext ctx = (ComponentServerContext)this.find_element_context_by_id("");
        if(ctx == null) {
            CCMException ex = new CCMException(CCMExceptionReason.OBJECT_NOT_FOUND);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
        return ctx;
    }
    
    public ApplicationElementContext create_element_context(String id, 
                            org.omg.CORBA.Object ref,
                            ConfigValue[] configValues){
        if(componentServerContext == null){
            componentServerContext = new ComponentServerElementContextImpl(configValues,
                    id, ref);
         }

        return componentServerContext;
        
    }

    public ApplicationElementContext find_element_context_by_id(String id) {
        return componentServerContext;
    }


    public void remove_element_context(String id) {
            componentServerContext = null;
        
    }
    
    public void clear(){
        
    }
    
    class ComponentElementContextFactory extends LocalObject 
                implements ComponentLifeCycleInterceptor, 
                AbstractApplicationElementContextFactory{

        private HashMap<String, ComponentElementContextImpl> componentElementContextsMap = 
            new HashMap<String, ComponentElementContextImpl> ();
        
        Cookie ck;
        
        
        public ComponentElementContextFactory(com.thalesgroup.CdmwCcmContainer.LifeCycleInterceptorRegistration reg){
           ck = reg.register_component_lifecycle_interceptor(this); 
        }
        
        public ApplicationElementContext create_element_context(String id,
                Object ref, ConfigValue[] configValues) {
            CDMW_LOG_FUNCTION(logger, "id= "+id);
            
            ComponentElementContextImpl comp_ctx = null;
            if(!componentElementContextsMap.containsKey(id)) {
                comp_ctx = new ComponentElementContextImpl(configValues, id, ref);
                componentElementContextsMap.put(id, comp_ctx);
            }
            else 
                CDMW_DEBUG(logger, "Component context is already created");
            
            return comp_ctx;
        }

        public ApplicationElementContext find_element_context_by_id(String id) {
            ApplicationElementContext ctx = null;
            if (!componentElementContextsMap.isEmpty()) 
                ctx =  componentElementContextsMap.get(id);
            
            return ctx;
        }

        public void remove_element_context(String id) {
            if(componentElementContextsMap.containsKey(id)){
                componentElementContextsMap.remove(id); 
            }else{
                CDMW_ERROR(logger, "Failed to remove the component context." +
                                "Component Context not found !!"); 
            }
        }
        
        public void clear(){
            if(!componentElementContextsMap.isEmpty()) componentElementContextsMap.clear();
            try {
                registration.unregister_component_lifecycle_interceptor(ck);
            } catch (InvalidRegistration e) {
                e.printStackTrace();
            }
            
        }

        public void after_component_activate(String id) {
        }

        public void after_component_connect(String arg0) {
        }

        public void after_component_create(String id, Object ref) {
            CDMW_LOG_FUNCTION(logger, "id = "+id);
            ComponentElementContextImpl ctx = (ComponentElementContextImpl)this.find_element_context_by_id(id); 
            if(ctx == null) {
                CDMW_ERROR(logger, "Component Context not found !!");
                return;
            }
            ctx.setRef(ref);
        }

        public void after_component_passivate(String arg0) {
        }

        public void after_component_remove(String id) {
            CDMW_LOG_FUNCTION(logger, "id = "+id);
            this.remove_element_context(id);
        }

        public void before_component_activate(String arg0) {
        }

        public void before_component_connect(String arg0, String arg1,
                Object arg2) {
        }

        public void before_component_create(String id, ConfigValuesHolder values) {
            CDMW_LOG_FUNCTION(logger, "id = "+id);
            this.create_element_context(id, null, values.value);
        }

        public void before_component_passivate(String arg0) {
        }

        public void before_component_remove(String arg0) {
        }

        public void destroy() {
        }

        public String name() {
            return "";
        }

        public short priority() {
            return Short.MIN_VALUE;
        }

        public IntegrationPoint registration_info() {
            return null;
        }

        public void set_slot_id(int arg0) {
        }
        
    }
    
    class HomeElementContextFactory extends LocalObject 
                    implements HomeLifeCycleInterceptor,
                            AbstractApplicationElementContextFactory{

        private HashMap<String, HomeElementContextImpl> homeElementContextsMap = 
            new HashMap<String, HomeElementContextImpl> ();
        
        Cookie ck ;
        
        public HomeElementContextFactory(com.thalesgroup.CdmwCcmContainer.LifeCycleInterceptorRegistration reg){
            ck = reg.register_home_lifecycle_interceptor(this);
        }
        
        public ApplicationElementContext create_element_context(String id,
                Object ref, ConfigValue[] configValues) {
            CDMW_LOG_FUNCTION(logger, "id= " + id);
            
            HomeElementContextImpl comp_ctx = new HomeElementContextImpl(configValues, id, ref);
            if(!homeElementContextsMap.containsKey(id))
                homeElementContextsMap.put(id, comp_ctx);
            else 
                CDMW_DEBUG(logger, "Home context is already created");
            
            return comp_ctx;
        }

        public ApplicationElementContext find_element_context_by_id(String id) {
            ApplicationElementContext ctx = null;
            if (!homeElementContextsMap.isEmpty()) 
                ctx = homeElementContextsMap.get(id);
            
            return ctx;
        }

        public void remove_element_context(String id) {
            if(homeElementContextsMap.containsKey(id)){
                homeElementContextsMap.remove(id); 
            }else{
                CDMW_ERROR(logger, "Failed to remove the home context." +
                                "Home Context not found !!"); 
            }
        }
        
        public void clear(){
            if(!homeElementContextsMap.isEmpty()) homeElementContextsMap.clear();
            try {
                registration.unregister_home_lifecycle_interceptor(ck);
            } catch (InvalidRegistration e) {
                e.printStackTrace();
            }
            
        }

        public void after_home_create(String id, Object ref) {
            HomeElementContextImpl ctx = (HomeElementContextImpl)this.find_element_context_by_id(id);
            if(ctx == null) {
                CDMW_ERROR(logger, "Home Context not found !!");
                return;
            }
            ctx.setRef(ref);
            
        }

        public void after_home_remove(String id) {
            this.remove_element_context(id);
        }

        public void before_home_create(String id, ConfigValuesHolder values,
                String entrypoint) {
            this.create_element_context(id, null, values.value);
        }

        public void before_home_remove(String arg0) {
        }

        public void destroy() {
        }

        public String name() {
            return "";
        }

        public short priority() {
            return Short.MIN_VALUE;
        }

        public IntegrationPoint registration_info() {
            return null;
        }

        public void set_slot_id(int arg0) {
        }
        
    }
    

}
