/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.qosenablers.dds;

import java.io.File;
import java.io.IOException;
import java.io.StringReader;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.logging.Logger;

import javax.xml.bind.JAXBException;

import org.omg.CCM_DDS.DDS_Base;
import org.omg.CCM_DDS.InternalError;
import org.omg.CORBA.BAD_OPERATION;
import org.omg.CORBA.BAD_PARAM;
import org.omg.CORBA.INTERNAL;
import org.omg.Components.CCMException;
import org.omg.Components.CCMExceptionReason;
import org.omg.Components.Cookie;
import org.omg.Components.ExtensionContext;
import org.omg.DDS.DomainParticipant;
import org.xml.sax.SAXException;


/**
 *
 */
public class DDS_BaseFragmentImpl extends org.omg.CORBA.LocalObject implements
        DDS_Base, org.omg.Components.ExtensionComponent{

    public static String DDS_DOMAIN_PARTICIPANT_SERVICE_PREFIX = "dds.domainparticipant";

    static int refCount;
    private Cookie participantCookie;

    protected Logger logger = Logger.getLogger(this.getClass().getName());

    private String domain_id = "";
    private String qos_profile_str;
    private QosProfile qosProfile;

    protected ExtensionContext extCtx;

    private boolean configured = false;

    DomainParticipantImpl participantImpl;

    /**
     * Attributes
     */

    public String domain_id() {
        return this.domain_id;
    }

    public void domain_id(String id) {
        if(!configured){
            logger.info("Domain Id set to "+ id);
            this.domain_id = new String(id);
            configured = true;
        }
        else {
            throw new BAD_OPERATION("The Domain Id is already set !");
        }
    }


    public String qos_profile() {
        return this.qos_profile_str;
    }

    /**
     *
     * @param profile File URL or XML string
     */
    public void qos_profile(String profile){

        if(this.qos_profile_str != null){
            logger.warning("The QoS Profile is already set to "+ this.qos_profile_str);
            throw new BAD_OPERATION("The QoS Profile is already set !");
        }

        if(profile==null){
            qos_profile_str = "DDS_DEFAULTQoS";
            logger.warning("null xml QoSProfile provided. Default QoS will be used.");
            return;
        }

        QoSProfileUnmarshaller m = null;
        try {
            m = new QoSProfileUnmarshaller();

        } catch (Throwable e1) {
            logger.fine("Failed to create the XML UnMarshaller");
            logger.throwing(this.getClass().getName(), "qos_profile", e1);
            throw new INTERNAL("Failed to create the XML UnMarshaller");
        }

        qosProfile = null;

        if(profile.startsWith("<")){
            StringReader xmlContent = new StringReader(profile);

            try {
             // to validate the XML File
                m.validate(xmlContent);

                // to parse the XML file
                qosProfile = m.getQosProfiles(xmlContent)[0];
            } catch (Throwable e) {
                logger.fine("Parsing Error: "+ e.getMessage());
                logger.throwing(this.getClass().getName(), "qos_profile", e);
                throw new BAD_PARAM("Parsing error: "+ e.getMessage());
            }
        }
        else{
            try{
                URI profileURI = new URI(profile);
                qosProfile = m.getQosProfiles(profileURI.getPath())[0];
                logger.info("qos_profile to set to "+profile);
            }catch(URISyntaxException e){
                logger.fine("The given QosProfile is not a valid File URI.");
                logger.throwing(this.getClass().getName(), "qos_profile", e);
                throw new BAD_PARAM("The given QosProfile is not a valid File URI."+ e.getMessage());
            }catch(Throwable e){
                logger.throwing(this.getClass().getName(), "qos_profile", e);
                throw new BAD_PARAM("Parsing error: "+ e.getMessage());
            }
        }

        this.qos_profile_str = new String(profile);
        logger.info("qos_profile set to "+profile);

    }

    public DDS_BaseFragmentImpl(){

    }

    public DDS_BaseFragmentImpl(String domain_id, String qos_profile) {
        this.domain_id = domain_id;
        this.qos_profile(qos_profile);
    }

    static String getDomainParticipantServiceId(String domain_id){
        return (DDS_DOMAIN_PARTICIPANT_SERVICE_PREFIX + "." + domain_id);
    }



    synchronized public DomainParticipant getDomainParticipant(DDS.DomainParticipantQos participantQos)
            throws InternalError {

        DomainParticipant participant = null;
        org.omg.CORBA.Object obj = null;
        String dpId = getDomainParticipantServiceId(domain_id());

        try{
            obj = extCtx.resolve_service_reference(dpId);
        }catch(CCMException ce){
            logger.warning("The DomainParticipant Service is not installed yet ! it will be installed ...");
        }

        if (obj == null) {
            participantImpl = new DomainParticipantImpl(domain_id, participantQos);
            participant = participantImpl;
            try{
                participantCookie = extCtx.install_service_reference(dpId, participant);
            }
            catch(CCMException e){
                logger.severe("Failed to install the DomainParticipant Service " + dpId);
                throw new InternalError("Failed to install the DomainParticipant Service "
                        + dpId, org.omg.DDS.RETCODE_ERROR.value, 0);
            }

            if(participantCookie != null){
                logger.warning("The DomainParticipant Service "+ dpId + " is installed now");
            }
            else{
                logger.severe("Failed to install the DomainParticipant Service " + dpId);
                throw new InternalError("Failed to install the DomainParticipant Service "
                        + dpId, org.omg.DDS.RETCODE_ERROR.value, 0);
            }
        } else {
            participant = org.omg.DDS.DomainParticipantHelper.narrow(obj);
        }
        return participant;

    }

    public void removeDomainParticipant() throws InternalError{
        try{
            extCtx.uninstall_service_reference(participantCookie);
        }
        catch(CCMException e){
            logger.throwing(this.getClass().getName(), "removeDomainParticipant", e);
            throw new InternalError("Failed to unistall the DomainParticipant service !",
                    org.omg.DDS.RETCODE_ERROR.value, 0);
        }
        participantImpl.remove();
    }

    public void ccm_remove() throws CCMException{
        // TODO Auto-generated method stub

    }

    public void set_extension_context(ExtensionContext ctx)
            throws CCMException {
        this.extCtx = ctx;

    }

    protected QosProfile getQosProfile() {
        return qosProfile;
    }

    

}
