/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccm.qosenablers.dds;

import java.util.logging.Logger;

import org.omg.CCM_DDS.BadParameter;
import org.omg.CCM_DDS.CCM_DDS_TopicBase;
import org.omg.CCM_DDS.InternalError;
import org.omg.CCM_DDS.NoSuchDDSEntity;
import org.omg.CCM_DDS.QueryFilter;
import org.omg.CCM_DDS.ReadInfoHolder;
import org.omg.CCM_DDS.ReadInfoSeqHolder;
import org.omg.CORBA.LocalObject;
import org.omg.Components.CCMException;
import org.omg.Components.ExtensionContext;
import org.omg.Components.SessionContext;
import org.omg.DDS.Duration_t;
import org.omg.DDS.RETCODE_OK;

import DDS.ANY_STATUS;
import DDS.DataReader;
import DDS.DataReaderQosHolder;
import DDS.DomainParticipant;
import DDS.HistoryQosPolicyKind;
import DDS.ReadCondition;
import DDS.SUBSCRIBER_QOS_DEFAULT;
import DDS.SampleInfo;
import DDS.Subscriber;
import DDS.TOPIC_QOS_DEFAULT;
import DDS.Topic;
import DDS.TypeSupport;
import DDS.WaitSet;


/**
 *    **********************************************************
     * Base implementation of the Getter interface              *
 ***************************************************************/

public abstract class DDS_GetterImpl extends LocalObject {

    Logger logger = Logger.getLogger("cdmw.ccm.qosenablers.dds.DDS_GetterImpl");

    protected WaitSet waits;
    protected ReadCondition readCond;
    protected ReadCondition userQuery;
    protected QueryFilter filter;
    protected Duration_t time_out = new Duration_t();
    protected DataReader reader;


    /**
     * Type-specific DDS entities
     */
    protected String dataTypeName;

    public DDS_GetterImpl(DataReader reader){
        this.waits = new WaitSet();
        this.reader = reader;
        this.readCond = reader.create_readcondition(
                DDS.NOT_READ_SAMPLE_STATE.value, DDS.NEW_VIEW_STATE.value
                        | DDS.NOT_NEW_VIEW_STATE.value,
                DDS.ALIVE_INSTANCE_STATE.value
                        | DDS.NOT_ALIVE_INSTANCE_STATE.value);
        if (readCond == null) {
            System.out.println("Error on create_readcondition");
            System.exit(-1);
        }
        this.userQuery = readCond;
        int status = waits.attach_condition(readCond);
        if (status != RETCODE_OK.value) {
            System.out.println("Error on attach_condition : " + status);
            System.exit(-1);
        }

    }


    /**
     * Attributes operations
     */
    synchronized public QueryFilter filter() {
        return this.filter;
    }

    synchronized public void set_filter(QueryFilter arg) throws BadParameter {
        this.filter = arg;
        if ("".equals(arg.query)) {
            userQuery = readCond;
            return;
        }
        userQuery = reader.create_querycondition(
                DDS.NOT_READ_SAMPLE_STATE.value, DDS.NEW_VIEW_STATE.value
                        | DDS.NOT_NEW_VIEW_STATE.value,
                DDS.ALIVE_INSTANCE_STATE.value
                        | DDS.NOT_ALIVE_INSTANCE_STATE.value, filter.query,
                filter.query_parameters);
        if (userQuery == null) {
            throw new BadParameter();
        }

    }

    public Duration_t time_out() {
        return this.time_out;
    }

    public void time_out(Duration_t arg) {
        this.time_out = arg;
    }

    public org.omg.CORBA.Object getDDSEntity(org.omg.CCM_DDS.DDSEntityKind e)
            throws NoSuchDDSEntity {
        return null;
    }

    /**
     * prepares the data
     *
     * @param <D>
     *            the data type
     * @param <H>
     *            the data sequence corba holder type
     * @param instances
     *            the read instances
     * @param infos
     * @param samples
     * @param readStatus
     * @return
     * @throws InternalError
     */
    public <D, H> boolean getAll(DataSeqHolder<D, H> instances,
            ReadInfoSeqHolder infos, SampleInfo[] samples, int readStatus)
            throws InternalError {

        return DataUtil.getData(instances, infos, samples, readStatus, true);
    }

    /**
     *
     * @param <D>
     * @param <H>
     * @param instances
     * @param infos
     * @param samples
     * @param readStatus
     * @return
     * @throws InternalError
     */
    public <D, H> boolean getAllHistory(DataSeqHolder<D, H> instances,
            ReadInfoSeqHolder infos, SampleInfo[] samples, int readStatus)
            throws InternalError {

        return DataUtil.getData(instances, infos, samples, readStatus, false);
    }

    /**
     *
     * @param infos
     * @param samples
     * @param readStatus
     * @return
     * @throws InternalError
     */
    public boolean getNext(ReadInfoHolder infos, SampleInfo[] samples,
            int readStatus) throws InternalError {

        if (samples.length != 1) {
            InternalError e = new InternalError();
            e.error_code = DDS.RETCODE_ERROR.value;
            e.index = 0;
            throw e;
        }

        return DataUtil.getData(infos, samples[0], readStatus);
    }

    public boolean getOne(ReadInfoHolder infos, SampleInfo[] samples,
            int readStatus) throws InternalError {

        if (readStatus != RETCODE_OK.value) {
            InternalError e = new InternalError();
            e.error_code = readStatus;
            e.index = 0;
            throw e;
        }

        for (SampleInfo sample : samples) {
            if (sample.sample_rank == 0) {
                return DataUtil.getData(infos, sample, readStatus);
            }
        }

        throw new InternalError("No samples found", DDS.RETCODE_ERROR.value, 0);
    }

    public boolean getOneHistory(ReadInfoSeqHolder infos, SampleInfo[] samples,
            int readStatus) throws InternalError {

        return DataUtil.getData(infos, samples, readStatus, false);

    }

    public void waitData() throws InternalError {
        DataUtil.waitData(waits, readCond,
                new DDS.Duration_t(time_out.sec, time_out.nanosec));
    }




}
