/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ccm.qosenablers.requestfollowup;

import org.omg.Components.ContainerPortableInterceptor.ContainerClientRequestInfo;
import org.omg.IOP.ServiceContext;

import cdmw.commonsvcs.requestfollowup.NDCFactory;
import cdmw.commonsvcs.requestfollowup.NDC;
import cdmw.commonsvcs.requestfollowup.NDCList;
import cdmw.commonsvcs.requestfollowup.NDCLogger;
import cdmw.logging.Logger;

import static cdmw.logging.FunctionLogger.*;
import static cdmw.logging.Logger.*;

/**
 * Purpose: <p> This class implements the ClientContainerInterceptor class.
 * 
 * Features: <p> Thread safe
 * 
 */
public class NDCClientContainerInterceptorImpl 
    extends org.omg.CORBA.LocalObject
    implements org.omg.Components.ContainerPortableInterceptor.ClientContainerInterceptor {

    /**
     * RFU Logging
     */
    private NDCLogger m_NDCLogger = NDCLogger.getInstance();

    /**
     * Internal logging
     */
    private Logger m_logger = m_NDCLogger.getLogger();

    // current request slot id
    private int m_ndcSlotId = 0;

    private static final String NDC_CLIENT_CONTAINER_INTERCEPTOR_NAME = "NDCClientContainerInterceptor";

    /**
     * Purpose: <p> Constructor
     * 
     * @param info
     *            ORB init information
     * @param ndcSlotId
     *            slot id used to store the date of the outgoing request.
     * 
     */
    public NDCClientContainerInterceptorImpl() {
        CDMW_LOG_FUNCTION(m_logger, "");
    }

    /**
     * Purpose: <p> When an exception occurs, this interception point is called.
     * It allows an Interceptor to query the exception's information before it
     * is raised to the client.
     * 
     * @param ccri
     *            Client request information
     */
    public void receive_exception(ContainerClientRequestInfo ccri) 
        throws org.omg.PortableInterceptor.ForwardRequest,
               org.omg.CORBA.SystemException {
        if ( ! NDCLogger.getInstance().isOperationToLog(ccri.request_info().operation()) ) {
            return;
        }

        // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
        // is used we enter in a cyclic recording of the log (infinite call to add_log)!
        // This explains why the macro is called only after the condition if()
        CDMW_LOG_FUNCTION(m_logger,
            "Operation=" + ccri.request_info().operation());

        
        CDMW_LOG(m_logger, CDMW_DEBUG_3,
            NDCLogger.getInstance().ipToString(NDCLogger.NDC_RECEIVE_EXCEPTION) + " (" + ccri.request_info().operation() + ")");
        logIncomingResponse(NDCLogger.NDC_RECEIVE_EXCEPTION, ccri);
    }
        

    /**
     * Purpose: <p> This interception point allows an Interceptor to query the
     * information available when a request results in something other than a
     * normal reply or an exception. For example, a request could result in a
     * retry (for example, a GIOP Reply with a LOCATION_FORWARD status was
     * received); or on asynchronous calls, the reply does not immediately
     * follow the request, but control shall return to the client and an ending
     * interception point shall be called.
     * 
     * @param ccri
     *            Client request information
     */
    public void receive_other(ContainerClientRequestInfo ccri) 
        throws org.omg.PortableInterceptor.ForwardRequest,
               org.omg.CORBA.SystemException {


        if ( ! NDCLogger.getInstance().isOperationToLog(ccri.request_info().operation()) ) {
            return;
        }

        // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
        // is used we enter in a cyclic recording of the log (infinite call to add_log)!
        // This explains why the macro is called only after the condition if()
        CDMW_LOG_FUNCTION(m_logger,
            "Operation=" + ccri.request_info().operation());
        
        CDMW_LOG(m_logger, CDMW_DEBUG_3,
            NDCLogger.getInstance().ipToString(NDCLogger.NDC_RECEIVE_OTHER) + " (" + ccri.request_info().operation() + ")");
        logIncomingResponse(NDCLogger.NDC_RECEIVE_OTHER, ccri);
    }
        
 
    /**
     * Purpose: <p> This interception point allows an Interceptor to query the
     * information on a reply after it is returned from the server and before
     * control is returned to the client. Here the date of the outgoing request
     * is reset and T0 of incoming request updated to remove the duration of the
     * outgoing request.
     * 
     * @param ccri
     *            Client request information
     */
    public void receive_reply(ContainerClientRequestInfo ccri)
        throws org.omg.CORBA.SystemException {
        if ( ! NDCLogger.getInstance().isOperationToLog(ccri.request_info().operation()) ) {
            return;
        }

        // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
        // is used we enter in a cyclic recording of the log (infinite call to add_log)!
        // This explains why the macro is called only after the condition if()
        CDMW_LOG_FUNCTION(m_logger,
            "Operation=" + ccri.request_info().operation());
        
       
        CDMW_LOG(m_logger, CDMW_DEBUG_3,
            NDCLogger.getInstance().ipToString(NDCLogger.NDC_RECEIVE_REPLY) + " (" + ccri.request_info().operation() + ")");
        logIncomingResponse(NDCLogger.NDC_RECEIVE_REPLY, ccri);
    }
    
    
    /**
     * Purpose: <p> This interception point allows an Interceptor to query
     * information during a Time-Independent Invocation (TII) polling get reply
     * sequence.
     * 
     * @param ccri
     *            Client request information
     * 
     */
    public void send_poll(ContainerClientRequestInfo ccri)
        throws org.omg.CORBA.SystemException {
            
        if ( ! NDCLogger.getInstance().isOperationToLog(ccri.request_info().operation()) ) {
            return;
        }

        // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
        // is used we enter in a cyclic recording of the log (infinite call to add_log)!
        // This explains why the macro is called only after the condition if()
        CDMW_LOG_FUNCTION(m_logger,
                "Operation=" + ccri.request_info().operation());
        
        CDMW_LOG(m_logger, CDMW_DEBUG_3,
            NDCLogger.getInstance().ipToString(NDCLogger.NDC_SEND_POLL) + " (" + ccri.request_info().operation() + ")");
        logOutgoingRequest(NDCLogger.NDC_SEND_POLL, ccri);
    }

    /**
     * Purpose: <p> This interception point allows an Interceptor to query
     * request information and modify the service context before the request is
     * sent to the server. Here, the date of the outgoing request is stored
     * here.
     * 
     * @param ccri
     *            Client request information
     * 
     */
    public void send_request(ContainerClientRequestInfo ccri) 
        throws org.omg.PortableInterceptor.ForwardRequest,
               org.omg.CORBA.SystemException {
        
        if ( ! NDCLogger.getInstance().isOperationToLog(ccri.request_info().operation()) ) {
            return;
        }

        // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
        // is used we enter in a cyclic recording of the log (infinite call to add_log)!
        // This explains why the macro is called only after the condition if()
        CDMW_LOG_FUNCTION(m_logger,
            "Operation=" + ccri.request_info().operation());
        
        
        CDMW_LOG(m_logger, CDMW_DEBUG_3,
            NDCLogger.getInstance().ipToString(NDCLogger.NDC_SEND_REQUEST) + " (" + ccri.request_info().operation() + ")");
        logOutgoingRequest(NDCLogger.NDC_SEND_REQUEST, ccri);
    }


    public void logOutgoingRequest(int ip, ContainerClientRequestInfo ccri) {
        CDMW_LOG_FUNCTION(m_logger,
            "InterceptionPoint=" + NDCLogger.getInstance().ipToString(ip) +
            ", Operation=" + ccri.request_info().operation());

        NDCList ndcList = new NDCList();
        
        try {
            // Get NDC Slot data
            CDMW_LOG(m_logger, CDMW_DEBUG_3,
                        "Get NDCs list from Request Service Context");
            org.omg.CORBA.Any any = ccri.request_info().get_slot(m_ndcSlotId);

            ndcList = NDCFactory.getInstance().fromAny(any);

            // Request Follow Up  not activated for that request
            if ( (ndcList.list.size() == 0) && !(m_logger.isEnabledFor(REQ_FOLLOWUP)) ) {
                CDMW_LOG(m_logger, CDMW_DEBUG_3,
                    "Request Follow Up not activated for that request");
                return;
            }

            CDMW_LOG(m_logger, CDMW_DEBUG_3,
                "Create a new NDC and add it into the NDCs list.");
            NDC ndc = NDCFactory.getInstance().createNewNDC(ccri.request_info().operation());
            ndcList.list.addLast(ndc);


            // Add NDCs to NDC Service Context of the request
            CDMW_LOG(m_logger, CDMW_DEBUG_3,
                    "Set Service Context from NDCs list : " + NDCFactory.getInstance().toString(ndcList));
            byte [] data = NDCFactory.getInstance().toOctetSeq(ndcList);
            ServiceContext context =  new ServiceContext(NDCLogger.CDMW_NDC_SERVICE_CONTEXT_ID, data);
            ccri.request_info().add_request_service_context(context, true);
        }
        catch (org.omg.PortableInterceptor.InvalidSlot ex) {
            CDMW_WARN(m_logger,
                "Catch PortableInterceptor.InvalidSlot exception - " + ex);
        }
        //  catch (UserException ex) {
        //      CDMW_WARN(m_logger,
        //              "Catch CORBA.UserException - " + ex);
        //  }
        catch (org.omg.CORBA.SystemException ex) {
            CDMW_WARN(m_logger,
                "Catch CORBA.SystemException - " + ex);
        }

        // log the Request Follow Up context
        NDCLogger.getInstance().logRequest(ip, ccri.request_info().operation(), ndcList);

    }



    public void logIncomingResponse(int ip, ContainerClientRequestInfo ccri) {
        CDMW_LOG_FUNCTION(m_logger,
            "InterceptionPoint=" + NDCLogger.getInstance().ipToString(ip) +
            ", Operation=" + ccri.request_info().operation());

        NDCList ndcList = new NDCList();
        
        try {
            //CDMW_LOG(m_logger, CDMW_DEBUG_3,
            //            "Get NDC slot: m_ndcSlotId=" +  m_ndcSlotId);
            //org.omg.CORBA.Any any = ccri.request_info().get_slot(m_ndcSlotId);
            //ndcList = NDCFactory.getInstance().fromAny(any);
            // Get NDC Service Context data
            CDMW_LOG(m_logger, CDMW_DEBUG_3,
                "Get NDCs list from Service Context");
            org.omg.IOP.ServiceContext sc =
                ccri.request_info().get_request_service_context(NDCLogger.CDMW_NDC_SERVICE_CONTEXT_ID);
            byte [] data = sc.context_data;

            ndcList = NDCFactory.getInstance().fromOctetSeq(data);
        }
        catch (org.omg.CORBA.BAD_PARAM ex) {
            // This is not an error while a request can come to a single reference
            // with no NDC service context.
            CDMW_LOG(m_logger, CDMW_DEBUG_3,
                "No NDC Service Context for that request");
        }
//        catch (org.omg.PortableInterceptor.InvalidSlot ex) {
//                CDMW_WARN(m_logger,
//                    "Catch PortableInterceptor.InvalidSlot exception - " + ex);
//        }
//        catch (UserException ex) {
//            CDMW_WARN(m_logger,
//                "Catch CORBA.UserException - " + ex);
//        }
        catch (org.omg.CORBA.SystemException ex) {
            CDMW_WARN(m_logger,
                    "Catch CORBA.SystemException - " + ex);
        }

        // Request Follow Up  not activated for that request
        if ( (ndcList.list.size() == 0) && !(m_logger.isEnabledFor(REQ_FOLLOWUP)) ) {
            CDMW_LOG(m_logger, CDMW_DEBUG_3,
                     "Request Follow Up not activated for that request");
            return;
        }

        if ( ndcList.list.size() == 0 ) {
            CDMW_LOG(m_logger, CDMW_DEBUG_3,
                     "Unexpected data returned by NDC Service Context, NDCs list is empty.");
        }

        NDCLogger.getInstance().logRequest(ip, ccri.request_info().operation(), ndcList);
        
    }


    /**
     * Purpose: <p> Return the name of the interceptor
     * 
     */
    public String name() 
        throws org.omg.CORBA.SystemException {
        CDMW_LOG_FUNCTION(m_logger, "");
        CDMW_LOG_FUNCTION_RETURN(m_logger,
            NDC_CLIENT_CONTAINER_INTERCEPTOR_NAME);
        return NDC_CLIENT_CONTAINER_INTERCEPTOR_NAME;
    }


    /**
     * Purpose: <p> Ddefine the slot of the Interceptor
     * 
     */
    public void set_slot_id(int slot_id)
    {
        this.m_ndcSlotId = slot_id;
        CDMW_LOG_FUNCTION(m_logger, "m_ndcSlotId " + this.m_ndcSlotId);
    }


    /**
     * Purpose: <p> Destroy the interceptor
     * 
     */
    public void destroy() 
        throws org.omg.CORBA.SystemException {
        CDMW_LOG_FUNCTION(m_logger, "");
    }

    
    /**
     * Purpose: <p> Return the RegistrationInfo
     * 
     */
    public org.omg.Components.ContainerPortableInterceptor.IntegrationPoint registration_info() 
        throws org.omg.CORBA.SystemException {
        CDMW_LOG_FUNCTION(m_logger, "");
        
        return null;
    }


    /**
     * Purpose: <p> Return the priority
     * 
     */
    public short priority() {
        CDMW_LOG_FUNCTION(m_logger, "");
        CDMW_LOG_FUNCTION_RETURN(m_logger, "");
        return 0;
    }
}