/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ccm.qosenablers.requestfollowup;

import org.omg.Components.ContainerPortableInterceptor.ContainerServerRequestInfo;
import org.omg.IOP.ServiceContext;

import cdmw.commonsvcs.requestfollowup.NDCFactory;
import cdmw.commonsvcs.requestfollowup.NDC;
import cdmw.commonsvcs.requestfollowup.NDCList;
import cdmw.commonsvcs.requestfollowup.NDCLogger;

import static cdmw.logging.FunctionLogger.*;
import static cdmw.logging.Logger.*;

/**
 * Purpose: <p> This class implements the ServerContainerInterceptor class.
 * 
 * Features: <p> Thread safe
 * 
 */
public class NDCServerContainerInterceptorImpl 
    extends org.omg.CORBA.LocalObject
    implements org.omg.Components.ContainerPortableInterceptor.ServerContainerInterceptor {
    
    private static final String NDC_SERVER_CONTAINER_INTERCEPTOR_NAME = "NDCServerContainerInterceptor";
    private int m_ndcSlotId = 0;

    /**
     * Purpose: <p> Constructor
     * 
     * @param info
     *            ORB init information
     * @param ndcSlotId
     *            slot id used to store the date of the outgoing request.
     * 
     */
    public NDCServerContainerInterceptorImpl() {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(), "");
    }


    /**
     * Purpose: <p> At this interception point, Interceptors must get their
     * service context information from the incoming request and transfer it to
     * PortableInterceptor.Current's slots.
     * 
     * @param csri
     *            Server request information
     */
    public void receive_request_service_contexts(ContainerServerRequestInfo csri)
        throws org.omg.PortableInterceptor.ForwardRequest,
               org.omg.CORBA.SystemException {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(),
            "Operation=" + csri.request_info().operation());
    }

    /**
     * Purpose: <p> This interception point allows an Interceptor to query
     * request information after all the information, including operation
     * parameters, are available.
     * 
     * @param csri
     *            Server request information
     */
    public void receive_request(ContainerServerRequestInfo csri)
        throws org.omg.PortableInterceptor.ForwardRequest,
               org.omg.CORBA.SystemException {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(),
            "Operation=" + csri.request_info().operation());

        if ( ! NDCLogger.getInstance().isOperationToLog(csri.request_info().operation()) ) {
            return;
        }
        
        CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
            NDCLogger.getInstance().ipToString(NDCLogger.NDC_RECEIVE_REQUEST) + " (" + csri.request_info().operation() + ")",
            new Throwable());
        logIncomingRequest(NDCLogger.NDC_RECEIVE_REQUEST, csri);
    }


    /**
     * Purpose: <p> This interception point allows an Interceptor to query reply
     * information and modify the reply service context after the target
     * operation has been invoked and before the reply is returned to the
     * Server.
     * 
     * @param csri
     *            Server request information
     */
    public void send_reply(ContainerServerRequestInfo csri) {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(),
            "Operation=" + csri.request_info().operation());

        if ( ! NDCLogger.getInstance().isOperationToLog(csri.request_info().operation()) ) {
            return;
        }
        
        CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
            NDCLogger.getInstance().ipToString(NDCLogger.NDC_SEND_REPLY) + " (" + csri.request_info().operation() + ")",
            new Throwable());
        logOutgoingRequest(NDCLogger.NDC_SEND_REPLY, csri);
    }

    /**
     * Purpose: <p> This interception point allows an Interceptor to query the
     * information available when a request results in something other than a
     * normal reply or an exception. A request could result in a retry (for
     * example, a GIOP Reply with a LOCATION_FORWARD status was received).
     * 
     * @param csri
     *            Server request information
     */
    public void send_other(ContainerServerRequestInfo csri)
        throws org.omg.PortableInterceptor.ForwardRequest,
               org.omg.CORBA.SystemException {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(),
            "Operation=" + csri.request_info().operation());

        if ( ! NDCLogger.getInstance().isOperationToLog(csri.request_info().operation()) ) {
            return;
        }
        
        CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
            NDCLogger.getInstance().ipToString(NDCLogger.NDC_SEND_OTHER) + " (" + csri.request_info().operation() + ")",
            new Throwable());
        logOutgoingRequest(NDCLogger.NDC_SEND_OTHER, csri);
    }

    /**
     * Purpose: <p> When an exception occurs, this interception point is called.
     * It allows an Interceptor to query the exception information and modify
     * the reply service context before the exception is raised to the Server.
     * 
     * @param csri
     *            Server request information
     */
    public void send_exception(ContainerServerRequestInfo csri)
        throws org.omg.PortableInterceptor.ForwardRequest,
               org.omg.CORBA.SystemException {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(),
                "Operation=" + csri.request_info().operation());

        if ( ! NDCLogger.getInstance().isOperationToLog(csri.request_info().operation()) ) {
            return;
        }
        
        CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
            NDCLogger.getInstance().ipToString(NDCLogger.NDC_SEND_EXCEPTION) + " (" + csri.request_info().operation() + ")",
            new Throwable());
        logOutgoingRequest(NDCLogger.NDC_SEND_EXCEPTION, csri);
    }


    public void logIncomingRequest(int ip, ContainerServerRequestInfo csri) {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(),
                "InterceptionPoint=" + NDCLogger.getInstance().ipToString(ip) +
                ", Operation=" + csri.request_info().operation());

        NDCList ndcList = new NDCList();
        
        try {
            CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
                    "Get NDCs list from Service Context",
                    new Throwable());
            ServiceContext context1 =
                csri.request_info().get_request_service_context(NDCLogger.CDMW_NDC_SERVICE_CONTEXT_ID);
            byte [] data1 = context1.context_data;
            ndcList = NDCFactory.getInstance().fromOctetSeq(data1);

            // Request Follow Up  not activated for that request
            if ( (ndcList.list.size() == 0) && !(NDCLogger.getInstance().getLogger().isEnabledFor(REQ_FOLLOWUP)) ) {
                CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
                        "Request Follow Up not activated for that request",
                        new Throwable());
                return;
            }
    
            // If the NDC Service Context is empty, the request come from a client 
            // with no Request Follow Up service activated. 
            // We create a new NDC to follow up the request, as the service is 
            // activated on this server. 
            if (ndcList.list.size() == 0) {
                CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
                        "Create a new NDC and add it into the NDCs list.",
                        new Throwable());
                NDC ndc = NDCFactory.getInstance().createNewNDC(csri.request_info().operation());
                ndcList.list.addLast(ndc);

                // Add NDCs to NDC Service Context of the request
                CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
                        "Set Service Context from NDCs list : " + NDCFactory.getInstance().toString(ndcList),
                        new Throwable());
                byte [] data2 = NDCFactory.getInstance().toOctetSeq(ndcList);
                ServiceContext context2 = 
                    new ServiceContext(NDCLogger.CDMW_NDC_SERVICE_CONTEXT_ID, data2);
                csri.request_info().add_reply_service_context(context2, true);
            }
            
            // Set NDC Slot Id
            CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
                    "Set Slot from NDCs list : " + NDCFactory.getInstance().toString(ndcList),
                    new Throwable());
            org.omg.CORBA.Any any = NDCFactory.getInstance().toAny(ndcList);
            csri.request_info().set_slot(m_ndcSlotId, any);
        }
        catch (org.omg.CORBA.BAD_PARAM ex) {
            // This is not an error while a request can come to a single reference
            // with no NDC service context.
            CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
                    "No NDC Service Context for that request",
                    new Throwable());
        }
        catch (org.omg.PortableInterceptor.InvalidSlot ex) {
            CDMW_WARN(NDCLogger.getInstance().getLogger(),
                   "Catch PortableInterceptor.InvalidSlot exception - " + ex, new Throwable());
        }
    //  catch (UserException ex) {
    //      CDMW_WARN(NDCLogger.getInstance().getLogger(),
    //              "Catch CORBA.UserException - " + ex, new Throwable());
    //  }
        catch (org.omg.CORBA.SystemException ex) {
            CDMW_WARN(NDCLogger.getInstance().getLogger(),
                    "Catch CORBA.SystemException - " + ex, new Throwable());
        }

        // log the Request Follow Up context
        NDCLogger.getInstance().logRequest(ip, csri.request_info().operation(), ndcList);
        
    }



    public void logOutgoingRequest(int ip, ContainerServerRequestInfo csri) {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(),
            "InterceptionPoint=" + NDCLogger.getInstance().ipToString(ip) +
            ", Operation=" + csri.request_info().operation());

        NDCList ndcList = new NDCList();
        
        try {
            CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
                        "Get NDCs list from NDC Slot", new Throwable());
            org.omg.CORBA.Any any = csri.request_info().get_slot(m_ndcSlotId);
            ndcList = NDCFactory.getInstance().fromAny(any);

//            CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
//                    "Get NDCs list from Service Context",
//                    new Throwable());
//            org.omg.IOP.ServiceContext sc =
//                csri.request_info().get_request_service_context(NDCLogger.CDMW_NDC_SERVICE_CONTEXT_ID);
//            byte [] data = sc.context_data;
//            ndcList = NDCFactory.getInstance().fromOctetSeq(data);
        }
        catch (org.omg.PortableInterceptor.InvalidSlot ex) {
                CDMW_WARN(NDCLogger.getInstance().getLogger(),
                    "Catch PortableInterceptor.InvalidSlot exception - " + ex, new Throwable());
        }
    //  catch (UserException ex) {
    //      CDMW_WARN(NDCLogger.getInstance().getLogger(),
    //              "Catch CORBA.UserException - " + ex, new Throwable());
    //  }
        catch (org.omg.CORBA.SystemException ex) {
            CDMW_WARN(NDCLogger.getInstance().getLogger(),
                    "Catch CORBA.SystemException - " + ex, new Throwable());
        }

        // Request Follow Up  not activated for that request
        if ( (ndcList.list.size() == 0) && !(NDCLogger.getInstance().getLogger().isEnabledFor(REQ_FOLLOWUP)) ) {
            CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
                "Request Follow Up not activated for that request",
                new Throwable());
            return;
        }

        if ( ndcList.list.size() == 0 ) {
            CDMW_WARN(NDCLogger.getInstance().getLogger(),
            "Unexpected data returned by NDC Slot, NDCs list is empty.",
            new Throwable());
        }

    // log the Request Follow Up context
    NDCLogger.getInstance().logRequest(ip, csri.request_info().operation(), ndcList);
        
    }


    /**
     * Purpose: <p> Return the name of the interceptor
     * 
     */
    public String name() throws org.omg.CORBA.SystemException {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(), "");
        CDMW_LOG_FUNCTION_RETURN(NDCLogger.getInstance().getLogger(),
            NDC_SERVER_CONTAINER_INTERCEPTOR_NAME);
        return NDC_SERVER_CONTAINER_INTERCEPTOR_NAME;
    }

    
    /**
     * Purpose: <p> Ddefine the slot of the Interceptor
     * 
     */
    public void set_slot_id(int slot_id)
    {
        this.m_ndcSlotId = slot_id;
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(), "m_ndcSlotId " + this.m_ndcSlotId);
    }


    /**
     * Purpose: <p> Destroy the interceptor
     * 
     */
    public void destroy() throws org.omg.CORBA.SystemException {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(),
            "PortableInterceptor=" + name());
    }


    /**
     * Purpose: <p> Return the RegistrationInfo
     * 
     */
    public org.omg.Components.ContainerPortableInterceptor.IntegrationPoint registration_info() 
        throws org.omg.CORBA.SystemException {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(), "");
        
        return null;
    }


    /**
     * Purpose: <p> Return the priority
     * 
     */
    public short priority() {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(), "");
        CDMW_LOG_FUNCTION_RETURN(NDCLogger.getInstance().getLogger(), "");
        return 0;
    }
}