/* ===================================================================== */
/*
 * This file is part of MOSIC (R). MOSIC is a System@tic project.
 * 
 * MOSIC is derivative work based on CARDAMOM (R). CARDAMOM is jointly
 * developed by THALES and SELEX SISTEMI INTEGRATI.
 * CARDAMOM is derivative work based on PERCO.
 * Copyright (C) THALES 2000-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ccm.qosenablers.requestfollowup;

import org.omg.CORBA.LocalObject;
import org.omg.CORBA.BooleanHolder;
import org.omg.CORBA.SystemException;
import org.omg.CORBA.Any;
import org.omg.CORBA.TCKind;
import org.omg.PortableInterceptor.Current;
import org.omg.PortableInterceptor.InvalidSlot;
import org.omg.PortableInterceptor.ForwardRequest;
import org.omg.Components.ContainerPortableInterceptor.StubContainerInterceptor;
import org.omg.Components.ContainerPortableInterceptor.ContainerStubRequestInfo;
import org.omg.Components.ContainerPortableInterceptor.CustomSlotItem;
import org.omg.Components.ContainerPortableInterceptor.CustomSlotItemSeqHelper;
import org.omg.Components.ContainerPortableInterceptor.IntegrationPoint;

import cdmw.commonsvcs.requestfollowup.NDCFactory;
import cdmw.commonsvcs.requestfollowup.NDC;
import cdmw.commonsvcs.requestfollowup.NDCList;
import cdmw.commonsvcs.requestfollowup.NDCLogger;
import cdmw.ccm.qosenablers.requestfollowup.ExtendedNDCExecutorImpl;

import cdmw.ccm.ccmcopi.COPIUtils;
import cdmw.logging.Logger;

import static cdmw.logging.FunctionLogger.*;
import static cdmw.logging.Logger.*;




/**
 * Purpose: <p> This class implements the StubContainerInterceptor class.
 *
 * Features: <p> Thread safe
 *
 */
public class NDCStubContainerInterceptorImpl
    extends LocalObject
    implements StubContainerInterceptor {

    /**
     * RFU Logging
     */
    private NDCLogger m_NDCLogger = NDCLogger.getInstance();

    /**
     * Internal logging
     */
    private Logger m_logger = m_NDCLogger.getLogger();
    
    /**
     * current request slot id
     */
    private int m_ndcSlotId = 0;

    /**
     * PICurrent
     */
    private Current m_PICurrent;

    /**
     * Interceptor name
     */
    private static final String NDC_STUB_CONTAINER_INTERCEPTOR_NAME = "NDCStubContainerInterceptor";




    /**
     * Purpose: <p> Constructor
     *
     * @param PICurrent PICurrent
     * 
     *
     */
    public NDCStubContainerInterceptorImpl(Current PICurrent) {
        CDMW_LOG_FUNCTION(m_logger, "");
        m_PICurrent = PICurrent;
        CDMW_LOG_FUNCTION_RETURN(m_logger); 
    }


    /**
     * Purpose: <p> This interception point allows an Interceptor to query
     * request information and modify the service context before the request is
     * sent to the server. Here, the date of the outgoing request is stored
     * here.
     *
     * @param csri
     *            Client request information
     *
     */
    public void stub_send_request(ContainerStubRequestInfo csri,
                                  BooleanHolder proceed_call)
        throws ForwardRequest,
               SystemException {

        proceed_call.value = true;

        if ( ! m_NDCLogger.isOperationToLog(csri.operation()) ) {
            return;
        }

        // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
        // is used we enter in a cyclic recording of the log (infinite call to add_log)!
        // This explains why the macro is called only after the condition if()
        CDMW_LOG_FUNCTION(m_logger,
                          "Operation=" + csri.operation());

        CDMW_LOG(m_logger, CDMW_DEBUG_3,
                 m_NDCLogger.ipToString(NDCLogger.NDC_SEND_REQUEST) + " (" + csri.operation() + ")");
        logOutgoingRequest(NDCLogger.NDC_SEND_REQUEST, csri);
    }


    /**
     * Purpose: <p> This interception point allows an Interceptor to query the
     * information on a reply after it is returned from the server and before
     * control is returned to the client. Here the date of the outgoing request
     * is reset and T0 of incoming request updated to remove the duration of the
     * outgoing request.
     *
     * @param csri
     *            Client request information
     */
    public void stub_receive_reply(ContainerStubRequestInfo csri,
                                   BooleanHolder proceed_call)
        throws SystemException {

        proceed_call.value = true;

        if ( ! m_NDCLogger.isOperationToLog(csri.operation()) ) {
            return;
        }

        // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
        // is used we enter in a cyclic recording of the log (infinite call to add_log)!
        // This explains why the macro is called only after the condition if()
        CDMW_LOG_FUNCTION(m_logger,
                          "Operation=" + csri.operation());

        CDMW_LOG(m_logger, CDMW_DEBUG_3,
                 m_NDCLogger.ipToString(NDCLogger.NDC_RECEIVE_REPLY) + " (" + csri.operation() + ")");
        logIncomingResponse(NDCLogger.NDC_RECEIVE_REPLY, csri);
    }


    /**
     * Purpose: <p> When an exception occurs, this interception point is called.
     * It allows an Interceptor to query the exception's information before it
     * is raised to the client.
     *
     * @param csri
     *            Client request information
     */
    public void stub_receive_exception(ContainerStubRequestInfo csri,
                                       BooleanHolder proceed_call)
        throws ForwardRequest,
               SystemException {

        proceed_call.value = true;

        if ( ! m_NDCLogger.isOperationToLog(csri.operation()) ) {
            return;
        }

        // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
        // is used we enter in a cyclic recording of the log (infinite call to add_log)!
        // This explains why the macro is called only after the condition if()
        CDMW_LOG_FUNCTION(m_logger,
                          "Operation=" + csri.operation());

        CDMW_LOG(m_logger, CDMW_DEBUG_3,
                 m_NDCLogger.ipToString(NDCLogger.NDC_RECEIVE_EXCEPTION) + " (" + csri.operation() + ")");
        logIncomingResponse(NDCLogger.NDC_RECEIVE_EXCEPTION, csri);
    }


    /**
     * Purpose: <p> This interception point allows an Interceptor to query the
     * information available when a request results in something other than a
     * normal reply or an exception. For example, a request could result in a
     * retry (for example, a GIOP Reply with a LOCATION_FORWARD status was
     * received); or on asynchronous calls, the reply does not immediately
     * follow the request, but control shall return to the client and an ending
     * interception point shall be called.
     *
     * @param csri
     *            Client request information
     */
    public void stub_receive_other(ContainerStubRequestInfo csri)
        throws ForwardRequest,
               SystemException {

        if ( ! m_NDCLogger.isOperationToLog(csri.operation()) ) {
            return;
        }

        // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
        // is used we enter in a cyclic recording of the log (infinite call to add_log)!
        // This explains why the macro is called only after the condition if()
        CDMW_LOG_FUNCTION(m_logger,
                          "Operation=" + csri.operation());

        CDMW_LOG(m_logger, CDMW_DEBUG_3,
                 m_NDCLogger.ipToString(NDCLogger.NDC_RECEIVE_OTHER) + " (" + csri.operation() + ")");
        logIncomingResponse(NDCLogger.NDC_RECEIVE_OTHER, csri);
    }


    public void logOutgoingRequest(int ip, ContainerStubRequestInfo csri) {
        CDMW_LOG_FUNCTION(m_logger,
                          "InterceptionPoint=" + m_NDCLogger.ipToString(ip) +
                          ", Operation=" + csri.operation());

        NDCList ndcList = new NDCList();
        NDCFactory ndcFactory = NDCFactory.getInstance();
        CustomSlotItem[] csi = null;
        Any any = COPIUtils.createAny();
        int idx = 0;

        try {
            // Get data from CustomSlotItemSeq Slot
            any = m_PICurrent.get_slot(m_ndcSlotId);

            // Manage extraction of CustomSlotItemSeq
            if (any != null && !any.type().kind().equals(TCKind.tk_null)) {
                csi = CustomSlotItemSeqHelper.extract(any);
                
                // Search the CustomSmotItem corresponding to the RFU Service
                for( idx = 0 ; idx <  csi.length ; idx++ ) {
                    if(csi[idx].identifier.equals(ExtendedNDCExecutorImpl.RFU_SERVICE_NAME)) {
                        ndcList = ndcFactory.fromAny(csi[idx].content);
                        break;
                    }
                }
            }
        } catch (InvalidSlot ex) {
            CDMW_WARN(m_logger,
                      "Catch PortableInterceptor.InvalidSlot exception - " + ex);
        }
        catch (SystemException ex) {
            CDMW_WARN(m_logger,
                      "Catch CORBA.SystemException - " + ex);
        }

        if ( ndcList.list.isEmpty() 
             && ! m_logger.isEnabledFor(REQ_FOLLOWUP) ) {
            // Request Follow Up not activated for that request
            CDMW_LOG(m_logger, CDMW_DEBUG_3,
                     "Request Follow Up not activated for that request");
            return;
        }

        try {
            // Create NDC for the current request
            NDC ndc = ndcFactory.createNewNDC(csri.operation());

            if ( ndcList.list.isEmpty() ) {
                // No CustomSlotItemSeq found ==> No RFU CustomSlotItem found
                ndcList.list.addLast(ndc);
                csi = new CustomSlotItem[1];
                csi[0] = new CustomSlotItem(ExtendedNDCExecutorImpl.RFU_SERVICE_NAME,
                                            ndcFactory.toAny(ndcList));
            } else {
                // CustomSlotItemSeq found
                ndcList.list.addLast(ndc);

                if ( idx < csi.length ) {
                    // And RFU CustomSlotItem found
                    csi[idx].content = ndcFactory.toAny(ndcList);
                } else {
                    // And RFU CustomSlotItem NOT found
                    CustomSlotItem[] tmp_csi = new CustomSlotItem[csi.length+1];
                    System.arraycopy(csi, 0, tmp_csi, 0, csi.length);
                    tmp_csi[idx] = 
                        new CustomSlotItem(ExtendedNDCExecutorImpl.RFU_SERVICE_NAME,
                                           ndcFactory.toAny(ndcList));
                    csi = tmp_csi;
                }
            }

            // Get data from CustomSlotItemSeq Slot
            CustomSlotItemSeqHelper.insert(any, csi);
            m_PICurrent.set_slot(m_ndcSlotId, any);
        }
        catch (InvalidSlot ex) {
            CDMW_LOG(m_logger, CDMW_DEBUG_3,
                     "No NDC Service Context for that request");
        }
        catch (SystemException ex) {
            CDMW_WARN(m_logger,
                      "Catch CORBA.SystemException - " + ex);
        }

        // log the Request Follow Up context
        m_NDCLogger.logRequest(ip, csri.operation(), ndcList);
        CDMW_LOG_FUNCTION_RETURN(m_logger);
    }


    public void logIncomingResponse(int ip, ContainerStubRequestInfo csri) {
        CDMW_LOG_FUNCTION(m_logger,
                          "InterceptionPoint=" + m_NDCLogger.ipToString(ip) +
                          ", Operation=" + csri.operation());

        NDCList ndcList = new NDCList();
        NDCFactory ndcFactory = NDCFactory.getInstance();

        try {
            // Get data from CustomSlotItemSeq slot
            Any any = m_PICurrent.get_slot(m_ndcSlotId);
            
            // Manage extraction of CustomSlotItemSeq
            if (any != null && ! any.type().kind().equals(TCKind.tk_null)) {
                CustomSlotItem[] csi = CustomSlotItemSeqHelper.extract(any);

                // Search the CustomSmotItem corresponding to the RFU Service
                for( int idx = 0 ; idx <  csi.length ; idx++ ) {
                    if(csi[idx].identifier.equals(ExtendedNDCExecutorImpl.RFU_SERVICE_NAME)) {
                        ndcList = ndcFactory.fromAny(csi[idx].content);

                        // remove item found
                        CustomSlotItem[] csi_tmp = new CustomSlotItem[csi.length - 1];
                        CDMW_DEBUG(m_logger, "new length["+csi_tmp.length+"]");
                        if (csi_tmp.length > 0 ) {
                            if(idx > 0)
                                System.arraycopy(csi, 0, csi_tmp, 0, idx);
                            if(idx < csi.length )
                                System.arraycopy(csi, idx+1, csi_tmp, idx, csi.length - (idx + 1));
                        }
                        csi = csi_tmp;
                        CustomSlotItemSeqHelper.insert(any, csi);
                        m_PICurrent.set_slot(m_ndcSlotId, any);

                        break;
                    }
                }

                // Get data from CustomSlotItemSeq slot
                any = m_PICurrent.get_slot(m_ndcSlotId);
                
                // Manage extraction of CustomSlotItemSeq
                if (any != null && ! any.type().kind().equals(TCKind.tk_null)) {
                    csi = CustomSlotItemSeqHelper.extract(any);
                    
                    // Search the CustomSmotItem corresponding to the RFU Service
                    for( int idx = 0 ; idx <  csi.length ; idx++ ) {
                        if(csi[idx].identifier.equals(ExtendedNDCExecutorImpl.RFU_SERVICE_NAME)) {
                            CDMW_DEBUG(m_logger, "RFU Found");
                            break;
                        }
                    }
                }
            }
        }
        catch (InvalidSlot ex) {
            // This is not an error while a request can come to a single reference
            // with no NDC service context.
            CDMW_LOG(m_logger, CDMW_DEBUG_3,
                     "No NDC Service Context for that request");
        }
        catch (SystemException ex) {
            CDMW_WARN(m_logger,
                      "Catch CORBA.SystemException - " + ex);
        }

        if ( ndcList.list.isEmpty() && ! m_logger.isEnabledFor(REQ_FOLLOWUP) ) {
            // Request Follow Up  not activated for that request
            CDMW_LOG(m_logger, CDMW_DEBUG_3,
                     "Request Follow Up not activated for that request");
        }

        if ( ndcList.list.isEmpty() ) {
            CDMW_LOG(m_logger, CDMW_DEBUG_3,
                     "Unexpected data returned by NDC Service Context, NDCs list is empty.");
        }

        // log the Request Follow Up context
        m_NDCLogger.logRequest(ip, csri.operation(), ndcList);

        CDMW_LOG_FUNCTION_RETURN(m_logger);
    }


    /**
     * Purpose: <p> Return the name of the interceptor
     *
     */
    public String name()
        throws SystemException {
        CDMW_LOG_FUNCTION(m_logger, "");
        CDMW_LOG_FUNCTION_RETURN(m_logger,
                                 NDC_STUB_CONTAINER_INTERCEPTOR_NAME);
        return NDC_STUB_CONTAINER_INTERCEPTOR_NAME;
    }


    /**
     * Purpose: <p> Ddefine the slot of the Interceptor
     *
     */
    public void set_slot_id(int slot_id)
    {
        CDMW_LOG_FUNCTION(m_logger, "slot_id [" + slot_id + "]");
        this.m_ndcSlotId = slot_id;
        CDMW_LOG_FUNCTION_RETURN(m_logger);
    }


    /**
     * Purpose: <p> Destroy the interceptor
     *
     */
    public void destroy()
        throws SystemException {
        CDMW_LOG_FUNCTION(m_logger, "");
    }


    /**
     * Purpose: <p> Return the RegistrationInfo
     *
     */
    public IntegrationPoint registration_info()
        throws SystemException {
        CDMW_LOG_FUNCTION(m_logger, "");

        return null;
    }


    /**
     * Purpose: <p> Return the priority
     *
     */
    public short priority() {
        CDMW_LOG_FUNCTION(m_logger, "");
        CDMW_LOG_FUNCTION_RETURN(m_logger, "");
        return 0;
    }
}