/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2010. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */

package cdmw.testftrecorder;

import com.acme.TestCcmFtRecorder.*;
import com.thalesgroup.CdmwRecording.*;

import cdmw.testutils.TestManager;
import org.junit.Before;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;


//
// IDL:acme.com/TestCcmFtRecorder/CCM_Client:1.0
//
public class CCMClientImpl extends org.omg.CORBA.LocalObject
    implements Session_CCM_Client 
{
    // session context
    private CCM_Client_Context sessionContext;
    
    RecordedOperationEvent lastReceivedEvent;
     
    // TestThread
    private class TestThread extends Thread {
        public TestManager testManager;
        
        TestThread(TestManager testManager) {
            this.testManager = testManager;
        }
        
        public void run() {
            this.testManager.runTests();
            this.testManager.printSummary();
        }
    }
    private TestThread testThread;
    
    
    public CCMClientImpl() 
    {
        this.sessionContext = null;
        this.testThread = null;
        this.lastReceivedEvent = null;
    }

    
    public void set_session_context(org.omg.Components.SessionContext ctx) 
    {
        System.out.println("   ... CCMClientImpl.set_session_context called!");
        this.sessionContext = CCM_Client_ContextHelper.narrow(ctx);
    }
    
    public CCM_Client_Context getContext()
    {
        return this.sessionContext;
    }
    
    
    public void ccm_activate() 
    {
        System.out.println("   ... CCMClientImpl.ccm_activate called!");

        try {
            TestManager testManager = TestManager.instance();
            TestManager.activateLocalInstance(testManager);
            
            testManager.init(
                "cdmw.testftrecorder.CCMClientImpl.CCMClientActivateTest");
            testManager.setLocalProperty(
                CCMClientActivateTest.CCM_CLIENT_IMPL_NAME,
                this);
            testManager.setLocalProperty(
                CCMClientActivateTest.SESSION_CONTEXT_NAME,
                this.sessionContext);
            testManager.addTest(CCMClientActivateTest.class);
                        
            this.testThread = new TestThread(testManager);
            this.testThread.start();
            
        } catch (Exception e) {
            System.err.println("Exception thrown during test: " + e);
        }
    }
    
    public void ccm_passivate() 
    {
        System.out.println("   ... CCMClientImpl.ccm_passivate called!");
        if (this.testThread != null) {
            try {
                this.testThread.join();
            } catch (java.lang.InterruptedException e) {
                // ignore
            }
        }
    }
    
    public void ccm_remove() 
    {
        System.out.println("   ... CCMClientImpl.ccm_remove called!");
    }
    
        
    
    // event_consumer sink
    public void push_event_consumer(RecordedOperationEvent evt)
    {
        System.out.println("   ... CCMClientImpl.push_event_consumer called!");
        System.out.println("       received " + evt.operation_name +
                           " from " + evt.recorder_name);
        this.lastReceivedEvent = evt;
    }
    
    
    //
    // CCMClientActivateTest class
    //
    public static class CCMClientActivateTest 
    {
        private static final String CCM_CLIENT_IMPL_NAME
            = "CCMClientImpl_object";

        private static final String SESSION_CONTEXT_NAME
            = "CCM_Client_Context_object";
            
        private CCMClientImpl ccmClientImpl;
        private CCM_Client_Context sessionContext;
        
        public CCMClientActivateTest()
        {
            this.ccmClientImpl = null;
            this.sessionContext = null;
        }
        
        
        @Before
        public void init () 
        {
            TestManager testManager = TestManager.instance();

            this.ccmClientImpl = 
                (CCMClientImpl)testManager.getLocalProperty(CCM_CLIENT_IMPL_NAME);
            assertTrue(this.ccmClientImpl != null);
            this.sessionContext = 
                (CCM_Client_Context)testManager.getLocalProperty(SESSION_CONTEXT_NAME);
            assertTrue(this.sessionContext != null);
        }
        
        private static final String MY_RECORDER_PRIMARY = "MyRecorder1";
        private static final String MY_RECORDER_BACKUP  = "MyRecorder2";
        private static final String OPERATION_NAME = "test_operation";        
        private static final String PARAM_A_S_VALUE = "value for 'a' param";
        private static final String PARAM_A_S_SUCIDE_VALUE = "MyRecorder: kill yourself !!";
        private static final short  PARAM_A_I_VALUE = 1;
        private static final String PARAM_B1_S_VALUE = "value for 'b' param (in)";
        private static final short  PARAM_B1_I_VALUE = 1;
        private static final String PARAM_B2_S_VALUE = "value for 'b' param (out)";
        private static final short  PARAM_B2_I_VALUE = 1;
        private static final String PARAM_C_S_VALUE = "value for 'c' param";
        private static final short  PARAM_C_I_VALUE = 1;
        private static final String PARAM_RET_S_VALUE = "value for return";
        private static final short  PARAM_RET_I_VALUE = 1;
        
        
        @org.junit.Test
        public void doTests() 
        {
            System.out.println("   ... CCMClientActivateTest.doTests called!");
            
            // wait previous connection requests to be recorded...
            try {
                cdmw.ossupport.OS.sleep(2000);
            } catch (InterruptedException e) {
                // ignore
            }
            
            // Send request to Server that should be recorded
            System.out.println("Send request to Server");
            try {
                Test test =
                    this.sessionContext.get_connection_test_receptacle();
                
                TestStruct a = new TestStruct(PARAM_A_S_VALUE, PARAM_A_I_VALUE);
                TestStructHolder b = new TestStructHolder(
                    new TestStruct(PARAM_B1_S_VALUE, PARAM_B1_I_VALUE));
                TestStructHolder c = new TestStructHolder();
                
                TestStruct ret = test.test_operation(a, b, c);
                
                assertTrue("Out value of b.s is '" + b.value.s + 
                           "' instead of '" + PARAM_B2_S_VALUE + "'",
                           b.value.s.equals(PARAM_B2_S_VALUE));
                assertTrue("Out value of b.i is '" + b.value.i + 
                           "' instead of '" + PARAM_B2_I_VALUE + "'",
                           b.value.i == PARAM_B2_I_VALUE);
                assertTrue("Out value of c.s is '" + c.value.s + 
                           "' instead of '" + PARAM_C_S_VALUE + "'",
                           c.value.s.equals(PARAM_C_S_VALUE));
                assertTrue("Out value of c.i is '" + c.value.i + 
                           "' instead of '" + PARAM_C_I_VALUE + "'",
                           c.value.i == PARAM_C_I_VALUE);
                assertTrue("Return.s value is '" + ret.s + 
                           "' instead of '" + PARAM_RET_S_VALUE + "'",
                           ret.s.equals(PARAM_RET_S_VALUE));
                assertTrue("Return.s value is '" + ret.i + 
                           "' instead of '" + PARAM_RET_I_VALUE + "'",
                           ret.i == PARAM_RET_I_VALUE);
            } catch (org.omg.CORBA.SystemException e) {
                e.printStackTrace();
                fail("Exception calling Server1's facet: " + e);
            }
            
            // wait requests to be recorded
            try {
                cdmw.ossupport.OS.sleep(5000);
            } catch (InterruptedException e) {
                // ignore
            }
            
            // check if request has been recorded
            System.out.println("Check if request has been recorded");
            assertTrue("No record has been received",
                       this.ccmClientImpl.lastReceivedEvent != null);
            assertTrue("Error at MyRecorder side: " + this.ccmClientImpl.lastReceivedEvent.operation_name,
                       this.ccmClientImpl.lastReceivedEvent.operation_name.equals(OPERATION_NAME));
            assertTrue("Operation was not recorded by primary recorder, but by " + 
                       this.ccmClientImpl.lastReceivedEvent.recorder_name,
                       this.ccmClientImpl.lastReceivedEvent.recorder_name.equals(MY_RECORDER_PRIMARY));
            assertTrue("Value of a.s is '" + this.ccmClientImpl.lastReceivedEvent.a_value.s + 
                       "' instead of '" + PARAM_A_S_VALUE + "'",
                       PARAM_A_S_VALUE.equals(
                       this.ccmClientImpl.lastReceivedEvent.a_value.s));
            assertTrue("Value of a.i is '" + this.ccmClientImpl.lastReceivedEvent.a_value.i + 
                       "' instead of '" + PARAM_A_I_VALUE + "'",
                       PARAM_A_I_VALUE == 
                       this.ccmClientImpl.lastReceivedEvent.a_value.i);
            assertTrue("Value of b1.s is '" + this.ccmClientImpl.lastReceivedEvent.b1_value.s + 
                       "' instead of '" + PARAM_B1_S_VALUE + "'",
                       PARAM_B1_S_VALUE.equals(
                       this.ccmClientImpl.lastReceivedEvent.b1_value.s));
            assertTrue("Value of b1.i is '" + this.ccmClientImpl.lastReceivedEvent.b1_value.i + 
                       "' instead of '" + PARAM_B1_I_VALUE + "'",
                       PARAM_B1_I_VALUE == 
                       this.ccmClientImpl.lastReceivedEvent.b1_value.i);
            assertTrue("Value of b2.s is '" + this.ccmClientImpl.lastReceivedEvent.b2_value.s + 
                       "' instead of '" + PARAM_B2_S_VALUE + "'",
                       PARAM_B2_S_VALUE.equals(
                       this.ccmClientImpl.lastReceivedEvent.b2_value.s));
            assertTrue("Value of b2.i is '" + this.ccmClientImpl.lastReceivedEvent.b2_value.i + 
                       "' instead of '" + PARAM_B2_I_VALUE + "'",
                       PARAM_B2_I_VALUE == 
                       this.ccmClientImpl.lastReceivedEvent.b2_value.i);
            assertTrue("Value of c.s is '" + this.ccmClientImpl.lastReceivedEvent.c_value.s + 
                       "' instead of '" + PARAM_C_S_VALUE + "'",
                       PARAM_C_S_VALUE.equals(
                       this.ccmClientImpl.lastReceivedEvent.c_value.s));
            assertTrue("Value of c.i is '" + this.ccmClientImpl.lastReceivedEvent.c_value.i + 
                       "' instead of '" + PARAM_C_I_VALUE + "'",
                       PARAM_C_I_VALUE == 
                       this.ccmClientImpl.lastReceivedEvent.c_value.i);
            assertTrue("Value of ret.s is '" + this.ccmClientImpl.lastReceivedEvent.ret_value.s + 
                       "' instead of '" + PARAM_RET_S_VALUE + "'",
                       PARAM_RET_S_VALUE.equals(
                       this.ccmClientImpl.lastReceivedEvent.ret_value.s));
            assertTrue("Value of ret.i is '" + this.ccmClientImpl.lastReceivedEvent.ret_value.i + 
                       "' instead of '" + PARAM_RET_I_VALUE + "'",
                       PARAM_RET_I_VALUE == 
                       this.ccmClientImpl.lastReceivedEvent.ret_value.i);
            // reset last received event
            this.ccmClientImpl.lastReceivedEvent = null;
            
            // get Recorder
            Recorder recorder = null;
            try {
                recorder = this.sessionContext.get_connection_recorder_receptacle();
                
                // Set up SYNC_WITH_SERVER policy on stub, since record()
                // operation is a oneway FT call (FT will work only with this policy)
                // This code should be removed when PCR-1429 is corrected

                // Dirty trick to get ORB (required for Valuetypes unmarshalling)
                org.omg.CORBA.ORB orb = ((org.omg.CORBA.portable.ObjectImpl) recorder)._orb();
                
                org.omg.CORBA.Policy[] syncScopePolicyList = 
                    new org.omg.CORBA.Policy[1];
                org.omg.CORBA.Any syncScopeAny = orb.create_any();
                syncScopeAny.insert_short(
                    org.omg.Messaging.SYNC_WITH_SERVER.value);
                syncScopePolicyList[0] = orb.create_policy(
                    org.omg.Messaging.SYNC_SCOPE_POLICY_TYPE.value, 
                    syncScopeAny);
    
                // Set the sync scope policy at the object level.
                org.omg.CORBA.Object obj = recorder._set_policy_override(
                    syncScopePolicyList,
                    org.omg.CORBA.SetOverrideType.ADD_OVERRIDE);
                recorder = RecorderHelper.narrow(obj);
                
            } catch (org.omg.CORBA.PolicyError e) {
                e.printStackTrace();
                fail("Exception getting recorder connection: " + e);
            } catch (org.omg.CORBA.SystemException e) {
                e.printStackTrace();
                fail("Exception getting recorder connection: " + e);
            }
            
            // directly record an request into Recorder
            System.out.println("Directly record a Request");
            try {
                GroupHeader header = new GroupHeader();
                header.base_time = 0; 
                header.hostname = cdmw.ossupport.OS.getHostname();
                header.flags = 0;
                header.num_pairs = 1;
                
                // fill Request struct
                RequestReplyPair pair = new RequestReplyPair();
                pair.request = new GIOP_BufferRequestComponent();
                pair.request.head = new GIOP_Header();
                pair.request.head.magic = 0;
                pair.request.head.GIOP_version = new Version((byte)1, (byte)2);
                pair.request.head.flags = 0;
                pair.request.head.message_type = 0;
                pair.request.head.message_size = 0;
                pair.request.server_id = "Client";
                pair.request.request_id = 55;
                pair.request.date = 0;
                pair.request.flags = 0;
                pair.request.endian = true;
                pair.request.interface_rep_id = "IDL:acme.com/TestCcmFtRecorder/Test:1.0";
                pair.request.operation = OPERATION_NAME;
                pair.request.service_contexts = new ServiceContext[0];

                // fill Request's body with a (in) and b (inout) attributes
                org.jacorb.orb.CDROutputStream ouput1 =
                    new org.jacorb.orb.CDROutputStream(org.omg.CORBA.ORB.init());
                TestStruct a = new TestStruct(PARAM_A_S_VALUE, PARAM_A_I_VALUE);
                TestStructHelper.write(ouput1, a);
                TestStruct b1 = new TestStruct(PARAM_B1_S_VALUE, PARAM_B1_I_VALUE);
                TestStructHelper.write(ouput1, b1);
                
                pair.request.data = ouput1.getBufferCopy();

                // fill Reply struct
                pair.reply = new GIOP_BufferReplyComponent();
                pair.reply.head = new GIOP_Header();
                pair.reply.head.magic = 0;
                pair.reply.head.GIOP_version = new Version((byte)1, (byte)2);
                pair.reply.head.flags = 0;
                pair.reply.head.message_type = 0;
                pair.reply.head.message_size = 0;
                pair.reply.request_id = pair.request.request_id;
                pair.reply.date = 0;
                pair.reply.endian = true;
                pair.reply.status = org.omg.GIOP.ReplyStatusType_1_2.NO_EXCEPTION.value();
                pair.reply.service_contexts = new ServiceContext[0];
                
                // fill Reply's body with return,  b (inout) and c (out) attributes
                org.jacorb.orb.CDROutputStream ouput2 =
                    new org.jacorb.orb.CDROutputStream(org.omg.CORBA.ORB.init());
                TestStruct ret = new TestStruct(PARAM_RET_S_VALUE, PARAM_RET_I_VALUE);
                TestStructHelper.write(ouput2, ret);
                TestStruct b2 = new TestStruct(PARAM_B2_S_VALUE, PARAM_B2_I_VALUE);
                TestStructHelper.write(ouput2, b2);
                TestStruct c = new TestStruct(PARAM_C_S_VALUE, PARAM_C_I_VALUE);
                TestStructHelper.write(ouput2, c);
                
                pair.reply.data = ouput2.getBufferCopy();
                
                // marshall RequestReplyPair
                org.jacorb.orb.CDROutputStream ouput3 =
                    new org.jacorb.orb.CDROutputStream(org.omg.CORBA.ORB.init());
                RequestReplyPairHelper.write(ouput3, pair);
                byte[] data = ouput3.getBufferCopy();
                
                // check record call is asynchronous: 
                // get time before call and after call.
                // since MyRecorder blocks 2 seconds in record operation,
                // time spent for call should be less than 2 seconds
                long timeBefore = System.currentTimeMillis();
                
                // send record
                recorder.record(header, data);
                
                long timeAfter = System.currentTimeMillis();
                
                assertTrue("Record request was not asynchronous, it spent more " +
                           "than 2 seconds: " + (timeAfter-timeBefore) + "ms",
                           (timeAfter-timeBefore) < 2000);
                
                
            } catch (org.omg.CORBA.SystemException e) {
                e.printStackTrace();
                fail("Exception calling record operation: " + e);
            }
            
            
            try {
                cdmw.ossupport.OS.sleep(4000);
            } catch (InterruptedException e) {
                // ignore
            }
            
            // check if request has been recorded
            System.out.println("Check if request has been recorded");
            assertTrue("No record has been received",
                       this.ccmClientImpl.lastReceivedEvent != null);
            assertTrue("Error at MyRecorder side: " + this.ccmClientImpl.lastReceivedEvent.operation_name,
                       this.ccmClientImpl.lastReceivedEvent.operation_name.equals(OPERATION_NAME));
            assertTrue("Operation was not recorded by primary recorder, but by " + 
                       this.ccmClientImpl.lastReceivedEvent.recorder_name,
                       this.ccmClientImpl.lastReceivedEvent.recorder_name.equals(MY_RECORDER_PRIMARY));
            assertTrue("Value of a.s is '" + this.ccmClientImpl.lastReceivedEvent.a_value.s + 
                       "' instead of '" + PARAM_A_S_VALUE + "'",
                       PARAM_A_S_VALUE.equals(
                       this.ccmClientImpl.lastReceivedEvent.a_value.s));
            assertTrue("Value of a.i is '" + this.ccmClientImpl.lastReceivedEvent.a_value.i + 
                       "' instead of '" + PARAM_A_I_VALUE + "'",
                       PARAM_A_I_VALUE == 
                       this.ccmClientImpl.lastReceivedEvent.a_value.i);
            assertTrue("Value of b1.s is '" + this.ccmClientImpl.lastReceivedEvent.b1_value.s + 
                       "' instead of '" + PARAM_B1_S_VALUE + "'",
                       PARAM_B1_S_VALUE.equals(
                       this.ccmClientImpl.lastReceivedEvent.b1_value.s));
            assertTrue("Value of b1.i is '" + this.ccmClientImpl.lastReceivedEvent.b1_value.i + 
                       "' instead of '" + PARAM_B1_I_VALUE + "'",
                       PARAM_B1_I_VALUE == 
                       this.ccmClientImpl.lastReceivedEvent.b1_value.i);
            assertTrue("Value of b2.s is '" + this.ccmClientImpl.lastReceivedEvent.b2_value.s + 
                       "' instead of '" + PARAM_B2_S_VALUE + "'",
                       PARAM_B2_S_VALUE.equals(
                       this.ccmClientImpl.lastReceivedEvent.b2_value.s));
            assertTrue("Value of b2.i is '" + this.ccmClientImpl.lastReceivedEvent.b2_value.i + 
                       "' instead of '" + PARAM_B2_I_VALUE + "'",
                       PARAM_B2_I_VALUE == 
                       this.ccmClientImpl.lastReceivedEvent.b2_value.i);
            assertTrue("Value of c.s is '" + this.ccmClientImpl.lastReceivedEvent.c_value.s + 
                       "' instead of '" + PARAM_C_S_VALUE + "'",
                       PARAM_C_S_VALUE.equals(
                       this.ccmClientImpl.lastReceivedEvent.c_value.s));
            assertTrue("Value of c.i is '" + this.ccmClientImpl.lastReceivedEvent.c_value.i + 
                       "' instead of '" + PARAM_C_I_VALUE + "'",
                       PARAM_C_I_VALUE == 
                       this.ccmClientImpl.lastReceivedEvent.c_value.i);
            assertTrue("Value of ret.s is '" + this.ccmClientImpl.lastReceivedEvent.ret_value.s + 
                       "' instead of '" + PARAM_RET_S_VALUE + "'",
                       PARAM_RET_S_VALUE.equals(
                       this.ccmClientImpl.lastReceivedEvent.ret_value.s));
            assertTrue("Value of ret.i is '" + this.ccmClientImpl.lastReceivedEvent.ret_value.i + 
                       "' instead of '" + PARAM_RET_I_VALUE + "'",
                       PARAM_RET_I_VALUE == 
                       this.ccmClientImpl.lastReceivedEvent.ret_value.i);
            // reset last received event
            this.ccmClientImpl.lastReceivedEvent = null;

            
            // directly record an request into Recorder,
            // but this request will provoke primary Recorder to suicide.
            System.out.println("Directly record a Request which will provoke primary MyRepcorder to crash");
            try {
                GroupHeader header = new GroupHeader();
                header.base_time = 0; 
                header.hostname = cdmw.ossupport.OS.getHostname();
                header.flags = 0;
                header.num_pairs = 1;
                
                // fill Request struct
                RequestReplyPair pair = new RequestReplyPair();
                pair.request = new GIOP_BufferRequestComponent();
                pair.request.head = new GIOP_Header();
                pair.request.head.magic = 0;
                pair.request.head.GIOP_version = new Version((byte)1, (byte)2);
                pair.request.head.flags = 0;
                pair.request.head.message_type = 0;
                pair.request.head.message_size = 0;
                pair.request.server_id = "Client";
                pair.request.request_id = 55;
                pair.request.date = 0;
                pair.request.flags = 0;
                pair.request.endian = true;
                pair.request.interface_rep_id = "IDL:acme.com/TestCcmFtRecorder/Test:1.0";
                pair.request.operation = OPERATION_NAME;
                pair.request.service_contexts = new ServiceContext[0];

                // fill Request's body with a (in) and b (inout) attributes
                org.jacorb.orb.CDROutputStream ouput1 =
                    new org.jacorb.orb.CDROutputStream(org.omg.CORBA.ORB.init());
                TestStruct a = new TestStruct(PARAM_A_S_SUCIDE_VALUE, PARAM_A_I_VALUE);
                TestStructHelper.write(ouput1, a);
                TestStruct b1 = new TestStruct(PARAM_B1_S_VALUE, PARAM_B1_I_VALUE);
                TestStructHelper.write(ouput1, b1);
                
                pair.request.data = ouput1.getBufferCopy();

                // fill Reply struct
                pair.reply = new GIOP_BufferReplyComponent();
                pair.reply.head = new GIOP_Header();
                pair.reply.head.magic = 0;
                pair.reply.head.GIOP_version = new Version((byte)1, (byte)2);
                pair.reply.head.flags = 0;
                pair.reply.head.message_type = 0;
                pair.reply.head.message_size = 0;
                pair.reply.request_id = pair.request.request_id;
                pair.reply.date = 0;
                pair.reply.endian = true;
                pair.reply.status = org.omg.GIOP.ReplyStatusType_1_2.NO_EXCEPTION.value();
                pair.reply.service_contexts = new ServiceContext[0];
                
                // fill Reply's body with return,  b (inout) and c (out) attributes
                org.jacorb.orb.CDROutputStream ouput2 =
                    new org.jacorb.orb.CDROutputStream(org.omg.CORBA.ORB.init());
                TestStruct ret = new TestStruct(PARAM_RET_S_VALUE, PARAM_RET_I_VALUE);
                TestStructHelper.write(ouput2, ret);
                TestStruct b2 = new TestStruct(PARAM_B2_S_VALUE, PARAM_B2_I_VALUE);
                TestStructHelper.write(ouput2, b2);
                TestStruct c = new TestStruct(PARAM_C_S_VALUE, PARAM_C_I_VALUE);
                TestStructHelper.write(ouput2, c);
                
                pair.reply.data = ouput2.getBufferCopy();
                
                // marshall RequestReplyPair
                org.jacorb.orb.CDROutputStream ouput3 =
                    new org.jacorb.orb.CDROutputStream(org.omg.CORBA.ORB.init());
                RequestReplyPairHelper.write(ouput3, pair);
                byte[] data = ouput3.getBufferCopy();
                
                // check record call is asynchronous: 
                // get time before call and after call.
                // since MyRecorder blocks 2 seconds in record operation,
                // time spent for call should be less than 2 seconds
                long timeBefore = System.currentTimeMillis();
                
                // send record, and check that Recorder failure don't block
                // the client.
                recorder.record(header, data);
                
                long timeAfter = System.currentTimeMillis();
                
                assertTrue("Record request was not asynchronous, it spent more " +
                           "than 2 seconds: " + (timeAfter-timeBefore) + "ms",
                           (timeAfter-timeBefore) < 2000);
                
            } catch (org.omg.CORBA.SystemException e) {
                e.printStackTrace();
                fail("Exception calling record operation: " + e);
            }
            
            try {
                cdmw.ossupport.OS.sleep(4000);
            } catch (InterruptedException e) {
                // ignore
            }

/************ PCR-1431: comment following check because of JacORB bug

            // check if request has been recorded
            System.out.println("Check if request has been recorded");
            assertTrue("No record has been received",
                       this.ccmClientImpl.lastReceivedEvent != null);
            assertTrue("Error at MyRecorder side: " + this.ccmClientImpl.lastReceivedEvent.operation_name,
                       this.ccmClientImpl.lastReceivedEvent.operation_name.equals(OPERATION_NAME));
            assertTrue("Operation was not recorded by backup recorder, but by " + 
                       this.ccmClientImpl.lastReceivedEvent.recorder_name,
                       this.ccmClientImpl.lastReceivedEvent.recorder_name.equals(MY_RECORDER_BACKUP));
            assertTrue("Value of a.s is '" + this.ccmClientImpl.lastReceivedEvent.a_value.s + 
                       "' instead of '" + PARAM_A_S_SUCIDE_VALUE + "'",
                       PARAM_A_S_VALUE.equals(
                       this.ccmClientImpl.lastReceivedEvent.a_value.s));
            assertTrue("Value of a.i is '" + this.ccmClientImpl.lastReceivedEvent.a_value.i + 
                       "' instead of '" + PARAM_A_I_VALUE + "'",
                       PARAM_A_I_VALUE == 
                       this.ccmClientImpl.lastReceivedEvent.a_value.i);
            assertTrue("Value of b1.s is '" + this.ccmClientImpl.lastReceivedEvent.b1_value.s + 
                       "' instead of '" + PARAM_B1_S_VALUE + "'",
                       PARAM_B1_S_VALUE.equals(
                       this.ccmClientImpl.lastReceivedEvent.b1_value.s));
            assertTrue("Value of b1.i is '" + this.ccmClientImpl.lastReceivedEvent.b1_value.i + 
                       "' instead of '" + PARAM_B1_I_VALUE + "'",
                       PARAM_B1_I_VALUE == 
                       this.ccmClientImpl.lastReceivedEvent.b1_value.i);
            assertTrue("Value of b2.s is '" + this.ccmClientImpl.lastReceivedEvent.b2_value.s + 
                       "' instead of '" + PARAM_B2_S_VALUE + "'",
                       PARAM_B2_S_VALUE.equals(
                       this.ccmClientImpl.lastReceivedEvent.b2_value.s));
            assertTrue("Value of b2.i is '" + this.ccmClientImpl.lastReceivedEvent.b2_value.i + 
                       "' instead of '" + PARAM_B2_I_VALUE + "'",
                       PARAM_B2_I_VALUE == 
                       this.ccmClientImpl.lastReceivedEvent.b2_value.i);
            assertTrue("Value of c.s is '" + this.ccmClientImpl.lastReceivedEvent.c_value.s + 
                       "' instead of '" + PARAM_C_S_VALUE + "'",
                       PARAM_C_S_VALUE.equals(
                       this.ccmClientImpl.lastReceivedEvent.c_value.s));
            assertTrue("Value of c.i is '" + this.ccmClientImpl.lastReceivedEvent.c_value.i + 
                       "' instead of '" + PARAM_C_I_VALUE + "'",
                       PARAM_C_I_VALUE == 
                       this.ccmClientImpl.lastReceivedEvent.c_value.i);
            assertTrue("Value of ret.s is '" + this.ccmClientImpl.lastReceivedEvent.ret_value.s + 
                       "' instead of '" + PARAM_RET_S_VALUE + "'",
                       PARAM_RET_S_VALUE.equals(
                       this.ccmClientImpl.lastReceivedEvent.ret_value.s));
            assertTrue("Value of ret.i is '" + this.ccmClientImpl.lastReceivedEvent.ret_value.i + 
                       "' instead of '" + PARAM_RET_I_VALUE + "'",
                       PARAM_RET_I_VALUE == 
                       this.ccmClientImpl.lastReceivedEvent.ret_value.i);
            // reset last received event
            this.ccmClientImpl.lastReceivedEvent = null;
************/
            
        }
        
    }
    
}

